; File: HV_2ds_over_lap_9_regionsG.pro
; Author:  Qixo Mo, modified by Ted Fisher <ted@specinc.com> <fishnuke@gmail.com>
; Purpose:  plots coincidense maps for the 2D-S and 3V-CPI probes made by SPEC Inc.  These plots show the
;   time difference between when a particle passes through each beam, which is used for alignment and evaluation.

;   Takes an array of the overlap timing in nanoseconds.  NaN is used as a flag, and is outputted to the screen as "unknown"






pro HV_2ds_over_lap_9_regionsG, data=data, title=title, bead_size=bead_size, outfile=outfile

    window,1, xs=750,ys=750
    !P.thick=1
    !P.CHARSIZE=3.0
    !P.multi=[0,1,1]
    loadct,0
    white=!D.N_COLORS-1
    !P.BACKGROUND=white

    Xsize=128
    Ysize=128
    x=[-128,-128,128,128,-128]
    y=[-128,128,128,-128,-128]

    plot,x,y,color=0,position=[0.2,0.2,0.8,0.8],xrange=[-140,140],yrange=[-140,140],$

        xticks=2,xtickname=[' ',' ',' '],yticks=2,ytickname=[' ',' ',' ']

    area0x=[2,2,82,82]
    area0y=[0,80,80,0]

    ;plot region 11
    polyfill,area0x-Xsize,area0y-80+1*Ysize,color=255
    ;plot region 12
    polyfill,area0x-42,area0y-80+1*Ysize,color=255
    ;plot region 13
    polyfill,area0x-84+Xsize,area0y-80+1*Ysize,color=255

    ;plot region 21
    polyfill,area0x-Xsize,area0y-80+0.5*Ysize-20,color=255
    ;plot region 22
    polyfill,area0x-42,area0y-80+0.5*Ysize-20,color=255
    ;plot region 23
    polyfill,area0x-84+Xsize,area0y-80+0.5*Ysize-20,color=255


    ;plot region 31
    polyfill,area0x-Xsize,area0y-60-0.5*Ysize,color=255
    ;plot region 32
    polyfill,area0x-42,area0y-60-0.5*Ysize,color=255
    ;plot region 33
    polyfill,area0x-84+Xsize,area0y-60-0.5*Ysize,color=255

    ;polyfill,[52,52,128,128],[-128,-52,-52,-128],color=255
    ;xyouts,-112,128,'7',charsize=0.8,color=0
    ;xyouts,-82,128,'27',charsize=0.8,color=0
    ;xyouts,-20,128,'54',charsize=0.8,color=0
    ;xyouts, 16,128,'74',charsize=0.8,color=0
    ;xyouts, 71,128,'100',charsize=0.8,color=0
    ;xyouts, 102,128,'120',charsize=0.8,color=0
    ;xyouts, 122,128,'128',charsize=0.8,color=0

    ;xyouts,-125,70,'27',charsize=0.8,color=0
    ;xyouts,-125,20,'54',charsize=0.8,color=0
    ;xyouts,-125,-20,'74',charsize=0.8,color=0
    ;xyouts,-127,-80,'100',charsize=0.8,color=0
    ;xyouts,-127,-118,'120',charsize=0.8,color=0

    ; row1 ticks
    ;xyouts, -100-20,90,'7.0',color=0,charthick=3,charsize=2.4
    ;xyouts, -5-20,  90,'6.0',color=0,charthick=3,charsize=2.4
    ;xyouts,  90-28, 90,'7.0',color=0,charthick=3,charsize=2.4

    ;row 2 ticks
    ;xyouts, -100-20,-5,'7.0',color=0,charthick=3,charsize=2.4
    ;xyouts,  -5-20, -5,'7.0',color=0,charthick=3,charsize=2.4
    ;xyouts,  90-28, -5,'7.0',color=0,charthick=3,charsize=2.4

    ; row 3 ticks
    ;xyouts, -100-20,-100,'7.0',color=0,charthick=3,charsize=2.4
    ;xyouts, -5-20,  -100,'7.0',color=0,charthick=3,charsize=2.4
    ;xyouts,  90-28, -100,'8.0',color=0,charthick=3,charsize=2.4

    ;xyouts,190,548,'2DS-9 Section H-V Beam Offset',color=0,charthick=2,charsize=2.0,/device
    ;xyouts,220,200,' Using 50um bead test',color=0,charthick=2,charsize=2.0,/device

    ;write_png,'C:\Fig9_region_overlap_separation_50um.png',tvrd(true=1)


    if 1 eq 2 then begin
        ; row1 ticks
        xyouts, -100-20,90,strcompress(string(data[0,0], format='(%"%5.1f")'), /rem),color=0,charthick=3,charsize=2.4
        xyouts, -5-20,  90,strcompress(string(data[0,1], format='(%"%5.1f")'), /rem),color=0,charthick=3,charsize=2.4
        xyouts,  90-28, 90,strcompress(string(data[0,2], format='(%"%5.1f")'), /rem),color=0,charthick=3,charsize=2.4

        ;row 2 ticks
        xyouts, -100-20,-5,strcompress(string(data[1,0], format='(%"%5.1f")'), /rem),color=0,charthick=3,charsize=2.4
        xyouts,  -5-20, -5,strcompress(string(data[1,1], format='(%"%5.1f")'), /rem),color=0,charthick=3,charsize=2.4
        xyouts,  90-28, -5,strcompress(string(data[1,2], format='(%"%5.1f")'), /rem),color=0,charthick=3,charsize=2.4

        ; row 3 ticks
        xyouts, -100-20,-100,strcompress(string(data[2,0], format='(%"%5.1f")'), /rem),color=0,charthick=3,charsize=2.4
        xyouts, -5-20,  -100,strcompress(string(data[2,1], format='(%"%5.1f")'), /rem),color=0,charthick=3,charsize=2.4
        xyouts,  90-28, -100,strcompress(string(data[2,2], format='(%"%5.1f")'), /rem),color=0,charthick=3,charsize=2.4
    endif


    if 1 eq 1 then begin
        ; row1 ticks
        xyouts, -100-20,90,strcompress(string(data[0,0], format='(%"%d")'), /rem),color=0,charthick=3,charsize=2.4
        xyouts, -5-20,  90,strcompress(string(data[0,1], format='(%"%d")'), /rem),color=0,charthick=3,charsize=2.4
        xyouts,  90-28, 90,strcompress(string(data[0,2], format='(%"%d")'), /rem),color=0,charthick=3,charsize=2.4

        ;row 2 ticks
        xyouts, -100-20,-5,strcompress(string(data[1,0], format='(%"%d")'), /rem),color=0,charthick=3,charsize=2.4
        xyouts,  -5-20, -5,strcompress(string(data[1,1], format='(%"%d")'), /rem),color=0,charthick=3,charsize=2.4
        xyouts,  90-28, -5,strcompress(string(data[1,2], format='(%"%d")'), /rem),color=0,charthick=3,charsize=2.4

        ; row 3 ticks
        xyouts, -100-20,-100,strcompress(string(data[2,0], format='(%"%d")'), /rem),color=0,charthick=3,charsize=2.4
        xyouts, -5-20,  -100,strcompress(string(data[2,1], format='(%"%d")'), /rem),color=0,charthick=3,charsize=2.4
        xyouts,  90-28, -100,strcompress(string(data[2,2], format='(%"%d")'), /rem),color=0,charthick=3,charsize=2.4
    endif




    ; row1 ticks
    ;xyouts, -100-20,90,strcompress(string(data[0,0]), /rem),color=0,charthick=3,charsize=2.4
    ;xyouts, -5-20,  90,strcompress(string(data[0,1]), /rem),color=0,charthick=3,charsize=2.4
    ;xyouts,  90-28, 90,strcompress(string(data[0,2]), /rem),color=0,charthick=3,charsize=2.4

    ;row 2 ticks
    ;xyouts, -100-20,-5,strcompress(string(data[1,0]), /rem),color=0,charthick=3,charsize=2.4
    ;xyouts,  -5-20, -5,strcompress(string(data[1,1]), /rem),color=0,charthick=3,charsize=2.4
    ;xyouts,  90-28, -5,strcompress(string(data[1,2]), /rem),color=0,charthick=3,charsize=2.4

    ; row 3 ticks
    ;xyouts, -100-20,-100,strcompress(string(data[2,0]), /rem),color=0,charthick=3,charsize=2.4
    ;xyouts, -5-20,  -100,strcompress(string(data[2,1]), /rem),color=0,charthick=3,charsize=2.4
    ;xyouts,  90-28, -100,strcompress(string(data[2,2]), /rem),color=0,charthick=3,charsize=2.4

    ; Graph Title title='2DS-9 Section H-V Beam Offset'
    xyouts,190,548,title,color=0,charthick=2,charsize=2.0,/device
    xyouts,220,200,' Using ' + strcompress(string(bead_size), /rem) + 'um bead test',color=0,charthick=2,charsize=2.0,/device

    write_png,outfile,tvrd(true=1)


    ;stop

end

