pro form_hist,pState
    ; procedure to create hist data from raw, 1 sec data

    ; new pointer
    pData = (*pState).pdata1

    ; 2DC Data
    ; define bins for 2DC spectrum

    TDCBins = (*pState).bins.twodc_bin
    TDCBinCnt = n_elements(TDCBins)
    SYS_time = *(*pData).timesys
    fsspBinCnt = (size(*(*pData).specfsp,/dim))[0]
    fsspBins = (*pState).bins.fssp_bin
    sysTCnt = n_elements(SYS_time)

    cpi_time = *(*(*pState).pdata1).cpi_time

    SYS_time = SYS_time + (cpi_time[0] - SYS_time[0])

    Bin_width = (*(*pState).pdata1).bin_width

    ; CPI data
    ; now modify spectral concentration files from knowledge of total concentration
    len = (size((*(*pData).cpi_concDT),/dim))[0]
    cpi_time = *(*pData).cpi_time
    Bin_width = (*pData).bin_width
    nSecs = (size(*(*pData).pSizeData,/dim))[0]
    cpiMaxBin = n_elements(*(*(*pState).pData1).cpi_bins) * Bin_width
    CPIBinCnt = ceil(cpiMaxBin/bin_width)
    spec_cpi = lonarr(CPIBinCnt,nSecs)
    Maxspec_cpi = lonarr(CPIBinCnt,nSecs)
    cpiMinBin = 5.0
    cpiBins = *(*(*pState).pData1).cpi_bins ; findgen(CPIBinCnt)*BIN_WIDTH + cpiMinBin

    for k = 0, nSecs-1 do begin
        pMeansizes = (*( * pData).pSizeData)[k]
        pMaxsizes = (*( * pData).pMaxSizeData)[k]
        if ptr_valid(pMeansizes) then begin
            spectrum = histogram(*pMeansizes,binsize = bin_width, min = cpiMinBin, max = cpiMaxBin)
            spec_cpi[0:n_elements(spectrum)-1,k] = spectrum
            Maxspectrum = histogram(*pMaxsizes,binsize = bin_width, min = cpiMinBin, max = cpiMaxBin)
            Maxspec_cpi[0:n_elements(Maxspectrum)-1,k] = Maxspectrum
        endif
    endfor

    ;define arrays
    spec_cpiDT = fltarr(CPIBinCnt,nSecs)
    spec_cpiTS = fltarr(CPIBinCnt,nSecs)
    Maxspec_cpiDT = fltarr(CPIBinCnt,nSecs)
    Maxspec_cpiTS = fltarr(CPIBinCnt,nSecs)

    ; scale to concentrations
    for i=0,len-1 do begin
        d = (total(spec_cpi[*,i])*Bin_width)

        if(d ne 0)then begin
            scale_factor1 = (*(*pData).cpi_concDT)[i]/d	; bin/(liter*micron)
            scale_factor2 = (*(*pData).cpi_concTS)[i]/d	; bin/(liter*micron)
            spec_cpiDT[*,i] = spec_cpi[*,i]*scale_factor1; #/liter/micron
            spec_cpiTS[*,i] = spec_cpi[*,i]*scale_factor2; #/liter/micron
            Maxspec_cpiDT[*,i] = Maxspec_cpi[*,i]*scale_factor1; #/liter/micron
            Maxspec_cpiTS[*,i] = Maxspec_cpi[*,i]*scale_factor2; #/liter/micron


        endif else begin
            spec_cpiDT[*,i] = spec_cpi[*,i]	; all zeros
            spec_cpiTS[*,i] = spec_cpi[*,i]
            Maxspec_cpiDT[*,i] = Maxspec_cpi[*,i]	; all zeros
            Maxspec_cpiTS[*,i] = Maxspec_cpi[*,i]
        endelse

    endfor

    ; find region of overlap
    dc_min = min(SYS_time,max = dc_max)
    cpi_min = min(cpi_time,max=cpi_max)
    if(dc_min le cpi_min)then begin
        ; SEA data starts first
        ;	SEA data |-----------------------------------|
        ;	CPI data	 |----------------------------------|
        sea_lead = 1
        if(cpi_min ge dc_max)then begin
            r=dialog_message('Datasets DO NOT overlap in time! This is not supported yet. Choose different datasets.',/info)
            return
        endif
        t_min = cpi_min
        if(cpi_max le dc_max)then t_max = cpi_max else t_max = dc_max
    endif else begin
        ; CPI data starts first
        ;	CPI data |-----------------------------------|
        ;	SEA data	 |----------------------------------|
        sea_lead = 0
        if(dc_min ge cpi_max)then begin
            r=dialog_message('Datasets DO NOT overlap in time! This is not supported yet. Choose different datasets.',/info)
            return
        endif
        t_min = dc_min
        if(dc_max le cpi_max)then t_max = dc_max else t_max = cpi_max
    endelse

    ;              t_min
    ;	SYS_time  |---X--------------------------------|
    ;	cpi_time	 |--------------------------------X--|
    ;												t_max
    ;						-OR-
    ;              t_min
    ;	CPI_time |---X--------------------------------|
    ;	SYS_time 	 |--------------------------------X--|
    ;												t_max

    ; this time defines the range of x axis and number of plots to show
    repeat begin
        time=[t_min/3600.,t_max/3600.]
        get_time1,time,/valid,/average
        time[0:1] = time[0:1]*3600.
    endrep until(time[0] ge t_min and time[1] le t_max)

    ;              t_min  time[0]		   time[1]
    ;	    data1|---X-----X------------------X---------|
    ;	    data2	 |-----X------------------X---------X--|
    ;												   t_max

    ; choose a time series based on SYS_time (could use either SYS_time or cpi_time here)
    sea_ind = where(SYS_time ge time[0] and SYS_time le time[1],cnt)
    cpi_ind = where(cpi_time ge time[0] and cpi_time le time[1])

    ;              t_min  time[0]		   time[1]
    ;	    data1|---X-----X------------------X---------|
    ;	    data2	 |-----X------------------X---------X--|
    ;												   t_max
    ; 	   SYS_time[ind] =  |------------------|

    if(time[2] lt 1)then begin
        r=dialog_message('Error: minimum time resolution is 1 sec. avergaging period adjusted',/info)
        time[2] = 1.0
    endif

    sec_avg = float(time[2])
    sea_sec_avg = sec_avg

    ; tstep is the number of steps for this averaging period
    tstep = round((time[1]-time[0])/sec_avg)
    time_range = sec_avg*findgen(tstep)/3600. + time[0]/3600.0
    dim = size(spec_cpi,/dimensions)

    ; average down to sec_avg sec steps
    cpiDTvol = fltarr(tstep)
    cpiHistDT = fltarr(dim[0],tstep)
    cpiHistTS = fltarr(dim[0],tstep)
    MaxcpiHistDT = fltarr(dim[0],tstep)
    MaxcpiHistTS = fltarr(dim[0],tstep)
    fsspHist = fltarr(fsspBinCnt,tstep)
    ;spp_spec = fltarr(20,tstep)
    TDCHist = fltarr(TDCBinCnt,tstep)
    conc = fltarr(tstep)

    max_dim_sea = (size(*(*(*pState).pdata1).specfsp,/dim))[1] - 1
    max_dim_cpi = dim[1] - 1

    ;perform averaging over user specified time period
    for i = 0, tstep-1 do begin
        sea_start = fix(i*sea_sec_avg + sea_ind[0])
        sea_stop  = fix(sea_start + sea_sec_avg - 1)
        cpi_start = fix(i*sec_avg + cpi_ind[0])
        cpi_stop  = fix(cpi_start + sec_avg - 1)

        sea_stop = sea_stop < max_dim_sea
        fsspHist[*,i] = congrid( (*(*pData).specfsp)[*,sea_start:sea_stop],fsspBinCnt,1)/sea_sec_avg/3.0	;bin width

        if((*pState).ver eq 99) then sppHist = congrid((*(*pData).SPP100Conc)[*,sea_start:sea_stop])/sea_sec_avg/3.0	;bin width
        TDCHist[*,i] = congrid((*(*pData).spec2dc)[*,sea_start:sea_stop],TDCBinCnt, 1)/sea_sec_avg/(TDCBinCnt)	;bin width

        conc[i] = total( (*(*(*pState).pdata1).conc_2dc)[sea_start:sea_stop] )/sea_sec_avg
        cpi_stop = cpi_stop < max_dim_cpi
        cpiDTvol[i] = total((*(*(*pState).pdata1).cpi_DTvol)[cpi_start:cpi_stop])
        cpiHistDT[*,i] = congrid(spec_cpiDT[*,cpi_start:cpi_stop],CPIBinCnt, 1)/sec_avg
        cpiHistTS[*,i] = congrid(spec_cpiTS[*,cpi_start:cpi_stop],CPIBinCnt, 1)/sec_avg
        MaxcpiHistDT[*,i] = congrid(Maxspec_cpiDT[*,cpi_start:cpi_stop],CPIBinCnt, 1)/sec_avg
        MaxcpiHistTS[*,i] = congrid(Maxspec_cpiTS[*,cpi_start:cpi_stop],CPIBinCnt, 1)/sec_avg

    endfor

    ; determine max and min Y
    (*pState).hist_data.yMax = max([max(cpiHistDT),max(cpiHistTS),max(fsspHist),max(TDCHist)])
    (*pState).hist_data.yMin = Min([Min(cpiHistDT),Min(cpiHistTS),Min(fsspHist),Min(TDCHist)]) > .0001

    ; save data
    (*pState).hist_data.pCpiDTvol = ptr_new(cpiDTvol,/no_copy)
    (*pState).hist_data.pCpiHistDT = ptr_new(CpiHistDT,/no_copy)
    (*pState).hist_data.pCpiHistTS = ptr_new(CpiHistTS,/no_copy)
    (*pState).hist_data.pCpiHistMaxDT = ptr_new(pCpiHistMaxDT,/no_copy)
    (*pState).hist_data.pCpiHistMaxTS = ptr_new(pCpiHistMaxTS,/no_copy)
    (*pState).hist_data.pFsspHist = ptr_new(FsspHist,/no_copy)
    (*pState).hist_data.pTDHist = ptr_new(TDCHist,/no_copy)

    (*pState).hist_data.pTime_range = ptr_new(time_range,/no_copy)

end
