; File: make_cpi_size_scale.pro
; Purpose:  generates a raster size scale for the CPI png pages




pro make_cpi_size_scale, image=image, scale_length=scale_length, endbar_length=endbar_length, $
    microns_per_pixel=microns_per_pixel, background_color=background_color, bar_color=bar_color, $
    draw_vertical=draw_vertical

; define the dimensions of the scale image
xsize = 115
ysize = 25

; scale length to display in microns
if(not keyword_set(scale_length)) then begin
    scale_length = 200
endif

; height of the end bars on the scale bar
if(not keyword_set(endbar_length)) then begin
    endbar_length = 6 ; length (height) of the end bars for the length scale
endif

if(not keyword_set(microns_per_pixel)) then begin
    ; make sure the microns per pixel system variable is defined
    defsysv, '!CONST_CPI_MICRONS_PER_PIXEL', exists=exists

    if(~exists) then begin
        defsysv, '!CONST_CPI_MICRONS_PER_PIXEL', 2.2 ; microns per pixel
    endif

    microns_per_pixel = !CONST_CPI_MICRONS_PER_PIXEL
endif

if(not keyword_set(background_color)) then begin
    background_color = 255 ; white using color table 39
endif

if(not keyword_set(bar_color)) then begin
    bar_color =   0 ; black using color table 39
endif

if(not keyword_set(draw_vertical)) then begin
    draw_vertical = 0
endif


xcenter = floor(xsize/2.)
ycenter = floor(ysize/2.)


; determine the length of the scale bar in pixels and calculate end points in image
bar_length = fix(scale_length / microns_per_pixel)

left_end_pix = fix((xsize - bar_length) / 2.)
right_end_pix = left_end_pix + bar_length

; determine the length of the endbars in pixels and calculate end points in image
barlo = fix((ysize - endbar_length) / 2.)
barhi = barlo + endbar_length


; draw the scale bar in the image
image = bytarr(xsize, ysize)

image[*] = background_color

image[left_end_pix:right_end_pix, ycenter] = bar_color

image[ left_end_pix, barlo:barhi] = bar_color
image[right_end_pix, barlo:barhi] = bar_color


if(draw_vertical) then begin
    image = transpose(image)
endif
end ; make_cpi_size_scale
