; Code used to process particle features for classification

pro process_frames, parameters, pStateInfo, multiple = multiple
; Purpose to process a series of frames defined by the user and extract feature information

; Parameters
;   parameters    INPUT  Info returned from GUI pop-up
;   pStateInfo    Input  all the info

;   Results : create an ascii file or write a min file
forward_function condition_image

if(not keyword_set(multiple))then multiple = !false

widget_control,/hourglass

; get YX aspect ratio from gui
widget_control,(*pStateInfo).aspectID, get_value = YXAspect

imgList = (*(*pStateInfo).pFileScan)[*(*pStateInfo).pImgPosIndx];  pfilescan is pointer, to each file that has been scanned.
BkgPosList = TEMPORARY(WHERE((*(*pStateInfo).pFileScan).ImgType eq 2, BkgCount))

if(BkgCount gt 1)then begin
    BackimgList = (*(*pStateInfo).pFileScan)[BkgPosList]
    currentBkgnd = 0
    backFrame = read_backGnd((*pStateInfo).InFileLun,(*pStateInfo).FileHead.Ver,BackimgList[currentBkgnd].fpos)
endif else begin
    if (PTR_VALID((*pStateInfo).BackFrame)) then begin
        BackimgList = (*(*pStateInfo).BackFrame).pBackImg
        BackFrame = *(*pStateInfo).backframe
        currentBkgnd = 1
    endif else begin
        BackFrame = {pBackImg:ptr_new()}
        currentBkgnd = 0
    endelse
endelse

print, 'current background #' ,currentBkgnd

; the objList marks the imglist with a true or false depending on the outcome of the extraction process, and the location in the obj file
if(not multiple)then (*pStateInfo).pObjList =ptr_new(replicate({valid:0B,fpos:0L,time:0.0D},n_elements(imgList)))

; process frames
cntr = 0
fpos = 0

start_scanPos = 0UL
stop_scanPos= n_elements((*((*pStateInfo).pFileScan))[*((*pStateInfo).pImgPosIndx)]) - 1
widget_control,(*pStateInfo).widgets.progress, map = 1
tvlct,255,0,0,255;    ;so that the bars are only red and background is white. with 255 levels ??, jan, 5, 2009 ?

;create loop along time determined from parameters
for frame = start_scanPos, stop_scanPos do begin
    error_no = 0

    if(error_no ne 0) then begin
        print, "caught error in process_features_lib"
        print, "error_no" + string(error_no)
        stop
    endif

    if(frame mod 4) eq 0 then begin ;  to display the progress of the processing.
        original = !d.window
        wset, (*pStateInfo).progressWID
        erase
        width =  fix((float(frame) / stop_scanPos)*120);  width increases as more frames are processed.
        bar = bytarr(width+1,30)+255B
        tv, bar
        wset, original
    endif

    if(error_no ne 0) then begin
        print, "caught error in process_features_lib"
        print, "error_no" + string(error_no)
        stop
    endif

    ; add frame number dialog for progress here
    ; get a frame
    ;FrameData = read_frame((*pStateInfo).InFileLun,(*pStateInfo).FileHead.Ver,imgList[frame].fpos,date=(*pStateInfo).date, TSBlanked = imgList[frame].TSBlanked )

    FrameData = read_frame((*pStateInfo).InFileLun,(*pStateInfo).FileHead.Ver,imgList[frame].fpos,date=(*pStateInfo).date, TSBlanked = imgList[frame].TSBlanked, nodata=nodata)


    if(n_elements(nodata) ne 0) then begin
        if(nodata) then begin
            print, 'read_frame returned nodata for frame ', frame

            if(parameters.ObjFile)then begin
                ; close and reopen as read only
                free_lun,(*pStateInfo).objLUN

                if( not multiple)then begin
                    openR,lun,(*pStateInfo).ObjFile,/get
                    (*pStateInfo).ObjLUN = lun
                endif
            endif

            widget_control,(*pStateInfo).widgets.progress, map = 0

            return
        endif
    endif


    if(error_no ne 0) then begin
        print, "caught error in process_features_lib"
        print, "error_no" + string(error_no)
        stop
    endif

    ; ROIArray is an array of pointers for each particle in the frame
    pROIArray = FrameData.pFrame

    ; How many ROIs?
    if ptr_valid(pROIArray) eq 0 then begin
        n_imgs=0
    endif else begin
        n_imgs = n_elements(*pROIArray)
    endelse

    ; write to a obj file if requested
    if((*pStateInfo).ObjLUN eq -1)then init_obj_file, pStateInfo   ;, roi=roi ;else begin

    if(error_no ne 0) then begin
        print, "caught error in process_features_lib"
        print, "error_no" + string(error_no)
        stop
    endif

    ; if still not valid, problem opening file... so bail out
    if((*pStateInfo).ObjLUN eq -1)then return

    if(not (fstat((*pStateInfo).ObjLUN)).write) then begin
        ; reopen as r/w
        free_lun,(*pStateInfo).objLUN
        openW,lun,(*pStateInfo).ObjFile,/get
        (*pStateInfo).ObjLUN = lun
    endif

    if(error_no ne 0) then begin
        print, "caught error in process_features_lib"
        print, "error_no" + string(error_no)
        stop
    endif

    if(currentBkgnd lt BkgCount-1)then begin
        if(error_no ne 0) then begin
            print, "caught error in process_features_lib"
            print, "error_no" + string(error_no)
            stop
        endif


        if(abs((*(*pStateInfo).pImgPosIndx)[frame] - BkgPosList[currentBkgnd]) gt abs((*(*pStateInfo).pImgPosIndx)[frame] - BkgPosList[currentBkgnd+1]))then begin

            if(error_no ne 0) then begin
                print, "caught error in process_features_lib"
                print, "error_no" + string(error_no)
                stop
            endif


            ; switch to a new background ; above if condition is looking for closest background frames, not  neccesary in time.
            currentBkgnd = currentBkgnd + 1

            if(currentBkgnd ne 1) then begin
                backFrame = read_backGnd((*pStateInfo).InFileLun,(*pStateInfo).FileHead.Ver,BackimgList[currentBkgnd].fpos)    ; contains the structure of the background
            endif

            print, 'current background #' ,currentBkgnd
        endif
    endif

    if(error_no ne 0) then begin
        print, "caught error in process_features_lib"
        print, "error_no" + string(error_no)
        stop
    endif

    ; NOTE: Need to deal here with ROIs which may have been incorrectly split
    ; by the real-time CPI code
    ; For now, we assume that each ROI image we process below is
    ; independent of all others

    FOR roiNum=0, n_imgs-1 DO BEGIN
        ; start an OBJ frame if needed
        if(roiNum eq 0)then begin
            fpos = init_obj_frame(pStateInfo, frame, FrameData.FrameHeader)
            totROIsSize = 104 ; size of frameheader

            if(not multiple) then begin
                (*(*pStateInfo).pObjList)[frame].fpos = fpos

                ; need to get calcStrobe time working for 3V-CPI
                ;(*(*pStateInfo).pObjList)[frame].time = float(FrameData.frameheader.time) + calcStrobeTime(FrameData.frameheader.arrivalT1,FrameData.frameheader.arrivalT2,FrameData.frameheader.Ver)/1000.0D
                (*(*pStateInfo).pObjList)[frame].time = float(FrameData.frameheader.time); mo change this further on August 15, 2012 to be following lines.
                ;*******************************************************
                ;frameData = read_frame((*pStateInfo).InFileLun,(*pStateInfo).FileHead.Ver,imgList[k].fpos,date=(*pStateInfo).date, TSBlanked = imglist[k].TSBlanked)
                ;frameTime = frameData.FrameHeader.time  ;  This time should be  corrected to = Htime*16^8+ ArivelaT1*16^4 + ArriveT2  August 6, 2010.
                ;time = convertFromSeconds(frameTime , (*pStateInfo).FileHead.year) ;  Time={month, day, hh, mm, ss, msec} ; change to 4/19/19:20:00.00 format.
                hour  =framedata.FrameHeader.hour
                minute=framedata.FrameHeader.minute
                sec   =framedata.FrameHeader.sec
                msec  =framedata.FrameHeader.msec; ;   HOUR =   14;  MINUTE=0; SEC= 11;MSEC=  791  from frameHeader.
                ;frameTimes(k) = Time.hour*3600.0 + time.minute*60.0 + time.sec + time.msec/1000.0D;  Changed to this August 6, 2010
                (*(*pStateInfo).pObjList)[frame].time = hour*3600.0D + minute*60.0D + sec*1.0D + msec/1000.0D
                ;********************************************************
            endif
        endif

        ; remove the background before processing
        pROIImg = (*pROIArray)[roiNum].pROI
        dim = size(*pROIImg,/dim)

        if(ptr_valid(BackFrame.pBackImg))then begin
            pBackImg = BackFrame.pBackImg
            backROI = (*pBackImg)[(*pROIArray)[roiNum].ImageHeader.sx:(*pROIArray)[roiNum].ImageHeader.ex,(*pROIArray)[roiNum].ImageHeader.sy:(*pROIArray)[roiNum].ImageHeader.ey]
            ; save this back into the data array
            Img = *pROIImg - fix(backROI)
        endif else begin
            Img = *pROIImg
            BackRoi=img*0; added this on Feb 22, 2008 for arron's new size calculation
        endelse

        sx = (*pROIArray)[roiNum].ImageHeader.sx
        sy = (*pROIArray)[roiNum].ImageHeader.sy

        Roundness2 = -5.
        Roundness3 = -5.

        ; compute particle properties
        particleInfo = FindParticles(Img, parameters.GradientThresh, parameters.Thresh, parameters.MedianFilters,YXAspect,[sx,sy],Roundness2=Roundness2,Roundness3=Roundness3)

        ;calculate new size parameters to replace teh old values

        cimage=condition_image(*pROIImg, backROI)

        ; Newdata=processparticles_ab(cimage,!CPI_MICRONS_PER_PIXEL,YXAspect);  new data is in pixels for len, wid, area, P.
        ;The values are Pob, Aob, Lob, Wob, They need to be corrected to
        ; Pab, Aab, Lab, Wab, and ect to be converted to Lcor, Pcor ....
        ; by focus values.
        ; comment above out on May, 4th 2009.
        if n_tags(Particleinfo) ne 0 then begin
            hole_count=particleinfo.hole_count
            total_hole_area=particleinfo.hole1_area + particleinfo.hole1_area + particleinfo.hole3_area
        endif else begin
            hole_count=0
            total_hole_area=0
        endelse

        ; TAF pipe Roundness2 and 3 into processparticles_ab, until we refactor the code to get rid of redundant calculations

        ;Newdata = processparticles_ab(cimage,!CPI_MICRONS_PER_PIXEL,YXAspect,*pROIImg,Parameters.gradientThresh, Parameters.Thresh,$
        ;          parameters.MedianFilters,[Sx,Sy],hole_count,total_hole_area, Roundness2=Roundness2, Roundness3=Roundness3)

        ; TAF 2013-09-07 temporarily restoring to just Roundness1 because we're getting all misclassified as ice
        Newdata = processparticles_ab(cimage,!CPI_MICRONS_PER_PIXEL,YXAspect,*pROIImg,Parameters.gradientThresh, Parameters.Thresh,$
            parameters.MedianFilters,[Sx,Sy],hole_count,total_hole_area, Roundness2=Roundness2, Roundness3=Roundness3)
        ;   *pRoiimg is not background subtracted.
        ;new data is in pixels for len, wid, area, P.
        ;The values are Pob, Aob, Lob, Wob, They need to be corrected to
        ; Pab, Aab, Lab, Wab, and ect to be converted to Lcor, Pcor ....
        ; by focus values.

        if n_tags(newData) ne 0 then begin;  replace particleino with new data.
            replace_particleinfo_with_new_values,particleinfo,newdata;     This statement takes less than 1us.
        endif

        ; if autoextract failed, use old header
        if(size(particleInfo,/type) ne 8)then begin
            valid = !false

            if(not multiple) then (*(*pStateInfo).pObjList)[frame].valid = valid

            objs = 0

        endif else begin
            ;mark objList as being valid

            objs = n_elements(particleInfo)
            valid = !true

            if(not multiple) then (*(*pStateInfo).pObjList)[frame].valid = valid
        endelse

        ; write OBJ frame
        write_obj_frame, particleInfo, (*pROIArray)[roiNum].ImageHeader, *pROIImg, $
            (*pStateInfo).ObjLUN, valid, totROIsSize, roi=roi

        if(parameters.asciiFile)then begin
            ;write a line of ASCII to the output file
            output_ascii,pStateInfo,particleInfo,frame,(not valid)
        endif

        cntr = cntr + 1

    ENDFOR    ; end particle ROI loop

    if(error_no ne 0) then begin
        print, "caught error in process_features_lib"
        print, "error_no" + string(error_no)
        stop
    endif

    ; complete OBJ frame if needed
    finish_obj_frame,(*pStateInfo).ObjLUN, fpos, totROIsSize

    if(error_no ne 0) then begin
        print, "caught error in process_features_lib"
        print, "error_no" + string(error_no)
        stop
    endif

    ; reset version number for delete
    FrameData.frameheader.ver = 25
    delete_frame, FrameData

    if(error_no ne 0) then begin
        print, "caught error in process_features_lib"
        print, "error_no" + string(error_no)
        stop
    endif

    heap_gc

    if(error_no ne 0) then begin
        print, "caught error in process_features_lib"
        print, "error_no" + string(error_no)
        stop
    endif
endfor     ; end frame loop

original = !d.window
wset, (*pStateInfo).progressWID
bar = bytarr(120,30)+255B
tv, bar
wset, original
tvlct,255,255,255,255 ;  goes back to original full color mode.

;free_lun,lun1
;   if(parameters.asciiFile)then begin
;     FREE_LUN,(*pStateInfo).AsciiLUN
;     (*pStateInfo).AsciiLUN = -1
;     (*pStateInfo).ASCIIFile = ''
;   endif

if(parameters.ObjFile)then begin
    ; close and reopen as read only
    free_lun,(*pStateInfo).objLUN

    if( not multiple)then begin
        openR,lun,(*pStateInfo).ObjFile,/get
        (*pStateInfo).ObjLUN = lun
    endif
endif

widget_control,(*pStateInfo).widgets.progress, map = 0
end ; process_frames



function LoadFileMin, filename, pStateInfo,backfilename

    ; Purpose ;to load a new file, selected by the user
    ; Parameters
    ;   pStateInfo  Main state structure
    ;   backfilename   output parameter with the name of the file used for the background

    ;   forward_function checkBackgnd

    backfilename = ''
    clean_up,pStateInfo

    ; open file
    err = 0
    OpenU, DFileUnit, FileName, /Get_lun, ERROR = err, /BUFSIZE,/swap_if_big_endian

    if(err NE 0) then begin
        PRINT, !ERR_STRING
        DFileUnit = -1
        return,-1
    endif

    (*pStateInfo).InFileName = FileName
    (*pStateInfo).InFileLun = DFileUnit

    WIDGET_CONTROL, /HOURGLASS

    ROI_INFO = ScanSPEC( DFileUnit, fileVers, scaleFactor = (*pStateInfo).scaleFactor, /reorder,labelID=(*pStateInfo).InstLabel) ;fileVers returns file version number or error code

    ; roi_info must be a structure
    if(size(roi_info,/type) ne 8)then return,-1

    (*pStateInfo).FileHead = ROI_INFO.ROIHeader

    if(size(ROI_INFO.scanlist,/type) eq 2)then begin   ; error condition
        ; try to continue
        message,'Error from scanSPEC',/ioerror
        return,-1
    endif

    if(ptr_valid((*pStateInfo).pFileScan))then PTR_FREE, (*pStateInfo).pFileScan

    (*pStateInfo).pFileScan = ptr_new(TEMPORARY(ROI_INFO.ScanList))

    imgListIndx = where((*(*pStateInfo).pFileScan).imgtype ge 0)
    imgList = (*(*pStateInfo).pFileScan)[imgListIndx]
    allTimes = imglist.HTime

    for k=0L,n_elements(allTimes)-2 do begin
        if(alltimes[k+1] lt alltimes[k])then begin
            (*(*pStateInfo).pFileScan)[imgListIndx[k]].HTime = alltimes[k+1]
            (*(*pStateInfo).pFileScan)[imgListIndx[k]].badframe = 1
            ShowMsg, (*pStateInfo).InstLabel,'Frame# '+strtrim(k,2)+' marked as bad... time out of sequence.'
        endif
    endfor

    (*pStateInfo).pImgPosIndx = ptr_new(where((*(*pStateInfo).pFileScan).imgtype gt 0))

    ; check to see if a background image is available
    checkBackgnd_namma,pStateInfo,/nosearch

    (*pStateInfo).backRemoved = !false

    if((*pStateInfo).BkgPos ge 0)then begin    ; BkgPos negative means either no background or we are using an external backgrnd
        ; GET A BACKGROUND from the current file

        ImgScanList = (*(*pStateInfo).pFileScan)[*(*pStateInfo).pImgPosIndx]
        BackPosIndx = where(ImgScanList.imgtype eq 2,cnt)
        position = 1   ;look near beginning of file

        if(not ptr_valid((*pStateInfo).pBackPosIndx))then begin
            if(cnt gt 0)then begin
                print,cnt,' Backgrounds found'
                position = BackPosIndx[0]

                if(cnt gt 1)then get_best_backGnd_pos,ImgScanList,BackPosIndx,position

                (*pStateInfo).pBackPosIndx = ptr_new(BackPosIndx,/no_copy)
                (*pStateInfo).Bkgs_exist = !true
            endif else print,'no backgrounds found'
        endif else get_best_backGnd_pos, imgScan, BackPosIndx, position

        ShowMsg, (*pStateInfo).InstLabel,'Current Background at Frame # '+ strtrim(position+1,2)

        BackFrame = read_backGnd((*pStateInfo).InFileLun,ROI_INFO.ROIHeader.Ver,ImgScanList[position].fpos)   ; contains the structure of the background

        if(size(BackFrame,/type) eq 8)then begin
            (*pStateInfo).BackFrame = ptr_new(BackFrame,/no_copy)
            (*pStateInfo).BkgPos =   position             ; contains the zero-based index number into scanList of the background to use
        endif
    endif else begin
        ; automatic search for backgrounds

        files = findFile((*pStateInfo).path + '*.roi',count=count)
        ; put in order
        files = files[sort(files)]
        ;print,count-1

        ; determine which is the current file
        thisFileIndx = where(files eq (*pStateInfo).inFileName)
        ;print,files[thisFileIndx[0]]

        ; search files first 1 back, then 1 forward, etc
        noBackground = !true
        offset = -1
        increment = indgen(count-1)+3
        sign = 1
        i = 0

        while(noBackground and i le count-2) do begin
            searchindex = thisFileIndx[0] + offset
            ;print,searchindex,files[searchindex]
            openr,lun,files[searchindex],/get

            ROI_INFO = ScanSPEC( lun, fileVers) ;fileVers returns file version number or error code

            if(size(roi_Info,/type) ne 8)then return,-1

            index = where(ROI_INFO.scanlist.ImgType eq 2,Backcnt)

            if(Backcnt gt 0)then begin
                print,backcnt,' Backgrounds found'
                noBackground = !false
                backfilename = files[searchindex]
            endif else begin
                ; move to another file
                offset = increment[i]/2 * sign
                sign = -sign
                i = i + 1
            endelse
        endwhile

        if(not noBackground)then begin
            if(backcnt gt 1 and sign lt 0)then fpos = ROI_INFO.scanlist[index[backcnt-1]].fpos else fpos = ROI_INFO.scanlist[index[0]].fpos

            (*pStateInfo).pBackPosIndx = ptr_new(backScanList,/no_copy)
            BackFrame = read_backGnd(lun,ROI_INFO.ROIHeader.Ver,fpos, frameheader=frameheader)   ; contains the structure of the background
            (*pStateInfo).BackFrame = ptr_new(BackFrame,/no_copy)
            (*pStateInfo).BkgPos =   -2    ; when BkgPos = -2 then do not search for another background!
            (*pStateInfo).Bkgs_exist = !true
        endif else begin
            ; no backgrounds found terminate!
            (*pStateInfo).BkgPos = -10
            r = dialog_message('No backgrounds found in this folder... terminating autoprocessing',/error)
        endelse
    endelse

    return,1
end ; LoadFileMin



pro process_multipleFiles, parameters, pStateInfo
    forward_function Process_timeSeries
    ; Purpose : to process multiple object files

    log = strarr(parameters.multipleFiles)

    for numFile = 0, parameters.multipleFiles-1 do begin
        ; get the file info
        flag = loadFileMin((*parameters.pFileList)[numFile], pStateInfo, backfilename)

        initpTimeInfo, pStateInfo, /multiple

        if(flag gt 0)then begin
            if(backfilename ne '')then begin
                ; file opened for background
                log[numfile] = 'The background for file ' + (*parameters.pFileList)[numFile] + ' was taken from file ' + backFileName
            endif else log[numfile] = 'File ' + (*parameters.pFileList)[numFile] + ' contained a background'

            if(parameters.extract) then  process_frames, parameters, pStateInfo, /multiple

            imgList = (*(*pStateInfo).pFileScan)[*(*pStateInfo).pImgPosIndx]
            dotloc = rstrpos((*pStateInfo).InFileName,'.')
            (*pStateInfo).ObjFile = strmid((*pStateInfo).InFileName,0,dotloc[0]) + '.obj'

            ; open file and test for error
            OpenU,ObjLUN,(*pStateInfo).ObjFile,/get, ERROR = err

            if(err ne 0)then begin
                (*pStateInfo).ObjFile = ''
                message=dialog_message('No OBJ file available to process. Cannot make ASCII files')
                return
            endif else begin
                ; ok to scan file
                (*pStateInfo).ObjLUN =ObjLUN
                ; clear ptr

                if(ptr_valid((*pStateInfo).pOBJList))then PTR_FREE, (*pStateInfo).pOBJList

                ; scan file. If this is valid, other code will use it
                roiversion_num=(*pStateInfo).roi_frameheader_version;   added by mo on Juanrya 8 2004
                (*pStateInfo).pOBJList=ptr_new(scanOBJ(ObjLUN,roiversion_num))  ;added by mo on Juanrya 8 2004
                objData = get_obj_header((*pStateInfo).ObjLUN,(*(*pStateInfo).pObjList)[(*pStateInfo).ImgPos-1].fPos)
                ; store obj info
                (*pStateInfo).pObjData = ptr_new(objData)
            endelse

            if(parameters.ASCIIFile) then begin
                ; add log files for each
                ; first do particle log file

                pLink = (*(*pStateInfo).pTimeInfo).pCurrent_link
                (*pLink).sendASCII = 1
                replace_particleinfo_with_new_values    (*pLink).output = 1
                ; TAF check the syntax of the above line -- might be corrupted
                (*pLink).sendASCII = 4
                (*pLink).Format = 'spec1'
                select = bytarr(50) + 1B
                select[47] = 0   ; skip boundary
                (*pLink).pSelect = ptr_new(select)
                ; call processing
                index = Process_timeSeries( pStateInfo)
                close_ascii, pStateInfo

                (*pLink).sendASCII = 2
                ; then do frame log
                (*pLink).Format = 'frame'
                index = Process_timeSeries( pStateInfo)
                close_ascii,pStateInfo

                ; finally do hertz log
                (*pLink).Format = 'hertz'
                index = Process_timeSeries( pStateInfo)
                close_ascii,pStateInfo
            endif
        endif else log[numfile] = 'File:' + (*parameters.pFileList)[numFile] + ' was skipped due to an error on input'
    endfor

    r = dialog_message(log,/info)

    ; load a file for the GUI
    r = dialog_message('Do you want to load a file in CPI View?',/question)
    if(r eq 'Yes')then loadFile, pStateInfo
end ; process_multipleFiles



pro  replace_particleinfo_with_new_values, particleinfo, newdata
    if n_tags(particleInfo) ne 0 then begin; there is a vald particle info for old_method.
        ;replace the old values with new values.

        particleinfo.BlockMark              = newdata.BlockMark
        particleinfo.Roiinfosize            = newdata.ROIinfosize
        particleinfo.Roiver                 = newdata.Roiver
        particleinfo.sx                     = newdata.sx
        particleinfo.sy                     = newdata.sy
        particleinfo.ex                     = newdata.ex
        particleinfo.ey                     = newdata.ey
        particleinfo.Pixbytes               = newdata.PixBytes
        particleinfo.Flags                  = newdata.Flags
        particleinfo.Len                    = newdata.Len
        particleinfo.Slen                   = newdata.Slen
        particleinfo.Elen                   = newdata.Elen
        particleinfo.wid                    = newdata.wid
        particleinfo.Dark                   = newdata.Dark
        particleinfo.area                   = newdata.area
        particleinfo.perim                  = newdata.perim
        particleinfo.centroid               = newdata.centroid
        particleinfo.roundness1             = newdata.roundness1
        particleinfo.x_moment               = newdata.x_moment
        particleinfo.y_moment               = newdata.y_moment
        particleinfo.rubber_band            = newdata.rubber_band
        particleinfo.fractal_dim            = newdata.fractal_dim
        particleinfo.harmonic_len           = newdata.harmonic_len
        particleinfo.bound_len              = newdata.Bound_len
        PARTICLEINFO.SAMPLE_PERIOD          = NEWDATA.SAMPLE_period
        particleinfo.focus                  = newdata.focus
        particleinfo.focusmin               = newdata.focusmin
        particleinfo.focusmax               = newdata.focuSmax
        particleinfo.focusStdDev            = newdata.focusStdDev
        particleinfo.focusGOODSAMPLES       = newdata.focusGOODSAMPLES

        Particleinfo.CUTOFF                 = newdata.CUTOFF
        Particleinfo.CRYSTAL                = newdata.crystal
        particleinfo.CONFIDENCE             = newdata.CONFIDENCE

        particleinfo.p1                     = newdata.p1
        particleinfo.Roundness2             = newdata.Roundness2
        particleinfo.Roundness3             = newdata.Roundness3
        particleinfo.p4                     = newdata.p4

        particleinfo.radii_len              = newdata.radii_len
        particleinfo.curvature_len          = newdata.curvature_len
        particleinfo.curvature_window       = newdata.curvature_window

        particleinfo.focus_stats            = newdata.focus_stats
        particleinfo.intensity_stats        = newdata.intensity_stats
        particleinfo.intensity_Holeless_stats = newdata.intensity_Holeless_stats
        particleinfo.gradient_stats         = newdata.gradient_stats
        particleinfo.radii_stats            = newdata.radii_stats
        particleinfo.Curvature_stats        = newdata.Curvature_stats

        ;     newdata.hole_count=particleinfo.hole_count
        ;     newdata.hole_area_stats=ParticleInfo.hole_area_stats
        ;     newdata.HOLE_INTENSITY_STATS=ParticleInfo.HOLE_INTENSITY_STATS
        ;     newdata.HOLE_OTHER_STATS=ParticleInfo.HOLE_OTHER_STATS

        ;     newdata.HOLE1_AREA=ParticleInfo.HOLE1_AREA
        ;     newdata.HOLE1_INTENSITY_STATS=ParticleInfo.HOLE1_INTENSITY_STATS
        ;     newdata.HOLE1_PERIM    =ParticleInfo.HOLE1_PERIM
        ;     newdata.HOLE1_OTHER_STATS=ParticleInfo.HOLE1_OTHER_STATS

        ;     newdata.HOLE2_AREA       =ParticleInfo. HOLE2_AREA
        ;     newdata.HOLE2_INTENSITY_STATS=ParticleInfo.HOLE2_INTENSITY_STATS
        ;     newdata.HOLE2_PERIM          =ParticleInfo.HOLE2_PERIM
        ;     newdata.HOLE2_OTHER_STATS    =ParticleInfo.HOLE2_OTHER_STATS

        ;     newdata.HOLE3_AREA           =ParticleInfo.HOLE3_AREA
        ;     newdata.HOLE3_INTENSITY_STATS=ParticleInfo.HOLE3_INTENSITY_STATS
        ;     newdata.HOLE3_PERIM          =ParticleInfo.HOLE3_PERIM
        ;     newdata.HOLE3_OTHER_STATS    =ParticleInfo.HOLE3_OTHER_STATS
        particleinfo.harmonics              = newdata.harmonics
        particleinfo.boundary               = newdata.boundary
        particleinfo.radii                  = newdata.radii
        ;particleinfo.wid=newdata.Curvature
    endif else begin
        particleinfo=newdata
    endelse
end ;  replace_particleinfo_with_new_values,particleinfo,newdata






