; Code used to process particle features for classification

pro process_frames, parameters, pStateInfo, multiple = multiple
; Purpose to process a series of frames defined by the user and extract feature information

; Parameters
;   parameters    INPUT  Info returned from GUI pop-up
;   pStateInfo    Input  all the info

;   Results : create an ascii file or write a min file

if(not keyword_set(multiple))then multiple = !false

widget_control,/hourglass

; get YX aspect ratio from gui
widget_control,(*pStateInfo).aspectID, get_value = YXAspect

imgList = (*(*pStateInfo).pFileScan)[*(*pStateInfo).pImgPosIndx]
BkgPosList = TEMPORARY(WHERE((*(*pStateInfo).pFileScan).ImgType eq 2, BkgCount))
if(BkgCount gt 1)then begin
    BackimgList = (*(*pStateInfo).pFileScan)[BkgPosList]
    currentBkgnd = 0
    backFrame = read_backGnd((*pStateInfo).InFileLun,(*pStateInfo).FileHead.Ver,BackimgList[currentBkgnd].fpos)
endif else begin
    if (PTR_VALID((*pStateInfo).BackFrame)) then begin
        BackimgList = (*(*pStateInfo).BackFrame).pBackImg
        BackFrame = *(*pStateInfo).backframe
        currentBkgnd = 1
    endif else begin
        BackFrame = {pBackImg:ptr_new()}
        currentBkgnd = 0
    endelse
endelse
print, 'current background #' ,currentBkgnd

; the objList marks the imglist with a true or false depending on the outcome of the extraction process, and the location in the obj file
if(not multiple)then (*pStateInfo).pObjList =ptr_new(replicate({valid:0B,fpos:0L,time:0.0D},n_elements(imgList)))

; process frames
cntr = 0
fpos = 0

start_scanPos = 0UL
stop_scanPos= n_elements((*((*pStateInfo).pFileScan))[*((*pStateInfo).pImgPosIndx)]) - 1
widget_control,(*pStateInfo).widgets.progress, map = 1
tvlct,255,0,0,255

;create loop along time determined from parameters
for frame = start_scanPos, stop_scanPos do begin

    if(frame mod 4) eq 0 then begin
        original = !d.window
        wset,(*pStateInfo).progressWID
        erase
        width =  fix((float(frame) / stop_scanPos)*120)
        bar = bytarr(width+1,30)+255B
        tv,bar
        wset,original
    endif

    ; add frame number dialog for progress here
    ; get a frame
    FrameData = read_frame((*pStateInfo).InFileLun,(*pStateInfo).FileHead.Ver,imgList[frame].fpos,date=(*pStateInfo).date, TSBlanked = imgList[frame].TSBlanked )

    ; ROIArray is an array of pointers for each particle in the frame
    pROIArray = FrameData.pFrame

    ; How many ROIs?
    n_imgs = n_elements( *pROIArray )

    ; write to a obj file if requested
    if((*pStateInfo).ObjLUN eq -1)then init_obj_file, pStateInfo   ;, roi=roi ;else begin

    ; if still not valid, problem opening file... so bail out
    if((*pStateInfo).ObjLUN eq -1)then return

    if(not (fstat((*pStateInfo).ObjLUN)).write) then begin
        ; reopen as r/w
        free_lun,(*pStateInfo).objLUN
        openW,lun,(*pStateInfo).ObjFile,/get
        (*pStateInfo).ObjLUN = lun
    endif


    if(currentBkgnd lt BkgCount-1)then begin
        if( abs((*(*pStateInfo).pImgPosIndx)[frame] - BkgPosList[currentBkgnd]) gt abs((*(*pStateInfo).pImgPosIndx)[frame] - BkgPosList[currentBkgnd+1]))then begin
            ; switch to a new background
            currentBkgnd = currentBkgnd + 1
            backFrame = read_backGnd((*pStateInfo).InFileLun,(*pStateInfo).FileHead.Ver,BackimgList[currentBkgnd].fpos)    ; contains the structure of the background
            print, 'current background #' ,currentBkgnd
        endif
    endif

    ; NOTE: Need to deal here with ROIs which may have been incorrectly split
    ; by the real-time CPI code
    ; For now, we assume that each ROI image we process below is
    ; independent of all others

    FOR roiNum=0, n_imgs-1 DO BEGIN

        ; start an OBJ frame if needed
        if(roiNum eq 0)then begin
            fpos = init_obj_frame(pStateInfo, frame, FrameData.FrameHeader)
            totROIsSize = 104 ; size of frameheader
            if(not multiple) then begin
                (*(*pStateInfo).pObjList)[frame].fpos = fpos
                (*(*pStateInfo).pObjList)[frame].time = float(FrameData.frameheader.time) + calcStrobeTime(FrameData.frameheader.arrivalT1,FrameData.frameheader.arrivalT2,FrameData.frameheader.Ver)/1000.0D
            endif
        endif

        ; remove the background before processing
        pROIImg = (*pROIArray)[roiNum].pROI
        dim = size(*pROIImg,/dim)

        if(ptr_valid(BackFrame.pBackImg))then begin
            pBackImg = BackFrame.pBackImg
            ;add this by mo on january to shrink teh back ground into that of the images.
            dddmo=size(*pbackimg,/dim)
            if dim(1) eq dddmo(1) and dim(0) eq dddmo(0) then begin
                backROI = (*pBackImg)[(*pROIArray)[roiNum].ImageHeader.sx:(*pROIArray)[roiNum].ImageHeader.ex,(*pROIArray)[roiNum].ImageHeader.sy:(*pROIArray)[roiNum].ImageHeader.ey]
            endif else begin
                mean_value_mo=round(mean((*pbackIMG)[*,*]))
                backROI = replicate(mean_value_mo,dim)
            endelse

            ; save this back into the data array
            Img = *pROIImg - fix(backROI)
        endif else begin
            Img = *pROIImg
        endelse

        sx = (*pROIArray)[roiNum].ImageHeader.sx
        sy = (*pROIArray)[roiNum].ImageHeader.sy

        ; compute particle properties
        particleInfo = FindParticles(Img, parameters.GradientThresh, parameters.Thresh, parameters.MedianFilters,YXAspect,[sx,sy])

        ; if autoextract failed, use old header
        if(size(particleInfo,/type) ne 8)then begin
            valid = !false
            if(not multiple) then (*(*pStateInfo).pObjList)[frame].valid = valid
            objs = 0
        endif else begin
            ;mark objList as being valid
            objs = n_elements(particleInfo)
            valid = !true
            if(not multiple) then (*(*pStateInfo).pObjList)[frame].valid = valid
        endelse

        ; write OBJ frame
        write_obj_frame, particleInfo, (*pROIArray)[roiNum].ImageHeader, *pROIImg, $
            (*pStateInfo).ObjLUN, valid, totROIsSize, roi=roi

        if(parameters.asciiFile)then begin
            ;write a line of ASCII to the output file
            output_ascii,pStateInfo,particleInfo,frame,(not valid)
        endif

        cntr = cntr + 1

    ENDFOR    ; end particle ROI loop

    ; complete OBJ frame if needed
    finish_obj_frame,(*pStateInfo).ObjLUN, fpos, totROIsSize

    ; reset version number for delete
    FrameData.frameheader.ver = 25
    delete_frame, FrameData

    heap_gc

endfor     ; end frame loop

original = !d.window
wset,(*pStateInfo).progressWID
bar = bytarr(120,30)+255B
tv,bar
wset,original
tvlct,255,255,255,255


;free_lun,lun1
;   if(parameters.asciiFile)then begin
;     FREE_LUN,(*pStateInfo).AsciiLUN
;     (*pStateInfo).AsciiLUN = -1
;     (*pStateInfo).ASCIIFile = ''
;   endif


if(parameters.ObjFile)then begin
    ; close and reopen as read only
    free_lun,(*pStateInfo).objLUN
    if( not multiple)then begin
        openR,lun,(*pStateInfo).ObjFile,/get
        (*pStateInfo).ObjLUN = lun
    endif
endif

widget_control,(*pStateInfo).widgets.progress, map = 0

end


function LoadFileMin, filename, pStateInfo,backfilename

    ; Purpose ;to load a new file, selected by the user
    ; Parameters
    ;   pStateInfo  Main state structure
    ;   backfilename   output parameter with the name of the file used for the background

    ;   forward_function checkBackgnd

    backfilename = ''
    clean_up,pStateInfo

    ; open file
    err = 0
    OpenU, DFileUnit, FileName, /Get_lun, ERROR = err, /BUFSIZE,/swap_if_big_endian

    if (err NE 0) then begin
        PRINT, !ERR_STRING
        DFileUnit = -1
        return,-1
    endif

    (*pStateInfo).InFileName = FileName
    (*pStateInfo).InFileLun = DFileUnit

    WIDGET_CONTROL, /HOURGLASS

    ROI_INFO = ScanSPEC( DFileUnit, fileVers, scaleFactor = (*pStateInfo).scaleFactor, /reorder,labelID=(*pStateInfo).InstLabel) ;fileVers returns file version number or error code

    ; roi_info must be a structure
    if(size(roi_info,/type) ne 8)then return,-1

    (*pStateInfo).FileHead = ROI_INFO.ROIHeader

    if(size(ROI_INFO.scanlist,/type) eq 2)then begin   ; error condition
        ; try to continue
        message,'Error from scanSPEC',/ioerror
        return,-1
    endif

    if(ptr_valid((*pStateInfo).pFileScan))then PTR_FREE, (*pStateInfo).pFileScan
    (*pStateInfo).pFileScan = ptr_new(TEMPORARY(ROI_INFO.ScanList))

    imgListIndx = where((*(*pStateInfo).pFileScan).imgtype ge 0)
    imgList = (*(*pStateInfo).pFileScan)[imgListIndx]
    allTimes = imglist.HTime

    for k=0L,n_elements(allTimes)-2 do begin
        if(alltimes[k+1] lt alltimes[k])then begin
            (*(*pStateInfo).pFileScan)[imgListIndx[k]].HTime = alltimes[k+1]
            (*(*pStateInfo).pFileScan)[imgListIndx[k]].badframe = 1
            ShowMsg, (*pStateInfo).InstLabel,'Frame# '+strtrim(k,2)+' marked as bad... time out of sequence.'
        endif
    endfor

    (*pStateInfo).pImgPosIndx = ptr_new(where((*(*pStateInfo).pFileScan).imgtype gt 0))

    ; check to see if a background image is available
    checkBackgnd,pStateInfo,/nosearch

    (*pStateInfo).backRemoved = !false

    if((*pStateInfo).BkgPos ge 0)then begin    ; BkgPos negative means either no background or we are using an external backgrnd
        ; GET A BACKGROUND from the current file

        ImgScanList = (*(*pStateInfo).pFileScan)[*(*pStateInfo).pImgPosIndx]
        BackPosIndx = where(ImgScanList.imgtype eq 2,cnt)
        position = 1   ;look near beginning of file

        if(not ptr_valid((*pStateInfo).pBackPosIndx))then begin

            if(cnt gt 0)then begin
                print,cnt,' Backgrounds found'
                position = BackPosIndx[0]
                if(cnt gt 1)then get_best_backGnd_pos,ImgScanList,BackPosIndx,position

                (*pStateInfo).pBackPosIndx = ptr_new(BackPosIndx,/no_copy)
                (*pStateInfo).Bkgs_exist = !true
            endif else print,'no backgrounds found'

        endif else get_best_backGnd_pos, imgScan, BackPosIndx, position

        ShowMsg, (*pStateInfo).InstLabel,'Current Background at Frame # '+ strtrim(position+1,2)

        BackFrame = read_backGnd((*pStateInfo).InFileLun,ROI_INFO.ROIHeader.Ver,ImgScanList[position].fpos)   ; contains the structure of the background

        if(size(BackFrame,/type) eq 8)then begin
            (*pStateInfo).BackFrame = ptr_new(BackFrame,/no_copy)
            (*pStateInfo).BkgPos =   position             ; contains the zero-based index number into scanList of the background to use
        endif
    endif else begin
        ; automatic search for backgrounds

        files = findFile((*pStateInfo).path + '*.roi',count=count)
        ; put in order
        files = files[sort(files)]
        ;print,count-1

        ; determine which is the current file
        thisFileIndx = where(files eq (*pStateInfo).inFileName)
        ;print,files[thisFileIndx[0]]

        ; search files first 1 back, then 1 forward, etc
        noBackground = !true
        offset = -1
        increment = indgen(count-1)+3
        sign = 1
        i = 0

        while( noBackground and i le count-2) do begin
            searchIndx = thisFileIndx[0] + offset
            ;print,searchIndx,files[searchIndx]
            openr,lun,files[searchIndx],/get

            ROI_INFO = ScanSPEC( lun, fileVers) ;fileVers returns file version number or error code
            if(size(roi_Info,/type) ne 8)then return,-1
            index = where(ROI_INFO.scanlist.ImgType eq 2,Backcnt)

            if(Backcnt gt 0)then begin

                print,backcnt,' Backgrounds found'
                noBackground = !false
                backfilename = files[searchIndx]

            endif else begin
                ; move to another file
                offset = increment[i]/2 * sign
                sign = -sign
                i = i + 1
            endelse
        endwhile

        if( not noBackground )then begin
            if(backcnt gt 1 and sign lt 0)then fpos = ROI_INFO.scanlist[index[backcnt-1]].fpos else fpos = ROI_INFO.scanlist[index[0]].fpos

            (*pStateInfo).pBackPosIndx = ptr_new(backScanList,/no_copy)
            BackFrame = read_backGnd(lun,ROI_INFO.ROIHeader.Ver,fpos, frameheader=frameheader)   ; contains the structure of the background
            (*pStateInfo).BackFrame = ptr_new(BackFrame,/no_copy)
            (*pStateInfo).BkgPos =   -2    ; when BkgPos = -2 then do not search for another background!
            (*pStateInfo).Bkgs_exist = !true

        endif else begin
            ; no backgrounds found terminate!
            (*pStateInfo).BkgPos = -10
            r = dialog_message('No backgrounds found in this folder... terminating autoprocessing',/error)
        endelse
    endelse

    return,1
end


pro process_multipleFiles, parameters, pStateInfo

    forward_function Process_timeSeries
    ; Purpose : to process multiple object files
    log = strarr(parameters.multipleFiles)

    for numFile = 0, parameters.multipleFiles-1 do begin

        ; get the file info
        flag = loadFileMin((*parameters.pFileList)[numFile], pStateInfo, backfilename)

        initpTimeInfo, pStateInfo, /multiple

        if(flag gt 0)then begin
            if(backfilename ne '')then begin
                ; file opened for background
                log[numfile] = 'The background for file ' + (*parameters.pFileList)[numFile] + ' was taken from file ' + backFileName
            endif else log[numfile] = 'File ' + (*parameters.pFileList)[numFile] + ' contained a background'

            if(parameters.extract) then  process_frames, parameters, pStateInfo, /multiple

            imgList = (*(*pStateInfo).pFileScan)[*(*pStateInfo).pImgPosIndx]
            dotloc = rstrpos((*pStateInfo).InFileName,'.')
            (*pStateInfo).ObjFile = strmid((*pStateInfo).InFileName,0,dotloc[0]) + '.obj'

            ; open file and test for error
            OpenU,ObjLUN,(*pStateInfo).ObjFile,/get, ERROR = err
            if(err ne 0)then begin
                (*pStateInfo).ObjFile = ''
                message=dialog_message('No OBJ file available to process. Cannot make ASCII files')
                return
            endif else begin
                ; ok to scan file
                (*pStateInfo).ObjLUN =ObjLUN
                ; clear ptr
                if(ptr_valid((*pStateInfo).pOBJList))then PTR_FREE, (*pStateInfo).pOBJList
                ; scan file. If this is valid, other code will use it
                roiversion_num=(*pStateInfo).roi_frameheader_version;   added by mo on Juanrya 8 2004
                (*pStateInfo).pOBJList=ptr_new(scanOBJ(ObjLUN,roiversion_num))  ;added by mo on Juanrya 8 2004
                objData = get_obj_header((*pStateInfo).ObjLUN,(*(*pStateInfo).pObjList)[(*pStateInfo).ImgPos-1].fPos)
                ; store obj info
                (*pStateInfo).pObjData = ptr_new(objData)
            endelse

            if(parameters.ASCIIFile) then begin
                ; add log files for each
                ; first do particle log file

                pLink = (*(*pStateInfo).pTimeInfo).pCurrent_link
                (*pLink).sendASCII = 1
                (*pLink).output = 1

                (*pLink).sendASCII = 4
                (*pLink).Format = 'spec1'
                select = bytarr(50) + 1B
                select[47]=0   ; skip boundary
                (*pLink).pSelect = ptr_new(select)
                ; call processing
                index = Process_timeSeries( pStateInfo)
                close_ascii,pStateInfo

                (*pLink).sendASCII = 2
                ; then do frame log
                (*pLink).Format = 'frame'
                index = Process_timeSeries( pStateInfo)
                close_ascii,pStateInfo

                ; finally do hertz log
                (*pLink).Format = 'hertz'
                index = Process_timeSeries( pStateInfo)
                close_ascii,pStateInfo
            endif

        endif else log[numfile] = 'File:' + (*parameters.pFileList)[numFile] + ' was skipped due to an error on input'

    endfor

    r = dialog_message(log,/info)

    ; load a file for the GUI
    r = dialog_message('Do you want to load a file in CPI View?',/question)
    if(r eq 'Yes')then loadFile, pStateInfo

end
