pro plot_2ds_1Hz_mean_psd
    ;select time period.
    time1='19:42:00'
    time2='19:42:30'
    filename=dialog_pickfile(filter='*archive*_NS.txt',get_path=pathnm)

    Get_2ds_time1totime2_mean_psd_from_1Hz_archive,filename,time1,time2,xpts,Hypts,Vypts


    window,1,xs=400,ys=400
    loadct,0
    !P.thick=1
    !P.CHARSIZE=1
    !P.multi=[0,1,1]
    white=!D.N_COLORS-1
    !P.BACKGROUND=white

    PLOT, xpts-5, Hypts, COLOR=0, XRANGE=[10,10000], /Xlog,$
        Xstyle = 1, YLOG=1,xtitle='Particle Size(um)',TITLE='2ds 1Hz mean psd ',$
        THICK=1, YRANGE=[0.01, 1.0],ystyle=1, charsize=1.0,$
        YTITLE='#/L/um';  Yticks=ytikn,Ytickn=myticknames
    ;stop
    ;Ok, SO far the Hconc, Hypts Hextn, Hmass agrees with 2dsview comp_psd results.
end; end of plot_2ds_1Hz_mean_psd
;*************************************************

pro Get_2ds_time1totime2_mean_psd_from_1Hz_archive,filename,time1,time2,xpts,Hypts,Vypts
    starttime=time1;   for this periods, caps don't see anything bigger than 10um.
    endtime  =time2;  is string format hh:mm:ss

    substr=strsplit(starttime,':',/extract)
    startseconds=substr(0)*3600LL+substr(1)*60LL+substr(2)*1LL

    substr=strsplit(endtime,':',/extract)
    endseconds=substr(0)*3600LL+substr(1)*60LL+substr(2)*1LL

    nseconds=(endseconds-startseconds)
    Hconc=fltarr(nseconds)
    Hextn=fltarr(nseconds)
    Hmass=fltarr(nseconds)
    Hmnsz=fltarr(nseconds)
    yptsH=fltarr(24);  for the mean
    LHypts=fltarr(24); for just 1 seconds ypts.

    Vconc=fltarr(nseconds)
    Vextn=fltarr(nseconds)
    Vmass=fltarr(nseconds)
    Vmnsz=fltarr(nseconds)
    yptsV=fltarr(24);   for the mean
    LVypts=fltarr(24);  flor just 1 seconds ypts

    combined_string=strtrim(startseconds,2)
    ;stop
    find_string_pos_newversion,filename,combined_string,pos; get the first header line position.
    ;stop
    HGood_count=0
    Vgood_count=0
    for i=0,nseconds-1 do begin
        ;read in the caps bin heights data
        data=read_ascii(filename,data_start=pos+1,Num_records=1)
        data=data.field1
        Hconc(i)=data(0)
        Hextn(i)=data(1)
        Hmass(i)=data(2)
        Hmnsz(i)=data(3)
        Vconc(i)=data(4)
        Vextn(i)=data(5)
        Vmass(i)=data(6)
        Vmnsz(i)=data(7)

        data=read_ascii(filename,data_start=pos+2,Num_records=1)
        LHypts(0:7)=data.field1

        data=read_ascii(filename,data_start=pos+3,Num_records=1)
        LHypts(8:15)=data.field1

        data=read_ascii(filename,data_start=pos+4,Num_records=1)
        LHypts(16:23)=data.field1

        data=read_ascii(filename,data_start=pos+5,Num_records=1)
        LVypts(0:7)=data.field1

        data=read_ascii(filename,data_start=pos+6,Num_records=1)
        LVypts(8:15)=data.field1

        data=read_ascii(filename,data_start=pos+7,Num_records=1)
        LVypts(16:23)=data.field1

        ;finished one periods (1Seconds) psd reading.
        pos=pos+8;  this new pos is the time in seconds from midnigt again.
        if Hconc(i) ne 9.999E+9 then begin
            Hgood_count=Hgood_count+1
            yptsH=yptsH+LHypts
        endif

        if Vconc(i) ne 9.999E+9 then begin
            Vgood_count=Vgood_count+1
            yptsV=yptsV+LVypts
        endif

    endfor;  i is ith second.

    index=where(Hconc ne 9.999*1E+9, contgood)
    if contgood lt nseconds then begin
        rrr=dialog_message('The selected time periods has xxx seconds data is missing',/information)
        Hconc=mean(Hconc(index))
        Hextn=mean(Hextn(index))
        Hmass=mean(Hmass(index))
        Hmnsz=mean(Hmnsz(index))
        Vconc=mean(Vconc(index))
        Vextn=mean(Vextn(index))
        Vmass=mean(Vmass(index))
        Vmnsz=mean(Vmnsz(index))
    endif else begin  ;all are good.
        Hconc=mean(Hconc)
        Hextn=mean(Hextn)
        Hmass=mean(Hmass)
        Hmnsz=mean(Hmnsz)
        Vconc=mean(Vconc)
        Vextn=mean(Vextn)
        Vmass=mean(Vmass)
        Vmnsz=mean(Vmnsz)
    endelse

    yptsH=yptsH/(Hgood_count*1.0 > 1.0)
    yptsV=yptsV/(Vgood_count*1.0 > 1.0)

    ;make then to be double values.
    edges=[10,20,30,40,50,60,70,80,90,100,120,150,200,250,320,400,500,600,700,800,900,1000,1200,1400,10010]
    xpts=fltarr(48); edges has 25 elements.
    xpts(0)=edges(0)
    xpts(47)=edges(24)
    for i=1,23 do begin
        xpts(2*i-1:2*i)=edges(i)
    endfor
    Hypts=fltarr(48)
    Vypts=fltarr(48)
    for i=0,23 do begin
        Hypts(2*i:2*i+1)=yptsH(i)
        Vypts(2*i:2*i+1)=yptsV(i)
    endfor
    ;stop
end; end of get_2ds_time1totime2_mean_psd_from_1Hz_archive,filename,time1,time2,xpts,ypts

;*********************************************************************
pro find_string_pos_newversion,filename,search_number,pos
    ;search_string='PDs =';'WYSIWYG:'
    ;filename='abcdefg.ascii'
    result=read_ascii(filename,count=n_lines)
    this_line='ABCD'
    count=1LL
    openr,1,filename
    while (not EoF(filename)) do begin
        readF,1,this_line
        this_number=this_line*1LL

        if this_number  eq search_number then begin
            pos=count-1LL
            break;   ;breakout of the for loop.
        endif

        if count eq n_Lines then begin
            print, 'Data line with ',search_number,' not found in file: ',filename
            pos=-1
        endif;  print, pos
        count=count+1LL
    endwhile
    close,1
end; ;end of function find_string_pos.
;*************************************************************************
