function MMOO_get_TAS_data,fileUnit,filePos,vers, roi_frameheader_version,DTError,labelID,extradata

    ; assume we are at the start of a header block + 2
    ; and filePos contains the location of the start of the block ( i.e. 2 bytes back from current location )

    ; returns the housekeeping time as a long integer
    ; Note also that the filePos is updated to point to the next element in the file


    ; file pointer is pointing to the first field (the block mark), save the beginning file pointer

    case vers of
        30:HKData = {HKver30}
        25:HKData = {HKver25}
        20:HKData = {HKver20}
        else: begin
            ;r = dialog_message('Error...HK block version not supported',/error)
            if(labelID ne 0)then ShowMsg, labelID,'Error...HK block version not supported'
        end
    endcase

    ; position file pointer at beginning of block ( blockMark was read in calling function to get here
    point_lun,fileunit,filepos

    ; store beginning of block position
    begFpos = filepos

    ; read HK Block
    err = 1
    on_ioerror, skipit1
    READU, fileUnit, HKData
    err = 0
    skipit1:
    if(err)then begin
        print, 'Warning: read past end of file'
    endif


    ; read extra data if it is there
    if(HKData.XtraSize gt 0)then begin
        ; to make this usable, need format of the extra data as to type ( integer, float, string etc)
        ; then make a structure to match that format and read data into structure in place of extraData byte array
        extraData =bytarr(HKData.XtraSize)
        err = 1
        on_ioerror, skipit2
        readU,fileUnit,extraData
        err = 0
        skipit2:
        if(err)then begin
            print, 'Warning: read past end of file'
        endif

    endif

    ; place pointer at end of block
    if(vers gt 20)then FilePos = begFpos + HKData.itemSize + HKData.XtraSize else FilePos = begFpos + HKData.itemSize

    if(HKLOG_request)then BEGIN
        SPP100SetMark = FIX('B1B6'X)
        SPP100DataMark = FIX('C1C6'X)
        SBCDataMark = FIX('D1D6'X)
        BlockMark = 0
        ulBlockSize = 0L
        bDone = !false

        on_ioerror, skipit3
        ; Read SPP100, SBC, if present
        WHILE (NOT EOF(fileUnit) AND NOT bDone) DO BEGIN
            readU, fileUnit, BlockMark, ulBlockSize

            CASE 1 OF
                (BlockMark EQ SPP100SetMark): BEGIN
                    IF ulBlockSize EQ 110 THEN BEGIN
                        SPP100Set = {SPP100SET_t}
                        POINT_LUN, fileUnit, FilePos
                        readU, fileUnit, SPP100Set
                        FilePos = FilePos + ulBlockSize

                    ENDIF ELSE bDone = !true
                END

                (BlockMark eq SPP100DataMark): BEGIN
                    CASE 1 OF
                        (ulBlockSize EQ 84): SPP100Data = {SPP100DATA_10CHAN_t}
                        (ulBlockSize EQ 124): SPP100Data = {SPP100DATA_20CHAN_t}
                        (ulBlockSize EQ 164): SPP100Data = {SPP100DATA_30CHAN_t}
                        (ulBlockSize EQ 204): SPP100Data = {SPP100DATA_40CHAN_t}
                        ELSE: BEGIN
                            PRINT, 'Warning: unexpected block size while trying to read SPP100 DATA', ulBlockSize
                            bDone = !true
                        END
                    ENDCASE

                    IF (NOT bDone) THEN BEGIN
                        POINT_LUN, fileUnit, FilePos
                        readU, fileUnit, SPP100Data
                        FilePos = FilePos + ulBlockSize
                    ENDIF
                END

                (BlockMark eq SBCDataMark): BEGIN
                    IF ulBlockSize EQ 52 OR ulBlockSize EQ 68 THEN BEGIN
                        IF ulBlockSize EQ 52 THEN WatchDogSBC = {WATCHDOG_SBC_t} $
                            ELSE WatchDogSBC = {WATCHDOG_SBC2_t}

                        POINT_LUN, fileUnit, FilePos
                        readU, fileUnit, WatchDogSBC
                        FilePos = FilePos + ulBlockSize

                    ENDIF ELSE bDone = !true
                END

                ELSE: BEGIN
                    ;PRINT, BlockMark, FORM='("Warning: unexpected block mark while trying to read SPP100, and SBC Data: ", Z10.4)'
                    bDone = !true
                END
            ENDCASE
        ENDWHILE

        skipit3:

    ENDIF

    ; NOTE: under normal conditions this step should not be needed.  In some cases, the HKblock read exceeds the length of the block
    Actualfilepos = (fstat(fileUnit)).cur_ptr
    if(filePos ne ActualFilePos)then begin
        ;print,(ActualFilePos-filePos)/2
        POINT_LUN, fileUnit, FilePos
        ;print,' Warning reading past end of HK block'
    endif

    return,{htime:HKData.time,fPos:begFPos,HouseSize:HKData.ItemSize + HKData.XtraSize}

end

