; Assumed overall file structure
; {HouseKeeping}{HouseKeeping}{HouseKeeping}{ImageData}{ROIIMG}{ROIIMG}{ROIIMG}{ROIIMG}{HouseKeeping}{HouseKeeping}{HouseKeeping}{HouseKeeping}...{HouseKeeping}...
; where the number of HK blocks varies, each ImageData block specidfies how many ROIIMG blocks will follow, each holding and ROI image.

; a frame is defineds as an ImageData header block, and its associated ROIIMG blocks

;*******************************************************************************************************************
pro read_rois, FileUnit, pad,  pFrame, nodata=noData

    ; Routine to read roi header and data blocks
    ; input parameters:
    ;  FileUnit:       Input        open and valid file logical unit number
    ;  pad:        Input        2 element array with ROI pad values from FrameHeader
    ;  pFrame:       Input/output   pointer to array of structures to hold data

    ; Keyword
    ;  no_data       Input       If defined, do not read ROI images
    ; This routine is valid for data versions 2.0, 2.3, and 2.5 only


    blocktype = FIX('B2E6'X)

    ; Get each ROI
    ROIsCount = (size(*pFrame,/dim))[0]

    ; create local struture for reading
    ImageHeader = ((*pFrame)[0]).ImageHeader
    ROIVer = ImageHeader.ROIVer

    if(ROIsCount gt 0) then begin ; test to make sure there are any ROIs before reading from the file
        for i=0LL, ROIsCount - 1LL do begin
            if(ROIVer lt 30)then begin
                ; Read the full Image header block
                readu, FileUnit, ImageHeader
                ;print,'Read Particle# ',i,' flags= ',ImageHeader.flags,format='(a15,1x,i3,a8,x,z4)'
            endif else begin
                ; read first 39 elements of structure
                for j=0,38 do readu, FileUnit, imageheader.(j)

                ; read last two variable length elements
                Harmonics = fltarr(ImageHeader.Harmonic_len)
                Boundary = intarr(Bound_len)
                readu, FileUnit, Harmonics, Boundary
            endelse

            ; Do some error checking
            ;if(ImageHeader.BlockMark NE blocktype )then r=1; Dialog_message('Missing Sub-Image. Quitting...')

            if(ImageHeader.BlockMark NE blocktype )then begin
                ; bad data? block mark incorrect
                nodata = 1

                break
            endif

            ; Image dimensions
            dx = ImageHeader.ex - ImageHeader.sx + 1
            dy = ImageHeader.ey - ImageHeader.sy + 1

            ; check that image dimensions are reasonable, flag if not and exit loop
            if(dx gt !MAX_DX or dy gt !MAX_DY) then begin
                nodata = 1

                break
            endif

            if(ImageHeader.ROIVer GE 21 )then begin
                area  = ImageHeader.area
                perim = ImageHeader.perim

                ; check for errors in area of perimeter date
                if(area LT 1 OR perim LT 1 OR area GT dx*dy OR perim GT 100*(dx+dy))then begin
                    ImageHeader.area = 0.0
                    ImageHeader.perim = 0.0
                endif
            endif

            if(ImageHeader.len EQ 0 AND ImageHeader.wid EQ 0)then begin
                ; Calculate corrected ROI length and width
                adjusted_dx = long(dx) - 2L*pad[0]
                if(adjusted_dx LE 0)then adjusted_dx = dx - pad[0]
                if(adjusted_dx LE 0)then adjusted_dx = dx

                adjusted_dy = long(dy) - 2L*pad[1]
                if(adjusted_dy LE 0)then adjusted_dy = dy - pad[1]
                if(adjusted_dy LE 0)then ady = dy

                ImageHeader.Len = FLOAT(adjusted_dy)
                ImageHeader.Wid = FLOAT(adjusted_dx)
            endif

            if(not keyword_Set(nodata))then begin
                ; Allocate a buffer for the ROI image
                case abs(ImageHeader.PixBytes) of
                    1: Buf = BYTARR(dx, dy)
                    2: Buf = INTARR(dx, dy)
                    4: Buf = LONARR(dx, dy)
                    else:r=1;dialog_message('Unsupported Image Data Type')
                endcase

                ; Read the ROI image
                readu, FileUnit, Buf

                (*pFrame)[i].pROI = PTR_NEW(TEMPORARY(Buf))
            endif ; nodata if

            ; save data into pointer array
            (*pFrame)[i].ImageHeader = ImageHeader

        endfor
    endif

end

