; Watchdog SBC Structure define routines
pro WATCHDOG_SBC_t__define

    house = {   WATCHDOG_SBC_t,           $ ; structure name
        usBlockNum:0,                $ ; specifies what kind of block this is
        ulItemSize:0L,               $ ; total bytes in the block
        usVer:0,                     $ ; block version number
        szVerName:BYTARR(16),        $
        faTemps:FLTARR(5),           $
        faPressures:FLTARR(2)}
end

pro WATCHDOG_SBC2_t__define

    house = {   WATCHDOG_SBC2_t,          $ ; structure name
        usBlockNum:0,                $ ; specifies what kind of block this is
        ulItemSize:0L,               $ ; total bytes in the block
        usVer:0,                     $ ; block version number
        szVerName:BYTARR(16),        $
        faTemps:FLTARR(5),           $
        faPressures:FLTARR(2),       $
        faVolts:FLTARR(4)}
end


; HouseKeeping Structure define routines   (currently not used)
; designed to read from a file
pro HKver30__define

    house = {   HKver30,                  $ ; name strucuture
        BlockNum:0,                  $ ; specifies what kind of block this is
        ItemSize:0L,                 $ ; total bytes in Housekeeping block
        Ver:0,                       $ ; ImageData version number
        XtraSize:0L,                 $ ; total bytes of extra data immediately after this block
        NewData:0,                   $ ; true if new data arrived, used for PC transfers
        ChkSumFlag:0,                $ ; check sum
        Head:INTARR(3),              $ ; 3 words with some header info
        Time:0L,                     $ ; seconds of year
        Readings:INTARR(46)  }         ; probe housekeeping block data
    ;      XtraData=BYTARR(XtraSize)        ; extra data
end

pro HKver3VCPI__define

    house = { HKver3VCPI,                 $ ; name strucuture
        BlockNum:0,                  $ ; specifies what kind of block this is
        ItemSize:0,                  $ ; total words in this block===so called packet length.  Itemsize should be 83*2.
        Readings1:INTARR(70),        $ ; 70 words from word3 to word72 for Forward Sample Tube Temp to PSD Alignment Info word.
        TimeMSW:0,                   $ ; Time Most Signigicaant Word.
        TimeISW:0,                   $
        TimeLSW:0,                   $ ; Time Least Sig Word.
        Readings:INTARR(8)  }          ; The last 8 words,  From TAS_floating word1 word2 to CheckSum
end

;===============================================================================

FUNCTION ScanSPEC, fileUnit, fileVers, scalefactor = scalefactor, reorder=reorder, labelID=labelID, HKLOG_request=HKLOG_request,path=path

; routine to scan a SPEC data file and determine where the frames are located, the image types, and the times
; paramters
;   fileUnit: Input     valid, open logical file unit number
;   fileVers: Output   variable to hold file version number
;Keyword
;   scaleFactor Input     defines the scale factor for deadtime Count to seconds
;   reorder     Input       selects reordering if true
;   labelID     Input       widgetID for output window in GUI
;   HKLOG_request Input     flag to output HK Log
;   path   Input     main file path
;   return value
;      scanlist:    an array of structures with the following elements, size determined by file
;      FPos:        file pointer for start of frame header
;      BlockMark:   type of blcok read
;      ImgType:     bitwise variable with image type info
;      HTime:       time of most recent housekeeping block
;      day:         time elements of block
;      hour:        time elements of block
;      minute:      time elements of block
;      sec:         time elements of block
;      msec:        time elements of block
;   on ERROR : fileVers returns an error code as follows
;     -1 : ??
;     -2 : block mark error FATAL
;     -3 : block mark error FATAL

; start with a array big enough for 100 roi's, grow as needed
ScanListLen = 100L
dummy = {FileScan1,      $
    BlockMark:0,    $
    FPos:0L,        $
    ImgType:0,      $
    HTime:0L,       $
    deadtime:0.0,   $
    PDSThresh45:0.0,$
    PDSThresh90:0.0,$
    day:0B,         $
    hour:0B,        $
    minute:0B,      $
    sec:0B,         $
    msec:0,         $
    ROIsCount:0,    $
    badframe:0,     $
    TSBlanked:0,    $; flag to force missed to 1 for 3 secs after a background is detected
    HouseSize:0}


BlockMark2 = fix(0) ;  added this by mo on August  4th 2010.

ScanList = REPLICATE({FileScan1}, ScanListLen)

if(not keyword_set(labelID)) then labelID = 0

; create a structure to hold data about the file
DataLength = 200L
conc = fltarr(DataLength)
data ={conc:ptr_new()}

; read the ROI file header
point_lun, fileUnit, 0
ROIHeader = {SPECHead, Ver:0, Year:0, Month:0, DX:0, DY:0, Text:BYTARR(70)}

readU, fileUnit, ROIHeader
fileVers = ROIHeader.Ver

;added this in may 10, 2005
;account for the UAV  roi_frameheader_version =40
;get the roi_frameheader_version first  Added May 8, 2005
date={date,year:0000,month:00,day:00}

defsysv, '!MAX_DX', ROIHeader.DX
defsysv, '!MAX_DY', ROIHeader.DY


frameData = read_frame(FileUnit, ROIHeader.Ver, 80, date=date, TSBlanked=0)
roi_frameheader_version=framedata.frameheader.ver;  ; ver=40 for 3vCpi.


; if scaleFactor eq 0.000341333 then roi_frameheader_version=40 else roi_frameheader_version=25

;skip over the rest of the PI File Header
filePos = 80L
point_lun, fileUnit, filePos

;define some variables
BlockMark = fix(0)
testread  = fix(0)
hk1 = FIX('484B'X);   % 3VCPI  HK  block    is 0x484B   has to be reversed. to 4B 48 in Hex edit.
hk2 = FIX('0053'X);   % 3VCPI  HK  block 0x0053 has to be reversed in 53 00 in Hexediter  0053=5*16+3=83 decimal.
im1 = FIX('A1D5'X);   % 3VCPI images mark has im3 and im1 in it.
im2 = FIX('A2D5'X)
im3 = FIX('A3D5'X);  which is  D5 A3 in Hex Editor in Roi file.
er1 = FIX('B2E6'X)

SPP100SetMark  = FIX('B1B6'X)
SPP100DataMark = FIX('C1C6'X)
SBCDataMark    = FIX('D1D6'X)




counter    = 0L
imgcnt     = 0L
badblocks  = 0L
maxROIsize = 0
istart     = 0

if(not keyword_set(HKLOG_request))then HKLOG_request = !false

if(HKLOG_request)then begin
    if(not keyword_set(path))then hkFileName ='_HKlog.txt' else hkFileName = path+'_HKlog.txt'
    openw,HKLUN,hkFileName,/get_lun
    hdr = '    Time ver    sync     length   type  fwdtubeTemp  upOptBlkT  lowOptBlkT centOptBlkT  aftOptBlkT    noseTemp   PylonTemp  draintubeT  APDAmpTemp outsideairT    DSPcardT  HKCardTemp  APCardTemp   inAirTemp  ' + $
        'CCDCameraT ImLaserTemp PDS45LaserT PDS90LaserT PDS45LaserP    PDS45Ldc PDS90LaserP    PDS90Ldc ImgLasCurnt    ImgLasPW   28Vsupply 5voltsupply   12vsupply  -12vsupply probemode heaterstat detPrt deadtime '+$
        'P45PSET  P45TSet  P90PSet  P90Tset  ILasSET ILASPWset minTTime checksum'
    printf,HKLUN,hdr

    ;Open a file for AirCraft data, just in case we need it
    if(not keyword_set(path))then PlaneLogFileName ='_Airplanelog.txt' else PlaneLogFileName = path+'_Airplanelog.txt'
    openw,PlaneLUN,PlaneLogFileName,/get_lun

    ;Open a file for SPP100 data, just in case we need it
    if(not keyword_set(path))then SPP100LogFileName ='_SPP100log.txt' else SPP100LogFileName = path+'_SPP100log.txt'
    openw,SPP100LUN,SPP100LogFileName,/get_lun

    ;Open a file for Watchdog SBC data, just in case we need it
    if(not keyword_set(path))then SBCLogFileName ='_WatchDogSBClog.txt' else SBCLogFileName = path+'_WatchDogSBClog.txt'
    openw,SBCLUN,SBCLogFileName,/get_lun
endif

BlockSize     = 0L
dterror       = !false
lastFrameTime = 0UL
test          = !true
bError        = !false


iteration_count = 0L

hk_count = 0L
image_count = 0L
else_count = 0L

point_lun, -fileUnit, filePos
lastfilePos = -1L

; This will scan through the entire file and gather information

; TAF NOTE:  this is implemented in an incredibly slow manner -- it will do potentially millions of individual byte reads directly from
; the roi file, instead of using buffers and larger IO operations

print, 'scanspec.pro starting file read at:', systime()
startseconds = systime(/seconds)

;stopmark = 0


WHILE(NOT EOF(fileUnit)) DO begin

    iteration_count++

    ;if(iteration_count mod 1000L eq 0) then begin
    ;    print, 'scanspec iteration: ', iteration_count, 100.*filePos/(fstat(fileunit)).size, '% ', filePos, ' / ', (fstat(fileunit)).size

    ;    wait, .005
    ;endif


    ;    if(stopmark) then begin
    ;        printh, filepos

    ;        stop
    ;    endif



    ;wait, .001

    ; read first integer from header
    ;if(test)then point_lun,fileUnit,filePos + 2
    ;test = !false
    ;print,(fstat(fileunit)).cur_ptr

    ; TAF really don't understand why this fails to increase on bad images -- looks like filepos should have increased!
    if(filePos le lastfilePos) then begin
        ;        print, 'filePos failed to increase'
        ;        printh, filePos
        ;        printh, lastfilePos


        ;        ;stop

        ;filePos += 2L
        ;point_lun, fileUnit, filePos
    endif

    point_lun, -fileUnit, filePos

    if(filePos + 2L ge (fstat(fileUnit)).size) then begin
        print, 'in scanspec main file read while, filePos + 2 exceeds file size, breaking out of while loop'
        printh, filePos
        printh, (fstat(fileUnit)).size

        break
    endif else begin
        readU, fileUnit, BlockMark
        point_lun, -fileUnit, filePos

        ;        print, 'Main read BlockMark returns:'
        ;        printh, BlockMark
        ;        printh, filePos

    endelse

    lastfilePos = filePos

    HouseKeepingBlock = 0
    ImageBlock = 0

    ; use appropriate reader based on block mark
    case BlockMark of
        hk1: begin  ; HouseKeeping Block the 4 words have to be 4B 48 and 53 00 to be sure it is a house keeping block.
            ;print,  filepos
            ;printh, filepos


            readU, fileUnit, BlockMark2
            point_lun, -fileUnit, filePos

            ;point_lun, -fileUnit, filePos

            if BlockMark2 eq '0053'X then begin ;   ; That is '0053'X = 83 words in the 3VCPI houskeeping Block.
                wait, .001

                HKVersion = fileVers

                HKfilePos = filePos

                ;print,  filepos
                ;printh, filepos
                ;stop

                r = read_HTime3VCPI(fileUnit,HKfilePos,HKVersion,roi_frameheader_version,HKLUN, PlaneLUN, SBCLUN, SPP100LUN, HKLOG_request,DTError,labelID)

                ;filePos += 176L

                ; point to next HK/image block, HK can be 0xA6 or 0xB4 long, so test

                if(filePos + 162L ge (fstat(fileUnit)).size) then begin
                    print, 'in scanspec HK file read while, filePos + 162 exceeds file size, breaking out of while loop'
                    printh, filePos
                    printh, (fstat(fileUnit)).size

                    break
                endif else begin
                    filePos += 162L
                    point_lun, fileUnit, filePos

                    readU, fileUnit, testread
                endelse



                ;if(testread eq im3 or testread eq hk1) then begin
                ;    print, 'short HK block found ending at:'
                ;    point_lun, fileUnit, filePos ; back up the file pointer
                ;    printh, filePos
                ;endif else begin
                ;    print, 'long HK block found ending at:'
                ;    filePos += 14L
                ;    point_lun, fileUnit, filePos
                ;    printh, filePos
                ;endelse

                ; test for next blockmark to determine short vs long HK
                if(testread ne im3 and testread ne hk1) then begin
                    filePos += 14L
                endif

                point_lun, fileUnit, filePos


                ;print,  filepos
                ;printh, filepos
                ;stop

                HouseKeepingBlock = 1

                hk_count++

                if(else_count gt 0) then begin
                    ;    print, 'iteration: ', iteration_count, 100.*filePos/(fstat(fileunit)).size, '% ', filePos, ' / ', (fstat(fileunit)).size, ' found valid hk1 BlockMark, after searching', else_count
                    else_count = 0L
                endif else begin
                    ;    print, 'iteration: ', iteration_count, 100.*filePos/(fstat(fileunit)).size, '% ', filePos, ' / ', (fstat(fileunit)).size, ' found valid hk1 BlockMark'
                endelse
            endif
        end
        im3: begin ; Image Block, for 3VCPI only. the image blockmark is 0xA3D5 (D5A3 in hexedit)

            ;print, 'in image start'
            ;print,  filepos
            ;printh, filepos
            ;stop

            ; could test for blockmark2 here 0x68 or 104
            ;readU, fileUnit, BlockMark2
            ;point_lun, fileUnit, filePos

            ;            if(else_count gt 0) then begin
            ;                print, 'found im3 BlockMark, after searching', else_count
            ;                else_count = 0L
            ;            endif
            ;stop
            ; test to make sure this is a valid image block
            ImgBlockSize = 0UL  ; 8 hex digits
            ImgDataVer   = 0U   ; 4 hex digits
            ROIsCount    = 0U   ; 4 hex digits
            TotROIsSize  = 0UL  ; 8 hex digits
            day          = 0B   ; 2 hex digits
            hour         = 0B   ; 2 hex digits
            minute       = 0B   ; 2 hex digits
            sec          = 0B   ; 2 hex digits
            msec         = 0U   ; 4 hex digits
            ImgType      = 0U   ; 4 hex digits

            point_lun, -fileunit, save_filepos ; save the current file position

            READU, fileUnit, ImgBlockSize, ImgDataVer, ROIsCount, TotROIsSize, day, hour, minute, sec, msec, ImgType


            ;            if(day eq 21 and hour eq 22 and minute eq 21 and sec eq 11 and msec eq 149) then begin
            ;                stopmark = 1

            ;                stop
            ;            endif

            ;            if(day eq 21 and hour eq 22 and minute eq 21 and sec eq 11 and msec eq 152) then begin
            ;                stopmark = 1

            ;                stop
            ;            endif

            ;            if(day eq 21 and hour eq 22 and minute eq 21 and sec eq 11 and msec eq 161) then begin
            ;                stopmark = 1

            ;                stop
            ;            endif

            ;point_lun, -fileUnit, filePos

            ; TAF 2014-09-03 adding to initially filter miss-parsed data
            ; 20140821232844.roi was getting ROIsCount = 18754 (0x4942) here,
            ; not sure yet where it is going wrong, but attempting to flag as bad and move on...
            ; We're going to limit the

            max_ROIsCount = 1000 ; maximum allowed ROIs in a frame before we decide it is just bad/missparsed data

            ;if(day lt 32 and hour lt 25 and minute lt 60 and sec lt 60 and ImgBlockSize lt 1000000000) then begin ; TAF should be a better test
            if(day lt 32 and hour lt 25 and minute lt 60 and sec lt 60 and ImgBlockSize eq ulong('0068'X) and ROIsCount lt max_ROIsCount) then begin ; TAF should be a better test
                ;if(day lt 32 and hour lt 25 and minute lt 60 and sec lt 60 and ImgBlockSize lt 1000000000 and $
                ;   ROIsCount lt max_ROIsCount and ROIsCount ne 0) then begin ; TAF should be a better test
                ; seemingly good image, keep

                point_lun, fileunit, save_filepos
                ;point_lun, -fileUnit, filePos

                r = scan_image_block(fileUnit,filePos,BlockMark,fileVers,ROIHeader.year)
                ;info[imgcnt].time = r.HTime   ; r is returned structure.


                ImageBlock = 1
                ;stop
                ;point_lun, -fileUnit, filePos
                ;stop
                ;print, 'iteration: ', iteration_count, 100.*filePos/(fstat(fileunit)).size, '% ', filePos, ' / ', (fstat(fileunit)).size, ' image ImgBlockSize:', ImgBlockSize, ' ROIsCount', ROIsCount, ' TotROIsSize', TotROIsSize

                image_count++
            endif else begin

                ; not an image, keep reading

                if(ROIsCount gt max_ROIsCount) then begin
                    print, 'ROIsCount exceeds max_ROIsCount = ', ROIsCount, ' > ', max_ROIsCount
                    print, 'treating as bad data and skipping frame'
                    printh, filepos

                    ;stop
                endif
                ; else if(ROIsCount eq 0) then begin
                ;    print, 'ROIsCount eq 0'
                ;    printh, filepos
                ;endif

                ;stop
                ;point_lun, fileunit, save_filepos + 2L

                point_lun, fileUnit, lastfilePos + 1L
                point_lun, -fileUnit, filePos

                ;                print, "ScanSPEC found 3V-CPI image BlockMark, but error checking says bad image, likely in data, skipping BlockMark:"
                ;                printh, BlockMark
                ;                print, "At filePos:"
                ;                printh, filePos

                ;                wait, .001


                ;point_lun, fileUnit, lastfilePos + 4L

                ;point_lun, fileunit, save_filepos+4L
                ;point_lun, -fileUnit, filePos

                ;printh, filepos
                ;printh, lastfilePos

                ;point_lun, fileUnit, lastfilePos + 2
                ;filePos = lastfilePos + 2


            endelse

            ;print, 'in image end'
            ;print,  filepos
            ;printh, filepos
            ;stop


        end
        else: begin
            else_count++
            ;stop
        end
    endcase



    ;increase length if needed
    if(counter ge ScanListLen) then begin
        if ScanlistLen gt 1000000LL  then begin
            ScanListLen = Counter + 10000LL
        endif else begin
            ScanListLen = 2*ScanListLen
        endelse

        Tmp = REPLICATE({FileScan1}, ScanListLen)
        Tmp[0] = ScanList
        ScanList = TEMPORARY(Tmp)

        ;        ;if counter gt 4000000LL  then begin
        ;        if counter gt 3990000LL  then begin
        ;            counter=counter-1LL
        ;            goto, skip;  bail out of the program. do just that much of the data of the file.
        ;        endif
    endif

    ;stop
    ;increase length if needed
    if(imgcnt ge DataLength) then begin
        DataLength = 2*DataLength
        Tmp = conc
        conc = fltarr(DataLength)
        conc[0] = TEMPORARY(Tmp)
    endif

    ; TAF 2014-09-03 adding to initially filter miss-parsed data
    ; 20140821232844.roi was getting ROIsCount = 18754 (0x4942) here,
    ; not sure yet where it is going wrong, but attempting to flag as bad and move on...

    ;if(r.ROIsCount gt 1000) then begin
    ;    print, 'r.ROIsCount = ', r.ROIsCount

    ;    stop
    ;endif

    ;assign results to scanList
    if(ImageBlock)then begin;  scanlist is the images data , not house keeping data.
        frameTime = r.HTime + r.msec/1000.0D

        if(r.day ne 0)then begin
            ScanList[counter].BlockMark = BlockMark
            ScanList[counter].fPos = r.fpos
            ScanList[counter].ImgType = r.ImgType
            ScanList[counter].HTime = r.HTime ; was hTime
            ScanList[counter].day = r.day
            ScanList[counter].hour = r.hour
            ScanList[counter].minute = r.minute
            ScanList[counter].sec = r.sec
            ScanList[counter].msec = r.msec

            ScanList[counter].ROIsCount = r.ROIsCount
            ScanList[counter].HouseSize = 0

            if(r.imgType eq 1)then begin
                conc[imgcnt] = r.ROIsCount
                imgcnt = imgcnt + 1
            endif

            if(frameTime eq lastFrameTime)then begin
                scanlist[counter].badframe = 1
                ScanList[counter].BlockMark = 'ffff'X
            endif else lastFrameTime = FrameTime

            counter = counter + 1L
        endif else begin
            badblocks = badblocks + 1
            ;counter = counter - 1LL
        endelse
    endif else if(HouseKeepingBlock) then begin; this is house keeping block for the scan list.
        ScanList[counter].BlockMark = BlockMark
        ScanList[counter].fPos = r.fpos
        ScanList[counter].ImgType = -1       ; imgType = -1 means this is a housekeeping block
        ScanList[counter].HTime = r.HTime ; was hTime
        ScanList[counter].day = r.HTime / 86400L
        days = ScanList[counter].day*86400L
        ScanList[counter].hour =  (r.HTime - days)/3600L
        hours = ScanList[counter].hour*3600L
        ScanList[counter].minute = (r.HTime - days - hours)/60L
        minutes = ScanList[counter].minute*60L
        ScanList[counter].sec = r.HTime - days - hours - minutes
        ScanList[counter].msec = 0
        ScanList[counter].ROIsCount = 0
        ScanList[counter].HouseSize = r.HouseSize
        ScanList[counter].deadtime = r.deadtime
        ScanList[counter].PDSThresh45 = r.PDSThresh45
        ScanList[counter].PDSThresh90 = r.PDSThresh90
        ;print,r.deadtime

        counter = counter + 1L
    endif

    ;print, ''
    ;print, 'At end of while loop:'
    ;printh, filePos
    ;point_lun, -fileunit, pointerfilepos
    ;printh, pointerfilepos

    ;skip:

ENDWHILE


print, 'scanspec read ', hk_count, ' HK blocks, ', image_count, ' image blocks', iteration_count, ' iterations for read'

print, 'scanspec.pro finished at:', systime()
endseconds = systime(/seconds)
print, 'elapsed seconds:', endseconds - startseconds


;stop


CLEANUP:
if (HKLOG_request) then BEGIN
    FREE_LUN, hklun

    ;Erase other log files if no data was stored into them
    SPP100FileSize = (fstat(SPP100LUN)).SIZE
    SBCFileSize = (fstat(SBCLUN)).SIZE
    PlaneFileSize = (fstat(PlaneLUN)).SIZE
    FREE_LUN, SPP100LUN
    FREE_LUN, SBCLUN
    FREE_LUN, PlaneLUN

    ; IDL versions less than 5.4 don't support FILE_DELETE
    IDLVersion = FLOAT(!VERSION.Release)
    IF IDLVersion GT 5.35 THEN BEGIN
        IF SPP100FileSize LT 10 THEN FILE_DELETE, SPP100LogFileName, /QUIET
        IF SBCFileSize LT 10 THEN FILE_DELETE, SBCLogFileName, /QUIET
        IF PlaneFileSize LT 10 THEN FILE_DELETE, PlaneLogFileName, /QUIET
    ENDIF
ENDIF

if(DTerror)then begin
    ;r = dialog_message('Deadtime errors occurred, check output log',/info)
    if(labelID ne 0)then ShowMsg, labelID,'Deadtime errors occurred'
    erase
    xyouts,.02,.40,'WARNING... see log window',/normal,charsize=6.0,font=0,color=255
endif


if(badblocks gt 0)then begin
    msg = strtrim(badblocks,2) + ' blocks were skipped due to errors in the file... day = 0'
    if(labelID ne 0)then ShowMsg, labelID,msg
    erase
    xyouts,.02,.40,'WARNING... see log window',/normal,charsize=6.0,font=0,color=255
endif

IF bError THEN RETURN, 0


if(keyword_set(reorder))then begin

    ; update deadtime values

    ScanList = ScanList[0:counter-1]

    ;search for a time match
    ; find HK frames
    HKIndex = where(ScanList.imgType eq -1,HKCnt)
    FrmIndex = where(Scanlist.imgType ge 0,frmCnt) ; includes empty frames, imaged frames and backgrounds
    if(HKCnt gt 0)then HKBlocks = ScanList[HKIndex]
    ;if(frmCnt gt 0)then begin
    if(HKCnt gt 0 AND frmCnt gt 0)then begin   ;changed to this by carlos nov 11 2004 MOqixu for icvs.

        FrmBlocks = ScanList[FrmIndex]

        ; the scalefactor fro deadtime to seconds depends on the counting clock ;modified on january 21/2004
        ; to account for that UAV deadtime is half of the old dead_time.
        if(not keyword_Set(scaleFactor))then begin
            if (*pstateinfo).roi_frameheader_version eq 40 then scaleFactor=.000341333 else scaleFactor = .000682667
            ;stop
        endif

        lastHTime = 0
        first = !true
        LastFrameEmpty = !false


        ; Loop thru all image type frames
        for i=0L,frmCnt-1 do begin

            ; if this frame does not have the same seconds of year as last frame then
            if(FrmBlocks[i].HTime ne lastHTime or LastFrameEmpty)then begin

                ; find all HouseKeeping blocks with the same HTime as the frame + 1
                index = where(HKBlocks.HTime eq FrmBlocks[i].HTime+1, matchCnt)

                if(matchCnt gt 0)then begin

                    ; record this time so we don't duplicate deadtimes in frames with the same seconds of year
                    if(not LastFrameEmpty)then lastHTime = FrmBlocks[i].HTime
                    if(scanList[FrmIndex[i]].imgtype eq 0)then lastFrameEmpty = !true else LastFrameEmpty = !false

                    ; assign the deadtime w/scaling to the frame
                    scanList[FrmIndex[i]].deadtime = scanList[HKIndex[index[0]]].deadtime * scaleFactor
                    scanList[FrmIndex[i]].PDSThresh45 = scanList[HKIndex[index[0]]].PDSThresh45 ;* .000610352
                    scanList[FrmIndex[i]].PDSThresh90 = scanList[HKIndex[index[0]]].PDSThresh90 ;* .000610352

                    ; check to see if this deadtime is zero
                    if(scanList[FrmIndex[i]].deadtime eq 0)then begin
                        if(first)then begin
                            print,'Frm HTIME   frame FPOS  scan index  frm#  HKindex    HK.deadtime   HK.HTIme'
                            first = !false
                        endif
                        ;print,scanList[FrmIndex[i]].htime,scanList[FrmIndex[i]].fpos,HKIndex[index[0]],i,index[0],scanList[HKIndex[index[0]]].deadtime,scanList[HKIndex[index[0]]].htime
                    endif

                endif
            endif
        endfor
    endif

    if(imgcnt gt 2)then    data.conc = ptr_new(conc[0:imgcnt-1])

endif

; look for frames within 3 secs of a background and mark to set missed to 1
BKIndex = where(ScanList.imgType eq 2,BKCnt)
for i = 0,BKCnt-1 do begin
    ; time of background
    BKTime = ScanList[BKIndex[i]].htime
    ;print,bkindex,bktime,scanlist.htime
    FrameIndex = where(ScanList.htime ge BKTime and ScanList.htime lt BKTime+3 and (ScanList.imgtype eq 1 or ScanList.imgtype eq 0),Cnt)
    ;if(cnt gt 0)then print,BKTime, scanlist[frameindex].htime
    for j = 0,Cnt-1 do scanlist[frameindex].TSBlanked = !true
endfor

return, {ROIHeader:ROIHeader,Scanlist:ScanList[0:counter-1],data:data,badblocks:badblocks}

END

