
;***********************************************************

pro plot_HK_data_DiodeV,ptr

    common basescan_common, show_images, particle, timestamp, prev_sec, timestamp_printed, basefile_lun, file_size, $
        raw_data_buffer, raw_data_buffer_size, buffer_size, buffer_ptr, raw_data, hk_array, mask_array, mask_timestamp_array, $
        timestamp_index, timestamp_array_size, timestamp_array, hk_timestamp_array, hk_probecount_array, $
        probecount_index, probecount_array_size, probecount_array, iteration_counter, $
        HK1_timestamp, HK2_timestamp, HK_timediff_nanos, block_timestamp, buffer
    forward_function SPEC_seconds2HMS

    ; TAF 2014-05-19 the Horizontal and Vertical element voltages were reversed in the documentation
    ; vs the actual HK from the probe, so they are switched accordingly below
    v_he0   = double(hk_array[*, 35]) * 0.00244140625
    v_he21  = double(hk_array[*, 36]) * 0.00244140625 ; Horizontal element  21 voltage
    v_he42  = double(hk_array[*, 37]) * 0.00244140625 ; Horizontal element  42 voltage
    v_he64  = double(hk_array[*, 38]) * 0.00244140625
    v_he85  = double(hk_array[*, 39]) * 0.00244140625 ; Horizontal element  85 voltage
    v_he106 = double(hk_array[*, 40]) * 0.00244140625 ; Horizontal element 106 voltage
    v_he127 = double(hk_array[*, 41]) * 0.00244140625

    v_ve0   = double(hk_array[*, 43]) * 0.00244140625
    v_ve21  = double(hk_array[*, 44]) * 0.00244140625 ; Vertical element  21 voltage
    v_ve42  = double(hk_array[*, 45]) * 0.00244140625 ; Vertical element  42 voltage
    v_ve64  = double(hk_array[*, 46]) * 0.00244140625
    v_ve85  = double(hk_array[*, 47]) * 0.00244140625 ; Vertical element  85 voltage
    v_ve106 = double(hk_array[*, 48]) * 0.00244140625 ; Vertical element 106 voltage
    v_ve127 = double(hk_array[*, 49]) * 0.00244140625


    vrt_lsr    = double(hk_array[*, 31]) * 0.000076294
    hrz_lsr    = double(hk_array[*, 32]) * 0.000076294
    raw_plusv  = double(hk_array[*, 33]) * 0.000152588
    raw_minusv = 2. * double(hk_array[*, 33]) * 0.000152588 - double(hk_array[*, 34]) * 0.00022889


    hk_count=n_elements(hk_timestamp_array)

    date_labels = label_date(date_format='%H:%I:%S')

    jultimes = timegen(start=julday(hk_timestamp_array[0].month,          hk_timestamp_array[0].day,             hk_timestamp_array[0].year,             $
        hk_timestamp_array[0].hour,           hk_timestamp_array[0].minute,          hk_timestamp_array[0].second),          $
        final=julday(hk_timestamp_array[hk_count-1].month, hk_timestamp_array[hk_count-1].day,    hk_timestamp_array[hk_count-1].year,    $
        hk_timestamp_array[hk_count-1].hour,  hk_timestamp_array[hk_count-1].minute, hk_timestamp_array[hk_count-1].second), $
        units='Seconds')


    info=*ptr
    HKdata=*(info.pHKdata)
    Time2DS=(HKdata.nanoseconds/1000000000LL); change to seconds now.

    StartTime0=Time2DS(0)
    EndTime0  =max(Time2DS)

    my_xrange=[StartTime0, EndTime0]

    ;plot out the data and save the png files.
    fullrange =long64(EndTime0-StartTime0); delta_s; ave_info.endNanoseconds - ave_info.startNanoseconds
    binwidth = fullrange/8LL
    curbin = long64(StartTime0);  ave_info.startNanoseconds

    tnames = StrArr(9)
    FOR i=0,8 DO BEGIN
        tnames(i) =  strtrim(strmid(SPEC_seconds2HMS(curbin),0,8),2)
        curbin= curbin+binwidth
    ENDFOR
    ;-------------------------------
    ;Set up X axis tick labels.
    ;-------------------------------
    !X.TICKNAME = tnames
    !X.TICKS = 8;      !charsize=1.5
    ;********************************************************
    device, decomposed = 0
    loadct, 39, /silent

    stored_pmulti = !P.MULTI
    !P.MULTI = [0, 1, 2] ; stack three plots vertically in window

    bluecolor =120;   GetColor("Navy", !D.Table_Size-2)
    redcolor = 254; GetColor("Red", !D.Table_Size-2)
    blackcolor=10;   getColor("Black",!D.Table_Size-2)
    whitecolor =0; GetColor("White", !D.Table_Size-2)
    greencolor =170; GetColor("Green", !D.Table_Size-2)
    Purplecolor  =180;GetColor("Purple",!D.Table_Size-2)
    Pinkcolor  =230;  GetColor("Pink",!D.Table_Size-2)

    Widget_Control, info.drawidHK, Get_Value=wid  ; get the window ID for drawing.
    WSet, wid
    ;erase,color='ffffff'x  ; get background to white

    !X.OMARGIN = [1.5,0.0]  ;We need this because the Area plot X axis label extends off the left edge of the window

    basefile=file_basename(info.currentFname);  basefile

    device, decomposed = 0
    loadct, 39, /silent

    color = 5
    color_step = 8

    plot, v_ve0, xrange = [0, hk_count-1], /xst, yrange=[0, 5], /yst, $
        pos=[0.05,0.45,0.975,0.95],xmargin=[3,0], charsize=1.3, $
        title='HK Voltages (H diodes Reds, V diodes Greens)', ytitle = 'Voltage (V)', xtitle = ' '

    ; plot V diodes in greens
    color = 170
    color_step = -11

    oplot, v_ve0, color = color
    color = (color + color_step) mod 255
    oplot, v_ve21, color = color
    color = (color + color_step) mod 255
    oplot, v_ve42, color = color
    color = (color + color_step) mod 255
    oplot, v_ve64, color = color
    color = (color + color_step) mod 255
    oplot, v_ve85, color = color
    color = (color + color_step) mod 255
    oplot, v_ve106, color = color
    color = (color + color_step) mod 255
    oplot, v_ve127, color = color
    color = (color + color_step) mod 255

    ; plot H diodes in reds
    color = 254
    color_step = -8

    oplot, v_he0, color = color
    color = (color + color_step) mod 255
    oplot, v_he21, color = color
    color = (color + color_step) mod 255
    oplot, v_he42, color = color
    color = (color + color_step) mod 255
    oplot, v_he64, color = color
    color = (color + color_step) mod 255
    oplot, v_he85, color = color
    color = (color + color_step) mod 255
    oplot, v_he106, color = color
    color = (color + color_step) mod 255
    oplot, v_he127, color = color
    color = (color + color_step) mod 255

    plot, vrt_lsr, xrange = [0, hk_count-1], /xst, yrange=[0, 8], /yst, $
        pos=[0.05,0.12,0.975,0.35],xmargin=[3,0], charsize=1.3,$
        title='HK Voltages (Lasers and Raw+-)', ytitle = 'Voltage (V)', xtitle = 'HK Packet'

    oplot, vrt_lsr, color = 30
    oplot, hrz_lsr, color = 70

    oplot, raw_plusv,  color = 120
    oplot, raw_minusv, color = 250

    !P.MULTI =  stored_pmulti

end; ********************************
;******************************************************

pro plot_HK_data_temp,ptr

    common basescan_common, show_images, particle, timestamp, prev_sec, timestamp_printed, basefile_lun, file_size, $
        raw_data_buffer, raw_data_buffer_size, buffer_size, buffer_ptr, raw_data, hk_array, mask_array, mask_timestamp_array, $
        timestamp_index, timestamp_array_size, timestamp_array, hk_timestamp_array, hk_probecount_array, $
        probecount_index, probecount_array_size, probecount_array, iteration_counter, $
        HK1_timestamp, HK2_timestamp, HK_timediff_nanos, block_timestamp, buffer
    forward_function SPEC_seconds2HMS


    stop

    ; convert the raw counts to physical units

    temp_fst   = counts2temp(hk_array[*,  1]) ; Forward Sample Tube Temp
    temp_uob   = counts2temp(hk_array[*,  2]) ; Upper Optics Block temp
    temp_lob   = counts2temp(hk_array[*,  3]) ; Lower Optics Block temp
    temp_cst   = counts2temp(hk_array[*,  4]) ; Central Sample Tube temp
    temp_ast   = counts2temp(hk_array[*,  5]) ; Aft Sample Tube temp
    temp_pyln1 = counts2temp(hk_array[*,  6]) ; Pylon 1 temp
    temp_pyln2 = counts2temp(hk_array[*,  7]) ; Pylon 2 temp
    temp_pyln3 = counts2temp(hk_array[*,  8]) ; Pylon 3 temp
    temp_ccd   = counts2temp(hk_array[*,  9]) ; CCD Camera temp
    temp_il    = counts2temp(hk_array[*, 10]) ; Imaging Lens Temp
    temp_ilz   = counts2temp(hk_array[*, 11]) ; Imaging Laser Temp

    ; apparently the lasers use a different temperature sensor -- not sure where this info came from, but doesn't seem accurate all the time...
    ;temp_vlz = double(hk_array[*, 12]) * 0.010173 - 273.15 ; Vertical Laser Temp
    ;temp_hlz = double(hk_array[*, 13]) * 0.010173 - 273.15 ; Horizontal Laser temp

    temp_vlz   = counts2temp(hk_array[*, 12])
    temp_hlz   = counts2temp(hk_array[*, 13])

    temp_pbd   = counts2temp(hk_array[*, 14]) ; Power Board temp
    temp_vpl   = counts2temp(hk_array[*, 15])
    temp_vop   = counts2temp(hk_array[*, 16])
    temp_hpl   = counts2temp(hk_array[*, 17])
    temp_hop   = counts2temp(hk_array[*, 18])
    temp_vmr   = counts2temp(hk_array[*, 19])
    temp_hmr   = counts2temp(hk_array[*, 20])
    temp_ipl   = counts2temp(hk_array[*, 21])
    temp_dsp   = counts2temp(hk_array[*, 22])
    temp_vayt  = counts2temp(hk_array[*, 23])
    temp_vayb  = counts2temp(hk_array[*, 24])
    temp_hayt  = counts2temp(hk_array[*, 25])
    temp_hayb  = counts2temp(hk_array[*, 26])

    rh         = double(hk_array[*, 27]) * 0.002515 - 0.2802198  ; Relative Humidity
    can_pres   = double(hk_array[*, 28]) * 0.000572205 -3.75     ; Internal pressure

    vrt_tec_i  = double(hk_array[*, 29]) * 0.000050498
    hrz_tec_i  = double(hk_array[*, 30]) * 0.000050498
    vrt_lsr    = double(hk_array[*, 31]) * 0.000076294
    hrz_lsr    = double(hk_array[*, 32]) * 0.000076294
    raw_plusv  = double(hk_array[*, 33]) * 0.000152588
    raw_minusv = 2. * double(hk_array[*, 33]) * 0.000152588 - double(hk_array[*, 34]) * 0.00022889

    ; TAF 2014-05-19 the Horizontal and Vertical element voltages were reversed in the documentation
    ; vs the actual HK from the probe, so they are switched accordingly below
    v_he0   = double(hk_array[*, 35]) * 0.00244140625
    v_he21  = double(hk_array[*, 36]) * 0.00244140625 ; Horizontal element  21 voltage
    v_he42  = double(hk_array[*, 37]) * 0.00244140625 ; Horizontal element  42 voltage
    v_he64  = double(hk_array[*, 38]) * 0.00244140625
    v_he85  = double(hk_array[*, 39]) * 0.00244140625 ; Horizontal element  85 voltage
    v_he106 = double(hk_array[*, 40]) * 0.00244140625 ; Horizontal element 106 voltage
    v_he127 = double(hk_array[*, 41]) * 0.00244140625

    img_lsr_i = double(hk_array[*, 42]) * 0.0268555

    v_ve0   = double(hk_array[*, 43]) * 0.00244140625
    v_ve21  = double(hk_array[*, 44]) * 0.00244140625 ; Vertical element  21 voltage
    v_ve42  = double(hk_array[*, 45]) * 0.00244140625 ; Vertical element  42 voltage
    v_ve64  = double(hk_array[*, 46]) * 0.00244140625
    v_ve85  = double(hk_array[*, 47]) * 0.00244140625 ; Vertical element  85 voltage
    v_ve106 = double(hk_array[*, 48]) * 0.00244140625 ; Vertical element 106 voltage
    v_ve127 = double(hk_array[*, 49]) * 0.00244140625

    img_lsr_pw      = double(hk_array[*, 50]) * 0.0268555
    img_lsr_i_set   = double(hk_array[*, 51]) * 0.014648
    img_lsr_pw_set  = double(hk_array[*, 52]) * 0.014648
    probe_mode_raw  =        hk_array[*, 53]  ; bits 6-7 CPI Trigger Source
    heater_status   = double(hk_array[*, 54]) * 1.
    optical_pwm_pct = double(hk_array[*, 55]) * 0.0268555
    num_hrz_dect    =        hk_array[*, 56]  ; Horizontal particles detected
    num_vrt_dect    =        hk_array[*, 57]  ; Vertical particles detected
    dead_time       = double(hk_array[*, 58]) ; dead time in second
    cpi_trig_set1   =        hk_array[*, 59]  ; CPI triggering settings
    cpi_trig_set2   =        hk_array[*, 60]  ; CPI triggering settings bits 11-16 min pixels before fire
    vrt_laser_drv   = double(hk_array[*, 61]) ; Vertical laser drive
    hrz_laser_drv   = double(hk_array[*, 62]) ; Horizontal laser drive
    vrt_masked_bits =        hk_array[*, 63]  ; # of vertical bits masked
    hrz_masked_bits =        hk_array[*, 64]  ; # of horizontal bits masked
    num_hrz_ovrld   =        hk_array[*, 65]  ; # of overload TWs found
    num_vrt_ovrld   =        hk_array[*, 66]  ; # of overload TWs found
    num_stereo_dect =        hk_array[*, 67]  ; # of overload TWs found
    comp_config     =        hk_array[*, 68]  ; Compression configuration

    pds_align_w1 = hk_array[*, 69] ; # of 4kb buffers sent last second
    pds_align_w2 = hk_array[*, 70] ; # of 4kb buffers sent last second

    TW_msw = hk_array[*, 71] ;  the three timing words
    TW_isw = hk_array[*, 72]
    TW_lsw = hk_array[*, 73]

    tas = two_uints2float(hk_array[*, 74], hk_array[*, 75])

    twods_cmd_acpt = hk_array[*, 76] ; % of 2ds commands accepted
    cpi_cmd_acpt = hk_array[*, 77] ; % of detected overlaps (from
    n2ds_data_blks = hk_array[*, 78] ; Number of 2D-S data blocks (4098 byte) sent over the last second
    array_skew_off = hk_array[*, 79] ; Array Skew Offset (six bits: 0-5)

    cpi_frame_rate = hk_array[*, 80] and '0FFF'X ; CPI Frame Rate

    hk_count=n_elements(hk_timestamp_array)

    date_labels = label_date(date_format='%H:%I:%S')

    jultimes = timegen(start=julday(hk_timestamp_array[0].month,          hk_timestamp_array[0].day,             hk_timestamp_array[0].year,             $
        hk_timestamp_array[0].hour,           hk_timestamp_array[0].minute,          hk_timestamp_array[0].second),          $
        final=julday(hk_timestamp_array[hk_count-1].month, hk_timestamp_array[hk_count-1].day,    hk_timestamp_array[hk_count-1].year,    $
        hk_timestamp_array[hk_count-1].hour,  hk_timestamp_array[hk_count-1].minute, hk_timestamp_array[hk_count-1].second), $
        units='Seconds')




    ; combine the 3 components of the 48 bit timing word

    hk_probecount_array = ulong64(ishft(ulong64(TW_msw), 32) or ishft(ulong64(TW_isw), 16) or ulong64(TW_lsw))

    info=*ptr
    HKdata=*(info.pHKdata)
    Time2DS=(HKdata.nanoseconds/1000000000LL); change to seconds now.

    StartTime0=Time2DS(0)
    EndTime0  =max(Time2DS)

    my_xrange=[StartTime0, EndTime0]

    ;plot out the data and save the png files.
    fullrange =long64(EndTime0-StartTime0); delta_s; ave_info.endNanoseconds - ave_info.startNanoseconds
    binwidth = fullrange/8LL
    curbin = long64(StartTime0);  ave_info.startNanoseconds

    tnames = StrArr(9)
    FOR i=0,8 DO BEGIN
        tnames(i) =  strtrim(strmid(SPEC_seconds2HMS(curbin),0,8),2)
        curbin= curbin+binwidth
    ENDFOR
    ;-------------------------------
    ;Set up X axis tick labels.
    ;-------------------------------
    !X.TICKNAME = tnames
    !X.TICKS = 8;      !charsize=1.5
    ;********************************************************
    device, decomposed = 0
    loadct, 39, /silent

    bluecolor =120;   GetColor("Navy", !D.Table_Size-2)
    redcolor = 254; GetColor("Red", !D.Table_Size-2)
    blackcolor=10;   getColor("Black",!D.Table_Size-2)
    whitecolor =0; GetColor("White", !D.Table_Size-2)
    greencolor =170; GetColor("Green", !D.Table_Size-2)
    Purplecolor  =180;GetColor("Purple",!D.Table_Size-2)
    Pinkcolor  =230;  GetColor("Pink",!D.Table_Size-2)

    Widget_Control, info.drawidHK, Get_Value=wid  ; get the window ID for drawing.
    WSet, wid
    ;erase,color='ffffff'x  ; get background to white

    !P.MULTI = [0, 1,1, 0, 0]
    !X.OMARGIN = [1.5,0.0]  ;We need this because the Area plot X axis label extends off the left edge of the window

    basefile=file_basename(info.currentFname);  basefile

    device, decomposed = 0
    loadct, 39, /silent

    color = 5
    color_step = 8

    ;plot, temp_fst, xrange = [0, hk_count-1], /xst, yrange=[-60, 60], /yst, title='HK Temperatures', ytitle = 'Temperature (C)', xtitle = 'HK Packet'

    plot, jultimes, temp_fst, yrange=[-60, 60], /yst, title='HK Temperatures', ytitle = 'Temperature (C)', $
        pos=[0.05,0.11,0.975,0.95],xmargin=[3,0], charsize=1.3,$
        xtickunits=['Time'], xtickformat='LABEL_DATE', xstyle=1, xticks=nxticks
    ;xtickunits=['Time','Time'], xtickformat='LABEL_DATE', xstyle=1, xticks=nxticks, ymargin=[6,2] ; this label scheme has the major hours picked out on a lower line

    oplot, jultimes, temp_fst, color = color
    color = (color + color_step) mod 255
    oplot, jultimes, temp_uob, color = color
    color = (color + color_step) mod 255
    oplot, jultimes, temp_uob, color = color
    color = (color + color_step) mod 255
    oplot, jultimes, temp_lob, color = color
    color = (color + color_step) mod 255
    oplot, jultimes, temp_cst, color = color
    color = (color + color_step) mod 255
    oplot, jultimes, temp_ast, color = color
    color = (color + color_step) mod 255
    oplot, jultimes, temp_pyln1, color = color
    color = (color + color_step) mod 255
    oplot, jultimes, temp_pyln2, color = color
    color = (color + color_step) mod 255
    oplot, jultimes, temp_pyln3, color = color
    color = (color + color_step) mod 255
    oplot, jultimes, temp_ccd, color = color
    color = (color + color_step) mod 255
    oplot, jultimes, temp_il, color = color
    color = (color + color_step) mod 255
    oplot, jultimes, temp_ilz, color = color
    color = (color + color_step) mod 255
    oplot, jultimes, temp_vlz, color = color
    color = (color + color_step) mod 255
    oplot, jultimes, temp_hlz, color = color
    color = (color + color_step) mod 255
    oplot, jultimes, temp_pbd, color = color
    color = (color + color_step) mod 255
    oplot, jultimes, temp_vpl, color = color
    color = (color + color_step) mod 255
    oplot, jultimes, temp_vop, color = color
    color = (color + color_step) mod 255
    oplot, jultimes, temp_hpl, color = color
    color = (color + color_step) mod 255
    oplot, jultimes, temp_hop, color = color
    color = (color + color_step) mod 255
    oplot, jultimes, temp_vmr, color = color
    color = (color + color_step) mod 255
    oplot, jultimes, temp_hmr, color = color
    color = (color + color_step) mod 255
    oplot, jultimes, temp_ipl, color = color
    color = (color + color_step) mod 255
    oplot, jultimes, temp_dsp, color = color
    color = (color + color_step) mod 255
    oplot, jultimes, temp_vayt, color = color
    color = (color + color_step) mod 255
    oplot, jultimes, temp_vayb, color = color
    color = (color + color_step) mod 255
    oplot, jultimes, temp_hayt, color = color
    color = (color + color_step) mod 255
    oplot, jultimes, temp_hayb, color = color
    color = (color + color_step) mod 255



end; ********************************
;******************************************************




pro plot_HK_data_RH_TAS_can_pressure,ptr

    common basescan_common, show_images, particle, timestamp, prev_sec, timestamp_printed, basefile_lun, file_size, $
        raw_data_buffer, raw_data_buffer_size, buffer_size, buffer_ptr, raw_data, hk_array, mask_array, mask_timestamp_array, $
        timestamp_index, timestamp_array_size, timestamp_array, hk_timestamp_array, hk_probecount_array, $
        probecount_index, probecount_array_size, probecount_array, iteration_counter, $
        HK1_timestamp, HK2_timestamp, HK_timediff_nanos, block_timestamp, buffer
    forward_function SPEC_seconds2HMS




    ; convert the raw counts to physical units

    temp_fst   = counts2temp(hk_array[*,  1]) ; Forward Sample Tube Temp
    temp_uob   = counts2temp(hk_array[*,  2]) ; Upper Optics Block temp
    temp_lob   = counts2temp(hk_array[*,  3]) ; Lower Optics Block temp
    temp_cst   = counts2temp(hk_array[*,  4]) ; Central Sample Tube temp
    temp_ast   = counts2temp(hk_array[*,  5]) ; Aft Sample Tube temp
    temp_pyln1 = counts2temp(hk_array[*,  6]) ; Pylon 1 temp
    temp_pyln2 = counts2temp(hk_array[*,  7]) ; Pylon 2 temp
    temp_pyln3 = counts2temp(hk_array[*,  8]) ; Pylon 3 temp
    temp_ccd   = counts2temp(hk_array[*,  9]) ; CCD Camera temp
    temp_il    = counts2temp(hk_array[*, 10]) ; Imaging Lens Temp
    temp_ilz   = counts2temp(hk_array[*, 11]) ; Imaging Laser Temp

    ; apparently the lasers use a different temperature sensor -- not sure where this info came from, but doesn't seem accurate all the time...
    ;temp_vlz = double(hk_array[*, 12]) * 0.010173 - 273.15 ; Vertical Laser Temp
    ;temp_hlz = double(hk_array[*, 13]) * 0.010173 - 273.15 ; Horizontal Laser temp

    temp_vlz   = counts2temp(hk_array[*, 12])
    temp_hlz   = counts2temp(hk_array[*, 13])

    temp_pbd   = counts2temp(hk_array[*, 14]) ; Power Board temp
    temp_vpl   = counts2temp(hk_array[*, 15])
    temp_vop   = counts2temp(hk_array[*, 16])
    temp_hpl   = counts2temp(hk_array[*, 17])
    temp_hop   = counts2temp(hk_array[*, 18])
    temp_vmr   = counts2temp(hk_array[*, 19])
    temp_hmr   = counts2temp(hk_array[*, 20])
    temp_ipl   = counts2temp(hk_array[*, 21])
    temp_dsp   = counts2temp(hk_array[*, 22])
    temp_vayt  = counts2temp(hk_array[*, 23])
    temp_vayb  = counts2temp(hk_array[*, 24])
    temp_hayt  = counts2temp(hk_array[*, 25])
    temp_hayb  = counts2temp(hk_array[*, 26])

    rh         = double(hk_array[*, 27]) * 0.002515 - 0.2802198  ; Relative Humidity
    can_pres   = double(hk_array[*, 28]) * 0.000572205 -3.75     ; Internal pressure

    vrt_tec_i  = double(hk_array[*, 29]) * 0.000050498
    hrz_tec_i  = double(hk_array[*, 30]) * 0.000050498
    vrt_lsr    = double(hk_array[*, 31]) * 0.000076294
    hrz_lsr    = double(hk_array[*, 32]) * 0.000076294
    raw_plusv  = double(hk_array[*, 33]) * 0.000152588
    raw_minusv = 2. * double(hk_array[*, 33]) * 0.000152588 - double(hk_array[*, 34]) * 0.00022889

    ; TAF 2014-05-19 the Horizontal and Vertical element voltages were reversed in the documentation
    ; vs the actual HK from the probe, so they are switched accordingly below
    v_he0   = double(hk_array[*, 35]) * 0.00244140625
    v_he21  = double(hk_array[*, 36]) * 0.00244140625 ; Horizontal element  21 voltage
    v_he42  = double(hk_array[*, 37]) * 0.00244140625 ; Horizontal element  42 voltage
    v_he64  = double(hk_array[*, 38]) * 0.00244140625
    v_he85  = double(hk_array[*, 39]) * 0.00244140625 ; Horizontal element  85 voltage
    v_he106 = double(hk_array[*, 40]) * 0.00244140625 ; Horizontal element 106 voltage
    v_he127 = double(hk_array[*, 41]) * 0.00244140625

    img_lsr_i = double(hk_array[*, 42]) * 0.0268555

    v_ve0   = double(hk_array[*, 43]) * 0.00244140625
    v_ve21  = double(hk_array[*, 44]) * 0.00244140625 ; Vertical element  21 voltage
    v_ve42  = double(hk_array[*, 45]) * 0.00244140625 ; Vertical element  42 voltage
    v_ve64  = double(hk_array[*, 46]) * 0.00244140625
    v_ve85  = double(hk_array[*, 47]) * 0.00244140625 ; Vertical element  85 voltage
    v_ve106 = double(hk_array[*, 48]) * 0.00244140625 ; Vertical element 106 voltage
    v_ve127 = double(hk_array[*, 49]) * 0.00244140625

    img_lsr_pw      = double(hk_array[*, 50]) * 0.0268555
    img_lsr_i_set   = double(hk_array[*, 51]) * 0.014648
    img_lsr_pw_set  = double(hk_array[*, 52]) * 0.014648
    probe_mode_raw  =        hk_array[*, 53]  ; bits 6-7 CPI Trigger Source
    heater_status   = double(hk_array[*, 54]) * 1.
    optical_pwm_pct = double(hk_array[*, 55]) * 0.0268555
    num_hrz_dect    =        hk_array[*, 56]  ; Horizontal particles detected
    num_vrt_dect    =        hk_array[*, 57]  ; Vertical particles detected
    dead_time       = double(hk_array[*, 58]) ; dead time in second
    cpi_trig_set1   =        hk_array[*, 59]  ; CPI triggering settings
    cpi_trig_set2   =        hk_array[*, 60]  ; CPI triggering settings bits 11-16 min pixels before fire
    vrt_laser_drv   = double(hk_array[*, 61]) ; Vertical laser drive
    hrz_laser_drv   = double(hk_array[*, 62]) ; Horizontal laser drive
    vrt_masked_bits =        hk_array[*, 63]  ; # of vertical bits masked
    hrz_masked_bits =        hk_array[*, 64]  ; # of horizontal bits masked
    num_hrz_ovrld   =        hk_array[*, 65]  ; # of overload TWs found
    num_vrt_ovrld   =        hk_array[*, 66]  ; # of overload TWs found
    num_stereo_dect =        hk_array[*, 67]  ; # of overload TWs found
    comp_config     =        hk_array[*, 68]  ; Compression configuration

    pds_align_w1 = hk_array[*, 69] ; # of 4kb buffers sent last second
    pds_align_w2 = hk_array[*, 70] ; # of 4kb buffers sent last second

    TW_msw = hk_array[*, 71] ;  the three timing words
    TW_isw = hk_array[*, 72]
    TW_lsw = hk_array[*, 73]

    tas = two_uints2float(hk_array[*, 74], hk_array[*, 75])

    twods_cmd_acpt = hk_array[*, 76] ; % of 2ds commands accepted
    cpi_cmd_acpt = hk_array[*, 77] ; % of detected overlaps (from
    n2ds_data_blks = hk_array[*, 78] ; Number of 2D-S data blocks (4098 byte) sent over the last second
    array_skew_off = hk_array[*, 79] ; Array Skew Offset (six bits: 0-5)

    cpi_frame_rate = hk_array[*, 80] and '0FFF'X ; CPI Frame Rate

    hk_count=n_elements(hk_timestamp_array)

    date_labels = label_date(date_format='%H:%I:%S')

    jultimes = timegen(start=julday(hk_timestamp_array[0].month,          hk_timestamp_array[0].day,             hk_timestamp_array[0].year,             $
        hk_timestamp_array[0].hour,           hk_timestamp_array[0].minute,          hk_timestamp_array[0].second),          $
        final=julday(hk_timestamp_array[hk_count-1].month, hk_timestamp_array[hk_count-1].day,    hk_timestamp_array[hk_count-1].year,    $
        hk_timestamp_array[hk_count-1].hour,  hk_timestamp_array[hk_count-1].minute, hk_timestamp_array[hk_count-1].second), $
        units='Seconds')




    ; combine the 3 components of the 48 bit timing word

    hk_probecount_array = ulong64(ishft(ulong64(TW_msw), 32) or ishft(ulong64(TW_isw), 16) or ulong64(TW_lsw))

    info=*ptr
    HKdata=*(info.pHKdata)
    Time2DS=(HKdata.nanoseconds/1000000000LL); change to seconds now.

    StartTime0=Time2DS(0)
    EndTime0  =max(Time2DS)

    my_xrange=[StartTime0, EndTime0]

    ;plot out the data and save the png files.
    fullrange =long64(EndTime0-StartTime0); delta_s; ave_info.endNanoseconds - ave_info.startNanoseconds
    binwidth = fullrange/8LL
    curbin = long64(StartTime0);  ave_info.startNanoseconds

    tnames = StrArr(9)
    FOR i=0,8 DO BEGIN
        tnames(i) =  strtrim(strmid(SPEC_seconds2HMS(curbin),0,8),2)
        curbin= curbin+binwidth
    ENDFOR
    ;-------------------------------
    ;Set up X axis tick labels.
    ;-------------------------------
    !X.TICKNAME = tnames
    !X.TICKS = 8;      !charsize=1.5
    ;********************************************************
    device, decomposed = 0
    loadct, 39, /silent

    bluecolor =120;   GetColor("Navy", !D.Table_Size-2)
    redcolor = 254; GetColor("Red", !D.Table_Size-2)
    blackcolor=10;   getColor("Black",!D.Table_Size-2)
    whitecolor =0; GetColor("White", !D.Table_Size-2)
    greencolor =170; GetColor("Green", !D.Table_Size-2)
    Purplecolor  =180;GetColor("Purple",!D.Table_Size-2)
    Pinkcolor  =230;  GetColor("Pink",!D.Table_Size-2)

    Widget_Control, info.drawidHK, Get_Value=wid  ; get the window ID for drawing.
    WSet, wid
    ;erase,color='ffffff'x  ; get background to white

    !P.MULTI = [0, 1,1, 0, 0]
    !X.OMARGIN = [1.5,0.0]  ;We need this because the Area plot X axis label extends off the left edge of the window

    basefile=file_basename(info.currentFname);  basefile

    device, decomposed = 0
    loadct, 39, /silent

    color = 5
    color_step = 8

    stored_pmulti = !P.MULTI
    !P.MULTI = [0, 1, 3] ; stack three plots vertically in window

    plot, jultimes, rh, yrange=[0, 100], /yst, title='HK Relative Humidity', ytitle = 'Relative Humidity (%)', $
        pos=[0.05,0.73,0.975,0.95],xmargin=[3,0], charsize=2.3,$
        xtickunits=['Time'], xtickformat='LABEL_DATE', xstyle=1, xticks=nxticks
    oplot, jultimes, rh, color = 170

    plot, jultimes, can_pres, yrange=[0, 15], /yst, title='HK Can Pressure', ytitle = 'Pressure (PSI)', $
        pos=[0.05,0.41,0.975,0.63],xmargin=[3,0], charsize=2.3, $
        xtickunits=['Time'], xtickformat='LABEL_DATE', xstyle=1, xticks=nxticks
    oplot, jultimes, can_pres, color = 70

    plot, jultimes, tas, yrange=[0, 200], /yst, title='HK True Air Speed', ytitle = 'TAS (m/s)', $
        pos=[0.05,0.10,0.975,0.32],xmargin=[3,0], charsize=2.3, $
        xtickunits=['Time'], xtickformat='LABEL_DATE', xstyle=1, xticks=nxticks
    oplot, jultimes, tas, color = 120

    !P.MULTI = stored_pmulti
end; plot_RH_TAS_can_pressure   ********************************
;******************************************************



pro plot_HK_data_mask_overload,ptr

    common basescan_common, show_images, particle, timestamp, prev_sec, timestamp_printed, basefile_lun, file_size, $
        raw_data_buffer, raw_data_buffer_size, buffer_size, buffer_ptr, raw_data, hk_array, mask_array, mask_timestamp_array, $
        timestamp_index, timestamp_array_size, timestamp_array, hk_timestamp_array, hk_probecount_array, $
        probecount_index, probecount_array_size, probecount_array, iteration_counter, $
        HK1_timestamp, HK2_timestamp, HK_timediff_nanos, block_timestamp, buffer
    forward_function SPEC_seconds2HMS



    dead_time       = double(hk_array[*, 58]) ; dead time in second

    vrt_masked_bits =        hk_array[*, 63]  ; # of vertical bits masked
    hrz_masked_bits =        hk_array[*, 64]  ; # of horizontal bits masked

    num_hrz_ovrld   =        hk_array[*, 65]  ; # of overload TWs found
    num_vrt_ovrld   =        hk_array[*, 66]  ; # of overload TWs found

    hk_count=n_elements(hk_timestamp_array)

    date_labels = label_date(date_format='%H:%I:%S')

    jultimes = timegen(start=julday(hk_timestamp_array[0].month,          hk_timestamp_array[0].day,             hk_timestamp_array[0].year,             $
        hk_timestamp_array[0].hour,           hk_timestamp_array[0].minute,          hk_timestamp_array[0].second),          $
        final=julday(hk_timestamp_array[hk_count-1].month, hk_timestamp_array[hk_count-1].day,    hk_timestamp_array[hk_count-1].year,    $
        hk_timestamp_array[hk_count-1].hour,  hk_timestamp_array[hk_count-1].minute, hk_timestamp_array[hk_count-1].second), $
        units='Seconds')
    info=*ptr
    HKdata=*(info.pHKdata)
    Time2DS=(HKdata.nanoseconds/1000000000LL); change to seconds now.

    StartTime0=Time2DS(0)
    EndTime0  =max(Time2DS)

    my_xrange=[StartTime0, EndTime0]

    ;plot out the data and save the png files.
    fullrange =long64(EndTime0-StartTime0); delta_s; ave_info.endNanoseconds - ave_info.startNanoseconds
    binwidth = fullrange/8LL
    curbin = long64(StartTime0);  ave_info.startNanoseconds

    tnames = StrArr(9)
    FOR i=0,8 DO BEGIN
        tnames(i) =  strtrim(strmid(SPEC_seconds2HMS(curbin),0,8),2)
        curbin= curbin+binwidth
    ENDFOR
    ;-------------------------------
    ;Set up X axis tick labels.
    ;-------------------------------
    !X.TICKNAME = tnames
    !X.TICKS = 8;      !charsize=1.5
    ;********************************************************
    device, decomposed = 0
    loadct, 39, /silent

    Widget_Control, info.drawidHK, Get_Value=wid  ; get the window ID for drawing.
    WSet, wid

    !X.OMARGIN = [1.5,0.0]  ;We need this because the Area plot X axis label extends off the left edge of the window

    basefile=file_basename(info.currentFname);  basefile

    device, decomposed = 0
    loadct, 39, /silent

    color = 5
    color_step = 8

    ; condition the data for the log plot -- don't output these values though

    temp_hrz_masked_bits = hrz_masked_bits
    temp_vrt_masked_bits = vrt_masked_bits

    temp_num_hrz_ovrld   = num_hrz_ovrld
    temp_num_vrt_ovrld   = num_vrt_ovrld


    index = where(temp_hrz_masked_bits eq 0, count)
    if(count gt 0) then begin
        temp_hrz_masked_bits[index] = 1
    endif

    index = where(temp_vrt_masked_bits eq 0, count)
    if(count gt 0) then begin
        temp_vrt_masked_bits[index] = 1
    endif

    index = where(temp_num_hrz_ovrld eq 0, count)
    if(count gt 0) then begin
        temp_num_hrz_ovrld[index] = 1
    endif

    index = where(temp_num_vrt_ovrld eq 0, count)
    if(count gt 0) then begin
        temp_num_vrt_ovrld[index] = 1
    endif

    stored_pmulti = !P.MULTI
    !P.MULTI = [0, 1, 2] ; stack two plots vertically in window

    nmax = max(hrz_masked_bits) > max(num_hrz_ovrld)

    plot, jultimes, temp_hrz_masked_bits, yrange=[0, nmax], ystyle=1, title='HK Overloads and Masks H (green) and Deadtime(ms, rightside axis) ', ytitle = 'Number of Masked Bits', $
        pos=[0.05,0.45,0.975,0.95],xmargin=[3,0], charsize=1.3,$
        xtickunits=['Time'], xtickformat='LABEL_DATE', xstyle=1, xticks=nxticks
    oplot,jultimes,temp_num_hrz_ovrld,   color = 110

    axis, yrange=[0, max(dead_time)], /yst, yaxis = 1, /save, ytitle='Dead Time (ms)' ; set up the right axis for dead time
    oplot, jultimes, dead_time, color = 70, linestyle=1

    ;axis, yrange=[0, nmax], /yst, yaxis = 0, /save ; reset to the left axis
    ;oplot, temp_hrz_masked_bits, color = 220, linestyle=1
    ;oplot, temp_num_hrz_ovrld,   color = 170


    nmax = max(vrt_masked_bits) > max(num_vrt_ovrld)
    plot, jultimes, temp_vrt_masked_bits, yrange=[0, nmax], ystyle=1, title='HK Overloads and Masks V (red) and DeadTime (ms, rightside axis)', ytitle = 'Number of Masked Bits', $
        pos=[0.05,0.12,0.975,0.35],xmargin=[3,0], charsize=1.3,$
        xtickunits=['Time'], xtickformat='LABEL_DATE', xstyle=1, xticks=nxticks
    oplot,jultimes,temp_num_vrt_ovrld,   color = 254

    axis, yrange=[0, max(dead_time)], /yst, yaxis = 1, /save, ytitle='Dead Time (ms)' ; set up the right axis for dead time
    oplot, jultimes, dead_time, color = 70, linestyle=1


    !P.MULTI = stored_pmulti




end; plot mask and overLoad   ********************************
;******************************************************



pro plot_HK_data_CPI_trigger_mode,ptr

    common basescan_common, show_images, particle, timestamp, prev_sec, timestamp_printed, basefile_lun, file_size, $
        raw_data_buffer, raw_data_buffer_size, buffer_size, buffer_ptr, raw_data, hk_array, mask_array, mask_timestamp_array, $
        timestamp_index, timestamp_array_size, timestamp_array, hk_timestamp_array, hk_probecount_array, $
        probecount_index, probecount_array_size, probecount_array, iteration_counter, $
        HK1_timestamp, HK2_timestamp, HK_timediff_nanos, block_timestamp, buffer
    forward_function SPEC_seconds2HMS




    ; convert the raw counts to physical units

    temp_fst   = counts2temp(hk_array[*,  1]) ; Forward Sample Tube Temp
    temp_uob   = counts2temp(hk_array[*,  2]) ; Upper Optics Block temp
    temp_lob   = counts2temp(hk_array[*,  3]) ; Lower Optics Block temp
    temp_cst   = counts2temp(hk_array[*,  4]) ; Central Sample Tube temp
    temp_ast   = counts2temp(hk_array[*,  5]) ; Aft Sample Tube temp
    temp_pyln1 = counts2temp(hk_array[*,  6]) ; Pylon 1 temp
    temp_pyln2 = counts2temp(hk_array[*,  7]) ; Pylon 2 temp
    temp_pyln3 = counts2temp(hk_array[*,  8]) ; Pylon 3 temp
    temp_ccd   = counts2temp(hk_array[*,  9]) ; CCD Camera temp
    temp_il    = counts2temp(hk_array[*, 10]) ; Imaging Lens Temp
    temp_ilz   = counts2temp(hk_array[*, 11]) ; Imaging Laser Temp

    ; apparently the lasers use a different temperature sensor -- not sure where this info came from, but doesn't seem accurate all the time...
    ;temp_vlz = double(hk_array[*, 12]) * 0.010173 - 273.15 ; Vertical Laser Temp
    ;temp_hlz = double(hk_array[*, 13]) * 0.010173 - 273.15 ; Horizontal Laser temp

    temp_vlz   = counts2temp(hk_array[*, 12])
    temp_hlz   = counts2temp(hk_array[*, 13])

    temp_pbd   = counts2temp(hk_array[*, 14]) ; Power Board temp
    temp_vpl   = counts2temp(hk_array[*, 15])
    temp_vop   = counts2temp(hk_array[*, 16])
    temp_hpl   = counts2temp(hk_array[*, 17])
    temp_hop   = counts2temp(hk_array[*, 18])
    temp_vmr   = counts2temp(hk_array[*, 19])
    temp_hmr   = counts2temp(hk_array[*, 20])
    temp_ipl   = counts2temp(hk_array[*, 21])
    temp_dsp   = counts2temp(hk_array[*, 22])
    temp_vayt  = counts2temp(hk_array[*, 23])
    temp_vayb  = counts2temp(hk_array[*, 24])
    temp_hayt  = counts2temp(hk_array[*, 25])
    temp_hayb  = counts2temp(hk_array[*, 26])

    rh         = double(hk_array[*, 27]) * 0.002515 - 0.2802198  ; Relative Humidity
    can_pres   = double(hk_array[*, 28]) * 0.000572205 -3.75     ; Internal pressure

    vrt_tec_i  = double(hk_array[*, 29]) * 0.000050498
    hrz_tec_i  = double(hk_array[*, 30]) * 0.000050498
    vrt_lsr    = double(hk_array[*, 31]) * 0.000076294
    hrz_lsr    = double(hk_array[*, 32]) * 0.000076294
    raw_plusv  = double(hk_array[*, 33]) * 0.000152588
    raw_minusv = 2. * double(hk_array[*, 33]) * 0.000152588 - double(hk_array[*, 34]) * 0.00022889

    ; TAF 2014-05-19 the Horizontal and Vertical element voltages were reversed in the documentation
    ; vs the actual HK from the probe, so they are switched accordingly below
    v_he0   = double(hk_array[*, 35]) * 0.00244140625
    v_he21  = double(hk_array[*, 36]) * 0.00244140625 ; Horizontal element  21 voltage
    v_he42  = double(hk_array[*, 37]) * 0.00244140625 ; Horizontal element  42 voltage
    v_he64  = double(hk_array[*, 38]) * 0.00244140625
    v_he85  = double(hk_array[*, 39]) * 0.00244140625 ; Horizontal element  85 voltage
    v_he106 = double(hk_array[*, 40]) * 0.00244140625 ; Horizontal element 106 voltage
    v_he127 = double(hk_array[*, 41]) * 0.00244140625

    img_lsr_i = double(hk_array[*, 42]) * 0.0268555

    v_ve0   = double(hk_array[*, 43]) * 0.00244140625
    v_ve21  = double(hk_array[*, 44]) * 0.00244140625 ; Vertical element  21 voltage
    v_ve42  = double(hk_array[*, 45]) * 0.00244140625 ; Vertical element  42 voltage
    v_ve64  = double(hk_array[*, 46]) * 0.00244140625
    v_ve85  = double(hk_array[*, 47]) * 0.00244140625 ; Vertical element  85 voltage
    v_ve106 = double(hk_array[*, 48]) * 0.00244140625 ; Vertical element 106 voltage
    v_ve127 = double(hk_array[*, 49]) * 0.00244140625

    img_lsr_pw      = double(hk_array[*, 50]) * 0.0268555
    img_lsr_i_set   = double(hk_array[*, 51]) * 0.014648
    img_lsr_pw_set  = double(hk_array[*, 52]) * 0.014648
    probe_mode_raw  =        hk_array[*, 53]  ; bits 6-7 CPI Trigger Source
    heater_status   = double(hk_array[*, 54]) * 1.
    optical_pwm_pct = double(hk_array[*, 55]) * 0.0268555
    num_hrz_dect    =        hk_array[*, 56]  ; Horizontal particles detected
    num_vrt_dect    =        hk_array[*, 57]  ; Vertical particles detected
    dead_time       = double(hk_array[*, 58]) ; dead time in second
    cpi_trig_set1   =        hk_array[*, 59]  ; CPI triggering settings
    cpi_trig_set2   =        hk_array[*, 60]  ; CPI triggering settings bits 11-16 min pixels before fire
    vrt_laser_drv   = double(hk_array[*, 61]) ; Vertical laser drive
    hrz_laser_drv   = double(hk_array[*, 62]) ; Horizontal laser drive
    vrt_masked_bits =        hk_array[*, 63]  ; # of vertical bits masked
    hrz_masked_bits =        hk_array[*, 64]  ; # of horizontal bits masked
    num_hrz_ovrld   =        hk_array[*, 65]  ; # of overload TWs found
    num_vrt_ovrld   =        hk_array[*, 66]  ; # of overload TWs found
    num_stereo_dect =        hk_array[*, 67]  ; # of overload TWs found
    comp_config     =        hk_array[*, 68]  ; Compression configuration

    pds_align_w1 = hk_array[*, 69] ; # of 4kb buffers sent last second
    pds_align_w2 = hk_array[*, 70] ; # of 4kb buffers sent last second

    TW_msw = hk_array[*, 71] ;  the three timing words
    TW_isw = hk_array[*, 72]
    TW_lsw = hk_array[*, 73]

    tas = two_uints2float(hk_array[*, 74], hk_array[*, 75])

    twods_cmd_acpt = hk_array[*, 76] ; % of 2ds commands accepted
    cpi_cmd_acpt = hk_array[*, 77] ; % of detected overlaps (from
    n2ds_data_blks = hk_array[*, 78] ; Number of 2D-S data blocks (4098 byte) sent over the last second
    array_skew_off = hk_array[*, 79] ; Array Skew Offset (six bits: 0-5)

    cpi_frame_rate = hk_array[*, 80] and '0FFF'X ; CPI Frame Rate

    hk_count=n_elements(hk_timestamp_array)

    date_labels = label_date(date_format='%H:%I:%S')

    jultimes = timegen(start=julday(hk_timestamp_array[0].month,          hk_timestamp_array[0].day,             hk_timestamp_array[0].year,             $
        hk_timestamp_array[0].hour,           hk_timestamp_array[0].minute,          hk_timestamp_array[0].second),          $
        final=julday(hk_timestamp_array[hk_count-1].month, hk_timestamp_array[hk_count-1].day,    hk_timestamp_array[hk_count-1].year,    $
        hk_timestamp_array[hk_count-1].hour,  hk_timestamp_array[hk_count-1].minute, hk_timestamp_array[hk_count-1].second), $
        units='Seconds')




    ; combine the 3 components of the 48 bit timing word

    hk_probecount_array = ulong64(ishft(ulong64(TW_msw), 32) or ishft(ulong64(TW_isw), 16) or ulong64(TW_lsw))

    info=*ptr
    HKdata=*(info.pHKdata)
    Time2DS=(HKdata.nanoseconds/1000000000LL); change to seconds now.

    StartTime0=Time2DS(0)
    EndTime0  =max(Time2DS)

    my_xrange=[StartTime0, EndTime0]

    ;plot out the data and save the png files.
    fullrange =long64(EndTime0-StartTime0); delta_s; ave_info.endNanoseconds - ave_info.startNanoseconds
    binwidth = fullrange/8LL
    curbin = long64(StartTime0);  ave_info.startNanoseconds

    tnames = StrArr(9)
    FOR i=0,8 DO BEGIN
        tnames(i) =  strtrim(strmid(SPEC_seconds2HMS(curbin),0,8),2)
        curbin= curbin+binwidth
    ENDFOR
    ;-------------------------------
    ;Set up X axis tick labels.
    ;-------------------------------
    !X.TICKNAME = tnames
    !X.TICKS = 8;      !charsize=1.5
    ;********************************************************
    device, decomposed = 0
    loadct, 39, /silent

    bluecolor =120;   GetColor("Navy", !D.Table_Size-2)
    redcolor = 254; GetColor("Red", !D.Table_Size-2)
    blackcolor=10;   getColor("Black",!D.Table_Size-2)
    whitecolor =0; GetColor("White", !D.Table_Size-2)
    greencolor =170; GetColor("Green", !D.Table_Size-2)
    Purplecolor  =180;GetColor("Purple",!D.Table_Size-2)
    Pinkcolor  =230;  GetColor("Pink",!D.Table_Size-2)

    Widget_Control, info.drawidHK, Get_Value=wid  ; get the window ID for drawing.
    WSet, wid
    ;erase,color='ffffff'x  ; get background to white

    !X.OMARGIN = [1.5,0.0]  ;We need this because the Area plot X axis label extends off the left edge of the window

    basefile=file_basename(info.currentFname);  basefile

    device, decomposed = 0
    loadct, 39, /silent

    color = 5
    color_step = 8


    stored_pmulti = !P.MULTI
    !P.MULTI = [0, 1, 2] ; stack two plots vertically in window

    ; CPI triggering settings bits 11-16 min pixels before fire  bits 6-7 CPI Trigger Source

    ; extract probe_mode, min pixels before fire, and

    probe_mode = ishft(probe_mode_raw, -6) and '0003'X

    cpi_min_pix = ishft(cpi_trig_set2, -11) and '03FF'X

    plot, jultimes, cpi_min_pix, title='CPI Modes, Standard vs Random (Green) and Min Pixels Before Fire (Red)', $
        pos=[0.05,0.65,0.975,0.95],xmargin=[3,0], charsize=1.3,$;   xmargin=[10,10], $
        xtickunits=['Time'], xtickformat='LABEL_DATE', xstyle=1, xticks=nxticks, $
        ytitle='Min Pixels Before Fire', yrange=[1, max(cpi_min_pix)+1], /yst

    oplot, jultimes, cpi_min_pix, color=254

    ; set up the right axis, and draw the axis labels in green to help identify the probe_mode line
    axis, yrange=[-1, 3], /yst, yaxis = 1, /save, yticks=4, ytickname=[' ', ' ', ' ', ' ', ' ']
    axis, yrange=[-1, 3], /yst, yaxis = 1, /save, yticks=4, ytickname=[' ', ' ', ' ', ' ', ' '], yticklayout=1, color=170
    oplot, jultimes, probe_mode, color=170
    xyouts, 0.92,0.865,'Random',color=170,/normal
    xyouts, 0.92,0.729,'Standard',color=170,/normal
    ; cpi_frame_rate = hk_array[*, 80] and '0FFF'X ; CPI Frame Rate

    plot, jultimes, cpi_frame_rate, title='CPI Frame Rate', $
        pos=[0.05,0.12,0.975,0.55],xmargin=[3,0], charsize=1.3,$;             xmargin=[10,10],
        ytitle='Frames/s', xtickunits=['Time'], xtickformat='LABEL_DATE', xstyle=1, xticks=nxticks

    oplot, jultimes, cpi_frame_rate, color=130


    !P.MULTI = stored_pmulti



end;  plot 5  Triger mode operation ********************************
;******************************************************

;@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
pro Spec_GraphClick_hkdata, event
    forward_function SPEC_seconds2HMS
    Widget_Control, event.top, Get_UValue=ptr, /No_Copy  ; get our info structure

    if ptr_valid(ptr) eq 0 then begin
        Widget_Control, event.top, Set_UValue=ptr, /No_Copy
        return
    endif

    if !zoom_state eq 1 then begin
        cursor,x,y,/data,/down
        if n_elements(x) eq 1 then begin
            (*ptr).EndTime=round(x)*1LL
            hhmmss=spec_seconds2HMS((*ptr).EndTime)
            hh=strmid(hhmmss,0,2)*1
            mm=strmid(hhmmss,3,2)*1
            ss=strmid(hhmmss,6,2)*1
            widget_control,(*ptr).EndHoursID,set_value=hh
            widget_control,(*ptr).EndminutesID,set_value=mm
            widget_control,(*ptr).EndsecondsID,set_value=ss
            marknumber=2
            plot_HK_data_to_the_mainbrowse_window, ptr,marknumber
            !zoom_state=0
        endif

    endif;!Zoom_State eq 1

    if !Zoom_State eq 2 then begin
        cursor,x,y,/data,/down
        if n_elements(x) eq 1 then begin
            (*ptr).StartTime=round(x)*1LL
            hhmmss=spec_seconds2HMS((*ptr).StartTime)
            hh=strmid(hhmmss,0,2)*1
            mm=strmid(hhmmss,3,2)*1
            ss=strmid(hhmmss,6,2)*1
            widget_control,(*ptr).StartHoursID,set_value=hh
            widget_control,(*ptr).StartminutesID,set_value=mm
            widget_control,(*ptr).StartsecondsID,set_value=ss
            marknumber=2
            plot_HK_data_to_the_mainbrowse_window, ptr,marknumber
            !zoom_state=1

        endif
    endif; !Zoom_State eq 2

    Widget_Control, event.top, Set_UValue=ptr, /No_Copy

end ;== PSD_graphicclick====================================================================

;***********************************************************
pro a3vcpiview_showHKdata_of_the_basefile_event,event
    Common  DataBlock,alldataH,alldataV,ADataH,ADataV,FirstData,LastData,StripData,FirstDataV,LastDataV,StripDataV,OneHzH,OneHzV
    Common  HKdatablock,HKseconds,HKtas,num_hrz_dect,num_vrt_dect,pHKtimefpos,H50micron
    Common  DataBlock3,PHdata,PVdata,Hpc,Vpc

    forward_function archive_ice_pop_up

    widget_control,event.top,get_uvalue=ptr,/no_copy
    widget_control, event.id,get_uvalue=uval


    sname = TAG_NAMES(event, /STRUCTURE_NAME)

    case sname of

        'WIDGET_KILL_REQUEST':begin
        widget_control, event.top,set_uvalue=ptr,/no_copy
        widget_control, event.top, /destroy
        return
    end
    else:
endcase

case uval of


    'zoom': begin
        !ZOOM_STATE = 2  ; once activate, user must select 2 point to go into a zoom for the Start Time and End Time.
        widget_control, (*ptr).DrawIDbrowse,get_value=wid;  Mo has to set this draw_widget ID to front for it to work.
        wset,wid
    end

    'zoomout': begin
        (*ptr).EndTime=max(HKseconds)
        hhmmss=spec_seconds2HMS((*ptr).EndTime)
        hh=strmid(hhmmss,0,2)*1
        mm=strmid(hhmmss,3,2)*1
        ss=strmid(hhmmss,6,2)*1
        widget_control,(*ptr).EndHoursID,set_value=hh
        widget_control,(*ptr).EndminutesID,set_value=mm
        widget_control,(*ptr).EndsecondsID,set_value=ss

        (*ptr).StartTime=min(HKseconds)
        hhmmss=spec_seconds2HMS((*ptr).StartTime)
        hh=strmid(hhmmss,0,2)*1
        mm=strmid(hhmmss,3,2)*1
        ss=strmid(hhmmss,6,2)*1
        widget_control,(*ptr).StartHoursID,set_value=hh
        widget_control,(*ptr).StartminutesID,set_value=mm
        widget_control,(*ptr).StartsecondsID,set_value=ss
        marknumber=2
        plot_HK_data_to_the_mainbrowse_window, ptr,marknumber
        widget_control, (*ptr).DrawIDbrowse,get_value=wid;  Mo has to set this draw_widget ID to front for it to work.
        wset,wid
    end


    'DiodeV': begin
        plot_HK_data_diodeV,ptr
    end
    'Temp': begin
        plot_HK_data_Temp,ptr
    end

    'RH-TAS':begin
    plot_HK_data_RH_TAS_can_pressure,ptr
end

'mask': begin
    plot_HK_data_mask_overload,ptr
end

'CpiMode': begin
    plot_HK_data_CPI_trigger_mode,ptr
end

'SavePng':begin
widget_control, (*ptr).DrawIDbrowse,get_value=wid;  Mo has to set this draw_widget ID to front for it to work.
wset,wid
imagetop=tvrd(true=1)

plot_HK_data_diodeV,ptr
Widget_Control, (*ptr).drawidHK, Get_Value=wid  ; get the window ID for drawing.
WSet, wid
bbb=tvrd(true=1)

window,6,xs=1400,ys=700
TVscl, bbb,0,0,true=1
TVscl, imagetop,0,550,true=1

basefile=file_basename((*ptr).currentFname)
substrs=strsplit(basefile,'.',/extract)
stripped_base_name=substrs(0)

outfilename=file_dirname((*ptr).currentFname)+'\'+stripped_base_name+'_Diode_laserV.jpg'
write_jpeg,outfilename, tvrd(true=1),true=1,quality=100


plot_HK_data_temp,ptr
Widget_Control, (*ptr).drawidHK, Get_Value=wid  ; get the window ID for drawing.
WSet, wid
bbb=tvrd(true=1)

window,6,xs=1400,ys=700
TVscl, bbb,0,0,true=1
TVscl, imagetop,0,550,true=1

outfilename=file_dirname((*ptr).currentFname)+'\'+stripped_base_name+'_temperature.jpg'
write_jpeg,outfilename, tvrd(true=1),true=1,quality=100

plot_HK_data_RH_TAS_can_pressure,ptr
Widget_Control, (*ptr).drawidHK, Get_Value=wid  ; get the window ID for drawing.
WSet, wid
bbb=tvrd(true=1)

window,6,xs=1400,ys=700
TVscl, bbb,0,0,true=1
TVscl, imagetop,0,550,true=1
outfilename=file_dirname((*ptr).currentFname)+'\'+stripped_base_name+'_RH_TAS_Press.jpg'
write_jpeg,outfilename, tvrd(true=1),true=1,quality=100

plot_HK_data_mask_overload,ptr
Widget_Control, (*ptr).drawidHK, Get_Value=wid  ; get the window ID for drawing.
WSet, wid
bbb=tvrd(true=1)

window,6,xs=1400,ys=700
TVscl, bbb,0,0,true=1
TVscl, imagetop,0,550,true=1
outfilename=file_dirname((*ptr).currentFname)+'\'+stripped_base_name+'_Overload_mask.jpg'
write_jpeg,outfilename, tvrd(true=1),true=1,quality=100

plot_HK_data_CPI_trigger_mode,ptr
Widget_Control, (*ptr).drawidHK, Get_Value=wid  ; get the window ID for drawing.
WSet, wid
bbb=tvrd(true=1)

window,6,xs=1400,ys=700
TVscl, bbb,0,0,true=1
TVscl, imagetop,0,550,true=1
outfilename=file_dirname((*ptr).currentFname)+'\'+stripped_base_name+'_CPI_trigger_mode.jpg'
write_jpeg,outfilename, tvrd(true=1),true=1,quality=100
wait,1
wdelete,6

end



else:
endcase
widget_control,event.top,set_uvalue=ptr,/no_copy;  put the info back to xmanager; must use no_copy so that the call9ng subroutine will work right away.
end;  ;end of event handler
;****************************************************

pro a3vcpiview_showHKdata_of_the_basefile,pHKdata,currentFname

    ; this is to browse 2DS images at any tim epoints to get a feel of the clouse passes.
    ; when cursor is click at a point, images at that point are show, click next will show images
    ; after that point.
    title_str='Display HKdata at different time periods.'

    ;create top level base
    tlb=widget_base(title=title_str,/column,xsize=800,ysize=11*72+40,xoffset=200,/TLB_KILL_REQUEST_EVENTS); 780 changed to 800.
    ;the above tiltle_str can be changed later in the plots.
    L1base=widget_base(tlb,/row);   ;xoffset to place in offset the computer screen.
    ;timetext=widget_text(L1base,xsize=25,ysize=1,value=time_periods); time_period='aaa'
    helpnote=widget_button(L1base,value='Help',uvalue='Help')

    sortbase=widget_base(L1base,/row,/frame)

    label1=widget_label(sortbase,value='T1: Day',FONT = 'ARIEL*BOLD*14')
    SStrDay             = cw_field(sortbase,title='',value=0,/int,xsize=2,uvalue ='startDay',/all)
    StartHoursID        = cw_field(sortbase,title=':',value=0,/int,xsize=2,uvalue ='starthh',/all)
    StartMinutesID      = cw_field(sortbase,title=':',value=0,/int,xsize=2,uvalue ='startmm',/all)
    StartSecondsID      = cw_field(sortbase,title=':',value=0,/int,xsize=2,uvalue ='startss',/all)

    ;sbutton=widget_button(b311,value='|',uvalue='ST0')
    label2=widget_label(sortbase,value='T2: Day',FONT = 'ARIEL*BOLD*14')
    EStrDay            = cw_field(sortbase,title='',value=0,/int,xsize=2,uvalue ='EndDay',/all)
    EndHoursID         = cw_field(sortbase,title=':',value=0,/int,xsize=2,uvalue ='etarthh',/all)
    EndMinutesID       = cw_field(sortbase,title=':',value=0,/int,xsize=2,uvalue ='etartmm',/all)
    EndSecondsID       = cw_field(sortbase,title=':',value=0,/int,xsize=2,uvalue ='etartss',/all)

    BUTTON_Plot_Zoom = WIDGET_BUTTON(sortbase, $
        FONT='9x15', $
        UVALUE='zoom', $
        XSIZE=55, $
        YSIZE=29, $
        TOOLTIP="select T1 and T2 by 2 clicks on the data area.", $aaaaaaaaaaa
    VALUE='ZoomIn')

    BUTTON_Plot_Zoomout = WIDGET_BUTTON(sortbase, $
        FONT='9x15', $
        UVALUE='zoomout', $
        XSIZE=60, $
        YSIZE=29, $
        TOOLTIP="click to reset to Start/EndTime of the base file.", $aaaaaaaaaaa
    VALUE='ZoomOut')

    label=   widget_label(L1base,value='display Options:')

    optbase1 = widget_base(L1base,/row,/frame)

    diodeID=widget_button(optbase1,value='DiodeV',Uvalue='DiodeV')
    tempID =widget_button(optbase1,value='Temp',Uvalue='Temp')
    maskID=widget_button(optbase1,value='RH-TAS',Uvalue='RH-TAS')
    maskID=widget_button(optbase1,value='mask',Uvalue='mask')
    overloadID =widget_button(optbase1,value='CpiMode',Uvalue='CpiMode')

    savepng=   widget_button(L1base,value='SavePng',Uvalue='SavePng',TOOLTIP="Save all HK plots.")

    ;***********************************************************************************************************

    starting_window_x = 1400
    starting_window_y = 730

    drawIDpart = Widget_Draw(tlb,XSize=starting_window_x, YSize=150, Event_Pro='SPEC_GraphClick_hkdata', /BUTTON_EVENTS, /MOTION_EVENTS)

    hkdata_base=widget_base(tlb,col=1)

    drawIDHK = Widget_Draw(hkdata_base,XSize=starting_window_x, YSize=550)


    Widget_Control, tlb,XSize=starting_window_x, YSize=starting_window_y,/Realize

    Widget_Control, drawIDPart, Get_Value=wid
    WSet,  wid
    WShow, wid

    ; Create the info structure with the information
    ; required to run the program.

    info = {pHKData:pHKdata, $          ; A pointer to the image data.
        Drawidbrowse:Drawidpart,$
        DrawidHK:DrawidHK,$
        fpos:0LL,$
        StartTime:0L,$
        EndTime:0L,$
        SStrDay: SStrDay,$
        StartHoursID: StartHoursID,$
        StartMinutesID :StartMinutesID,$
        StartSecondsID :StartSecondsID,$
        EStrDay: EStrDay,$
        EndHoursID:   EndHoursID,$
        EndMinutesID :EndMinutesID,$
        EndSecondsID :EndSecondsID,$
        CurrentFname :CurrentFname,$
        DateStamp:'****'}
    ;************************************************************************************************************
    ;use a pointer to pass teh data
    ptr=ptr_new(info,/no_copy); no extra copy is made.
    widget_control,/hourglass
    marknumber=1
    plot_HK_data_to_the_mainbrowse_window, ptr,marknumber
    plot_HK_data_DiodeV,ptr
    widget_control,tlb,set_uvalue=ptr,/no_copy;    info;  this replace above ***=info

    xmanager,'a3vcpiview_showHKdata_of_the_basefile',tlb,/just_reg,/no_block, event_handler='a3vcpiview_showHKdata_of_the_basefile_event'
    return
    ;*****************************************************
end;   ;end of main program
;***********************************************************
