; File:  calc_jul_date.pro
; Author:  Ted Fisher (Ted.Fisher@colorado.edu) 19 June 2001
; Purpose:  returns a long int containing the Julian date in the
;   format YYYYDDD, for example 1998079 was the spring equinox in 1998

;   Takes year, month, day format or year, day of year format



function calc_jul_date, year, month=month, day

    if(keyword_set(month)) then begin
        if(day gt 31) then begin
            message, 'invalid day value: ' + string(day)
        endif

        case month of
            '1' : ndays = day
            '2' : ndays = 31 + day
            '3' : ndays = 31 + 28 + day
            '4' : ndays = 31 + 28 + 31 + day
            '5' : ndays = 31 + 28 + 31 + 30 + day
            '6' : ndays = 31 + 28 + 31 + 30 + 31 + day
            '7' : ndays = 31 + 28 + 31 + 30 + 31 + 30 + day
            '8' : ndays = 31 + 28 + 31 + 30 + 31 + 30 + 31 + day
            '9' : ndays = 31 + 28 + 31 + 30 + 31 + 30 + 31 + 31 + day
            '10' : ndays = 31 + 28 + 31 + 30 + 31 + 30 + 31 + 31 + 30 + day
            '11' : ndays = 31 + 28 + 31 + 30 + 31 + 30 + 31 + 31 + 30 + 31 + day
            '12' : ndays = 31 + 28 + 31 + 30 + 31 + 30 + 31 + 31 + 30 + 31 + 30 + day
            else : message, 'Invalid month value: ' + string(month)
        endcase

        if((month gt 2) and ((year mod 4) eq 0)) then begin ; set up for leap years
            ndays = ndays + 1
        endif
    endif else begin
        if((year mod 4) eq 0) then begin
            max = 366
        endif else begin
            max = 365
        endelse

        if(day gt max) then begin
            message, 'Invalid day value: ' + string(day)
        endif    

        ndays = day
    endelse


    return, long(year) * 1000L + long(ndays)
end

