; File: calc_timediff_nanos.pro
; Author:  Ted Fisher ~2014? <ted@specinc.com> <fishnuke@gmail.com>
; Pupose: calculates the time difference in nanoseconds between two hires_timestamp structures

; in_time1 and in_time2 are hires_timestamp structs
; returns a ulong64 which is the difference, in_time2 - in_time1, in nanoseconds

; first converts both dates to Julian for easy comparison that is year/month/day robust, but will not work correctly if the
; input years are more than 584 years appart, so don't try comparing data from 2014 with data from 1430 or from 2598, unless you use a different function


; Note long64 is sufficient to hold these values, since 584 years of nanoseconds can be held comfortably in a 64 bit number

; print, long64(365)*long64(24)*long64(3600)*long64(1e9)
; 31536000000000000


; printh,long64(365)*long64(24)*long64(3600)*long64(1e9)
; 0x007009D32DA30000

; print, long64('7FFFFFFFFFFFFFFF'X) / (long64(365)*long64(24)*long64(3600)*long64(1e9))
; 292


function calc_timediff_nanos, in_time1, in_time2
    juldate1 = calc_jul_date(in_time1.year, month=in_time1.month, in_time1.day)
    juldate2 = calc_jul_date(in_time2.year, month=in_time2.month, in_time2.day)

    day_diff = juldate2 - juldate1

    julnanos2 = long64(day_diff) * 24LL * 3600LL * 1000000000LL + $
        long64(in_time2.hour)   * 3600LL * 1000000000LL + $
        long64(in_time2.minute) *   60LL * 1000000000LL + $
        long64(in_time2.second)          * 1000000000LL + $
        long64(in_time2.ms)              *    1000000LL + $
        long64(in_time2.us)              *       1000LL + $
        long64(in_time2.ns)

    julnanos1 = long64(in_time1.hour)    * 3600LL * 1000000000LL + $
        long64(in_time1.minute)  *   60LL * 1000000000LL + $
        long64(in_time1.second)           * 1000000000LL + $
        long64(in_time1.ms)               *    1000000LL + $
        long64(in_time1.us)               *       1000LL + $
        long64(in_time1.ns)

    ; also should check that julnanos2 gt julnanos1

    timediff_nanos = julnanos2 - julnanos1 + 1LL

    return, timediff_nanos
end ; calc_timediff_nanos
