; File:  plane3.pro
; Author:  Ted Fisher <ted@specinc.com> <fishnuke@gmail.com>
; Purpose:  test procedure to display 3 intersecting planes with images on them



Pro Texture_Surface_Cleanup, tlb
    ; Come here when program dies. Free all created objects.

    Widget_Control, tlb, Get_UValue=info
    IF N_Elements(info) NE 0 THEN Obj_Destroy, info.thisContainer
END ; Texture_Surface_Cleanup




PRO Texture_Surface_Draw_Events, event

    ; Draw widget events handled here: expose events and trackball
    ; events. The trackball uses RSI-supplied TRACKBALL oject.

    Widget_Control, event.top, Get_UValue=info, /No_Copy

    drawTypes = ['PRESS', 'RELEASE', 'MOTION', 'SCROLL', 'EXPOSE']
    thisEvent = drawTypes(event.type)

    CASE thisEvent OF

        'EXPOSE':  ; Nothing required except to draw the view.
        'PRESS':   Widget_Control, event.id, Draw_Motion_Events=1 ; Motion events ON.
        'RELEASE': Widget_Control, event.id, Draw_Motion_Events=0 ; Motion events OFF.
        'MOTION':  ; Just update trackball.
        ELSE:

    ENDCASE

    ; Does the trackball need updating? If so, update.

    needUpdate = info.thisTrackball->Update(event, Transform=thisTransform)
    IF needUpdate THEN BEGIN
        info.thisModel->GetProperty, Transform=modelTransform
        info.thisModel->SetProperty, Transform=modelTransform # thisTransform
    ENDIF

    ; Draw the view.

    info.thisWindow->Draw, info.thisView

    ;Put the info structure back.

    Widget_Control, event.top, Set_UValue=info, /No_Copy
END ; Texture_Surface_Draw_Events




PRO Texture_Surface_Style, event

    ; Event handler to select surface style.

    Widget_Control, event.top, Get_UValue=info, /No_Copy

    ; What style is wanted?

    Widget_Control, event.id, Get_UValue=newStyle
    CASE newStyle OF

        'DOTS': info.cpi_surface->SetProperty, Style=0
        'MESH': info.cpi_surface->SetProperty, Style=1
        'SOLID': info.cpi_surface->SetProperty, Style=2, Shading=1
        'XPARALLEL': info.cpi_surface->SetProperty, Style=3
        'YPARALLEL': info.cpi_surface->SetProperty, Style=4
        'HIDDEN': BEGIN
            Widget_Control, event.id, Get_Value=buttonValue
            IF buttonValue EQ 'Hidden Lines OFF' THEN BEGIN
                setting = 0
                hlvalue = 'Hidden Lines ON'
            ENDIF ELSE BEGIN
                setting = 1
                hlvalue = 'Hidden Lines OFF'
            ENDELSE
            Widget_Control, event.id, Set_Value=hlvalue
            info.cpi_surface->SetProperty, Hidden_Lines=setting
        ENDCASE

    ENDCASE

    ; Redraw the graphic.

    info.thisWindow->Draw, info.thisView

    ;Put the info structure back.

    Widget_Control, event.top, Set_UValue=info, /No_Copy
END ; Texture_Surface_Style




PRO Plane3_Select_Image, event

    ; Event handler to select images

    Widget_Control, event.top, Get_UValue=info, /No_Copy

    ; What style is wanted?

    Widget_Control, event.id, Get_UValue=newStyle
    CASE newStyle OF

        'c1': begin
            print, "setting to c1"

            info.h_image = congrid(info.h1, info.h_x, info.h_y)
            info.v_image = congrid(info.v1, info.v_x, info.v_y)

            xs=(size(info.c1))[1]
            ys=(size(info.c1))[2]

            info.c_image[*] = 0

            info.c_image[0:xs-1,0:ys-1] = info.c1



            ;info.cpi_surface->SetProperty, Style=0
        endcase
        'c2': begin
            print, "setting to c2"

            info.h_image = congrid(info.h2, info.h_x, info.h_y)
            info.v_image = congrid(info.v2, info.v_x, info.v_y)

            xs=(size(info.c2))[1]
            ys=(size(info.c2))[2]

            info.c_image[*] = 0

            info.c_image[0:xs-1,0:ys-1] = info.c2

            ;info.cpi_surface->SetProperty, Style=0
        endcase

        'c3': begin
            print, "setting to c3"

            info.h_image = congrid(info.h3, info.h_x, info.h_y)
            info.v_image = congrid(info.v3, info.v_x, info.v_y)

            xs=(size(info.c3))[1]
            ys=(size(info.c3))[2]

            info.c_image[*] = 0

            info.c_image[0:xs-1,0:ys-1] = info.c3

            stop

            ;info.cpi_surface->SetProperty, Style=0
        endcase

        ;   'c2': begin
        ;            print, "setting to c2"

        ;            h_image = congrid(h2, h_x, h_y)
        ;            v_image = congrid(v2, v_x, v_y)

        ;            c_image = c2


        ;            ;begininfo.cpi_surface->SetProperty, Style=1
        ;         endcase


        ;'HIDDEN': BEGIN
        ;    Widget_Control, event.id, Get_Value=buttonValue
        ;    IF buttonValue EQ 'Hidden Lines OFF' THEN BEGIN
        ;       setting = 0
        ;       hlvalue = 'Hidden Lines ON'
        ;    ENDIF ELSE BEGIN
        ;       setting = 1
        ;       hlvalue = 'Hidden Lines OFF'
        ;    ENDELSE
        ;    Widget_Control, event.id, Set_Value=hlvalue
        ;    info.cpi_surface->SetProperty, Hidden_Lines=setting
        ;    ENDCASE

    ENDCASE


    h_image = info.h_image
    v_image = info.v_image

    ;c_image = c1
    ;xs=(size(c5))[1]
    ;ys=(size(c5))[2]
    ;c_image[*] = 0
    ;c_image[0:xs-1,0:ys-1] = c5


    c_image = info.c_image


    pixels = 128 ; # pixels for the 2DS images

    h_slices = (size(h_image))[1]
    v_slices = (size(v_image))[1]

    ;c_x = (size(c_image))[1]
    ;c_y = (size(c_image))[2]

    c_x = (size(info.c1))[1]
    c_y = (size(info.c1))[2]


    ;stop

    ; rescale 2DS images to fit CPI
    scale = 5

    h_x = h_slices * scale
    h_y = pixels   * scale
    v_x = v_slices * scale
    v_y = pixels   * scale

    h_image = congrid(h_image, h_x, h_y)
    v_image = congrid(v_image, v_x, v_y)


    ; offset in pixels to center images -- put in a slider in GUI
    h_off = v_y/2
    v_off = h_y/2
    c_off = v_y/2



    oPalette = OBJ_NEW('IDLgrPalette')
    oPalette->Loadct, 1
    oPalette -> SetRGB, 255,255,255,255

    o_h = OBJ_NEW('IDLgrImage', h_image, PALETTE=oPalette, BLEND_FUNCTION=[3,4], ALPHA_CHANNEL=0.001250)
    o_v = OBJ_NEW('IDLgrImage', v_image, PALETTE=oPalette, BLEND_FUNCTION=[3,4], ALPHA_CHANNEL=0.001250)
    o_c = OBJ_NEW('IDLgrImage', c_image, PALETTE=oPalette, BLEND_FUNCTION=[3,4], ALPHA_CHANNEL=0.001250)

    ; Get dimensions of the surface data.

    h_surfaceData = bytarr(h_x, h_y)
    h_surfaceData[*] = -c_off/2
    ;v_surfaceData = bytarr(v_x, v_y)

    v_surfaceData = bytarr(2, h_y)
    v_surfaceData[0, *] = 0
    v_surfaceData[1, *] = -c_off

    ;v_surfaceData = bytarr(v_x, v_y)
    ;;v_surfaceData[0, *] = 0
    ;v_surfaceData[*] = -c_off/2

    ;stop
    ;c_surfaceData = bytarr(c_x, c_y)
    c_surfaceData = (findgen(c_x)#replicate(1.,c_y))

    ;s = Size(surfaceData, /Dimensions)

    ; Fill out X and Y vectors if necessary.
    h_xx = findgen(h_x)
    h_yy = findgen(h_y)

    ;v_xx = findgen(v_x)
    v_xx = [128,128]
    v_yy = findgen(v_y)

    c_xx = findgen(c_x)
    c_yy = findgen(c_y)


    ; Create a view. Use RGB color. Charcoal background.
    ; The coodinate system is chosen so that (0,0,0) is in the
    ; center of the window. This will make rotations easier.


    thisView = OBJ_NEW('IDLgrView', Color=[200,200,200], $
        Viewplane_Rect=[-1.2,-1.1,2.3,2.1])

    ; I want the surface to have the same aspect ratio as the data itself
    ; in the X and Y directions.

    surfaceAspect = Float(c_y) / c_x
    windowAspect = 1.0
    pos = Aspect(surfaceAspect, WindowAspect=windowAspect, Margin=0)
    pos = pos - 0.5
    ;pos = pos - .75

    ; Create a model for the surface and axes and add it to the view.
    ; This model will rotate under the direction of the trackball object.


    thisModel = OBJ_NEW('IDLgrModel')
    thisView->Add, thisModel


    IF N_Elements(xtitle) EQ 0 THEN xtitle='2DS H'
    IF N_Elements(ytitle) EQ 0 THEN ytitle='2DS V'
    IF N_Elements(ztitle) EQ 0 THEN ztitle='CPI'
    IF N_Elements(zscale) EQ 0 THEN zscale = 1.0 ELSE zscale = (0.001 > zscale < 1.0)
    hidden_lines = Keyword_Set(hidden_lines)
    landscape    = Keyword_Set(landscape)
    vector       = Keyword_Set(vector)

    ; Create helper objects. First, create title objects
    ; for the axes and plot. Color them green.

    xTitleObj = Obj_New('IDLgrText', xtitle, Color=[0,200,0])
    yTitleObj = Obj_New('IDLgrText', ytitle, Color=[0,200,0])
    zTitleObj = Obj_New('IDLgrText', ztitle, Color=[0,200,0])

    ; Create font objects.

    helvetica10pt = Obj_New('IDLgrFont', 'Helvetica', Size=10)
    helvetica14pt = Obj_New('IDLgrFont', 'Helvetica', Size=14)

    ; Create a trackball for surface rotations. Center it in
    ; the 400-by-400 window. Give it a 200 pixel diameter.

    thisTrackball = OBJ_NEW('Trackball', [200, 200], 200)

    thisImage = o_c



    ; Create a surface object. Add the image to it as a texture map.
    ; The image is positioned with the Texture_Coord keyword and the
    ; texcoords values.


    ;s = Size(surfaceData, /Dimensions)
    ;texcoords = FltArr(2, s[0], s[1])
    ;texcoords[0,*,*] = (Findgen(s[0])#Replicate(1,s[1])) / (s[0]-1)
    ;texcoords[1,*,*] = (Replicate(1,s[1])#Findgen(s[0])) / (s[0]-1)

    h_texcoords = FltArr(2, h_x, h_y)
    h_texcoords[0,*,*] = (Findgen(h_x)#Replicate(1,h_y)) / (h_x-1)
    h_texcoords[1,*,*] = (Replicate(1,h_y)#Findgen(h_x)) / (h_x-1)

    h_surface = OBJ_NEW('IDLgrSurface', SHADING = 1, h_surfaceData, h_xx, h_yy, Style=2, $
        Color=[255,255,255], _Extra=extra, Hidden_Lines=hidden_lines)
    h_surface->SetProperty, Texture_Map=o_h, Texture_Coord=h_texcoords

    v_texcoords = FltArr(2, v_x, v_y)
    v_texcoords[0,*,*] = (Findgen(v_x)#Replicate(1,v_y)) / (v_x-1)
    v_texcoords[1,*,*] = (Replicate(1,v_y)#Findgen(v_x)) / (v_x-1)

    ;v_surface = OBJ_NEW('IDLgrSurface', SHADING = 1, v_surfaceData, v_xx, v_yy, Style=2, $
    v_surface = OBJ_NEW('IDLgrSurface', SHADING = 1, v_surfaceData, v_xx, v_yy, Style=2, $
        Color=[255,255,255], _Extra=extra, Hidden_Lines=hidden_lines)
    v_surface->SetProperty, Texture_Map=o_v, Texture_Coord=v_texcoords

    c_texcoords = FltArr(2, c_x, c_y)
    c_texcoords[0,*,*] = (Findgen(c_x)#Replicate(1,c_y)) / (c_x-1)
    c_texcoords[1,*,*] = (Replicate(1,c_y)#Findgen(c_x)) / (c_x-1)

    c_surface = OBJ_NEW('IDLgrSurface', SHADING = 1, c_surfaceData, c_xx, c_yy, Style=2, $
        Color=[255,255,255], _Extra=extra, Hidden_Lines=hidden_lines)
    c_surface->SetProperty, Texture_Map=o_c, Texture_Coord=c_texcoords




    ; Get the data ranges of the surface.

    c_surface->GetProperty, XRange=xrange, YRange=yrange, ZRange=zrange

    xrange = [-5,5]
    yrange = [-5,5]
    zrange = [0,128]

    ; Create axes objects for the surface. Color them green.
    ; Axes are created after the surface so the range can be
    ; set correctly. Note how I set the font to 10 point Helvetica
    ; by creating the axis with the title object, then getting the
    ; actual axis text from the axis object itself, and switching it.



    xAxis = Obj_New("IDLgrAxis", 0, Color=[0,255,0], Ticklen=0.1, $
        Minor=4, Title=xtitleObj, Range=xrange, Exact=Keyword_Set(exact))
    xAxis->GetProperty, Ticktext=xAxisText
    xAxisText->SetProperty, Font=helvetica10pt

    yAxis = Obj_New("IDLgrAxis", 1, Color=[0,255,0], Ticklen=0.1, $
        Minor=4, Title=ytitleObj, Range=yrange, Exact=Keyword_Set(exact))
    yAxis->GetProperty, Ticktext=yAxisText
    yAxisText->SetProperty, Font=helvetica10pt

    zAxis = Obj_New("IDLgrAxis", 2, Color=[0,255,0], Ticklen=0.1, $
        Minor=4, Title=ztitleObj, Range=zrange, Exact=Keyword_Set(exact))
    zAxis->GetProperty, Ticktext=zAxisText
    zAxisText->SetProperty, Font=helvetica10pt

    ; The axes may not use exact axis scaling, so the ranges may
    ; have changed from what they were originally set to. Get
    ; and update the range variables.

    xAxis->GetProperty, CRange=xrange
    yAxis->GetProperty, CRange=yrange
    zAxis->GetProperty, CRange=zrange

    ; Set scaling parameters for the surface and axes so that everything
    ; is scaled into the range -0.5 to 0.5. We do this so that when the
    ; surface is rotated we don't have to worry about translations. In
    ; other words, the rotations occur about the point (0,0,0).

    xs = FSC_Normalize(xrange, Position=[pos[0], pos[2]])
    ys = FSC_Normalize(yrange, Position=[pos[1], pos[3]])
    xs = FSC_Normalize(xrange, Position=[-.5,.5]*zscale)
    ys = FSC_Normalize(yrange, Position=[-.5,.5]*zscale)
    zs = FSC_Normalize(zrange, Position=[-0.5,0.5]*zscale)

    ; Scale the axes and place them in the coordinate space.
    ; Note that not all values in the Location keyword are
    ; used. (I've put really large values into the positions
    ; that are not being used to demonstate this.) For
    ; example, with the X axis only the Y and Z locations are used.

    xAxis->SetProperty, Location=[9999.0, pos[1], -0.5*zscale], XCoord_Conv=xs
    yAxis->SetProperty, Location=[pos[0], 9999.0, -0.5*zscale], YCoord_Conv=ys
    zAxis->SetProperty, Location=[pos[0], pos[3],      9999.0], ZCoord_Conv=zs

    ; Scale the surface. Notice the surface is scaled *AFTER* the
    ; actual data range is obtained from the axes (CRANGE, above).
    ; Failure to do this can result in inaccurate results.

    h_surface->SetProperty, XCoord_Conv=xs, YCoord_Conv=ys, ZCoord_Conv=zs
    v_surface->SetProperty, XCoord_Conv=xs, YCoord_Conv=ys, ZCoord_Conv=zs
    c_surface->SetProperty, XCoord_Conv=xs, YCoord_Conv=ys, ZCoord_Conv=zs

    ;v_surface

    ; Add the surface and axes objects to the model.


    info.thisModel->Add, xAxis
    info.thisModel->Add, yAxis
    info.thisModel->Add, zAxis

    ;thisModel->Add, h_surface

    ;T3D, ROT = [ 90,0,0]
    ;thisModel->Rotate,[1,0,0],  90
    ;thisModel->Add, v_surface
    ;thisModel->Rotate,[1,0,0], -90
    ;T3D, ROT = [-90,0,0]

    ;T3D, ROT = [0, 45,0]
    ;thisModel->Rotate,[0,1,0],  45
    ;thisModel->Add, c_surface
    ;thisModel->Rotate,[0,1,0], -45
    ;T3D, ROT = [0,-45,0]




    ; Rotate the surface model to the standard surface view.

    info.thisModel->Rotate,[1,0,0], -90  ; To get the Z-axis vertical.
    info.thisModel->Rotate,[0,1,0],  30  ; Rotate it slightly to the right.
    info.thisModel->Rotate,[1,0,0],  30  ; Rotate it down slightly.


    info.thisModel->Add, h_surface
    info.thisModel->Add, v_surface
    info.thisModel->Add, c_surface

    ; Redraw the graphic.

    info.thisWindow->Draw, info.thisView

    ;Put the info structure back.

    Widget_Control, event.top, Set_UValue=info, /No_Copy
END ; Plane3_Select_Image



PRO Texture_Surface_Output, event

    ; This event handler creates GIF and JPEG files.

    Widget_Control, event.top, Get_UValue=info, /No_Copy

    ; Get a snapshop of window contents. (TVRD equivalent.)

    info.thisWindow->GetProperty, Image_Data=snapshot

    ; What kind of file is wanted?

    Widget_Control, event.id, GET_UValue=whichFileType
    CASE whichFileType OF

        'GIF': BEGIN

            ; Because we are using RGB color for this model, we have
            ; a 3-m-n array. Use Color_Quan to create a 2D image and
            ; appropriate color tables for the GIF file.

            image2D = Color_Quan(snapshot, 1, r, g, b)
            filename = Dialog_Pickfile(/Write, File='texture_surface.gif')
            IF filename NE '' THEN Write_GIF, filename, image2d, r, g, b
        END

        'JPEG': BEGIN

            filename = Dialog_Pickfile(/Write, File='texture_surface.jpg')
            IF filename NE '' THEN Write_JPEG, filename, snapshot, True=1
        END


        'TIFF': BEGIN

            filename = Dialog_Pickfile(/Write, File='texture_surface.tif')
            IF filename NE '' THEN BEGIN

                ; TIFF files should have their Y direction reversed for
                ; compatibility with most other software.

                Write_TIFF, filename, Reverse(snapshot,3)
            ENDIF
        END

    ENDCASE

    ;Put the info structure back.

    Widget_Control, event.top, Set_UValue=info, /No_Copy
END ; Texture_Surface_Output



PRO Texture_Surface_Exit, event

    ; Exit the program. This will cause the CLEANUP
    ; routine to be called automatically.

    Widget_Control, event.top, /Destroy
END ; Texture_Surface_Exit



PRO Texture_Surface_Printing, event

    ; PostScript printing and printer setup handled here.

    Widget_Control, event.top, Get_UValue=info, /No_Copy

    ; Configure printer and print if user OKs.

    result = Dialog_PrinterSetup(info.thisPrinter)

    IF result EQ 1 THEN BEGIN

        ; Background colors can use a lot of toner. Change background
        ; color to white and axes to black before printing.

        info.xaxis->GetProperty, Color=axisColor
        info.thisView->GetProperty, Color=backgroundColor
        info.cpi_surface->GetProperty, Color=surfaceColor

        info.xaxis->SetProperty, Color=[0,0,0]
        info.yaxis->SetProperty, Color=[0,0,0]
        info.zaxis->SetProperty, Color=[0,0,0]
        info.thisView->SetProperty, Color=[255, 255, 255]
        info.cpi_surface->SetProperty, Color=[70,70,70]

        ; I want the output on the page to have the same aspect ratio
        ; (ratio of height to width) as I see in the display window.
        ; I use the Aspect function to calculate the correct viewport
        ; position in normalized coordinates. The return value of
        ; Aspect is the position of the viewport on the output page.

        info.thisWindow->GetProperty, Dimensions=wdims
        info.thisPrinter->GetProperty, Dimensions=pdims
        plotAspect = Float(wdims[1]) / wdims[0]
        printerAspect = Float(pdims[1]) / pdims[0]
        position = Aspect(plotAspect, WindowAspect=printerAspect, Margin=0)

        ; Change the dimensions and postion of the viewport on the output page.
        ; Be sure to use normalized coordinates (units=3).

        info.thisView->SetProperty, Dimensions=[position[2]-position[0], position[3]-position[1]], $
            Location=[position[0], position[1]], Units=3

        ; Print the document.

        Widget_Control, Hourglass=1
        info.thisPrinter->Draw, info.thisView, Vector=info.vector
        info.thisPrinter->NewDocument
        Widget_Control, Hourglass=0

        ; Set colors and the view back to original values.

        info.xaxis->SetProperty, Color=axisColor
        info.yaxis->SetProperty, Color=axisColor
        info.zaxis->SetProperty, Color=axisColor
        info.thisView->SetProperty, Color=backgroundColor, Location=[0,0], Dimensions=[0,0]
        info.cpi_surface->SetProperty, Color=surfaceColor

    ENDIF

    ; Put the info structure back.

    Widget_Control, event.top, Set_UValue=info, /No_Copy
END ; Texture_Surface_Printing



PRO Texture_Surface_Resize, event

    ; The only events generated by this simple program are resize
    ; events, which are handled here.

    ; Get the info structure.

    Widget_Control, event.top, Get_UValue=info, /No_Copy

    ; Resize the draw widget. This is the proper way to do this
    ; in object graphics, but it does not always work in UNIX
    ; versions of IDL. If it doesn't work for you, comment the
    ; first line out and try the second. The second line is more
    ; portable, but not exactly the proper "object" way. :-(

    info.thisWindow->SetProperty, Dimension=[event.x, event.y]
    ;Widget_Control, info.drawID, Draw_XSize=event.x, Draw_YSize=event.y


    ; Redisplay the graphic.

    info.thisWindow->Draw, info.thisView

    ; Update the trackball objects location in the center of the
    ; window.

    info.thisTrackball->Reset, [event.x/2, event.y/2], $
        (event.y/2) < (event.x/2)

    ;Put the info structure back.

    Widget_Control, event.top, Set_UValue=info, /No_Copy
END ; Texture_Surface_Resize




PRO plane3

    ; Check for keywords.

    IF N_Elements(xtitle) EQ 0 THEN xtitle='2DS H'
    IF N_Elements(ytitle) EQ 0 THEN ytitle='2DS V'
    IF N_Elements(ztitle) EQ 0 THEN ztitle='CPI'
    IF N_Elements(zscale) EQ 0 THEN zscale = 1.0 ELSE zscale = (0.001 > zscale < 1.0)
    hidden_lines = Keyword_Set(hidden_lines)
    landscape    = Keyword_Set(landscape)
    vector       = Keyword_Set(vector)


    ; Importing image into IDL.
    ;file = FILEPATH('worldelv.dat', $
    ;   SUBDIRECTORY = ['examples', 'data'])
    ;image = READ_BINARY(file, DATA_DIMS = [360, 360])


    ; define the images and their dimensions

    ; Test image names:

    ;2DS_H_174.bmp
    ;2DS_H_190.bmp
    ;2DS_H_196.bmp
    ;2DS_H_211.bmp
    ;2DS_H_222.bmp
    ;2DS_H_227.bmp

    ;2DS_V_174.bmp
    ;2DS_V_190.bmp
    ;2DS_V_196.bmp
    ;2DS_V_211.bmp
    ;2DS_V_222.bmp
    ;2DS_V_227.bmp

    ;CPI_174_g.bmp
    ;CPI_190_g.bmp
    ;CPI_196_g.bmp
    ;CPI_211_g.bmp
    ;CPI_222_g.bmp
    ;CPI_227_g.bmp

    h=read_bmp("../3views/2DS_H_000.bmp")
    h0=reform(h[0,*,*])
    h=read_bmp("../3views/2DS_H_174.bmp")
    h1=reform(h[0,*,*])
    h=read_bmp("../3views/2DS_H_190.bmp")
    h2=reform(h[0,*,*])
    h=read_bmp("../3views/2DS_H_196.bmp")
    h3=reform(h[0,*,*])
    h=read_bmp("../3views/2DS_H_211.bmp")
    h4=reform(h[0,*,*])
    h=read_bmp("../3views/2DS_H_222.bmp")
    h5=reform(h[0,*,*])
    h=read_bmp("../3views/2DS_H_227.bmp")
    h6=reform(h[0,*,*])

    v=read_bmp("../3views/2DS_V_000.bmp")
    v0=reform(v[0,*,*])
    v=read_bmp("../3views/2DS_V_174.bmp")
    v1=reform(v[0,*,*])
    v=read_bmp("../3views/2DS_V_190.bmp")
    v2=reform(v[0,*,*])
    v=read_bmp("../3views/2DS_V_196.bmp")
    v3=reform(v[0,*,*])
    v=read_bmp("../3views/2DS_V_211.bmp")
    v4=reform(v[0,*,*])
    v=read_bmp("../3views/2DS_V_222.bmp")
    v5=transpose(reform(v[0,*,*]))
    v=read_bmp("../3views/2DS_V_227.bmp")
    v6=reform(v[0,*,*])

    c0=read_bmp("../3views/CPI_000_g.bmp")
    c1=read_bmp("../3views/CPI_174_g.bmp")
    c2=read_bmp("../3views/CPI_190_g.bmp")
    c3=read_bmp("../3views/CPI_196_g.bmp")
    c4=read_bmp("../3views/CPI_211_g.bmp")
    c5=read_bmp("../3views/CPI_222_g.bmp")
    c6=read_bmp("../3views/CPI_227_g.bmp")

    device, decomposed=0
    loadct, 1

    ;stop


    ;h_image = transpose(h5)
    ;v_image = transpose(v5)
    h_image = h0
    v_image = v0

    ;c_image = c1
    ;xs=(size(c5))[1]
    ;ys=(size(c5))[2]
    ;c_image[*] = 0
    ;c_image[0:xs-1,0:ys-1] = c5


    c_image = c0


    pixels = 128 ; # pixels for the 2DS images

    h_slices = (size(h_image))[1]
    v_slices = (size(v_image))[1]

    ;c_x = (size(c_image))[1]
    ;c_y = (size(c_image))[2]

    c_x = (size(c1))[1]
    c_y = (size(c1))[2]


    ;stop

    ; rescale 2DS images to fit CPI
    scale = 5

    h_x = h_slices * scale
    h_y = pixels   * scale
    v_x = v_slices * scale
    v_y = pixels   * scale

    h_image = congrid(h_image, h_x, h_y)
    v_image = congrid(v_image, v_x, v_y)


    ; offset in pixels to center images -- put in a slider in GUI
    h_off = v_y/2
    v_off = h_y/2
    c_off = c_y/2



    oPalette = OBJ_NEW('IDLgrPalette')
    oPalette->Loadct, 1
    oPalette -> SetRGB, 255,255,255,255

    o_h = OBJ_NEW('IDLgrImage', h_image, PALETTE=oPalette, BLEND_FUNCTION=[3,4], ALPHA_CHANNEL=0.001250)
    o_v = OBJ_NEW('IDLgrImage', v_image, PALETTE=oPalette, BLEND_FUNCTION=[3,4], ALPHA_CHANNEL=0.001250)
    o_c = OBJ_NEW('IDLgrImage', c_image, PALETTE=oPalette, BLEND_FUNCTION=[3,4], ALPHA_CHANNEL=0.001250)

    ; Get dimensions of the surface data.

    h_surfaceData = bytarr(h_x, h_y)
    h_surfaceData[*] = -h_off/2
    ;v_surfaceData = bytarr(v_x, v_y)



    v_surfaceData = bytarr(2, h_y)
    ;v_surfaceData = bytarr(v_x, v_y)
    v_surfaceData[0, *] = 0
    v_surfaceData[1, *] = -v_off

    ;v_surfaceData = bytarr(v_x, v_y)
    ;;v_surfaceData[0, *] = 0
    ;v_surfaceData[*] = -c_off/2

    ;stop
    ;c_surfaceData = bytarr(c_x, c_y)
    c_surfaceData = (findgen(c_x)#replicate(1.,c_y))

    ;s = Size(surfaceData, /Dimensions)

    ; Fill out X and Y vectors if necessary.
    h_xx = findgen(h_x)
    h_yy = findgen(h_y)

    ;v_xx = findgen(v_x)
    v_xx = [128,128]
    v_yy = findgen(v_y)

    c_xx = findgen(c_x)
    c_yy = findgen(c_y)


    ; Create a view. Use RGB color. Charcoal background.
    ; The coodinate system is chosen so that (0,0,0) is in the
    ; center of the window. This will make rotations easier.


    thisView = OBJ_NEW('IDLgrView', Color=[200,200,200], Viewplane_Rect=[-1.2,-1.1,2.3,2.1])

    ; I want the surface to have the same aspect ratio as the data itself
    ; in the X and Y directions.

    surfaceAspect = Float(c_y) / c_x
    windowAspect = 1.0
    pos = Aspect(surfaceAspect, WindowAspect=windowAspect, Margin=0)
    pos = pos - 0.5
    ;pos = pos - .75

    ; Create a model for the surface and axes and add it to the view.
    ; This model will rotate under the direction of the trackball object.


    thisModel = OBJ_NEW('IDLgrModel')
    thisView->Add, thisModel


    ; Create helper objects. First, create title objects
    ; for the axes and plot. Color them green.

    xTitleObj = Obj_New('IDLgrText', xtitle, Color=[0,170,0])
    yTitleObj = Obj_New('IDLgrText', ytitle, Color=[0,170,0])
    zTitleObj = Obj_New('IDLgrText', ztitle, Color=[0,170,0])

    ; Create font objects.

    helvetica10pt = Obj_New('IDLgrFont', 'Helvetica', Size=10)
    helvetica14pt = Obj_New('IDLgrFont', 'Helvetica', Size=14)

    ; Create a trackball for surface rotations. Center it in
    ; the 400-by-400 window. Give it a 200 pixel diameter.

    thisTrackball = OBJ_NEW('Trackball', [200, 200], 200)

    ;thisImage = o_c



    ; Create a surface object. Add the image to it as a texture map.
    ; The image is positioned with the Texture_Coord keyword and the
    ; texcoords values.


    ;s = Size(surfaceData, /Dimensions)
    ;texcoords = FltArr(2, s[0], s[1])
    ;texcoords[0,*,*] = (Findgen(s[0])#Replicate(1,s[1])) / (s[0]-1)
    ;texcoords[1,*,*] = (Replicate(1,s[1])#Findgen(s[0])) / (s[0]-1)

    h_texcoords = FltArr(2, h_x, h_y)
    h_texcoords[0,*,*] = (Findgen(h_x)#Replicate(1,h_y)) / (h_x-1)
    h_texcoords[1,*,*] = (Replicate(1,h_y)#Findgen(h_x)) / (h_x-1)

    h_surface = OBJ_NEW('IDLgrSurface', SHADING = 1, h_surfaceData, h_xx, h_yy, Style=2, $
        Color=[255,255,255], _Extra=extra, Hidden_Lines=hidden_lines)
    h_surface->SetProperty, Texture_Map=o_h, Texture_Coord=h_texcoords

    v_texcoords = FltArr(2, v_x, v_y)
    v_texcoords[0,*,*] = (Findgen(v_x)#Replicate(1,v_y)) / (v_x-1)
    v_texcoords[1,*,*] = (Replicate(1,v_y)#Findgen(v_x)) / (v_x-1)

    ;v_surface = OBJ_NEW('IDLgrSurface', SHADING = 1, v_surfaceData, v_xx, v_yy, Style=2, $
    v_surface = OBJ_NEW('IDLgrSurface', SHADING = 1, v_surfaceData, v_xx, v_yy, Style=2, $
        Color=[255,255,255], _Extra=extra, Hidden_Lines=hidden_lines)
    v_surface->SetProperty, Texture_Map=o_v, Texture_Coord=v_texcoords

    c_texcoords = FltArr(2, c_x, c_y)
    c_texcoords[0,*,*] = (Findgen(c_x)#Replicate(1,c_y)) / (c_x-1)
    c_texcoords[1,*,*] = (Replicate(1,c_y)#Findgen(c_x)) / (c_x-1)

    c_surface = OBJ_NEW('IDLgrSurface', SHADING = 1, c_surfaceData, c_xx, c_yy, Style=2, $
        Color=[255,255,255], _Extra=extra, Hidden_Lines=hidden_lines)
    c_surface->SetProperty, Texture_Map=o_c, Texture_Coord=c_texcoords


stop

    ; Get the data ranges of the surface.

    c_surface->GetProperty, XRange=xrange, YRange=yrange, ZRange=zrange

    ;xrange = [-5,5]
    ;yrange = [-5,5]
    ;zrange = [0,128]

    ; Create axes objects for the surface. Color them green.
    ; Axes are created after the surface so the range can be
    ; set correctly. Note how I set the font to 10 point Helvetica
    ; by creating the axis with the title object, then getting the
    ; actual axis text from the axis object itself, and switching it.



    xAxis = Obj_New("IDLgrAxis", 0, Color=[0,255,0], Ticklen=0.1, $
        Minor=4, Title=xtitleObj, Range=xrange, Exact=Keyword_Set(exact))
    xAxis->GetProperty, Ticktext=xAxisText
    xAxisText->SetProperty, Font=helvetica10pt

    yAxis = Obj_New("IDLgrAxis", 1, Color=[0,255,0], Ticklen=0.1, $
        Minor=4, Title=ytitleObj, Range=yrange, Exact=Keyword_Set(exact))
    yAxis->GetProperty, Ticktext=yAxisText
    yAxisText->SetProperty, Font=helvetica10pt

    zAxis = Obj_New("IDLgrAxis", 2, Color=[0,255,0], Ticklen=0.1, $
        Minor=4, Title=ztitleObj, Range=zrange, Exact=Keyword_Set(exact))
    zAxis->GetProperty, Ticktext=zAxisText
    zAxisText->SetProperty, Font=helvetica10pt

    ; The axes may not use exact axis scaling, so the ranges may
    ; have changed from what they were originally set to. Get
    ; and update the range variables.

    xAxis->GetProperty, CRange=xrange
    yAxis->GetProperty, CRange=yrange
    zAxis->GetProperty, CRange=zrange

    ; Set scaling parameters for the surface and axes so that everything
    ; is scaled into the range -0.5 to 0.5. We do this so that when the
    ; surface is rotated we don't have to worry about translations. In
    ; other words, the rotations occur about the point (0,0,0).

    ;xs = FSC_Normalize(xrange, Position=[pos[0], pos[2]])
    ;ys = FSC_Normalize(yrange, Position=[pos[1], pos[3]])
    xs = FSC_Normalize(xrange, Position=[-.5,.5]*zscale)
    ys = FSC_Normalize(yrange, Position=[-.5,.5]*zscale)
    zs = FSC_Normalize(zrange, Position=[-0.5,0.5]*zscale)

    ; Scale the axes and place them in the coordinate space.
    ; Note that not all values in the Location keyword are
    ; used. (I've put really large values into the positions
    ; that are not being used to demonstate this.) For
    ; example, with the X axis only the Y and Z locations are used.

    xAxis->SetProperty, Location=[9999.0, pos[1], -0.5*zscale], XCoord_Conv=xs
    yAxis->SetProperty, Location=[pos[0], 9999.0, -0.5*zscale], YCoord_Conv=ys
    zAxis->SetProperty, Location=[pos[0], pos[3],      9999.0], ZCoord_Conv=zs

    ; Scale the surface. Notice the surface is scaled *AFTER* the
    ; actual data range is obtained from the axes (CRANGE, above).
    ; Failure to do this can result in inaccurate results.

    h_surface->SetProperty, XCoord_Conv=xs, YCoord_Conv=ys, ZCoord_Conv=zs
    v_surface->SetProperty, XCoord_Conv=xs, YCoord_Conv=ys, ZCoord_Conv=zs
    c_surface->SetProperty, XCoord_Conv=xs, YCoord_Conv=ys, ZCoord_Conv=zs

    ;v_surface

    ; Add the surface and axes objects to the model.


    thisModel->Add, xAxis
    thisModel->Add, yAxis
    thisModel->Add, zAxis

    thisModel->Add, h_surface

    ;T3D, ROT = [ 90,0,0]
    ;thisModel->Rotate,[1,0,0],  90
    thisModel->Add, v_surface
    ;thisModel->Rotate,[1,0,0], -90
    ;T3D, ROT = [-90,0,0]

    ;T3D, ROT = [0, 45,0]
    ;thisModel->Rotate,[0,1,0],  45
    thisModel->Add, c_surface
    ;thisModel->Rotate,[0,1,0], -45
    ;T3D, ROT = [0,-45,0]




    ; Rotate the surface model to the standard surface view.

    thisModel->Rotate,[1,0,0], -90  ; To get the Z-axis vertical.
    thisModel->Rotate,[0,1,0],  30  ; Rotate it slightly to the right.
    thisModel->Rotate,[1,0,0],  30  ; Rotate it down slightly.

    ; Create some lights to view the surface. Surfaces will look
    ; best if there is some ambient lighting to illuminate them
    ; uniformly, and some positional lights to give the surface
    ; definition. We will create three positional lights: one,
    ; non-rotating light will provide overhead definition. Two
    ; rotating lights will provide specific surface definition.

    ; First create the ambient light. Don't turn it on too much,
    ; or the surface will appear washed out. If you have an image
    ; as a texture map, turn it on full.

    if(0) then begin

        ambientLight = Obj_New('IDLgrLight', Type=0, Intensity=0.2)
        IF N_Elements(image) NE 0 THEN ambientLight->SetProperty, Intensity=1.0
        thisModel->Add, ambientLight

        ; Shaded surfaces will not look shaded unless there is a
        ; positional light source to give the surface edges definition.
        ; This light will rotate with the surface.

        rotatingLight = Obj_New('IDLgrLight', Type=1, Intensity=0.60, $
            Location=[xrange[1], yrange[1], 4*zrange[1]], $
            Direction=[xrange[0], yrange[0], zrange[0]])
        thisModel->Add, rotatingLight

        ; Create a fill light source so you can see the underside
        ; of the surface. Otherwise, just the top surface will be visible.
        ; This light will also rotate with the surface.

        fillLight = Obj_New('IDLgrLight', Type=1, Intensity=0.4, $
            Location=[(xrange[1]-xrange[0])/2.0, (yrange[1]-yrange[0])/2.0, -2*Abs(zrange[0])], $
            Direction=[(xrange[1]-xrange[0])/2.0, (yrange[1]-yrange[0])/2.0, zrange[1]])
        thisModel->Add, fillLight

        ; Create a non-rotating overhead side light.

        nonrotatingLight = Obj_New('IDLgrLight', Type=1, Intensity=0.8, $
            Location=[-xrange[1], (yrange[1]-yrange[0])/2.0, 4*zrange[1]], $
            Direction=[xrange[1], (yrange[1]-yrange[0])/2.0, zrange[0]])
        nonrotatingModel = Obj_New('IDLgrModel')
        nonrotatingModel->Add, nonrotatingLight

        ; Be sure to add the non-rotating model to the view, or it won't be visualized.

        thisView->Add, nonrotatingModel

        ; Scale the light sources.

        rotatingLight->SetProperty, XCoord_Conv=xs, YCoord_Conv=ys, ZCoord_Conv=zs
        fillLight->SetProperty, XCoord_Conv=xs, YCoord_Conv=ys, ZCoord_Conv=zs
        nonrotatingLight->SetProperty, XCoord_Conv=xs, YCoord_Conv=ys, ZCoord_Conv=zs

        ; Rotate the non-rotating model to the standard surface view.

        nonrotatingModel->Rotate,[1,0,0], -90  ; To get the Z-axis vertical.
        nonrotatingModel->Rotate,[0,1,0],  30  ; Rotate it slightly to the right.
        nonrotatingModel->Rotate,[1,0,0],  30  ; Rotate it down slightly.

    endif



    ; Check for availability of GIF files.

    thisVersion = Float(!Version.Release)
    IF thisVersion LT 5.4 THEN haveGif = 1 ELSE haveGIF = 0

    ; Create the widgets to view the surface. Set expose events
    ; on the draw widget so that it refreshes itself whenever necessary.
    ; Button events are on to enable trackball movement.

    tlb = Widget_Base(Title='Texture Surface Example', Column=1, $
        TLB_Size_Events=1, MBar=menubase)
    drawID = Widget_Draw(tlb, XSize=400, YSize=400, Graphics_Level=2, Retain=0, $
        Expose_Events=1, Event_Pro='Texture_Surface_Draw_Events', Button_Events=1)

    ; Create FILE menu buttons for printing and exiting.

    fileID = Widget_Button(menubase, Value='File')

    outputID = Widget_Button(fileID, Value='Save As...', /Menu)
    IF havegif THEN dummy = Widget_Button(outputID, Value='GIF File', $
        UValue='GIF', Event_Pro='FSC_Surface_Output')
    dummy = Widget_Button(outputID, Value='JPEG File', $
        UValue='JPEG', Event_Pro='FSC_Surface_Output')
    dummy = Widget_Button(outputID, Value='TIFF File', $
        UValue='TIFF', Event_Pro='FSC_Surface_Output')

    dummy = Widget_Button(fileID, Value='Print', Event_Pro='Texture_Surface_Printing')

    dummy = Widget_Button(fileID, /Separator, Value='Exit', $
        Event_Pro='Texture_Surface_Exit')

    ; Create STYLE menu buttons for surface style.

    styleID = Widget_Button(menubase, Value='Style', Event_Pro='Texture_Surface_Style', /Menu)
    dummy = Widget_Button(styleID, Value='Dot Surface', UValue='DOTS')
    dummy = Widget_Button(styleID, Value='Wire Mesh', UValue='MESH')
    dummy = Widget_Button(styleID, Value='Solid', UValue='SOLID')
    dummy = Widget_Button(styleID, Value='Parallel X Lines', UValue='XPARALLEL')
    dummy = Widget_Button(styleID, Value='Parallel Y Lines', UValue='YPARALLEL')
    IF hidden_lines THEN hlValue = 'Hidden Lines OFF' ELSE hlValue='Hidden Lines ON'
    dummy = Widget_Button(styleID, Value=hlvalue, UValue='HIDDEN')

    ; Create the Image Selector 1 - 6
    imageID = Widget_Button(menubase, Value='Select Image', Event_Pro='Plane3_Select_Image', /Menu)
    dummy = Widget_Button(imageID, Value='CPI 174', UValue='c1')
    dummy = Widget_Button(imageID, Value='CPI 190', UValue='c2')
    dummy = Widget_Button(imageID, Value='CPI 196', UValue='c3')
    dummy = Widget_Button(imageID, Value='CPI 211', UValue='c4')
    dummy = Widget_Button(imageID, Value='CPI 222', UValue='c5')
    dummy = Widget_Button(imageID, Value='CPI 227', UValue='c6')

    ; Realize the widgets.

    Widget_Control, tlb, /Realize

    ; Get the window destination object, which is the value of
    ; an object draw widget. The view will be drawn in the window
    ; when the window is exposed.

    Widget_Control, drawID, Get_Value=thisWindow

    ; Get a printer object for this graphic.

    thisPrinter = Obj_New('IDLgrPrinter', Print_Quality=2, Landscape=landscape)

    ; Create a container object to hold all the other
    ; objects. This will make it easy to free all the
    ; objects when we are finished with the program.

    thisContainer = Obj_New('IDL_Container')

    ; Add created objects to the container. No need to add objects
    ; that have been added to the model, since a model object is
    ; a subclass of a container object. But helper objects that
    ; are NOT added to the model directly MUST be destroyed properly.

    thisContainer->Add, thisView
    thisContainer->Add, thisPrinter
    thisContainer->Add, thisTrackball
    thisContainer->Add, xTitleObj
    thisContainer->Add, yTitleObj
    thisContainer->Add, zTitleObj
    thisContainer->Add, thisModel
    thisContainer->Add, helvetica10pt
    thisContainer->Add, helvetica14pt
    IF Obj_Valid(o_c) THEN thisContainer->Add, o_c
    IF Obj_Valid(imgpal) THEN thisContainer->Add, imgpal

    ; Create an INFO structure to hold needed program information.

    info = { thisContainer:thisContainer, $ ; The object container.
        thisWindow:thisWindow, $       ; The window object.
        thisPrinter:thisPrinter, $     ; The printer object.
        c_surface:c_surface, $         ; The surface object.
        thisTrackball:thisTrackball, $ ; The trackball object.
        thisModel:thisModel, $         ; The model object.
        thisView:thisView, $           ; The view object.
        xaxis:xaxis, $                 ; The X axis object.
        yaxis:yaxis, $                 ; The Y axis object.
        zaxis:zaxis, $                 ; The Z axis object.
        drawID:drawID, $               ; The widget identifier of the draw widget.
        landscape:landscape, $         ; A flag for landscape output.
        vector:vector, $               ; A flag for vector output.
        h1:h1, $
        h2:h2, $
        h3:h3, $
        h4:h4, $
        h5:h5, $
        h6:h6, $
        v1:v1, $
        v2:v2, $
        v3:v3, $
        v4:v4, $
        v5:v5, $
        v6:v6, $
        c1:c1, $
        c2:c2, $
        c3:c3, $
        c4:c4, $
        c5:c5, $
        c6:c6, $
        h_x:h_x, $
        h_y:h_y, $
        v_x:v_x, $
        v_y:v_y, $
        c_x:c_y, $
        c_y:c_y, $
        h_image:h_image, $
        v_image:v_image, $
        c_image:c_image  $
        }


    ; Store the info structure in the UValue of the TLB.

    Widget_Control, tlb, Set_UValue=info, /No_Copy

    ; Call XManager. Set a cleanup routine so the objects
    ; can be freed upon exit from this program.

    XManager, '3VCPI 3 Views', tlb, Cleanup='Texture_Surface_Cleanup', /No_Block, $
        Event_Handler='Texture_Surface_Resize', Group_Leader=groupLeader
END ; plane3





;stop

;oModel = OBJ_NEW('IDLgrModel')


;oModel -> ADD, o_h

;oModel -> ROTATE, [0, 0, 1],  45
;oModel -> ADD, o_v
;oModel -> ROTATE, [0, 0, 1], -45

;oModel -> ROTATE, [0, 0, 1],  90
;oModel -> ADD, o_c
;oModel -> ROTATE, [1, 0, 1], -90
;oModel -> ADD, o_h

; display model in interactive viewer
;XOBJVIEW, oModel, /BLOCK



;stop











; create mesh objects for each image

;mesh_obj, 1,   h_vertices,   h_polygons,   h_image
;mesh_obj, 1,   v_vertices,   v_polygons,   v_image
;mesh_obj, 1, c_vertices, c_polygons, c_image


;mesh_obj, 1,   h_vertices,   h_polygons, bytarr(h_x, h_y)
;mesh_obj, 1,   v_vertices,   v_polygons, bytarr(v_x, v_y)
;mesh_obj, 1, c_vertices, c_polygons, bytarr(c_x, c_y)


;stop

; create a model object to contain the display
;oModel = OBJ_NEW('IDLgrModel')

;  o_h_model = OBJ_NEW('IDLgrModel')
;  o_v_model = OBJ_NEW('IDLgrModel')
;o_c_model = OBJ_NEW('IDLgrModel')


; create image and palette objects to contain the image and color table
; using blue/white linear as it is used natively by cView, and has 0 black and 255 white for the 2DS images
;o_h_palette = OBJ_NEW('IDLgrPalette')
;o_h_palette -> LoadCT, 1
;o_h_palette -> SetRGB, 255,255,255,255
;o_h_image = OBJ_NEW('IDLgrImage', h_image, PALETTE = o_h_palette)

;o_v_palette = OBJ_NEW('IDLgrPalette')
;o_v_palette -> LoadCT, 1
;o_v_palette -> SetRGB, 255,255,255,255
;o_v_image = OBJ_NEW('IDLgrImage', v_image, PALETTE = o_v_palette)

;o_c_palette = OBJ_NEW('IDLgrPalette')
;o_c_palette -> LoadCT, 1
;o_c_palette -> SetRGB, 255,255,255,255
;o_c_image = OBJ_NEW('IDLgrImage', c_image, PALETTE = o_c_palette)


; derive texture map coordinates
;vector = FINDGEN(101)/100.
;texure_coordinates = FLTARR(2, 101, 101)
;texure_coordinates[0, *, *] = vector # REPLICATE(1., 101)
;texure_coordinates[1, *, *] = REPLICATE(1., 101) # vector

;xvector = FINDGEN(h_x+1)/float(h_x)
;yvector = FINDGEN(h_y+1)/float(h_y)
;h_texture_coordinates = FLTARR(2, h_x+1, h_y+1)
;h_texture_coordinates[0, *, *] = xvector # REPLICATE(1., h_y+1)
;h_texture_coordinates[1, *, *] = REPLICATE(1., h_x+1) # yvector

;xvector = FINDGEN(v_x+1)/float(v_x)
;yvector = FINDGEN(v_y+1)/float(v_y)
;v_texture_coordinates = FLTARR(2, v_x+1, v_y+1)
;v_texture_coordinates[0, *, *] = xvector # REPLICATE(1., v_y+1)
;v_texture_coordinates[1, *, *] = REPLICATE(1., v_x+1) # yvector

;xvector = FINDGEN(c_x+1)/float(c_x)
;yvector = FINDGEN(c_y+1)/float(c_y)
;c_texture_coordinates = FLTARR(2, c_x+1, c_y+1)
;c_texture_coordinates[0, *, *] = xvector # REPLICATE(1., c_y+1)
;c_texture_coordinates[1, *, *] = REPLICATE(1., c_x+1) # yvector

;stop
;v_vertices = v_vertices ## [

;a = fltarr(3, 4)

;a[*, 0] = [0,0,0]
;a[*, 1] = [0,1,0]
;a[*, 2] = [1,1,0]
;a[*, 3] = [1,0,0]

;h_vertices = a

;a[*, 0] = [0,0,0]
;a[*, 1] = [0,0,1]
;a[*, 2] = [1,0,1]
;a[*, 3] = [1,0,0]

;v_vertices = a

;a[*, 0] = [0,0,0]
;a[*, 1] = [0,0,1]
;a[*, 2] = [0,1,1]
;a[*, 3] = [0,1,0]

;c_vertices = a



; create the polygon object containing the data
;o_h_polygons = OBJ_NEW('IDLgrPolygon', SHADING = 1, $
;   DATA = h_vertices, POLYGONS = h_polygons, $
;   COLOR = [255,255,255], $
;   TEXTURE_COORD = h_texture_coordinates, $
;   TEXTURE_MAP = o_h_image, /TEXTURE_INTERP)

;o_v_polygons = OBJ_NEW('IDLgrPolygon', SHADING = 1, $
;   DATA = v_vertices, POLYGONS = v_polygons, $
;   COLOR = [255,255,255], $
;   TEXTURE_COORD = v_texture_coordinates, $
;   TEXTURE_MAP = o_v_image, /TEXTURE_INTERP)

;o_c_polygons = OBJ_NEW('IDLgrPolygon', SHADING = 1, $
;   DATA = c_vertices, POLYGONS = c_polygons, $
;   COLOR = [255,255,255], $
;   TEXTURE_COORD = c_texture_coordinates, $
;   TEXTURE_MAP = o_c_image, /TEXTURE_INTERP)


; add polygons to model container
; rotate model to align the 3 planes
;oModel -> ADD, o_h_polygons

;oModel -> ROTATE, [0, 1, 0], 90
;oModel -> ADD, o_v_polygons

;oModel -> ROTATE, [0, 0, 1],  45
;oModel -> ADD, o_c_polygons
;oModel -> ROTATE, [0, 0, 1], -45

;oModel -> ADD, o_h

;oModel -> ROTATE, [0, 0, 1],  90
;oModel -> ADD, o_v
;oModel -> ROTATE, [0, 0, 1], -90

;oModel -> ROTATE, [1, 0, 1],  45
;oModel -> ADD, o_c
;oModel -> ROTATE, [1, 0, 1], -45


; display model in interactive viewer
;XOBJVIEW, oModel, /BLOCK

; clean up object references
;OBJ_DESTROY, [oModel,   o_h_image,   o_h_palette, $
;                        o_v_image,   o_h_palette, $
;                      o_c_image, o_c_palette]

;END ; plane3
