; File: plotmask.pro
; Author:  Ted Fisher <ted@specinc.com> <fishnuke@gmail.com>
; Purpose: displays an annotated binary plot of a 128 bit 2D array mask passed in as
;          an 8 element unsigned integer array (4 hex digits, 16 bits per element)

; The mask input is read left to right, bit 0 of the mask is the leftmost, and bit 127 is the rightmost.

; The mask is ANDed with the pixel array values, so 0 is masked out, and 1's are allowed

; plotmask, uint(['0000'X, '0000'X, '0000'X, '0000'X, '0000'X, '0000'X, '0000'X, '0000'X]) ; all zeros
; plotmask, uint(['FFFF'X, 'FFFF'X, 'FFFF'X, 'FFFF'X, 'FFFF'X, 'FFFF'X, 'FFFF'X, 'FFFF'X]) ; all ones
; plotmask, uint(['AAAA'X, 'AAAA'X, 'AAAA'X, 'AAAA'X, 'AAAA'X, 'AAAA'X, 'AAAA'X, 'AAAA'X]) ; 1010... alternating
; plotmask, uint(['5555'X, '5555'X, '5555'X, '5555'X, '5555'X, '5555'X, '5555'X, '5555'X]) ; 0101... alternating
; plotmask, uint(['0000'X, '0000'X, '0000'X, '0000'X, 'FFFF'X, 'FFFF'X, 'FFFF'X, 'FFFF'X]) ; half zero, half one
; plotmask, uint(['FFFF'X, 'FFFF'X, 'FFFF'X, 'FFFF'X, '0000'X, '0000'X, '0000'X, '0000'X]) ; half one,  half zero
; plotmask, uint(['FFFF'X, '0000'X, 'FF00'X, 'FF00'X, 'F0F0'X, 'F0F0'X, 'CC33'X, '55AA'X]) ; test pattern






pro plotmask, hex_input

    xscale = 5
    yscale = 10

    npixels = 128

    mask = bytarr(npixels)

    printh, hex_input

    for i=0, npixels -1 do begin
        current_element = fix(i / 16)

        current_bit = 15 - (i - current_element * 16)

        mask[i] = byte(1) and ishft(hex_input[current_element], -current_bit)
    endfor

    ; needed to do the rescale for display in 2 steps,
    ; as the rebin expanded the 0's not the 1's,
    ; and the congrid can't expand a linear array
    mask_image = rebin(mask, npixels, yscale)
    mask_image = congrid(mask_image, xscale*npixels, yscale)


    ted, mask_image


    stop
end ; plotmask
