
pro read_3vcpi_hk_data, HK_array,basefile, H50micron

    ;basefile = dialog_pickfile(title='Select 3V-CPI 2D-S Base File', filter='base*.2DSCPI', get_path=infilepath)

    ; read a 3V-CPI Housekeeping file into memory, generate arrays of HK, Masks, and timestamps, and convert the raw
    ; HK counts into physical values

    ; fills the basescan_common hk_ arrays

    ; hk_count is the hk_ array size

    ; H50micron is set to 1 for H 50 micron probes, 0 for H 10 micron probes

    ; setting /show_plots to 1 shows plots of HK data, setting to 0 overrides /save_plots
    ; setting /save_plots saves the HK plots

    ; note, should update to check second word = 83 and the checksums, and make parsing more robust

    ; need to make sure hk_array and hk_timestamp_array match up -- maybe put the timestamp struct in the hk_array???

    ; need to detect H 50 micron from Housekeeping and put it into output files/sets flag H50micron

    ;    (from the c++)
    ;    // test for the Hawkeye 50 micron flag in HK Word 81, bit 15
    ;    if((hk->data2[3] & 0x8000) != 0)
    ;    {
    ;        *H50micron_flag = 1
    ;        H50micron = true
    ;    }
    ;    else
    ;    {
    ;        *H50micron_flag = 0
    ;        H50micron = false
    ;    } // end if


    ;    hpos = 512
    ;    vpos = 512

    ;    reprint_raw_data = 1

    ;    inword = uint(0) ; read 2 bytes
    ;    Hwords = uint(0)
    ;    Vwords = uint(0)

    ;    total_Hpart     = 0L
    ;    total_Vpart     = 0L
    ;    total_Hmulti    = 0L
    ;    total_Vmulti    = 0L
    ;    total_Hoverflow = 0L
    ;    total_Voverflow = 0L
    ;    total_Hcpi_trigger = 0L
    ;    total_Vcpi_trigger = 0L
    ;    total_Hflag_bit13  = 0L
    ;    total_Vflag_bit13  = 0L
    ;    total_Hflag_bit12  = 0L
    ;    total_vflag_bit12  = 0L
    ;    total_NH_and_NV_ne_zero = 0L
    ;    total_NULL_count   = 0L

    ;        Hpart_this_second = 0L
    ;        Vpart_this_second = 0L
    ;       Hmulti_this_second = 0L
    ;       Vmulti_this_second = 0L
    ;    Hoverflow_this_second = 0L
    ;    Voverflow_this_second = 0L
    ;         NULL_this_second = 0L

    ;    probecountflag_set = 0 ; debug flag for stopping at specified probecount values

    ;    probecount = long64(0)


    infilename = strcompress(basefile + 'HK', /rem)
    infilepath = strmid(basefile, 0, strpos(basefile, '\', /reverse_search) + 1)

    stripped_infilename = strmid(infilename, strlen(infilepath)) ; the file name with the path removed, for generating the output file names


    ;    print, ''
    ;    print, 'read_hk opening file: ', stripped_infilename
    ;    print, ''
    ;    print, 'Scan starting at:'
    ;    print, systime()

    startseconds = systime(/seconds)



    openr, lun, /get, infilename

    infilefstat = fstat(lun)

    hk_file_size = infilefstat.size

    hk_lun = lun

    ; Housekeeping ASCII Flag 'HK' (0x484B)
    ; Mask         ASCII Flag 'MK' (0x4D4B)

    flag = uint(0)

    hk_words = 82 ; 83 including ID, second word is Packet Length, which should = 83
    mask_words = 27

    hk_frame = uintarr(hk_words)
    mask_frame = uintarr(mask_words)

    defsysv, '!array_growth_factor', 100000L 

    timestamp  = { _timestamp_struct, $
        year  : uint(0), $
        month : uint(0), $
        dow   : uint(0), $
        day   : uint(0), $
        hour  : uint(0), $
        minute: uint(0), $
        second: uint(0), $
        ms    : uint(0)  }



    array_size = !array_growth_factor

    empty_timestamp = timestamp

    hk_timestamp_array = replicate(empty_timestamp, array_size)

    hk_array = uintarr(array_size,   hk_words)
    mask_array = uintarr(array_size, mask_words)

    hk_count = 0L
    mask_count = 0L
    timestamp_count = 0L


    done = 0

    while(not done) do begin
        ; read the 16 byte MS timestamp
        ; test explicitly for end of file before each read
        point_lun, -hk_lun, current_pos

        if(current_pos eq hk_file_size) then begin
            print, 'at end of file, exiting'
            break
        endif

        next_read_length = n_tags(timestamp, /data_length)
        if(current_pos + next_read_length gt hk_file_size) then begin
            print, 'at end of file, exiting'
            break
        endif

        readu, hk_lun, timestamp

        hk_timestamp_array[timestamp_count] = timestamp

        timestamp_count++

        point_lun, -hk_lun, current_pos

        next_read_length = n_tags(timestamp, /data_length)
        if(current_pos + next_read_length gt hk_file_size) then begin
            print, 'at end of file, exiting'
            break
        endif

        point_lun, -hk_lun, current_pos

        next_read_length = n_tags(flag, /data_length)
        if(current_pos + next_read_length gt hk_file_size) then begin
            print, 'at end of file, exiting'
            break
        endif

        readu, hk_lun, flag

        if(flag eq '484B'X) then begin ; 0x484B HK Housekeeping
            ;print, 'HK found'

            point_lun, -hk_lun, current_pos

            next_read_length = n_tags(hk_frame, /data_length)
            if(current_pos + next_read_length gt hk_file_size) then begin
                print, 'at end of file, exiting'
                break
            endif

            readu, hk_lun, hk_frame

            hk_array[hk_count, *] = hk_frame

            hk_count++
        endif else if(flag eq '4D4B'X) then begin ; 0x4D4B MK Mask
            print, 'Mask found ', mask_count+1

            point_lun, -hk_lun, current_pos

            next_read_length = n_tags(mask_frame, /data_length)
            if(current_pos + next_read_length gt hk_file_size) then begin
                print, 'at end of file, exiting'
                break
            endif

            readu, hk_lun, mask_frame

            mask_array[mask_count, *] = mask_frame

            mask_count++


            ; Word       Mask Data Description
            ; 1          Flag ASCII "MK" (0x4D4B)
            ; 2          Packet Length (in words) - 28 (0x1C)
            ; 3-5        48 bit Timing Word (read from the FPGA)
            ; 6-13       Horizontal mask bits -  Directly Read from FPGA (hz_mask_?)
            ; 14 - 21    Vertical mask bits - Directly Read from FPGA (vt_mask_?)
            ; 22 - 24    Timing Word at start of mask routine - Directly Read from FPGA (tw_?sb)
            ; 25 - 27    Timing Word at end of mask routine - Directly Read from FPGA (tw_?sb)
            ; 28         Checksum Word


        endif

        ; test the counts against the array size and expand all arrays if needed
        if(timestamp_count ge array_size-1 or hk_count ge array_size-1 or mask_count ge array_size-1) then begin
            stop

            array_size += 10000L

            if(timestamp_count gt 0) then begin
                new_timestamp_array = replicate(empty_timestamp, array_size)
                new_timestamp_array[0:timestamp_count-1] = temporary(hk_timestamp_array[0:timestamp_count-1])
                hk_timestamp_array = temporary(new_timestamp_array)
            endif

            if(hk_count gt 0) then begin
                new_hk_array = uintarr(array_size, hk_words)
                new_hk_array[0:hk_count-1, *] = temporary(hk_array[0:hk_count-1, *])
                hk_array = temporary(new_hk_array)
            endif

            if(mask_count gt 0) then begin
                new_mask_array = uintarr(array_size, mask_words)
                new_mask_array[0:mask_count-1, *] = temporary(mask_array[0:mask_count-1, *])
                mask_array = temporary(new_mask_array)
            endif

            stop
        endif
    endwhile

    free_lun, hk_lun

    ; truncate the arrays to the actual counts
    if(timestamp_count gt 0) then begin
        new_timestamp_array = replicate(empty_timestamp, timestamp_count)
        new_timestamp_array[0:timestamp_count-1] = temporary(hk_timestamp_array[0:timestamp_count-1])
        hk_timestamp_array = temporary(new_timestamp_array)
    endif

    if(hk_count gt 0) then begin
        new_hk_array = uintarr(hk_count, hk_words)
        new_hk_array[0:hk_count-1, *] = temporary(hk_array[0:hk_count-1, *])
        hk_array = temporary(new_hk_array)
    endif

    if(mask_count gt 0) then begin
        new_mask_array = uintarr(mask_count, mask_words)
        new_mask_array[0:mask_count-1, *] = temporary(mask_array[0:mask_count-1, *])
        mask_array = temporary(new_mask_array)
    endif

    print, ''
    print, 'Read ',        hk_count, ' HK packets'
    print, 'Read ',      mask_count, ' mask packets'
    print, 'Read ', timestamp_count, ' timestamps'
    print, ''

    num_hrz_dect    =        hk_array[*, 56]  ; Horizontal particles detected
    num_vrt_dect    =        hk_array[*, 57]  ; Vertical particles detected


    ; detect Hawykeye H 50 micron flag in HK Word 81, bit 15
    if((uint(hk_array[0, 79]) and uint('8000'X)) gt 0) then begin
        H50micron = 1
    endif else begin
        H50micron = 0
    endelse



end ; read_hk


