; File a3VCPIView.pro
; Entry point for 3V-CPI View, use @compile3vcpi to run


pro define_current_3vcpiview_version ; run from compile3vcpi.pro to define the current version for the program
    defsysv, '!SPEC_VERSION', '1.1.82 TAF'

    defsysv, '!CPI_MICRONS_PER_PIXEL',         2.3 ; microns/pixel -- this is CPI probe specific, measured using 500 micron bead data

    defsysv, '!disable_time_popups',            1  ; 0 or 1 -- disable "If time is synced" popup in CPI View side

    defsysv, '!manual_override_H50vs10micron',  0  ; 0 or 1 -- manually override the automatic 50 vs 10 micron detection
    ;(for files prior to Playback 1.1.17)
    defsysv, '!microns_per_pixelh', 10.
    defsysv, '!microns_per_pixelv', 10.

    defsysv, '!H_resolution_microns',          10  ; 10 or 50 microns for the H channel resulotion.
    ; Use with !manual_override_H50vs10micron = 1 to manually set the H resolution

    defsysv, '!H_res_set',                      0  ; flag to determine if the H resolution has been set this run, to avoid stops in manual override
    defsysv, '!H_res_set_manually',             0  ; flag for internal logic to detect if the manual selection window has been used or not

    defsysv, '!RANDOM_SEED',                   42 ; a fixed random seed for producing repeatable outputs
    defsysv, '!USE_RANDOM_DIFFUSER',            0 ; boolean, 1 use the random diffuser, 0 do not use the random diffuser
    defsysv, '!mass_optionIWC',                 1 ; set 1 to calculate mass using regular options , otherwise,m set it to 0.  added this part on 2012-10-04--Mo
    defsysv, '!mass_optionIcsp',                0 ; set 1 to calculate mass using ICSP, other wise set it to 0.  The maybe overwrite by new guis.
    defsysv, '!mass_optionLiq',                 0 ; set 1 to calculate iwc use csp.  but it will e overwrite by pop up otion clicks..
    defsysv, '!ct_2DS',                        39 ; 2D-S color table
    defsysv, '!ct_CPI',                         1 ; CPI color table

    defsysv, '!csp_alpha',                  0.135 ; from Paul and Brad's paper of  Januray 2006-improvement in Determination of ice Water Content JAPMandClimat.
    defsysv, '!csp_beta',                   0.793

    defsysv, '!Project_name',          'Testing'      ; which project the file is from.
    defsysv, '!Platform',              'Lear'       ; which airplane the probe is on.
    defsysv, '!Revision_X',                  'R0' ;   R0,  R1, R2, revision number for archive sumbitting.

    defsysv, '!Add_Classification_to_pngs',   'N' ; 'Y' print sph/ice on png page images, 'N' do not print classification on images

    defsysv, '!enable_image_diagnostic',        0 ; 0 or 1 -- CPI View enables a stop in gui_lib.pro with an array of images, used for viewing raw data
    defsysv, '!diagnostics_min_size',         300 ; size used in enable_image_diagnostics, to limit stopping on small images

    defsysv, '!flag_3VCPI',                     1; define this system variable so that processparticle.pro handles the 3V-CPI type data format instead of 2D-S/HVPS format (slice value 0x7FFF handled differently)
    ; only set to 1 in 3V-CPI View, in 2D-Sview  and HVPSview  should have this set to 0
    defsysv, '!flag_uCOPP',       0 ; sets microns per pixel and dww for uCOPP

    ;defsysv, '!dww',        -1.0D ; mm Distance Window to Window for defining the Depth of Field (DOF), 50.8mm for 3V-CPI, 63.0mm for 2D-S,  162.5mm for HVPS, ??? for 2D-128
    defsysv, '!dww_3V2DS',  50.8D ; mm Distance Window to Window for defining the Depth of Field (DOF) 50.8mm for 3V-2DS
    defsysv, '!dww_uCOPP',  38.1D ; mm Distance Window to Window for defining the Depth of Field (DOF) 38.1mm for uCOPP
    defsysv, '!dww_main',   -1.0D ; mm Distance Window to Window, gets set later

    defsysv, '!stop_from_here_on',              0  ; default is set to 0. otherwise to set  it  be 1.

    defsysv, '!Do_M7_area_ratio',               0 ; set it to be 1 to include doing M7 area ration output file. that will be 20 files more. take a long time.
    ; set 0 to do regular archive.

    defsysv, '!IMAGE_ROTATION',                 4 ; used in called to the function rotate for image display and printing, 4 is transpose with no rotation, 1 is no transpose with 90 degree rotation counterclockwise
    defsysv, '!use_external_TAS',               0 ; used exteral TAS like MMS data file to replace the wrong TAS data inbedded in the base file--for attrext hawkeye 10,10um 2DS only.
    ; set it to 1 to use external TAS. set it to 0 which is default to use the internal 2DS readed TAS to process data.

    defsysv, '!clean_option',   0; Note:   0 is default option, it mean clean up all noise, stuck bits streackers and shattered particles..
    ;       1 is cleans out noise and stuck bits only,  that mean keep the shattered particle in the acceted archive data.
    ;       2 is doing no clean at all, include all images, stuck bits, noise and shattered particle in the archive data fle.

    defsysv, '!just_HK',          0 ; just process the HK file and display the HK plots, then exit.  Useful for quick look at the HK file.
    defsysv, '!show_HK_plots',    1 ; set to 1 to show plots of the HK data, set to 0 to not show plots (overrides !save_HK_plots below)
    defsysv, '!save_HK_plots',    1 ; set to 1 to save images of the HK plots in the output directory, set to 0 to not save plots


    defsysv, '!mask_bit_filling', 1 ; set to 1 to enable mask bit filling, 0 to disable
    defsysv, '!mask_bit_width',   5 ; the maximum number of consecutive pixels to fill for mask bit filling, generally 5 seems to work well
    defsysv, '!manual_mask',      1 ; flag for using a manual mask -- ANDed with the probe masks (ZERO is masked, ONE is clear)

    ; The manual masks -- use the Realtime 2.X Pixel Histogram to identify pixels requiring masking, and the Mask control to generate the hex strings
    ; all F's -- no masked bits
    ;defsysv, '!manual_H_mask',    uint(['FFFF'X,'FFFF'X,'FFFF'X,'FFFF'X,'FFFF'X,'FFFF'X,'FFFF'X,'FFFF'X]) ; all F's -- no masked bits
    ;defsysv, '!manual_V_mask',    uint(['FFFF'X,'FFFF'X,'FFFF'X,'FFFF'X,'FFFF'X,'FFFF'X,'FFFF'X,'FFFF'X]) ; all F's -- no masked bits

    ; for base110711180731.2DSCPI -- pixel 29 in H manually masked
    defsysv, '!manual_H_mask',    uint(['FFFF'X,'FFFF'X,'FFFF'X,'FFFF'X,'FFFF'X,'FFFF'X,'DFFF'X,'FFFF'X])
    defsysv, '!manual_V_mask',    uint(['FFFF'X,'FFFF'X,'FFFF'X,'FFFF'X,'FFFF'X,'FFFF'X,'FFFF'X,'FFFF'X])

    ; edge testing for bitmap alignment
    ;defsysv, '!manual_H_mask',    uint(['5FFF'X,'FFFF'X,'FFFF'X,'FFFF'X,'FFFF'X,'FFFF'X,'FFFF'X,'FFFA'X])
    ;defsysv, '!manual_V_mask',    uint(['5FFF'X,'FFFF'X,'FFFF'X,'FFFF'X,'FFFF'X,'FFFF'X,'FFFF'X,'FFFA'X])


    defsysv, '!DATA_BLOCK_LENGTH',   4096L ; in bytes, note that raw_data_buffer and raw_data are indexed in words
    ; NOTE  ; + 2 for checksum = 4098 byte blocks + 16 bytes for the ms timestamps

    defsysv, '!MSTIMESTAMP_LENGTH',    16  ; in bytes

    defsysv, '!CHECKSUM_LENGTH',        2  ; in bytes

    defsysv, '!hk_count',           0
    defsysv, '!mask_count',         0
    defsysv, '!timestamp_count',    0


    defsysv, '!megaflaga', 0 ; you don't want to know
end ; define_current_3vcpiview_version



pro define_Pixel_resolutions
    ;  This subroutine is called when data are loaded.  if it it a 150um, or 50/10um or 10/10um probe.


    ;defsysv, '!MICRONS_PER_PIXELH',     10. ; default. calibrated size of pixels for calculating real lengths in microns, temporary for 2D-S
    defsysv, '!MICRONS_PER_PIXELV',      10. ; default. calibrated size of pixels for calculating real lengths in microns, temporary for 2D-S


    if(!manual_override_H50vs10micron eq 1 or !H_res_set_manually eq 1) then begin ; user has selected to manually override automatic detection of H 50 vs 10 microns
        print, ""                                      ; use the value set in !H_resolution_microns
        print, "!!!WARNING!!!
        print, "Using manual override for H 50 vs 10 microns"
        print, "!manual_override_H50vs10micron is set for manual override"
        print, "If you want to change this, set !manual_override_H50vs10micron = 0 and restart program!!"
        print, ""

        if(!H_res_set_manually ne 1) then begin
            ; if we're going for a user setup manual override,
            ; the popup will not have been displayed yet,
            ; if the automatic detetction in SORTED or binary fails, the popup will already have been shown
            H50micron_pop_up ; pop up window for User to manually select H 10 or 50 microns
        endif

        if(!H_resolution_microns eq 10) then begin
            !microns_per_pixelh = 10.

            print, "using manually set H resolution: ", !microns_per_pixelh
            print, ""
            ;print, ".cont to continue, or .res to reset IDL, then restart program with new value"
            ;print, ""
        endif else if(!H_resolution_microns eq 50) then begin
            !microns_per_pixel = 50.

            print, "using manually set H resolution: ", !microns_per_pixelh
            print, ""
            ;print, ".cont to continue, or .res to reset IDL, then restart program with new value"
            ;print, ""
        endif else begin
            print, "Error, !H_resolution_microns must be either 10 or 50 microns"
            print, "correct value then restart program"
            print, ""

            stop
        endelse

        !dww_main = !dww_3V2DS

        ;stop ; use .cont to continue from here or .res to reset IDL
        ; if you want to use automatic 50 vs 10 micron detection for SORTED files from Playback 1.1.17 or later,
        ; set !manual_override_H50vs10micron to 0
        ; otherwise, set !H_resolution_microns to 10 or 50 microns
    endif else begin ; using automatic detection was successful
        if(!flag_uCOPP) then begin
            !microns_per_pixelh = 25.
            !microns_per_pixelv = 25.

            !dww_main = !dww_uCOPP

            print, ""
            print, "Set for uCOPP processing"
            print, "H resolution: ", !microns_per_pixelh, " microns"
        endif else begin
            if(!H_resolution_microns eq 10) then begin
                !microns_per_pixelh = 10.
            endif else if(!H_resolution_microns eq 50) then begin
                !microns_per_pixelh = 50.
            endif

            !dww_main = !dww_3V2DS

            print, ""
            print, "Using automatic H resolution detection"
            print, "H resolution: ", !microns_per_pixelh, " microns"
        endelse
    endelse

    print, "V resolution: ", !microns_per_pixelv, " microns"
    print, ""

    !H_res_set = 1 ; set the flag to prevent repeating setting the H resolution
end ; define_current_3vcpiview_version







; NOTE:

; We need to discontinue use of forward_function.  It is being used incorrectly.

; fisher@fisher1 /cygdrive/c/fisher/SPEC/3VCPI/3VCPIView/3VCPIView_1.1.78_TAF
; $ grep forward_function ./* | wc -l
; 61

; In the latest 3V-CPI View 1.1.77 Mo, it is being used 61 times incorrectly.
; forward_function should only be used on a cross-platform compilation where a platform-specific function will not be present on all platforms.
; It is not a way to define a function so that another function will compile.  If the function will not compile, it means that the structure of
; the source file is incorrect.  Correcting the ordering and naming of the files and functions will allow IDL to auto-compile them as needed,
; and will help give the code a readable structure.  Finally, it will allow the code to be run from the IDL Virtual Machine.


; MORE IMPORTANTLY IS FILE NAMING AND PROCEEDURE/FUNCTION ORDERING!!!!!
;
; The *reason* forward_function is being used at all is because the file naming and the proceedures and functions in each file
; are in the incorrect order.  There are also repeats of names of proceedures and functions -- depending on the compile order,
; you get different versions of these running.  So you end up with unpredictable program behavior.  We need to reorder the proceedures
; and functions in each file, clean up and fix redundant naming, and make sure the files are named after the primary pro or function.
; When this is done, the program will run with autocompile, and will not need to use the @compile script.  Forward_function will no longer
; be needed to compile correctly.  And the compilation order and repeat names will be fixed.


; From Coyote:

; http://www.idlcoyote.com/code_tips/mostcommon.html
; http://www.idlcoyote.com/tips/namefiles.html

; By far the most common IDL error I see are IDL programs that are named incorrectly. If you have to
; compile an IDL program first to get it to run, it is named incorrectly. Think of IDL programs as IDL
; commands that you are constructing. The IDL module (procedure or function) that corresponds to the
; command name should be the last module in the file. If there are other modules in the file, they should
; be placed in front (toward the top of the file) of the command module and should be utility programs
; for the command module, not commands of their own. The names of these utility modules should reflect
; their utility status by having the command name prepended to their names. If a utility module gains
; status and importance beyond its modest service to the command module, it should be removed from the
; command module file, renamed, and promoted to a command module. The name of the file should be exactly
; the same as the name of the command module, and should be spelled with all lowercase letters, since
; that is what IDL looks for when it is auto-compiling code on all platforms.







; 1.1.82 TAF 2016-03-31 basescan fixes -- was throwing out CPI trigger images, still needs work on multi-part and overloads for full SORTED generation
; 1.1.81 TAF 2016-03-29 updating 3 Viewer for erronious hex times on CPI images (word1_b was being repeated), also expanded CPI Size min/max constants
;                       major work on the basescan generating SORTED files
; 1.1.80 TAF 2016-03-23 (in progress) removed debugging checks in processparticle, refined basescan reads
;                       fixed redundantly named getangle function from CPI Libaries, which was causing floating errors, and also uses differnt logic than
;                       the 3V-2DS getangle function.  This is a serious problem caused by the poor naming practices in these programs, and needs to be fixed permanently.
;                       basescan needs to be fixed or reverted before release
; 1.1.79 TAF 2016-03-18 refined image recompression after mask bit filling, reduced SORTED read particle count update from 100 to 1000, removed
;                       SORTED read "back up one byte to try for realignment", removed some popup dialogs, removed undefined !use_nav_tas_correct_L1n (again),
; 1.1.78 TAF 2016-03-15 merged with 1.1.77MO, got rid of "temp" from 3 Viewer, some cleanup, corrected "Eevent" in initial time series plot for base->SORTED and changed y axis scaling,
;                       got initial mask bit filling and image recompression working in SORTED file generation
; 1.1.77 TAF 2016-02-22 updated processparticle.pro with 1 pixel mask bit filling and new vectorized image decompression -- NOTE the SORTED files do not
;                       have output mask bit filling yet, it is handled after images are read -- the images should be recompressed after filling and stored
;                       removed "decompressimagemm.pro" as redundant code -- although I agree we should make these into subroutines -- it just wasn't complete enough yet,
;                       copied the T1 to T2 functionality from basescan_mo.pro into basescan.pro then removed basescan_mo.pro
; 1.1.76 TAF 2015-11-20 updated for uCOPP 25 microns per pixel, as well as 38.1 mm distance window to window (for correct sample volume calculation)
;                       modified concate_all_2ds_Mxice_txt_files.pro which was crashing during uCOPP processing because 1) it was not assigning
;                       outfile11 variable a value for 25 microns/pixel, and it was not closing file unit 11 before trying to open file unit 11
;                       Couple of notes on this:  File units should be managed using the get/free lun model, and error handling should be put in to ensure
;                       the file is opened properly.  Also, the logic structure (a set of independent if/endifs) used to define the filename outfile11,
;                       should be modified so that a value is always assigned, ie a default case.  Additionally, constants are embedded throughout this
;                       logic structure, which should be replaced with named variables.  Finally, the concate_all_2ds_Mxice_txt_files.pro does not follow
;                       IDL file/procedure naming convention and order, so it must be restructured or compiled -- it will not run using the standard IDL
;                       calling and compiling convention.
; 1.1.75 TAF 2015-11-20 fixed minor bug dealing with LastData not being defined for uCOPP processing
; 1.1.74 TAF 2015-11-19  - PRELIMINARY VERSION, NEEDS EXTENSIVE TESTING ON STANDARD TEST DATASETS
;                        - updating to deal with uCOPP data, or any data with only one channel.  Also put work around for file copy bug in plot_current_1Hz_ascii_file_data.pro line ~224,
;                          however this is NOT a fix to the incorrect file naming, just a work around.
;                        - removed non-IDL syntax extraneous semi-colons from end of lines, removed extraneous um_per_pixel which was being mixed with microns_per_pixel,
;                        - extracted some functions from being file-embedded and put them back into separate .pro files to follow IDL naming conventions, calling, and compiling standards.
;                        - some cleanup of repeated functions which should be in their own files,
;                        - moved the basescan.pro and other programs to main directory, removed basescan subdirectory
;                        - removed 2 extraneous prints from processparticle.pro, added slightly expedited single pixel particle handling (ie earlier return with defaults defined)


;1.1.77MO  2016-03-08    add png image page 200um scaler.

;1.1.76MO  2016-02-08    modify to do the same as 12dsview 202Mo to do liquid archiving same format as ice archiving.

;1.1.75MO  2016-01-28    addapt changes pf 2dsview 201Mo version to make calculation fast:  auto_clean*.pro,  calculate_L7, L8*.pro and the do_Nsecond.pro
;                        and the a2dsview_methodHzoom.pro  to change the png page output image filename  from  *H_RA.png to *H_yyyymmdd_hhmmss_R1.png.

; 1.1.74MO  2015-12-04   fixed inital plot of plot_unsorted**.pro if doing bacth processing.  xtick tick label used other figutres labels if bacth processing.

; 1.1.73MO  2015-10-20   changed to always generate new bin files if !use_external_TAS is set to 1.---swood requested.

; 1.1.72Mo  2015-10-06   Fixed a bug that PSD display on the screen is not the same as matlab got from the short_csv_cob.txt file. spectral_lib.pro changed.

; 1.1.71Mo  2015-10-01   Modified concat_all_small_Mxice.pro to generate matlab loadbale files too. make it usefull of 2ds and HVPS too. add more notes to !system_variables.
; 1.1.70 Mo 2015-0921    add no plottting for H-channnel for 2D128 and changed do_Nsecond_archiveHorV.pro and concate all smaller_files.
; 1.1.69Mo  2015-09-15 based on 168MO put back archive option for  (1) no-cleaning( keeps shattering , stuck bit, noise , taht is keep all particle), and
;                                                                  (2) clean noise only(that mean keep Shattered particle in the accepted group)
; by using  globale variabble !clean_option

; 1.1.68Mo  2015-07-15 based on 167MO modified so that plot_unsort*.pro's plots labels better and add summary plots at end of 1Hz archiving and remove
;non_work buttons and label the buttons better. as suggested by Chris roden. affected files a3v2dsview. @compile.pro, plot_current_1Hz.pro
; psd2d_mo.pro  do_Nsecond_ultimate*.pro. zoomchartevent.pro.

; 1.1.67Mo  2015-06-11 based on 166MO modified sio that It won't crash when less than 2M bytes no image Roi files are loaded , also "time is synced png page label'.
;;1.1.66Mo  2015-05-29 based on 165Mo modified all method of SV adjustment when particle are removed.  n-bin-counts are final remainging particle numbers.
;           clean up un-neccesary codes ( subroutines) that does not used any more.---codes are  concised.

;;1.1.65Mo  2015-05-26 based on 164Mo modified Do_Dof_calibrartion for CPI partile Glass bead calibrations, particles goes through by vacume sucker.
;           use all particle option. Correct Dcor  calculations and plot os the Dia and Dapp.

;1.1.64Mo  2015-05-14 based on 163Mo add batch process all base files and added M7 Sample volume correct when particle are removed
;          using all in M7 method---adjustment factor = Total/total_accepted.

;1.1.63Mo  2015-04-09 based on 162Mo.modified a3vcpiview_browse_image.pro  for enhanced feature.--the qui interface.

;1.1.62Mo  2015-03-18 based on 161Mo.fixing load_external_tas cancel button crash and two time poping up of loading the external; TAS data file and added new features of basescan.---Mo
;          the new featuer of basescan ac browse the images and do 1setp from making sorted file to finaly one Hz archive and png pages.

;1.1.61Mo  2015-02-03 based on 160Mo. modified a3vcpiview_3_view_widget_temp.pro because M6 was still using M2 with L4 rather than L5. Also changed if L1 gt 40 to if L1 gt 265um then L6 =L1.
;1.1.60Mo  2015-02-02 based on 159Mo. modified a3vcpiview_3_view_widget_temp to update 3viewer display
;1.1.59Mo  2015-01-28 based on 158Mo. added batch process png page with input criteria instead of default criteria.

;1.1.58Mo  2015-01-27 based on 157Mo. fixed the individual corected L7 size or infocu and size gt 40 pixle particles.
;1.1.57Mo  2015-01-27 based on 156Mo. merge with attrext !uexe_external_TAS  for correctinion teh size, area, icsp and update using L5 for alexei-size correction.

;1.1.56Mo  2015-1-27  based on 155Mo. added CPI habit criteria for making the png pages in Batch processing mode---do_overnight_works.pro subroutine change. Sarah wood need that.


;1.1.55Mo  2015-1-26  based on 154Mo. changed teh 3viewwr to 3viewer_temp  for temporay development use. ---old 3viewer is kept, just need replcae the subrountine to revert old bad good one.

;1.1.54Mo  2015-1-23  based on 153Mo. fixed bug when loading second roi file to make png page, it crashes. have to reset

;;1.1.53Mo  2015-1-23  based on 152Mo. fixed criteria with hole area not define bug.

;1.1.52Mo  2015-1-23  based on 151Mo. added 3viewreTemp for temporary use for developemnt to see whch method is teh best for mass calculation.


; 1.1.51Mo 2014-12-30  based on 150Mo. added h1 to h6 to criteria window for habit classification ---on process_time_lib.pro and timeperiodgui.pro.


; 1.1.50Mo 2014-12-30  based on 148Mo. correct a few labelings.

; 1.1.48Mo 2014-12-30  based on 147Mo. fixed 3viewer output file *.csv CPI time printout correctly.

; 1.1.47Mo  2014-12-30  based on 146Mo.added more displaydata and re-arranged the buttons, take off allpng file.

; 1.1.46Mo  2014-12-29  based on 145o. further debuging the crashes

; 1.1.45Mo  2014-12-22  based on 144Mo. modified 3viwer widegt_progam and fixed other crashing bugs in the gui for showing Tms and psd compare button.


; 1.1.44Mo  2014-12-09  based on 143Mo.  fixed add background not recognized when reload the new roi file.

; 1.1.43Mo  2014-12-04  based on 142Mo.  fix color display bug and added more features, add ascii outfile and saved mosaic image slides.
;                        add hourglass to let user know it is busy calculating.

; 1.1.42 Mo 2014-11-19  based on 141TAF, added CPI dof calibration subroutine for analyzing how focus values and size are changed when glass bead slides
;                       are moved of of depthe field.  a button was added in teh mainCPIview window, a few other subrotines are change by adding a few
;                       lines and the main subroutine added is DO_DOF_calibration.pro

; 1.1.41 TAF 2014-11-18 changed image rotation constant, added a system variable !IMAGE_ROTATION to calls to the rotate function.
;                       The file _zoomedHstrip apparently holds the calls to processparticle (with subsequent rotations) used for both
;                       displaying the H and V image strips, as well as the png image outputs.  I'm not sure what the other calls to processparticle
;                       are supposed to do, however their assocation rotations all now use !IMAGE_ROTATION
;                       Also, changed the HK H/V laser temperature calculation in read_hk.pro, seemed to be using equations for a different temperature sensor.

; 1.1.40 Mo 2014-11-11 change generate _zoomed-ascii from perimeter to perimeteru. base on 139Mo.
; 1.1.39 Mo 2014-11-11 just update  generate_zoomed_ascii.pro for particle by particle ascii out file.
;                      base on 138 TAF.
; 1.1.38 TAF 2014-10-14 csp was being calculated incorrectly, largest_satellite_area was already in units of microns^2, but was being multiplied by
;                    microns_per_pixel * microns_per_pixel again
; 1.1.37 TAF 2014-10-13 fixed !flag_3VCPI not being defined in processparticle.pro, by explicitely defining it in the system variables section of 2D-S and HVPS Views
;                       this was brought on by simulated files with no H channel and possibly with incorrect 0x7FFF flags in 2D-S data, but
;                       should be handled more robustly anyway, so this fix should make the code more stable to data errors
;                    This 0x7FFF in the 2D-S comes from running Playback 1.1.2 on Win7 -- different SORTED files are produced on WinXP.
; 1.1.36 TAF 2014-10-09 updated processparticle handling for single pixel case, number of satellites from 0 to 1, and largest_satellite_area set to 1
; 1.1.35 TAF 2014-10-08 modified processparticle to output largest_satellite_area, and to use largest_satellite_area in the csp calculation
;                       modified _DerivedDataRev2 to hold largest_satellite_area and zoomed H/V strips to display, replacing "areaold",
;                       which is an obsolete variable, with largest_satellite_area, so that the bin files will remain compatible
; 1.1.34 TAF 2014-10-03 updated the various open button icons (BASE, SORTED, BIN) to better illustrate functionality after basescan integration
;                       NOTE:  Not sure why the overlapping system variables aren't conflicting/crashing, will keep an eye on them during further testing
; 1.1.33 TAF 2014-10-03 initial integration with basescan_2.0.23, to convert base*.2DSCPI(HK) files into SORTED files
;                       this isn't a streamlined integration for batch processing, but just a way to call basescan directly from the View,
;                       and makes base->SORTED processing more consistent with SORTED->bin and bin->archive processing
;                       the new basescan button is between the CPI button and the SORTED->bin button
; 1.1.32 TAF 2014-10-03 renamed to reduce ambiguity
; 1.1.31 TAF 2014-10-02 added more array boundary checking in processparticle.pro
;                       set up to use universal processparticles.pro (same file as basescan, 3V-CPI View, 2D-S View, and HVPS View now all use)
; 1.1.31Mo   2014-09-27 ---added M7 methods ---all calculations---affective subbroutines.  zoomchartevent.pro. 3vcpiview_psd_methods.pro  **tms.method.pro
;                       --Do_Neseond*.pro do_overnight*.pro archive_ice_pop_up.pro  a3vcpiview.pro
; 1.1.30 TAF 2014-09-26 merged 1.1.29 TAF and 1.1.29 Mo, changes made to separate files
; 1.1.29 TAF 2014-09-19 moved processparticle to separate file, copied from basescan 2.0.20 -- already was separate file, and identical to functions in a3VCPIView.pro
;                       so this move makes the programs easier to maintain and cleaner,
;                       updates to 3 Viewer, with png and csv outputs, fixed error in CPI csp calculation
;                       resuming 1.1.27 debugging
; 1.1.29Mo 2014-09-24 base on 1.1.28TAF  just fix do_manaul_habit_change.pro. in teh past It crashed when no criteria is input and
;      only sph or ice is selected.
; 1.1.28 TAF 2014-09-19 removed some debugging stops from 1.1.27 for temporary internal release due to popular demand,
;                       still need to fix skipping some images and large pngs displaying as grayscale
; 1.1.27 TAF 2014-09-16 fixed png output bug of single grayscale image in process_time_lib2.pro
; 1.1.26 TAF 2014-09-15 finished update of scanspec.pro, tested on problem files
; 1.1.25 TAF 2014-09-09 ongoing testing of scanspec.pro, massive upgrades to scanspec, and reading HK and images
; 1.1.24 TAF 2014-09-05 minor coment and printout cleanup in "Do_overnight_works.pro", ongoing testing of scanspec.pro

; 1.1.23 TAF 2014-09-04 1.1.22 seems to get to the end of files, but skips a lot of data, trying to be more conservative in when to skip
;                       or when to advance the file pointer
; 1.1.22 TAF 2014-09-04 continued work on 1.1.21 work-around, including detection of image types 0xA1D5 and 0xA2D5, which are now being
;                       skipped, with only 0xA3D5 being processed, currently a beta release, testing on other data files
;                       gets to end of file successfuly on 20140821233145.roi, stopping around frame 1761 instead of ~400...

; 1.1.21 TAF 2014-09-02 added work-around in scanspec for ROIsCount > 1000 to skip the frame -- this is a temporary workaround
;                       for miss-parsed data, and the root cause should be deteremined and handled better,
;                       however, this prevents a crash as well as skipping time periods, so is a step forward
; 1.1.20 TAF 2014-08-28 process_features_lib.pro crashes on pROIImg empty -- program wasn't checking return value in call to read_frame
;            when searching for a background.  Also updated to use the system variable !CPI_MICRONS_PER_PIXEL instead of constants embedded throughout code
;            Fixed incorrect font name "ARIEL" to "ARIAL"
;            Fixed display of image mean, pixel value and location, for GUI display with and without obj files

; 1.1.19Mo   2014-08-26 based on 1.18Mo. Fixed bug of obj farme not found crash. extractable obj frame list usually is a lot less than all fraem list.
;            because, csv file, image classification are all based of extractable obj frame ( length, wid, focus, ect), if the processed data
;            are searched by the obj data, teh obj frame will be found, otherwise, it will be not found and program crashes.

; 1.1.18Mo   2014-08-14 added PSDs spectral and Time series plots under the generate tool. The original not implemented yet. added process_timeseries_3vcpi_stral.pro
;            process_N_timeseries_only.pro , modified mainevent.pro and a few other subroutines.  changed and improve the times series y-axes silder.--get rid of it.
;            use cw_field.
; 1.1.17 TAF 2014-07-01 cleaned up stops leftover from window pop fix, tested more thoroughly and made sure concatenate program ran
; 1.1.16 TAF 2014-06-30 fixed truncated first slice in process_particles image decompression
;                       also removed a call to wshow, wid which was causing the back window to pop to the front
; 1.1.15 TAF 2014-06-02 reset Wordcount variable to Unsigned Int before SORTED file reads -- was reading the wrong lengths (as signed Longs) from the file on occasion

; 1.1.14 Mo  2014-04-22 fix M6 archive not working. just chnage HorV in the calling Do_Nseconds**HorV.pro and Do_overnight_*.pro.
; 1.1.13 MO  2014-04-18 fix T1_T2_data in do_Nsecond_***HorV.pro  it crashes when M1 QC are make if there is bad data, T1_T2_data not defined.
; 1.1.12 Mo  2014-04-09 Just fixed the ice water separation to work.  don't use roundness 1, 2,3, use old technique.
;                       laso fix the .csb file time not correct---changed ascii_lib.pro  and shapetype_mo---not using roundness1.
; 1.1.11 TAF 2014-03-28 merging with 1.1.11 Mo

; 1.1.11MO  2014-02-25 clean up output_lib.pro to make it easier to handle, to modify in the future.
; 1.1.10Mo  2014-02-19 change file name png Datetimestring to be yyyymmddhhmmssxxx_R0.png   where xxx is miliseconds.

; 1.1.10 TAF 2014-03-28 added !enable_image_diagnostic to toggle building an array of raw image data and a stop, for examining background data,
;                       removed an old HASP check from left_event.pro
;                       changed the behavior so that pixel values are always shown in CPI View mouse over image, even without an obj file
;                       2DS time popups are also disabled
; 1.1.9 TAF 2014-02-27 BETA, merged 1.1.5 TAF and 1.1.9 Mo, added functionallity to the 3 Viewer, added !disable_time_popups to CPI do remove the various time
;                      popup windows so that the program can be run without constant interruption -- a better solution to this problem must be found,
;                      as the functionality behind the time popups is needed for archiving
;                      NOTE: the time popups in the 2D-S are not being handled and may have stops in them, 1.1.9 TAF is therefore BETA
; 1.1.9 Mo  2014-02-19 Change "ariel" to "Arial" for the png output font size and name.
; 1.1.8 Mo  2014-02-19 modified to fixed some pngs has smaller fonts. becasue in seac4rs only first place in output_lib.pro are changed aerial 11. other places are not changed.
; 1.1.7 Mo  2014-02-18 modified to fixed un-supported image type "0"  in Guam.  Changed read_frame, obj_lib.pro, process_timelib2.pro etc.
; 1.1.6 Mo  2014-02-07 modified output_lib.pro to label the CPI roi image's time label  to be 05:06:19.563 instead of 5: 6:19:563--Chris requested
; 1.1.5 TAF 2013-11-19 evaluating STREAKER, deadtime fixes, image slice display missing last (first) slice???
; 1.1.5 Mo  2013-12-27 modified Do_overnight_work.pro of CPI part to zero pageDX pageDy to fix batch-Roi_process Crash, it only do one file before.
;                      also modified Do_Nseconds_ultimate_archiveHorV.pro of 2DS part.
; 1.1.4 TAF 2013-10-28 modified Do_overnight_auto_archiving_works_tc4.pro to prevent repeat png images
; 1.1.3 TAF 2013-10-21 verifying [H]V_DEADTIME_HEADER format and handling, checking image problem with Mask handling in PB 1.1.18

; 1.1.2 TAF 2013-10-09 added masking on edges/F1 variable for proper edge flag detection (to zoomchartevents.pro routines)
;                      disabled setting edges flag in process particles
; 1.1.1 TAF 2013-10-08 added stub handling for new H/V_DEADTIME_HEADER packets (not fully implemented, but the files can be read)
;                      removed a stop from line 2710 in a3VCPIView_psd_methods.pro
;                      removed a stop from process particles associated with "image with no satellites detected (empty image)"
;                      removed a stop from line 5027 in a3VCPIView.pro testing images -- Playback 1.1.18 seems to have image problems,
;                      and these stops were preventing analysis of the data files
;                      sent this version to CW with Playback 1.1.18 for testing/fixing Playback issues
; 1.1.0 TAF 2013-10-02 automated 10 vs 50 micron H channel selection for 3V-CPI, Hawkeye 10 and 50 micron probes
;                      3V-CPI/Hawkeye can be built with two 10 micron channels, or one 10 micron (on V) and one 50 micron (on H)
;                      Please note that this is still somewhat kludgey, and is NOT a general solution to arbitrarily assigning either
;                      channel to either size.  Mo made major contributions to being able to change sizes on the H channel.

;                      Added some new state variables for handling 50 vs 10 micron, using automatic detection (Playback 1.1.17 and later) vs
;                      manual specification.  If 3V-CPI View does not detect the 50 vs 10 micron flag, it will pop up a message to the user
;                      explaining that they need to use the manual 50 vs 10 micron flag.

;                      The 50 vs 10 micron flags will be generated in SORTED files produced by Playback after 1.1.17,
;                      as well as the binary files produced after 3V-CPI View 1.1.0.
;                      SORTED or binary files before these versions will pop up the manual selection window for the user to decide.

; 1.0.105 TAF   2013-10-01 updated 1.0.104 processparticle calls to standard, checked pointers/memory leaks, fixed error in Auto_clean_oneParticle_by_oneParticle_mos.pro update from 1.0.104
; 1.0.104 TAF   2013-09-27
;               - major revision.  Combined 101 TAF, 95 TAF, and 101 Mo, 95 Mo
;               serious steps backward with pointers, csps, etc being dropped in interval
;               revisions during SEAC4RS -- 104 TAF has brought back several months of
;               improvements that are necessary

; 1.0.101 TAF - removed "Fulltiks" printout from a3vcpiview_psd_methods.pro
;             - removed 50 micron stops from SORTED file archive multi ascii -- added printouts to make them easier to find again
;               with the plan to finish automating 50 vs 10 micron detection in 3V-CPI View -- search for "put in 50" in a3vcpiview.pro to find
; 1.0.100 TAF - fixed automatic CPI particle criteria in processParticles.pro
;             - based on 1.0.99MO, which had Ariel font size 11 selected for png pages
;             - changed Overnight to BatchROI once again -- this keeps getting reverted when Mo uses an old version of his instead of a newer version as a basis
;             - added !Add_Classification_to_pngs to toggle the classification on the png images to Do_Overnight_Works.pro
; 1.0.97Mo. ; Base or 1.0.96Mo.  Just chnage the font szie from 8 points to 10 points  for CPI png page image Time labels.--as of 2013-09-05.
; 1.0.96Mo : version95Mo has a error in processtimeseries_lib2.pro  that eliminate all images when making teh png pages.  TeD chased out that
;          ; it is a line Start_K and END_K that caused the problem, where start_K and eND_K is from plink.Times.  Reverted to 1.0.91vesrion
;          ; that will print out all index of the times from 0 to nnnn-1LL images.  Chaneg on this day of 2013-08-23  by Qixu Mo

; 1.0.95 TAF merged 1.0.94 with 1.0.95 Mo, added support for autodecting 50/10 micron H channel
;        based on the flag in the SORTED file

; 1.0.95Mo ; This Version Merges 50um and 10um Channel together, base on 94Mo and also  using L5 instead of L4 for M4, M2 and M6'sM2 method.
; 1.094Mo  update  to use L5 instead of L4 for M4, M2 and M6'sM2 methods.  Started from version 1.0.93Mo.

; 1.0.91 Update to output 1Hz file of overlaod deadtime, total_deadTime, Sample,volume, adjfactor, TAS, H_particle, rejected, accepted--MO.
; 1.0.89 updated 3Viewer,
;        changed CPI Scanspec to handled larger number of roi's after crash on GV RF04 2012 data -- but this looks like a hacked region of code
;        to incorrectly lengthen the data arrays and bail arbitrarily if they're "too big" ie over 4000000 elements -- need to clean this up correctly
; 1.0.87 improvements in 3 viewer, incremental backup
; 1.0.84-86 work on a3vcpiview_3_view_widget which can now display CPI and 2DS images and scroll through them based on particle ID,
;           some minor fixes to the CPI time display (hex words displayed in wrong order, no leading zeros in time fields)
;           Added line length in pixel and microns to the Cross Section plot
; 1.0.83 cleaned up debug printouts, attempted to catch end-of-archive hang bug
; 1.0.82 updated the get_spec_ticks to get_spec_ticks2, to use the same particle for starttime and endtime, instead of two different calls to
;        get_spec_ticks which could use different base particles and mess up the sample volume calculations
; 1.0.80-81 upgraded SORTED format to Playback 1.1.7, Get_spec_ticks needs to be updated with correct equations which need HK1 HK2 time and ticks
; 1.0.79 incremental release with Playback 1.1.3, added change to detect and ignore corrupted HK_TXT lines (sometimes last line is incomplete and would crash batch processing, OK now)
; 1.0 78 updated ProbeCounter calculation to 64 bit:  OD.ProbeCounter = ishft(Long64(ProbeCount2), 32) or Long64(ProbeCount)
; 1.0.77 removed calls to correct_HV_particles_nanotimes, which were screwing up the particle times.
;        still have problem with the sample volume and concentration in the archive files.
; 1.0.76 implemented getangle, which had been left unimplemented and hung on Win7 execution
; 1.0.75 removed stop in SORTED file reads left over from previous version, cleaned up comments in that section as well
; 1.0.74 work around for GV hang ups in process particles, prints "slice size exceeded error, truncating slice"
; 1.0 73 minor incrimental
; 1.0.72 fixed several batch processing bugs in 2D-S and CPI, cleaned up needless printouts in CPI, handled out of memory error in CPI
; 1.0.71 fixed bug in png page generation
; 1.0.68 added perimeter images to zoomed H and V strips, fixed program so that .sav files can be run with IDL Virtual Machine again
; 1.0.66 2012-11-09 updated with 1.0.65 Mo, new Roundness 1-3 for edge, dual edge particles, temporary workaround for crash bug in decompress in Process Particles dll,

; 1.0.61 2012-08-24 added Mo's improved CPI manual image selection
; 1.0.58 2012-07-24 Updated to Roundness1-3, added Roundness criteria to boolean, fixed copy/paste in boolean
;  - CPI can now make png pages without timestamps on the images
;  - many other minor bug fixes/updates,

; version 1.0.48, added Roundness criteria and modified dialogs to 2D-S processing, removed trailing ; from lines

; version 1.0.47, merged with version42Mo

;version42mo:    modified on 20120710---to fix  manualfeature  changing habit so that it can be saved and changed.
;                (1)   left_event.pro      change   roitime=(frameheader.time ;  +  (ArrivalT1, T2,  version)/1000.0;-----line   348.
;                (2) change obj_lib.pro       line 271.  same as above.
;                          Line 451     objscanList[frame].time=frameheader.time ;    (T1,T2, version)/1000.0
;                                                          In  Function scanOBJ,LUN,roiversion_num
;                (3)  process_feature_lib.pro      line 109 ------ (*(*pStateInfo).pObjList)[frame].time = float(FrameData.frameheader.time) ;+
;                (4)  process_time_lib2.pro ;  259 lines
;                          objTime = float(objData.frameHeader.time); + calcStrobeTime(objData.***** + *****)/1000.0D
;                          roiTime = float(framedata.frameheader.time); + calcStrobeTime(framedata.frameheader.arrivalT1,****Ver)/1000.0D

; version 1.0.42 merged Mo 1.0.39 with Ted version 1.0.40 2012-06-13

;version39  is 1.033Mo and 1.038  Ted version merged.---as of  May 26. 2012.

;version 1.0.32:  (1)  Mo added  catch onError to to while loop when reading the sorted ---so that it bails out when reads too much before End of File.
;                 (2)  added if n_elements(HKTAS) eq 0 then return becaus esome HK_txt*.2dsCPi file don't have TAs data--so No Tas is corrected.
;                      These are added on April 24th 2012.
;                 (3)  added  !DT_CPI in a3vcpiview.pro  for Time syncing CPI images with other time and also added total_seconds in PrintROIs.pro of outputlib.pro.
;version 73: added sample volume to the ice_archive file and added new w8_t and rejected type to zoomed_ascii particles file.
;Version 72; added output_png_criteria.pro --for saudi-duncan---affetcted a3vcpiview pngdroplist--lines about 5844
;, do_overnight_rico_archive_or liq and ice if !auto-archive=1, or rico and work_TC4,on august 26, 2009.
; affected are also: get_allindex_8******Clean.pro in zoomevent.pro.
;Version 71;  added !fiter=1 for autoarchive which multiply adj0 to archived data.
;            April. 22, 2009. added Free_lun,22 to avoid logical units overflow.
;version 71  added no_clean_archive, with WS shattering for both liq and ice multiple archiving methods buttons.
;            fixed teh big is Hchannel auto_clean  end_ticks0 to ticcks(4999) if part2_conts gt 5000.

;            April. 22, 2009. added Free_lun,22 to avoid logical units overflow.
;Veriosn 70--March,23, 2009. Chris changed DLL file so that images leading edges aND TAIL SWITCH ON DISPLAY.  ROTATE(XX,1,) TO ROTATE(XX,4).
;Dec 31, 2008: Modify do_a_new_clean to keep the reject=2 non-monotonic particles if a new cleaning is started.
;Dec 23, 2008  Fixed the intial zoom taking too long a time to wait for the additional zoom to be ready to start.
;Oct 17, 2008: added code to calculaet-_w8t_part that  if the tcn1 < tcp2 then re-assigne the times of the hump.
;                   the report the special case to mo and brad part.
;version 68  Sept. 3, 2008:  added manually adjust the time of ICE_L lear corrupt data.  In the same time, change the ouput displays
;            and made the code more robust to handle some special cases--like IAn's 2ds_128 mode ect. change plot_unsorted_ into two plot_unsort.
;Version 66  Aug. 11, 2008:  added M6 to the zoom window TMS plots.  also cleanup the codes so that it more concise--some subroutines are deleted.
;            affected programs are: zoomchartevent.pro, get_one_tms_trace_300bin.pro, createzoom.pro, draw_tms_hires_mo.pro get_m6_tms_area_adjs.pro
;            a3vcpiview_tms_method.pro

;            July 22:  fixed  data data1 data 2 multi-archive multi_png problem for brayn use.--sorted files are in different places.
;                      also fixed teh h-channel has no images or just one image wthat will crash problem.--add first few up to max png per second.
;version65   July 1, 2008:    get_alexei_correct_size so that when a Area/Area_max ratio is not exactly as the alexei table
;            when interplate the enlargement factor instead of using the closest one in the table as the enlargement factors.
;             Change the ST (stereo particle search program to out stereo-pair's time, pc1, L1, L2 and focus values.
;             use deltaT=4 ticks and Lpercent=0.5

;version 64, May16, 2008.  cmp_psd M1M2 binc debugged because F1 ne  0. cmp_tms Nbins if not=300 debugged. added cmps_labeling methods.
;            Chand the cleaning method for big splase, bad_pc1 removal, and w8_t remove to keep more images.
;            Rb corrected by a factor of 100. change cmp_tms output to include times.

;Version63   March 4,2008:  add Radar values for Fairbank projects. added lable_all_saw_tooth_events. moved multi_ice pop_up
;            before hand. add free_all,


;Version 62:  modify adding area -200, -201 ect while reading the sorted file  and 1Hz time series and Remove_non-monotonic
;             TAS sample volume correction, while archive if time is wrong, put 9.99E+30 to archive file, ect.
;             add HK_TAS_factor data to common block.January 29,2008
;             add keyword precip, shatter to make ignore_has_big and input factor for auto_clean.
;                         !ignore_has_big_eq_1, !w8_t_shatter_factor on Feb. 15, 2008.

;version 61   fixed chris 1Hz tiem series and psds. also comment out tick and time correction ect. Jan3,2008.
;version 60.  Add TAS correction for L1 sizes and use that for TC4 M9 method archiving--nov. 2007
;             Which need to read in an additional data file for Time and TAS values.
;             to compare it with the each particles TAS and then determine L9 correct or not.

;version 0.59a modified accDTTick to include total_DeadTime accummulations. accDTTick=DeadOver+L1_dead.

; version 0.59  added accDTtick Deadtime calculation improvement for speed. in if contaVe1, contAve2 ****
; The speed increase twice. oct 10, 2009.

;version 57B2  oct 8.2007.  add accDT, noisetype, w8_t_clean to the data for fast processing
;  like that of version 58.

;Oct3-2007  Talkwith Paul, and Bryan, Change M4 method to make no-Alexei-coorections for size greater than 400um.

;vER57b  CHANEG TO CLEANALL, CLEAN_NOISE AND 4 ARCHIVE BUTTONS---AUGUST--20-2007
;****
; version 57a  added save only first 50 images of each second. 0-----on july 11, 2007.
; auto archive  M6 method data only.  use this version because it is faster and tested good. compare to version 58.

;Vesrion 57  Modified in May 10, 2007  to change w8_t cleaning algorithm.   using threshold=(0.05,0.2 )*Averaged_W8t  and has_big_particle
;do eliminate particles of w8t less than threshold. Don't use a fixed wait-time threshold anymore.
;also added adj0 calculation so that later all adjs will be multiplied by adj0 to account for w8t clean effect.

;Modified in May 2 2007 to put three buttons for Namma Noclean, clean and clean_but_keep_splashes
;clean_but_keep_splashes is simply clean_with_comments_out_the w8t lines and splashes lines
;of the namma_H, namma_V subroutine in the auto_clean_***mo.pro
;affected subrotuin is auto_clean***mo.pro and the a3vcpiview.pro.


;Modified on April-23,2007 to fixed 1-pixel, 2-pixel 3,4-pixel size statistical cleaning part.
;Only affect the auto_clean_one_particlebyparticle.pro of the Namma_H Namma_V cleaning subroutine.


;modified on Feb 28,  flip back to old definition of datatype 9236, and 9252 and added -109 -110
;for the backup type dat area. Changed corresponding places in the a3vcpiview_psd_method.pro and zoomchartevent.pro
;where -105 and bad2, copied to make -109, -110 add DT_bad3.

;vesrion 0.56 Fen 7, 2007  added lastH and LastV =0, -103, -106, -108 to make decision for wipping out the last
;                           non-monotonic time stamps. in the makeing of Hbinfiles when reading the sorted file.

;version 0.55 Jan 24 2007.  take away the  sort-rountine and change it to label the non-montonic particles as warning
;                           particles.  It does not make sensor to sorted teh particles and shovel them around.
;                           if a particle's tick is larger than it's next one, I will change its  time to be the
;  same as its earlier one particle and assign it L1 L2,L4, L7, L8 Area=0 and reject it.
;  Following two rountine will be modified:
;  correct_HV_particles_nanotimes;  assigned warn=warn+1000; to time can not recovered particles--add remove_HV_non_monontic
;  sort_HV_particles_and_assign_warning1_values;---this is for ver9662.
;  Change roundeness calculation to round=L1>0.49L5 and (L5 >0.49L1 or L5 >50pixels).

;version 0.54  Jan 16, 2007  added back the wipe off the non-monotonic particles----for playback vers 996.

;version 0.53 Jan 5, 2007 added button clean4. so that Namma can be done by a combination of clean4 and clean2.

;Version 0.52 Dec 10 2006  Fix M6 using DOF of infocus coeff also fixed teh read sorted part fox verion 992.


;Version 51  Nov 28 2006  For Playback version 94-991 with new *.DLL . Chris removed those backup fake
;                         particles from the sorted data file and skip processing the L1>128 pixel
;                         particles. H and V Tas is separatedly readed from the sorted file now.
;                         Implemented M6 for Psd and RICO archive files and added streakers.

;Version 50   Nov22 2006  This verions Chris fixed the placyback so that Deadtime and Mask times
;                         are in pairs. We simply sort, calculate w8_t. To implement new irregular
;                         Particle count.  (1) after sort, if  the sorted index are not increased by
;                         1 compared with earlier particle, this particle will get a warning values
;                          equals 1.  (2) if a particles w8_t is less that 0, or equal to 1. This
;                         particle will be rejected. its wait time will be assigned as zero.
;                          and with warnig values eq 2. (3) in any particle
;            whose area ne 0, and they get put in between a start Marke and end-mark, tehy get reject
;                            and get a warning values=3. they w8_t will be assigned as 0 also.
;            This version never used by anyone execept Mo using it to develop code and testing.
;Version 49,  Nov3-2006   This version is to used to process Playback version 92 which include rollover counter 2
;                         and that fixed many other things, hope to resolve all timing issues. Note that the acceptable
;                         Tas Limits are specified on teh playback.  please see chris e-mail on Nov 3, 2006 on the
;                         release of version 92 playback.

;Version 48,  09-25-2006 added auto-clean3 and lots of stuff and sorted the data by ticks, and
;                         recalculat the nanoseconds, add warning
;                         1, 10, 100 value sto particles and reject value=2 to irregualre particles.
;                         nodified the a3vcpiview to make it take any format of sorted file name.
;                         and modified how Tiem is recalculated---using same start_nano, start_ticks in both H and V.
;                         on Oct., 2006. added the four dummy H-deadtime  for playback version 91.
;Version 0.47 08-14-2006  added auto clean1 autoclean2 methods and re-do teh calculation of sample
;                         volume to include all particle's L1's time as deadtimes. so if there is a
;                         particles in the time period, there will always be deadtimes. But not
;                         neccessary of overflow times.  Re-assign zero Tas values to the all fake particles
;                         according to the House Keeping time Tas.

;version 0.45 06-27-2006  added option of using the infocus particle only, added the auto-clean-one_particle-by one particle
;                         methods.


;version 0.44 06-01 2006  added code to re-assign tas values for bad tas values in the data.
;                         added code to add additional deadtime and sample volume for rejected particles.
;                         fixed the bug of that some time alldataH(index0).w8_t=w8_t is not working right.
;                         added auto cleaning programs as discussed with brad. That works good for rico data.
;version 0.43 05-16 2006  Program can not handle Rico data file of half or one hour long---out of memory problem
;                         Modified the program so that it won't use too much memory to run.


;version 0.43  05-05 2006  added L7, L8, W8_t, reject to zoomed ascii file
;                          modified do_2ds_fishing.pro for quality check with brad and modified zoomevent.pro
;-------------------------------------
;Version  0.42 04-20 2006  Added House keeping Plots and CRAVE, RICO Archiving Procedures in the Codes.
;                          Also added several Parts  for  automatic Cleaning the noisy and modified
;                          and add more subroutines fro cleaning.
;                          added the generating the png pages of the images for archiving.
;-------------------------------------
; Version 0.38 01-26 2006. started to add the label spurious particles to give an option to do clean and noisy
;                          data anlysis.  affected is a3vcpiview.pro zoomzhart.pro and creatzoom.pro
;                          to add buttons and selection choises.
;                          also added other widgets to show particle's pixel location distribution
;                          and showing the images at a particular pixel location.
;                          added action log file and and boolean logic for user to label the some
;                          particles that statisfy the criteria as bad images.
; Version 0.37 01-19_2006.  added the fast calculation of 6 tms trace and 8 psd traces and fixed some bugs.
;             January 5, note;  Multiply 1.8  to L3 values in all programs which includes these
;             a3vcpiview.pro a3vcpiview_zoomedHstrip.pro; *a3vcpiview_zoomedVstrip.pro
;             generate_ascii.pro, generate_zoomed_ascii.pro; zoomchartevents.pro.
;             These needs to be undone in the future after chris fix the problem.
; version 0.36 01-15-06  added stereo-pair-image shows
; version 0.35 12-29-05  Added a lot more improvement: able to see H aand V strip charts at the same time.
;                        added Boolean logic; added New 5 methods M1 M2, M3, M4 M5, and event time series plot.
;                        added Housekeeping plots. added able to printer the plots to printer and save them as
;                        ascii and png files. --mo
; Version 0.34 11-21-05  Fixed error in generate_1h_file that crashes when a 1Hz bin has no particles due to elimination based on bad TAS values
; Version 0.33 11-14-05  Variety of changes from Mo including accepting wider range of TAS. Fix to 1Hz big for accumulating dead time
; Version 0.32 10-6-05   More new code from Mo and integrate changes to drawaveragechart_specifyrange to add both Mo and Chris code changes together
; Version 0.31 10-4-05   New code from Mo and Brad
; Version 0.30 9-28-05   Multiple changes
; Version 0.29 9-10-05    Minor fix for reformatting of image array for Alexei processing
; Version 0.28 9-8-05     Added L3,L4,L5,A3,PC3x,y,PC4x,y parameters, added Alexei parameters, added stereo pairs code from Mo
; Version 0.27 8-17-05     Made initial zoom plots all created from 1Hz file, updated
; Version 0.26 7-8-05     Made initial zoom plots all created from 1Hz file, updated
; drawaveragechart_specifyrange.pro, drawtimeseries.pro, and zoomchartevents.pro
; with improvements from Mo
; Version 0.25 6-28-05    Modified for creation of 1Hz file creation and dropdown menus options in zoom
; Version 0.24            Minor bug fixes
; Version 0.23 5-25-05    Added 9 new modules, and updated drawaveragechart_specifyrange.pro all from Mo
; Version 0.22 5-15-05    Moved button events that are used in the zooming code (drawaveragechart_specifyrange.pro)
;                           from a3vcpiview.pro into the above routine for independent coding by Mo
; Version 0.21 05-12-05   Added two button on zoom window and set up code to provide arrays for new routine for timeshift testing
; Version 0.20 05-06-05   Fixed problem with IWC plotting where lines were missing, Fixed display problem for the
;                          "mouse-over" display values shown in the lower left corner of the time series display.
; Version 0.19 03-11-05   Fixed swapped Slices and WordCount for particle strip display
; Version 0.18 03-05-05   Fixed swapped Slices and WordCount variables and relabel zoomed time series charts
; Version 0.17 02-18-05   Added handling for zero Slice or Wordcount particles read in from SORTED data file
; Version 0.16 02-17-05
; Version 0.15 02-02-05
; Version 0.14 01-26-05
; Version 0.12 01-11-05
; Version 0.11 01-08-05
; Version 0.10 01-06-05
; Version 0.09 01-03-05
; Version 0.08 12-30-04
; Version 0.07 12-27-04
; Version 0.06 12-20-04
; Version 0.05 12-15-04
;-------------------------------------
; a3vcpiview.pro
; 7/24/04
; C.Wieland

; Program to process SORTEDyymmddhhmmss.2DS files that are the sorted data output files from the 2DS data collection program.
; This program reads SORTEDyymmddhhmmss.2DS files, sorts the data, and creates the following files:
;  HORIZyymmddhhmmss.bin
;  vert.bin
;  onehz.bin
;  mask.bin

; The files are created in a directory directly beneath the location of the selected data file.

; These newly created horiz.bin and vert.bin files contain a single structure of data for each particle
;  This structure if of the format


; struct = {_DerivedData, $
;             AllMonths:0 , $
;             NanoSeconds:0ULL , $
;             ImageOffset: 0UL , $
;             ImageWords: 0 , $
;             Area: 0L , $
;             L1: 0 , $
;             L2: 0 , $
;             F1: 0 , $
;             Adj1: 0.0, $
;             Pc1: 0.0, $
;             TAS: 0.0 }



;***********************************************************************

pro ZoomChartHiRes_event, event

    Common  DataBlock,AllDataH,AllDataV,ADataH,ADataV,FirstData,LastData,StripData,FirstDataV,$
        LastDataV,StripDataV,OneHzH,OneHzV,avestripdata,avestripdataV,SHstripdata,SVstripdata,$
        Mstripdata
    ; Toggle HI_LO_STATE to match the radio button setting
    if (event.select eq 1) then begin
        !HI_LO_STATE = 0
    endif else begin
        !HI_LO_STATE = 1
    endelse

    ;==========================================================
    ; The following code permits updates whenever the user changes
    ;  the radio button for the hi/lo res setting. It was disabled in version 0.30
    ;==========================================================
    ;Widget_Control, event.top, Get_UValue=ave_info, /No_Copy  ; get our info structure
    ;if (!HI_LO_STATE eq 1) then begin  ;1=hires
    ;  DrawTimeSeries, ave_info ; must be called after call to DrawAverageChart_SpecifyRange as this routine fills in the start/end indexes of ave_info
    ;endif else begin
    ;     ;----------------------------------------------------------------------
    ;     ; Now that we've got the user requested timestamps lets round those
    ;     ; timestamps down at the start, and up at the end so that we start
    ;     ;  and stop at exactly the "top" of the second. This permits us to
    ;     ;  pull out the 1Hz data cleanly..
    ;     ;----------------------------------------------------------------------
    ;     NanosecondsPerSecond = 1000000000.0D  ; use double for some calculations
    ;     NPS = 1000000000ULL
    ;     ave_info.StartNanoSeconds = ave_info.StartNanoSeconds - (ave_info.StartNanoSeconds mod NPS)  ; Round down to time with milli,micro, and nano seconds = 0

    ;     temp = ave_info.EndNanoSeconds - (ave_info.EndNanoSeconds mod NPS)      ; Round down to time with milli, micro and nano seconds = 0
    ;     ;  If the upper limit happened to already be at exactly one second, don't bother to round up to next second
    ;     if (temp NE ave_info.EndNanoSeconds) then temp = temp + NPS  ; Round up to next full second
    ;     ave_info.EndNanoSeconds = temp
    ;     OneHzStartBin = (ave_info.StartNanoSeconds - FirstData.Nanoseconds)/NPS
    ;     OneHzEndBin = (OneHzStartBin + (ave_info.EndNanoSeconds - ave_info.StartNanoSeconds)/NPS)

    ;     num = OneHzEndBin - OneHzStartBin ;  + 1   ; do +1 to include thru the end of the last bin

    ;     DrawTimeSeries_1Hz_Zoom, info, ave_info,OneHzStartBin, OneHzEndBin
    ;endelse
    Widget_Control, event.top, Set_UValue=ave_info, /No_Copy  ; release our info structure
end
;*****************************************************************************
pro Bitmap_Playback_Speed_event, event

    if event.index eq 0 then !PLAYBACK_SPEED = 0.1
    if event.index eq 1 then !PLAYBACK_SPEED = 0.25
    if event.index eq 2 then !PLAYBACK_SPEED = 0.50
    if event.index eq 3 then !PLAYBACK_SPEED = 1.0

end
;*****************************************************************************
pro play_forward_timer_event, event

    forward_function  FillStrip

    IF (TAG_NAMES(event, /STRUCTURE_NAME) EQ 'WIDGET_TIMER') THEN BEGIN
        if (!PLAYBACK_ACTIVE eq 0) then begin



            Widget_Control, event.top, Get_UValue=strip_info, /No_Copy  ; get our info structure

            foffset = !FOFFSET
            pre_offset = foffset
            newoffset = FillStrip(foffset, strip_info,1,'H')
            ; print, "pre,post",pre_offset,newoffset
            if pre_offset EQ foffset then begin
                WIDGET_CONTROL ,strip_info.stripLabel ,SET_VALUE= "END OF FILE REACHED"
                !PLAYBACK_ACTIVE = 1
                widget_control, event.ID, SET_VALUE = !SPEC_3VCPI_DIR + '/FF.bmp', /BITMAP

                WIDGET_CONTROL, strip_info.SSR_button, SENSITIVE=1
                WIDGET_CONTROL, strip_info.SSF_button, SENSITIVE=1
                WIDGET_CONTROL, strip_info.FR_button, SENSITIVE=1

            end else begin
                WIDGET_CONTROL ,strip_info.stripLabel ,SET_VALUE= ""
                WIDGET_CONTROL, event.ID, TIMER= !PLAYBACK_SPEED
            endelse


            Widget_Control, event.top, Set_UValue=strip_info, /No_Copy  ; release our info structure

        endif
    endif else begin
        Widget_Control, event.top, Get_UValue=strip_info, /No_Copy  ; get our info structure
        if (!PLAYBACK_ACTIVE eq 1) then begin
            !PLAYBACK_ACTIVE = 0
            widget_control, event.ID, SET_VALUE = !SPEC_3VCPI_DIR + '/FFon.bmp', /BITMAP
            WIDGET_CONTROL, event.ID, TIMER= !PLAYBACK_SPEED

            WIDGET_CONTROL, strip_info.SSR_button, SENSITIVE=0
            WIDGET_CONTROL, strip_info.SSF_button, SENSITIVE=0
            WIDGET_CONTROL, strip_info.FR_button, SENSITIVE=0


            foffset = !FOFFSET
            pre_offset = foffset
            newoffset = FillStrip(foffset, strip_info,1,'H')
            ; print, "pre,post",pre_offset,newoffset
            if pre_offset EQ foffset then begin
                WIDGET_CONTROL ,strip_info.stripLabel ,SET_VALUE= "END OF FILE REACHED"
            end else begin
                WIDGET_CONTROL ,strip_info.stripLabel ,SET_VALUE= ""
            endelse




        endif else begin
            !PLAYBACK_ACTIVE = 1
            widget_control, event.ID, SET_VALUE = !SPEC_3VCPI_DIR + '/FF.bmp', /BITMAP

            WIDGET_CONTROL, strip_info.SSR_button, SENSITIVE=1
            WIDGET_CONTROL, strip_info.SSF_button, SENSITIVE=1
            WIDGET_CONTROL, strip_info.FR_button, SENSITIVE=1
        endelse
        Widget_Control, event.top, Set_UValue=strip_info, /No_Copy  ; release our info structure
    endelse
end
;*****************************************************************************
pro play_reverse_timer_event, event

    ;forward_function  FillStrip

    IF (TAG_NAMES(event, /STRUCTURE_NAME) EQ 'WIDGET_TIMER') THEN BEGIN
        if (!REWIND_ACTIVE eq 0) then begin


            Widget_Control, event.top, Get_UValue=strip_info, /No_Copy  ; get our info structure

            foffset = !FOFFSET
            newoffset = FillStrip(foffset, strip_info,-1,'H')

            if !FOFFSET EQ 0UL then begin
                WIDGET_CONTROL ,strip_info.stripLabel ,SET_VALUE= "BEGINNING OF FILE REACHED"
                !REWIND_ACTIVE = 1
                widget_control, event.ID, SET_VALUE = !SPEC_3VCPI_DIR + '/RR.bmp', /BITMAP
                WIDGET_CONTROL, strip_info.SSR_button, SENSITIVE=1
                WIDGET_CONTROL, strip_info.SSF_button, SENSITIVE=1
                WIDGET_CONTROL, strip_info.FF_button, SENSITIVE=1

            end else begin
                WIDGET_CONTROL ,strip_info.stripLabel ,SET_VALUE= ""
                WIDGET_CONTROL, event.ID, TIMER= !PLAYBACK_SPEED
            endelse



            Widget_Control, event.top, Set_UValue=strip_info, /No_Copy  ; release our info structure

        endif
    endif else begin
        Widget_Control, event.top, Get_UValue=strip_info, /No_Copy  ; get our info structure
        if (!REWIND_ACTIVE eq 1) then begin
            !REWIND_ACTIVE = 0
            widget_control, event.ID, SET_VALUE = !SPEC_3VCPI_DIR + '/RRon.bmp', /BITMAP
            WIDGET_CONTROL, event.ID, TIMER= !PLAYBACK_SPEED

            WIDGET_CONTROL, strip_info.SSR_button, SENSITIVE=0
            WIDGET_CONTROL, strip_info.SSF_button, SENSITIVE=0
            WIDGET_CONTROL, strip_info.FF_button, SENSITIVE=0

            foffset = !FOFFSET
            newoffset = FillStrip(foffset, strip_info,-1,'H')

            if !FOFFSET EQ 0UL then begin
                WIDGET_CONTROL ,strip_info.stripLabel ,SET_VALUE= "BEGINNING OF FILE REACHED"
            end else begin
                WIDGET_CONTROL ,strip_info.stripLabel ,SET_VALUE= ""
            endelse




        endif else begin
            !REWIND_ACTIVE = 1
            widget_control, event.ID, SET_VALUE = !SPEC_3VCPI_DIR + '/RR.bmp', /BITMAP
            WIDGET_CONTROL, strip_info.SSR_button, SENSITIVE=1
            WIDGET_CONTROL, strip_info.SSF_button, SENSITIVE=1
            WIDGET_CONTROL, strip_info.FF_button, SENSITIVE=1

        endelse
        Widget_Control, event.top, Set_UValue=strip_info, /No_Copy  ; release our info structure
    endelse
end

;*****************************************************************************

pro ZoomChartUpdate_event, event

    Common  DataBlock,alldataH,alldataV,ADataH,ADataV,FirstData,LastData,StripData,FirstDataV,$
        LastDataV,StripDataV,OneHzH,OneHzV,avestripdata,avestripdataV,SHstripdata,SVstripdata,$
        Mstripdata


    Widget_Control, event.top, Get_UValue=ave_info, /No_Copy  ; get our info structure


    Widget_Control, ave_info.IDAveXMin, GET_VALUE = val
    ave_info.AveXMin = fix(val)

    val = Widget_Info(ave_info.IDAveXLog, /button_set)
    ave_info.AveXLog= byte(val)

    val = Widget_Info(ave_info.IDAveYLog, /button_set)
    ave_info.AveYLog= byte(val)

    Widget_Control, ave_info.IDAveXMax, GET_VALUE = val
    ave_info.AveXMax = fix(val)


    Widget_Control, ave_info.IDAveMinBinSize, GET_VALUE = val
    ave_info.AveMinBinSize = fix(val)

    Widget_Control, ave_info.IDAveMinCountPerBin, GET_VALUE = val
    ave_info.AveMinCountPerBin = fix(val)

    x = 0
    ts = ({_time_struct})
    Widget_Control, ave_info.StartHoursID, GET_VALUE = x
    ts.Hours = x
    Widget_Control, ave_info.StartMinutesID, GET_VALUE = x
    ts.Minutes = x
    Widget_Control, ave_info.StartSecondsID, GET_VALUE = x
    ts.Seconds = x
    Widget_Control, ave_info.StartMillisecondsID, GET_VALUE = x
    ts.Milliseconds= x
    ts.Microseconds = 0
    ts.Nanoseconds = 0


    z = SPEC_HMS2Nano_struct(ns, ts)
    ave_info.StartNanoSeconds = ns

    ts = ({_time_struct})
    Widget_Control, ave_info.EndHoursID, GET_VALUE = x
    ts.Hours = x
    Widget_Control, ave_info.EndMinutesID, GET_VALUE = x
    ts.Minutes = x
    Widget_Control, ave_info.EndSecondsID, GET_VALUE = x
    ts.Seconds = x
    Widget_Control, ave_info.EndMillisecondsID, GET_VALUE = x
    ts.Milliseconds= x
    ts.Microseconds = 0
    ts.Nanoseconds = 0


    z = SPEC_HMS2Nano_struct(ns, ts)
    ave_info.EndNanoSeconds = ns

    if (ave_info.EndNanoSeconds le ave_info.StartNanoSeconds) then begin
        r = dialog_message(['Cannot Update!','Start Time later than End Time.','Please correct Start/End values.'],TITLE = 'Update Error')
    endif else begin
        NPS = 1000000000ULL
        range_in_seconds =  ( ave_info.EndNanoSeconds-ave_info.StartNanoSeconds)/NPS
        if (!HI_LO_STATE eq 0) and (range_in_seconds lt 2) then begin ; 0=lo-res
            r = dialog_message(['Cannot Update!','Selected time range too small (< 2 secs)for lo-res display.', 'Please increase time range or switch to hi-res mode.'],TITLE = 'Update Error')
        endif else begin
            ;*********************************************************************************************
            ;added the boolean logic and calculte the deadtime and deadvolume just one time for the period.
            ;*********************************************************************************************
            Get_allindexs_and_boolean_subsetindexs,ave_info,Hallindex,Hsubindex,Hfactor,Vallindex,VsubIndex,Vfactor

            starttime=ave_info.startnanoseconds
            endtime  =ave_info.endnanoseconds

            Get_JustDeadtime_between_time1_time2,'H',starttime,endtime,Fullticks,SampleVolume,ThisTAS,ovldcnts,$
                StartTicks,EndTicks,DT_mask,DT_ovfl,DT_L1,DT_bad1,DT_bad2,DT_bad3,total_AccDeadtime
            ave_info.Hdeadtime=total_accdeadtime;HDT_ovfl+HDT_mask+HDT_L1+HDT_bad1+HDT_bad2; Hdeadtimemo
            ave_info.HDT_ovfl=DT_ovfl
            ave_info.HDT_mask=DT_mask; seconds_per_tick
            ave_info.HDT_L1=DT_L1;*seconds_per_tick
            ave_info.HliveV=sampleVolume
            ave_info.HDT_bad1=DT_bad1;*seconds_per_tick
            ave_info.HDT_bad2=DT_bad2;*seconds_per_tick


            Get_JustDeadtime_between_time1_time2,'H',starttime,endtime,Fullticks,SampleVolume,ThisTAS,ovldcnts,$
                StartTicks,EndTicks,DT_mask,DT_ovfl,DT_L1,DT_bad1,DT_bad2,DT_bad3,total_AccDeadtime

            ave_info.Vdeadtime=total_AccDeadtime; VDT_ovfl+VDT_mask+VDT_L1+VDT_bad1+VDT_bad2;Vdeadtimemo
            ave_info.VDT_ovfl=DT_ovfl;*seconds_per_tick
            ave_info.VDT_mask=DT_mask;*seconds_per_tick
            ave_info.VDT_L1=DT_L1;*seconds_per_tick
            ave_info.VliveV=SampleVolume
            ave_info.VDT_bad1=DT_bad1;*seconds_per_tick
            ave_info.VDT_bad2=DT_bad2;*seconds_per_tick

            ;*********************************************************************************************
            ;*********************************************************************************************

            if (!CURRENT_CHANNEL EQ !H_CHART) then  DrawAverageChart_SpecifyRange, ave_info, !ZOOM_START_INDEX_H, !ZOOM_END_INDEX_H
            if (!CURRENT_CHANNEL EQ !V_CHART) then  DrawAverageChart_SpecifyRange, ave_info, !ZOOM_START_INDEX_V, !ZOOM_END_INDEX_V

            if (!HI_LO_STATE eq 0) then begin ; 1=lo-res   if lo res, then round start/stop time to exactly 1 second divisions
                ;----------------------------------------------------
                ; Now see if the zoom should produce a 1Hz time series display
                ;  or if the user zoomed in on a very short period and we can
                ;  go straight to the hi-res display
                ;------------------------------------------------------------
                ;----------------------------------------------------------------------
                ; Now that we've got the user requested timestamps lets round those
                ; timestamps down at the start, and up at the end so that we start
                ;  and stop at exactly the "top" of the second. This permits us to
                ;  pull out the 1Hz data cleanly..
                ;----------------------------------------------------------------------
                NanosecondsPerSecond = 1000000000.0D  ; use double for some calculations

                ave_info.StartNanoSeconds = ave_info.StartNanoSeconds - (ave_info.StartNanoSeconds mod NPS)  ; Round down to time with milli,micro, and nano seconds = 0

                temp = ave_info.EndNanoSeconds - (ave_info.EndNanoSeconds mod NPS)      ; Round down to time with milli, micro and nano seconds = 0
                ;  If the upper limit happened to already be at exactly one second, don't bother to round up to next second
                if (temp NE ave_info.EndNanoSeconds) then temp = temp + NPS  ; Round up to next full second
                ave_info.EndNanoSeconds = temp
                OneHzStartBin = (ave_info.StartNanoSeconds - FirstData.Nanoseconds)/NPS
                OneHzEndBin = (OneHzStartBin + (ave_info.EndNanoSeconds - ave_info.StartNanoSeconds)/NPS)

                num = OneHzEndBin - OneHzStartBin ;  + 1   ; do +1 to include thru the end of the last bin
                DrawTimeSeries_1Hz_Zoom, info, ave_info,OneHzStartBin, OneHzEndBin
            endif else begin
                DrawTimeSeries_hires_mo, ave_info,'M1';   ;plot the default L1 time series.
                ;DrawTimeSeries, ave_info ; must be called after call to DrawAverageChart_SpecifyRange as this routine fills in the start/end indexes of ave_info
            endelse
        endelse

    endelse

    Widget_Control, event.top, Set_UValue=ave_info, /No_Copy  ; release our info structure
end
;***************************************************************
; Execute when the "N" button is pressed on the main toolbar
;-------------------------------------------------------------
pro InitiateZoom, info, start_index, end_index, OneHzStartBin, OneHzEndBin
    Common  DataBlock,alldataH,alldataV

    forward_function DrawTimeSeries_1Hz_Zoom, SPEC_Nano2HMS_struct
    ; TAF changing for .sav

    if not widget_info(info.ZoomWindowID, /valid_id) then begin
        ;   widget_control, info.ZoomWindowID, /destroy
        ;if info.ZoomWindowID eq 0 then begin



        IDAveMinBinSize = 0L
        IDAveMinCountPerBin = 0L
        IDAveXMin = 0L
        IDAveXMax = 0L
        IDAveXLog = 0L
        IDAveYLog = 0L

        AveMinBinSize = 10L
        AveMinCountPerBin = 100L
        AveXMin = 10L
        AveXMax = 1000L
        AveXLog = 1b
        AveYLog = 1b
        ;creat a sturcture to hold the criteria data.
        criteria={text:'L1',$
            offID:1,$
            onID:0,$
            variables:ptr_new(),$
            vDlistID:0,$
            operators:ptr_new(),$
            oDlistID:0,$
            baccept1ID:0,$
            baccept2ID:0,$
            criteriaID:0,$
            on:0,$
            Hfactor:1.0,$
            Vfactor:1.0,$
            label:'Criteria=None Applied'}

        ave_info = { IDAveXMin:0L, $
            IDAveXMax:0L, $
            IDAveMinBinSize:0L, $
            IDAveMinCountPerBin:0L, $
            IDAveXLog: 0L, $
            IDAveYLog: 0L, $
            FirstIndex: 0L, $
            LastIndex: 0L, $
            ZoomWindowID: 0L, $
            ZoomWindowDrawID: 0L, $
            ZoomWindowSeriesDrawID: 0L, $
            AveMinBinSize:AveMinBinSize, $
            AveMinCountPerBin:AveMinCountPerBin, $
            AveXMin:AveXMin, $
            AveXMax:AveXMax , $
            AveXLog:AveXLog, $
            AveYLog:AveYLog, $
            StartHoursID: 0L, $
            StartMinutesID: 0L, $
            StartSecondsID: 0L, $
            StartMillisecondsID: 0L, $
            EndHoursID: 0L, $
            EndMinutesID: 0L, $
            EndSecondsID: 0L, $
            EndMillisecondsID: 0L ,$
            StartNanoseconds: 0ULL, $
            EndNanoseconds: 0ULL, $
            StartTimeString: '', $
            EndTimeString: '', $
            BaseFilenameString: '', $
            NumDataPointsH: '',  $
            NumDataPointsV: '' , $
            H_First_Index: 0L , $
            H_Last_Index: 0L , $
            V_First_Index: 0L, $
            V_Last_Index: 0L , $
            statusID: 0L,  $
            DropdownFunction1: 0, $  ; current setting for the top dropdown list on the zoom window
            DropdownFunction2: 0, $ ; current setting for the middledropdown list on the zoom window
            DropdownFunction3: 0, $ ; current setting for the bottom dropdown list on the zoom window
            binfilename:' ',$ ;              add this part for psd data on june 2, 2005
            plot_string:'plot_string',$
            delta_s:'delta_s',$
            Hpartcont:'Hpartcont',$
            Vpartcont:'Vpartcont',$
            pxpts:ptr_new(),$
            Hbinc:ptr_new(),$
            Hconc   :ptr_new(),$
            Hextn   :ptr_new(),$
            Hiwc   :ptr_new(),$;;   for ice.
            HLwc   :ptr_new(),$;;  ;for liquid
            Vbinc:ptr_new(),$
            Vconc  :ptr_new(),$
            Vextn   :ptr_new(),$
            Viwc  :ptr_new(),$
            VLwc   :ptr_new(),$   ; for liquid
            Hdeadtime:0.0,$  ;   added on Nov 15, 2005
            HDT_ovfl:0.0,$      ;overflow_deadtime for -100 -101 markers
            HDT_mask:0.0,$     ; mask deadtime for -102 -103 markers
            HDT_L1:0.0,$       ; L1 deadtime of all images.
            HliveV:0.0,$     ;livetime volume.
            Vdeadtime:0.0,$
            VDT_ovfl:0.0,$
            VDT_Mask:0.0,$
            VDT_L1:0.0,$
            VliveV:0.0,$
            HDT_bad1:0.0,$  ; H and V bad type-1 and bad type-2 particle deadtimes.
            HDT_bad2:0.0,$
            VDT_bad1:0.0,$
            VDT_bad2:0.0,$
            Hallindex:ptr_new(),$
            Hsubindex:ptr_new(),$
            Vallindex:ptr_new(),$
            Vsubindex:ptr_new(),$
            Hsv:ptr_new(),$
            Vsv:ptr_new(),$; to hold the 300 bin SE_volume values once a start_time endtime period is set.
            criteria:criteria, $; added this on Nov 8 for boolean logic
            filter:0, $  ;added on january 26 to filter spurious particles.
            filterID:0 $
            }

        CreateZoom, info, ave_info


    endif else begin

        Widget_Control, info.ZoomWindowID, Get_UValue=ave_info

        Widget_Control, ave_info.IDAveXMin, GET_VALUE = val
        ave_info.AveXMin = fix(val)

        val = Widget_Info(ave_info.IDAveXLog, /button_set)
        ave_info.AveXLog= byte(val)

        val = Widget_Info(ave_info.IDAveYLog, /button_set)
        ave_info.AveYLog= byte(val)

        Widget_Control, ave_info.IDAveXMax, GET_VALUE = val
        ave_info.AveXMax = fix(val)


        Widget_Control, ave_info.IDAveMinBinSize, GET_VALUE = val
        ave_info.AveMinBinSize = fix(val)

        Widget_Control, ave_info.IDAveMinCountPerBin, GET_VALUE = val
        ave_info.AveMinCountPerBin = fix(val)

        ; Setting the Start/EndNanoSeconds to zero will trigger a refilling of the
        ;  HH:MM:SS:mm boxes when we call drawaveragechart_specifyrange()
        ave_info.StartNanoSeconds = 0ULL
        ave_info.EndNanoSeconds = 0ULL
        ;stop
    endelse




    ;----------------------------------------------------
    ; Now see if the zoom should produce a 1Hz time series display
    ;  or if the user zoomed in on a very short period and we can
    ;  go straight to the hi-res display
    ;------------------------------------------------------------
    num = OneHzEndBin - OneHzStartBin + 1   ; do +1 to include thru the end of the last bin
    DrawTimeSeries_1Hz_Zoom, info, ave_info,OneHzStartBin, OneHzEndBin

    ; add following on Dec 22, 2008.
    ave_info.StartNanoSeconds = !ZOOM_START_NS
    ave_info.EndNanoSeconds = !ZOOM_END_NS

    !zoom_start_string = SPEC_Nano2HMS_struct(!ZOOM_START_NS,starting_time_struct)
    !ZOOM_END_STRING =  SPEC_Nano2HMS_struct(!ZOOM_END_NS, ending_time_struct)

    Widget_Control, ave_info.StartHoursID, SET_VALUE = string(starting_time_struct.Hours)
    Widget_Control, ave_info.StartMinutesID, SET_VALUE = string(starting_time_struct.Minutes)
    Widget_Control, ave_info.StartSecondsID, SET_VALUE = string(starting_time_struct.Seconds)
    Widget_Control, ave_info.StartMillisecondsID, SET_VALUE = string(starting_time_struct.Milliseconds)

    ;a = SPEC_Nano2HMS_struct((*alldataH)[last_point].Nanoseconds, ending_time_struct)
    Widget_Control, ave_info.EndHoursID, SET_VALUE = string(ending_time_struct.Hours)
    Widget_Control, ave_info.EndMinutesID, SET_VALUE = string(ending_time_struct.Minutes)
    Widget_Control, ave_info.EndSecondsID, SET_VALUE = string(ending_time_struct.Seconds)
    Widget_Control, ave_info.EndMillisecondsID, SET_VALUE = string(ending_time_struct.Milliseconds)

    ;************************** finishi adding all aobve lines, DEc22, 2008.


    ;mo nov_15_2005 here ave_info.startnanoseconds is not defined yet.

    ; DrawAverageChart_SpecifyRange, ave_info, start_index, end_index;  This line is comment out on Dec. 22, 2008.

    ;DrawTimeSeries, ave_info ; must be called after call to DrawAverageChart_SpecifyRange as this routine fills in the start/end indexes of ave_info
    ; mo nov_15_2005 here ave_info.startnanoseconds is defined now.

    ;**********************************************************************
    ave_info.Hsubindex=ptr_new(where((*alldataH).nanoseconds ge !ZOOM_START_NS and (*alldataH).nanoseconds lt !ZOOM_END_NS))
    ave_info.Vsubindex=ptr_new(where((*alldataV).nanoseconds ge !ZOOM_START_NS and (*alldataV).nanoseconds lt !ZOOM_END_NS))
    ave_info.HdeadTime=0.0
    ave_info.VdeadTime=0.0
    ave_info.HliveV=(!ZOOM_END_NS-!ZOOM_START_NS)*10.0/1000000000.0;  assume 10Litere/per_seconds
    ave_info.VliveV=(!ZOOM_END_NS-!ZOOM_START_NS)*10.0/1000000000.0
    Draw_Default_averaged_psd,ave_info,'M1'
    ;**** ******************************************************************
    Widget_Control, info.ZoomWindowID, Set_UValue=ave_info
    Widget_Control, info.ZoomWindowDrawID, Set_UValue=ave_info

end



;***************************************************************


FUNCTION RemoveBadValues, results,data

    FOR i=1,n_elements(results)-1 DO BEGIN
        newTAS = data(results(i)).TAS
        IF (newTAS LT 10.0 or newTAS GT 300.0) THEN BEGIN
            ; remove this index as it's a bad TAS value
            a = results[0:i-1]
            a = [a, results[i+1:n_elements(results)-1]]
            results = a
            return, 1
        END


    ENDFOR
    return, 0
end

;***************************************************************
FUNCTION SPEC_TestForFile, fname

    result = 0L
    OPENR, lun, fname, /Get_Lun, ERROR = err
    IF err NE 0 then begin
        result = -1
    END ELSE BEGIN
        close, lun
        Free_Lun,lun
    ENDELSE
    return, result
end
;***************************************************************

PRO GetTasChanges, data,starting,ending,results

    ; Need to verify that data(starting).TAS is a valid TAS value!!!
    results_TAS = [data(starting).TAS]
    results = [0]  ; first TAS is always at location 0
    count = 1L
    currentTAS = starting

    WHILE (count GT 0) DO BEGIN
        a = where(data[currentTAS:ending-1].TAS NE data(currentTAS).TAS,count) ;find next instance of TAS change
        IF count GT 0 then BEGIN
            currentTAS += a(0) ; record the new TAS index
            results = [results,currentTAS] ; add the index of this change to our results array
            results_TAS = [results_TAS,data(currentTAS).TAS]
            ; if currentTAS GT 0 then    print, currentTAS,data(currentTAS-1).TAS,data(currentTAS).TAS
        ENDIF
    ENDWHILE


    ; remove the index values where bad TAS changes occur, we'll assume that we keep the same
    ;  TAS until a new "good" index tells us to change the TAS
    removed = 1
    WHILE removed EQ 1 DO BEGIN
        removed = RemoveBadValues (results,data)
    ENDWHILE



end

;***************************************************************

pro AveChartUpdate_event, event
    Widget_Control, event.top, Get_UValue=ave_info, /No_Copy  ; get our info structure
    print, "Updating Average Chart"

    Widget_Control, ave_info.IDAveXMin, GET_VALUE = val
    ave_info.AveXMin = fix(val)

    val = Widget_Info(ave_info.IDAveXLog, /button_set)
    ave_info.AveXLog= byte(val)

    val = Widget_Info(ave_info.IDAveYLog, /button_set)
    ave_info.AveYLog= byte(val)

    Widget_Control, ave_info.IDAveXMax, GET_VALUE = val
    ave_info.AveXMax = fix(val)


    Widget_Control, ave_info.IDAveMinBinSize, GET_VALUE = val
    ave_info.AveMinBinSize = fix(val)

    Widget_Control, ave_info.IDAveMinCountPerBin, GET_VALUE = val
    ave_info.AveMinCountPerBin = fix(val)

    SPEC_DrawAverageChart, ave_info

    Widget_Control, event.top, Set_UValue=ave_info, /No_Copy  ; release our info structure
end

;*****************************************************
;*****************************************************

pro ave_events, event

    Widget_Control, event.top, Get_UValue=ave_info, /No_Copy  ; get our info structure

    theEvent = Tag_Names(event, /Structure_Name)

    if theEvent EQ 'WIDGET_BASE' then Begin
        ;----------- Handle resize event
        Widget_Control, event.top, Get_UValue=info, /No_Copy  ; get our info structure
        !CHAR_SIZE = (event.x / 300.0) * 0.3
        Widget_Control, ave_info.aveID, Draw_XSize=event.x, Draw_YSize=event.y

        ;//Widget_Control, ave_info.chart2controlpos, /UPDATE, XSIZE = 295  ; doesn't seem to work, need another solution

        SPEC_DrawAverageChart, ave_info

    end


    Widget_Control, event.top, Set_UValue=ave_info, /No_Copy  ; release our info structure
end



function SPEC_Nano2HMS, in_ns
    ; accepts ULL values of nanoseconds
    ; returns string of HH:MM:SS.mmm.mmm.nnn

    ns = in_ns

    NanosecondsPerMicro  = 1000ULL
    NanosecondsPerMilli  = 1000000ULL
    NanosecondsPerSecond = 1000000000ULL
    NanosecondsPerMinute = NanosecondsPerSecond * 60ULL
    NanosecondsPerHour   = NanosecondsPerMinute * 60ULL

    hours   = ULONG64(ns/NanosecondsPerHour)
    ns = ns - (hours   * NanosecondsPerHour)

    minutes = ULONG64(ns/NanosecondsPerMinute)
    ns = ns - (minutes * NanosecondsPerMinute)

    seconds = ULONG64(ns/NanosecondsPerSecond)
    ns = ns - (seconds * NanosecondsPerSecond)

    milli   = ULONG64(ns/NanosecondsPerMilli)
    ns = ns - (milli   * NanosecondsPerMilli)

    micro   = ULONG64(ns/NanosecondsPerMicro)
    ns = ns - (micro   * NanosecondsPerMicro)

    if(hours gt 23) then begin
        print, "SPEC_Nano2HMS: hours out of range ", hours, in_ns
        hours = 99

    endif

    if(minutes gt 59) then begin
        print, "SPEC_Nano2HMS: minutes out of range ", minutes, in_ns
        minutes = 99
    endif

    a = string(format = '(%"%2.2d:%2.2d:%2.2d.%3.3d.%3.3d.%3.3d")',hours,minutes,seconds,milli,micro,ns)

    if(strmatch(a, '*\**')) then begin
        ; TAF this check has been precluded by setting the hours to 99 as a flag,
        ; hopefully fixing the reads will prevent any of this from happening
        print, "nanosecond input out of range in SPEC_Nano2HMS, replacing * with X for file naming"
        ;stop

        strreplace, a, '*', 'X'
    endif

    return, a
end



;function SPEC_Nano2HMS, ns
; accepts ULL values of nanoseconds
; returns string of HH:MM:SS.mmm.mmm.nnn

; orig_ns = ns

; NanosecondsPerMicro  = 1000ULL
; NanosecondsPerMilli  = 1000000ULL
; NanosecondsPerSecond = 1000000000ULL
; NanosecondsPerMinute = NanosecondsPerSecond * 60ULL
; NanosecondsPerHour   = NanosecondsPerMinute * 60ULL

; hours=ns/NanosecondsPerHour
; if hours GT 0 then  ns = ns - (hours * NanosecondsPerHour)
; minutes=ns/NanosecondsPerMinute
; if minutes GT 0 then  ns = ns - (minutes * NanosecondsPerMinute)
; seconds=ns/NanosecondsPerSecond
; if seconds GT 0 then  ns = ns - (seconds * NanosecondsPerSecond)
; milli=ns/NanosecondsPerMilli
; if milli GT 0 then  ns = ns - (milli * NanosecondsPerMilli)
; micro=ns/NanosecondsPerMicro
; if micro GT 0 then  ns = ns - (micro * NanosecondsPerMicro)
; a = string(format = '(%"%2.2d:%2.2d:%2.2d.%3.3d.%3.3d.%3.3d")',hours,minutes,seconds,milli,micro,ns)

; ns = orig_ns

; return,a
;end




;*****************************************************
;*****************************************************
function  SPEC_Nano2HMS_struct, ns, time_struct
    ; accepts ULL values of nanoseconds
    ; returns string of HH:MM:SS.mmm.mmm.nnn

    ts  = { _time_struct, $
        Hours: 0, $
        Minutes: 0, $
        Seconds: 0, $
        Milliseconds: 0, $
        Microseconds: 0, $
        Nanoseconds: 0 }

    time_struct = ({_time_struct})

    NanosecondsPerMicro  = 1000ULL
    NanosecondsPerMilli  = 1000000ULL
    NanosecondsPerSecond = 1000000000ULL
    NanosecondsPerMinute = NanosecondsPerSecond * 60ULL
    NanosecondsPerHour   = NanosecondsPerMinute * 60ULL

    hours=ns/NanosecondsPerHour
    if hours GT 0 then  ns = ns - (hours * NanosecondsPerHour)
    minutes=ns/NanosecondsPerMinute
    if minutes GT 0 then  ns = ns - (minutes * NanosecondsPerMinute)
    seconds=ns/NanosecondsPerSecond
    if seconds GT 0 then  ns = ns - (seconds * NanosecondsPerSecond)
    milli=ns/NanosecondsPerMilli
    if milli GT 0 then  ns = ns - (milli * NanosecondsPerMilli)
    micro=ns/NanosecondsPerMicro
    if micro GT 0 then  ns = ns - (micro * NanosecondsPerMicro)

    time_struct.Hours = hours
    time_struct.Minutes = minutes
    time_struct.Seconds = seconds
    time_struct.Milliseconds = milli
    time_struct.Microseconds = micro
    time_struct.Nanoseconds = ns

    a = string(format = '(%"%2.2d:%2.2d:%2.2d.%3.3d.%3.3d.%3.3d")',hours,minutes,seconds,milli,micro,ns)

    return,a


end


;*****************************************************
;*****************************************************

function FillStrip, foffset, info, direction,HorV

    Common  DataBlock,alldataH,alldataV,ADataH,ADataV,FirstData,LastData,StripData,FirstDataV,$
        LastDataV,StripDataV,OneHzH,OneHzV,avestripdata,avestripdataV,SHstripdata,SVstripdata,$
        Mstripdata

    main_fname = ""
    ;if (!CURRENT_CHANNEL EQ !H_CHART) THEN  main_fname = !currentHFname
    ;if (!CURRENT_CHANNEL EQ !V_CHART) THEN  main_fname = !currentVFname
    if (HorV EQ 'H') THEN  main_fname = !currentHFname
    if (HorV EQ 'V') THEN  main_fname = !currentVFname
    OpenR, lun, main_fname, /Get_Lun

    ; Now we must figure out how many particles are in the opened file that
    ;  occur after our selected fileoffset location. We want to avoid reading past the
    ;  end of the file.
    fdata = FSTAT(lun)
    if (!CURRENT_VERSION eq 1.0) then    StructSize = N_TAGS( {_DerivedData} , /DATA_LENGTH )
    if (!CURRENT_VERSION eq 3.0) then    StructSize = N_TAGS( {_DerivedDataRev2} , /DATA_LENGTH )


    ;   foffset *= StructSize ; with the 1Hz file, we now receive foffset as the structure index
    ;                      ; into the (*alldataH)/V files (i.e. the Horiznnn.Bin file's contents)
    ;                      ; rather than the byte offset, so here we convert from the structure index to the byte offset


    ; if  direction=1, then scan forward, if -1, then backward
    ;-------------------------------------------------------------
    ; Handle Rewind
    ;  Reposition the foffset from the current position so that
    ;  we fill the strip with the last particle as the one currently
    ;  pointed to.
    ;-------------------------------------------------------------

    if (direction EQ -1) then begin
        yoffset = 0
        current_index = foffset/StructSize  ; generate index of which structure we're pointing at so we can move backwards
        ;if (!CURRENT_VERSION eq 1.0) then    OneD = ({_DerivedData})
        if (!CURRENT_VERSION eq 3.0) then    OneD = ({_DerivedDataRev2})

        ;if (!CURRENT_VERSION eq 1.0) then POINT_LUN, lun, foffset
        if (!CURRENT_VERSION eq 3.0) then POINT_LUN, lun, foffset + 256 ; include the header

        ReadU, lun, OneD
        display = 0

        WHILE (display EQ 0) DO BEGIN



            if (foffset EQ 0) then begin
                ; We've reached the start of the file, display starting at 0
                ;foffset = 0
                ; Go do normal "forward" display with foffset=0
                display = 1
            end else begin
                foffset -= StructSize          ; point back one structure
                ;if (!CURRENT_VERSION eq 1.0) then POINT_LUN, lun, foffset
                if (!CURRENT_VERSION eq 3.0) then POINT_LUN, lun, foffset + 256 ; include the header
                ReadU, lun, OneD
                if ((yoffset + OneD.L1n + 1) GT !STRIP_WIDTH) then begin
                    foffset += StructSize   ; we went back one too many for the contents of the strip, move forward one and display
                    display = 1
                end else begin
                    yoffset +=  (OneD.L1n + 1)
                endelse
            endelse

        ENDWHILE
        reverse_foffset = foffset  ; return this value when doing a reverse display
    end

    ;----------------------------------------------------------------

    number_left = ((fdata.Size-256) - foffset)/StructSize
    if number_left LT 0 then begin
        ; foffset if past the end of the file, return and don't do anything
        return, foffset
    end

    ;if (!CURRENT_VERSION eq 1.0) then POINT_LUN, lun, foffset
    if (!CURRENT_VERSION eq 3.0) then POINT_LUN, lun, foffset + 256 ; include the header

    if (number_left GT 500) then number_left = 500; there is 1000 pixels in strip. can display a max of 500 particles.

    ;if (!CURRENT_VERSION eq 1.0) then OD = REPLICATE({_DerivedData},number_left)
    if (!CURRENT_VERSION eq 3.0) then OD = REPLICATE({_DerivedDataRev2},number_left)



    ReadU, lun, OD
    Close ,lun
    Free_Lun,lun

    substrsF=strsplit(!currentHFname,'\',/extract)
    up_pathnm=strjoin(substrsF(0:n_elements(substrsF)-3),'\')+'\'
    short_filename=substrsF(n_elements(substrsF)-1)
    datestamp = strmid(Short_filename,5,strlen(Short_filename)-9)
    ;fname =up_pathnm+ 'SORTED' + datestamp+ '.2DS'
    fname =up_pathnm+ 'SORTED' + datestamp+ '.2DSCPI'

    OpenR, lun, fname, /Get_Lun

    ; draw the graph in the Strip Widget
    if HorV eq 'H' then begin
        Widget_Control, info.stripID, Get_Value=wid
    endif
    if HorV eq 'V' then begin
        Widget_Control, info.VstripID, Get_Value=wid
    endif
    WSet, wid

    erase,color='ffffff'x  ; get background to white

    ; Initialize our variables for the display
    yoffset = 0
    firstnano = 0ULL
    lastnano = 0ULL
    strip_index = 0U


    FOR i=0U,number_left-1 DO BEGIN  ; the number_left is not neccessary equal to n_elements(stripdata(*)).
        if (OD(i).Nslice GT 0) then begin  ;------------------ ; at begining yoffset=0, as it goes, yoffset increases.
            if firstnano eq 0ULL then firstnano = OD(i).Nanoseconds
            if ((yoffset + OD(i).Nslice + 1) GT !STRIP_WIDTH) then begin  ;util yoffset reached teh stripwidth.
                lastnano = OD(i).Nanoseconds
                val = i     ; so I can stop filling if OD(i).nanosecond is greater than ave_info.endnansseconds.
                foffset += (val * StructSize)
                i = uint(number_left + 100)
                !FOFFSET = foffset

            end else begin ; only fill up to the top of the strip

                WordCount = OD(i).ImageWords
                Slices = OD(i).Nslice

                Image_data = BytArr(WordCount * 2)

                POINT_LUN, lun, OD(i).ImageOffset
                ReadU, lun, Image_data

                ;out_array = bytarr(16L * Slices)           ; 128 bits = 16 bytes * 8 bits
                out_byte_array = BytArr(128L, Slices)

                ;****************************************************
                ;Catch, onError
                ;IF onError NE 0 THEN BEGIN
                ; Catch, /Cancel
                ; result = Error_Message(Traceback=1)
                ;stop
                ;Hsums=0
                ;ENDIF

                ;Hsums=0

                ;make a color display of the images
                ;******************************************************

                HSums = IntArr(Slices)
                HMax  = 0
                Area = 0UL
                Midpoint = 0
                Edges = 0
                L3 = 0.0
                L4 = 0
                L5 = 0
                PC3x = 0.0
                PC3y = 0.0
                PC4x = 0.0
                PC4y = 0.0
                A3 = 0.0
                Roundness1 = 0.0
                Roundness2 = 0.0
                Roundness3 = 0.0
                L1r = 0.0
                maxLength = 0.0
                maxWidth  = 0.0
                perimeter = 0UL
                perimeterum = 0.
                csp = 0.
                num_satellites = 0.
                largest_satellite_area = 0.


                Err=0

                nstop = 1

                ;ProcessParticle, Image_data, out_array, out_byte_array, Wordcount,Slices, HSums, HMax, Area, Midpoint, Edges, Err, $
                ;                 L3,L4,L5,PC3x,PC3y,PC4x,PC4y,A3, Roundness1, Roundness2, Roundness3, L1r, maxLength, maxWidth, perimeter, crystal_type, perimeterum, csp

                !mask_bit_filling = 0 ; disable for slight speedup, SORTED files already have the mask bits filled...

                ProcessParticle, Image_data, out_byte_array, Wordcount,Slices, HSums, HMax, Area, Midpoint, Edges,Err , $
                    L3,L4,L5,PC3x,PC3y,PC4x,PC4y,A3, Roundness1, Roundness2, Roundness3, L1r, maxLength, maxWidth, perimeter, crystal_type, perimeterum, csp, $
                    ml_p1x=ml_p1x, ml_p1y=ml_p1y, ml_p2x=ml_p2x, ml_p2y=ml_p2y, mw_p1x=mw_p1x, mw_p1y=mw_p1y, mw_p2x=mw_p2x, mw_p2y=mw_p2y, $
                    perimeter_points_x=perimeter_points_x, perimeter_points_y=perimeter_points_y, num_satellites=num_satellites, HorV=HorV, largest_satellite_area=largest_satellite_area


                ;****************************************************
                ; Catch, onError
                ;IF onError NE 0 THEN BEGIN
                ;Catch, /Cancel
                ;result = Error_Message(Traceback=1)
                ;print, ''
                ;print, 'error caught after out_byte_array swap, after ProcessParticle call in a3vcpiview'
                ; print, ''
                ;stop
                ;Hsums=0
                ;ENDIF

                tv, ROTATE(out_byte_array, !IMAGE_ROTATION), yoffset, 0

                ;    clip = TVRD(0,yoffset,OD(i).Imagewords,128)
                ;    print, clip


                ; Save the pertinent data for each tick on the graph for later display
                FOR z=yoffset, (yoffset + OD(i).Nslice+1) DO BEGIN
                    if (z LT !STRIP_WIDTH) then  StripData(z) = OD(i)
                ENDFOR
                yoffset += OD(i).Nslice + 1

            endelse

        end  ;-------------------------------------------
    ENDFOR
    ;*************************************************************************************
    ;added by mo to asigned stripdataV which is updated every time this fucntion is called.
    if HorV eq 'V' then stripdataV=stripdata; else stripdataH=stripdata
    ;*************************************************************************************

    if (i EQ number_left) then lastnano = OD(number_left-1).Nanoseconds
    Close ,lun
    Free_Lun,lun

    channel_name = ""
    ;if (!CURRENT_CHANNEL EQ !H_CHART) THEN channel_name = 'Horizontal'
    ;if (!CURRENT_CHANNEL EQ !V_CHART) THEN channel_name = 'Vertical'
    if HorV eq 'H' then begin
        channel_name = 'Horizontal'
        Widget_Control, info.stripBase, BASE_SET_TITLE="Showing " + channel_name + " channel from " + SPEC_Nano2HMS(firstnano) + " to " +  SPEC_Nano2HMS(lastnano)
    endif

    if HorV eq 'V' then begin
        channel_name = 'Vertical'
        Widget_Control, info.VstripBase, BASE_SET_TITLE="Showing " + channel_name + " channel from " + SPEC_Nano2HMS(firstnano) + " to " +  SPEC_Nano2HMS(lastnano)
    endif

    if (direction EQ -1) then !FOFFSET = reverse_foffset
    return, foffset
end


;********************************************************************
;********************************************************************

PRO SPEC_Ave_event, event


    Widget_Control, event.top, Get_UValue=ave_info, /No_Copy  ; get our info structure

    theEvent = Tag_Names(event, /Structure_Name)

    if theEvent EQ 'WIDGET_BASE' then Begin
        ;----------- Handle resize event


        Widget_Control, event.top, Get_UValue=info, /No_Copy  ; get our info structure
        info.xscale(1) = event.x
        info.yscale(1) = event.y
        !CHAR_SIZE = (event.x / 300.0) * 0.3
        Widget_Control, ave_info.aveID, Draw_XSize=event.x, Draw_YSize=event.y
    end
    Widget_Control, event.top, Set_UValue=ave_info, /No_Copy  ; release our info structure

END

;*************************************************************************
PRO SPEC_DrawAverageChart, ave_info

    Common  DataBlock,alldataH,alldataV,ADataH,ADataV,FirstData,LastData,StripData,FirstDataV,$
        LastDataV,StripDataV,OneHzH,OneHzV,avestripdata,avestripdataV,SHstripdata,SVstripdata,$
        Mstripdata

    Widget_Control, ave_info.aveID, Get_Value=wid


    ; #1 thing to do is to make it visible
    Widget_Control, ave_info.aveBase, /UPDATE, SHOW = 1
    ;============================================
    ; Here's some code that demonstrates a quick
    ;  way to search for TAS changes and flag the
    ;  ones that are not real.
    ; The results are the indexes to the good TAS
    ;  changes in the array "changes". This routine
    ;  is for testing only and is not used yet.
    ;============================================


    IF (!CURRENT_CHANNEL EQ !H_CHART) THEN begin
        num_data_points = n_elements((*alldataH))
        changes = ULINDGEN(10)
        GetTasChanges, (*alldataH),0,num_data_points,changes
    ENDIF
    IF (!CURRENT_CHANNEL EQ !V_CHART) THEN  BEGIN
        num_data_points = n_elements(*alldataV)
        changes = ULINDGEN(10)
        GetTasChanges, (*alldataV),0,num_data_points,changes
    ENDIF
    ; the array "changes" now contains the index of where the TAS (really) changes
    ; We'll use this to calculate the SV default for the entire data period
    ;  SV default = TAS * DeltaT * 128 * pxs * dww  (where pxs=0.01mm and dww = 63mm)
    ; Since the TAS is changing throughout this time, we're going to calculate the
    ;  entire SV default for the full data time period as a sum of the
    ;  individual portions for each TAS speed. We'll use the changes array to split up the data


    ;---------------------------------------------------------
    ; Now calculate the total air volume for this data file (SVdefault)
    ;  Include the TAS changes in the calculation
    ;---------------------------------------------------------
    NanosecondsPerSecond = 1000000000.0D
    pxsV = !microns_per_pixelV/1000.0
    dww = !DWW_main; 63.0mm in 2D-S, 50.8 mm in 3V-CPI adjust in a3VCPIView.pro
    SAdefaultV = 128.0D * pxsV * dww
    SVdefault = 0.0D

    pxsH = !microns_per_pixelH/1000.0;    0.05D;  50um pixel
    SAdefaultH = 128.0D * pxsH * dww
    SVdefaultH = 0.0D

    chart_name = ""
    IF (!CURRENT_CHANNEL EQ !H_CHART) THEN chart_name = 'Horizontal'
    IF (!CURRENT_CHANNEL EQ !V_CHART) THEN chart_name = 'Vertical'

    if (!CURRENT_CHANNEL EQ !H_CHART) THEN BEGIN
        IF (n_elements(changes) GE 1) then BEGIN
            FOR i = 0, n_elements(changes)-2 DO BEGIN
                timeperiod =  (*alldataH)(changes(i+1)).NanoSeconds  - (*alldataH)(changes(i)).NanoSeconds
                SVdefaultH += (*alldataH)(i).TAS * (timeperiod/NanosecondsPerSecond) * SAdefaultH
                ;print, "begin ",changes(i),(*alldataH)(changes(i)).NanoSeconds
                ;print, "end ",changes(i+1),(*alldataH)(changes(i+1)).NanoSeconds
                ;print, "timediff,TAS,SVdefault ",(timeperiod/NanosecondsPerSecond),(*alldataH)(changes(i)).TAS,SVdefault
                ;print, "===="
            ENDFOR
            ; and pick up the last time period that finishes out the data file
            timeperiod = (*alldataH)(num_data_points-1LL).NanoSeconds  -  (*alldataH)(changes(n_elements(changes)-1)).NanoSeconds
            SVdefaultH += (*alldataH)(i).TAS * (timeperiod/NanosecondsPerSecond) * 128.0D * pxsH * dww
        END ELSE BEGIN  ; Same TAS throughout the entire file, just measure end-start time for SVdefault calculation
            timeperiod =  (*alldataH)(num_data_points-1LL).NanoSeconds  - (*alldataH)(0).NanoSeconds
            SVdefaultH = (*alldataH)(i).TAS * (timeperiod/NanosecondsPerSecond) * 128.0D * pxsH * dww
        endelse
    ENDIF


    if (!CURRENT_CHANNEL EQ !V_CHART) THEN BEGIN
        IF (n_elements(changes) GE 1) then BEGIN
            FOR i = 0, n_elements(changes)-2 DO BEGIN
                timeperiod =  (*alldataV)(changes(i+1)).NanoSeconds  - (*alldataV)(changes(i)).NanoSeconds
                SVdefault += (*alldataV)(i).TAS * (timeperiod/NanosecondsPerSecond) * SAdefault
                ;print, "begin ",changes(i),(*alldataH)(changes(i)).NanoSeconds
                ;print, "end ",changes(i+1),(*alldataH)(changes(i+1)).NanoSeconds
                ;print, "timediff,TAS,SVdefault ",(timeperiod/NanosecondsPerSecond),(*alldataH)(changes(i)).TAS,SVdefault
                ;print, "===="
            ENDFOR
            ; and pick up the last time period that finishes out the data file
            timeperiod = (*alldataV)(num_data_points-1LL).NanoSeconds  -  (*alldataV)(changes(n_elements(changes)-1)).NanoSeconds
            SVdefault += (*alldataV)(i).TAS * (timeperiod/NanosecondsPerSecond) * 128.0D * pxs * dww
        END ELSE BEGIN  ; Same TAS throughout the entire file, just measure end-start time for SVdefault calculation
            timeperiod =  (*alldataV)(num_data_points-1LL).NanoSeconds  - (*alldataV)(0).NanoSeconds
            SVdefault = (*alldataV)(i).TAS * (timeperiod/NanosecondsPerSecond) * 128.0D * pxs * dww
        endelse
    ENDIF

    ; TAF added to prevent empty Window 0 from popping up from device call
    Widget_Control, info.drawID, Get_Value=wid
    WSet,  wid
    WShow, wid

    ;WSet, wid
    device, CURSOR_STANDARD= !HOURGLASS_CURSOR  ; show hourglass while we process the data
    bluecolor = GetColor("Navy", !D.Table_Size-2)
    whitecolor = GetColor("White", !D.Table_Size-2)

    erase,color='ffffff'x  ; get background to white

    XYOUTS,20,17000,"REDRAWING ",color=bluecolor, size=3
    FOR i=0,5 DO BEGIN  ; seems to be needed to get the "REDRAW" to fully appear
        PRINT, " "
    ENDFOR


    pxsV    =!microns_per_pixelV/1000.0;  0.01D ; in mm
    strbszV =!microns_per_pixelV/1000.0;  0.01D ; in mm

    pxsH    =!microns_per_pixelH/1000.0;  0.01D ; in mm
    strbszH =!microns_per_pixelH/1000.0;  0.01D ; in mm

    alpha = 0.115D
    beta = 1.218D
    microns_per_pixelV = !MICRONS_PER_PIXELV
    microns_per_pixelH = !MICRONS_PER_PIXELH


    current_multi = !P.MULTI
    num_plots_per_row = 2
    num_plots_per_column = 2
    current_plot_position = 0
    !P.MULTI = [current_plot_position, num_plots_per_row, num_plots_per_column, 0, 0]
    !X.OMARGIN = [1.5,0.0]  ; We need this because the Area plot X axis label extends off the left edge of the window


    ;-------------------------------------------------------------------
    ; Determine which particles we're plotting, H or V
    ;-------------------------------------------------------------------


    mincntsperbin = ave_info.AveMinCountPerBin
    minbinwid = ave_info.AveMinBinSize
    xmin = ave_info.AveXMin
    xmax = ave_info.AveXMax
    xlog_setting = ave_info.AveXLog
    ylog_setting = ave_info.AveYLog

    IF (!CURRENT_CHANNEL EQ !H_CHART) THEN BEGIN

        ; prepare arrays and variables to pass to unihist for a Size distribution
        sizes = (*alldataH).L1n  * microns_per_pixelH ; length of particles in microns
        adj   = (*alldataH).adj1

        vals=dblarr(2*minbinwid) ; prepare arrays for results from unihist
        xpts=dblarr(2*minbinwid)

        ;sizes = [TEMPORARY(sizes),0]  ; Add one zero value element to get the array output correct
        ;adj = [TEMPORARY(adj),0.0]
        unihist_adj, sizes,mincntsperbin,minbinwid,xpts,vals,adj


        Plot_Numbers = fltarr(n_elements(xpts))
        Plot_Size = fltarr(n_elements(xpts))
        Plot_Area = fltarr(n_elements(xpts))
        Plot_Mass = fltarr(n_elements(xpts))

        for j=0,n_elements(xpts)-1,2 do begin
            ; Pull out the particles that are in each of the bing
            if j+1 eq n_elements(xpts)-1 then somarr=H_(sizes ge xpts[j] ) $  ; Handle the last bin
                else somarr=where(sizes lt xpts[j+1] and sizes ge xpts[j]) ; Handle all but last bins

            if (n_elements(somarr) GE 2) then begin
                Plot_Numbers[j] = n_elements(somarr) / SVdefaultH / (xpts[j+1]-xpts[j]) ;   #/Liter/um
                Plot_Numbers[j+1] = Plot_Numbers[j]

                Plot_Size[j] =  total((sizes(somarr)  * adj(somarr)) / SVdefaultH / (xpts[j+1]-xpts[j])) ;   Size/Liter/um
                Plot_Size[j+1] = Plot_Size[j]

                aa = (*alldataH).Area
                aa = temporary(aa(somarr)) * pxsH * strbszH / 1000000.0

                Plot_Area[j] =  total( (aa * adj(somarr))  / SVdefaultH / (xpts[j+1]-xpts[j])) ;   Area/Liter/um
                Plot_Area[j+1] = Plot_Area[j]

                Plot_Mass[j] =  total( ( (aa ^ beta)  * alpha * adj(somarr)) / SVdefaultH / (xpts[j+1]-xpts[j]) ) ;   Mass/Liter/um
                Plot_Mass[j+1] = Plot_Mass[j]


                ; Using this approach causes an out of memory error
                ;tArea = (*alldataH).Area(somarr)   * pxs * strbsz / 1000000.0D
                ;Plot_Area[j] =  total((tArea * adj(somarr)) / SVdefault / (xpts[j+1]-xpts[j])) ;   Area/Liter/um
                ;Plot_Area[j+1] = Plot_Area[j]

                ;Plot_Mass[j] =  total(((tArea ^ beta)  * alpha * adj(somarr))/ SVdefault / (xpts[j+1]-xpts[j])) ;   Mass/Liter/um
                ;Plot_Mass[j+1] = Plot_Mass[j]

            endif else begin
                Plot_Numbers[j] =   0.0     ;   #/Liter/um
                Plot_Numbers[j+1] = 0.0

                Plot_Size[j] =   0.0;    Size/Liter/um
                Plot_Size[j+1] = 0.0


                Plot_Area[j] =   0.0;   Area/Liter/um
                Plot_Area[j+1] = 0.0

                Plot_Mass[j] =   0.0;   Mass/Liter/um
                Plot_Mass[j+1] = 0.0
            endelse

        endfor

        ;wset,0

        Plot, xpts, Plot_Numbers, $
            background= whitecolor, $
            Title = ' Horiz.Particle Count Distribution', $
            XTitle = 'microns' , $
            YTitle = '#/Liter/um'  , $
            xrange = [xmin,xmax], $
            color = bluecolor, $
            XLOG = xlog_setting, $
            YLOG = ylog_setting

        Plot, xpts, Plot_Size, $
            background= whitecolor, $
            Title = ' Horiz.Particle Size Distribution', $
            XTitle = 'microns' , $
            YTitle = 'microns/Liter/um'  , $
            xrange = [xmin,xmax], $
            color = bluecolor, $
            XLOG = xlog_setting, $
            YLOG = ylog_setting

        Plot, xpts, Plot_Area, $
            background= whitecolor, $
            Title = ' Horiz.Particle Area Distribution', $
            XTitle = 'microns' , $
            YTitle = 'sq.mm./Liter/um'  , $
            xrange = [xmin,xmax], $
            color = bluecolor, $
            XLOG = xlog_setting, $
            YLOG = ylog_setting

        Plot, xpts, Plot_Mass, $
            background= whitecolor, $
            Title = ' Horiz.Particle Mass Distribution', $
            XTitle = 'microns' , $
            YTitle = 'g/m^3/um'  , $
            xrange = [xmin,xmax], $
            color = bluecolor, $
            XLOG = xlog_setting, $
            YLOG = ylog_setting

        !P.MULTI = current_multi



    ENDIF

    IF (!CURRENT_CHANNEL EQ !V_CHART) THEN BEGIN

        ; prepare arrays and variables to pass to unihist for a Size distribution
        sizes = (*alldataV).L1n  * microns_per_pixel ; length of particles in microns
        adj   = (*alldataV).adj1

        vals=dblarr(2*minbinwid) ; prepare arrays for results from unihist
        xpts=dblarr(2*minbinwid)

        ;sizes = [TEMPORARY(sizes),0]  ; Add one zero value element to get the array output correct
        ;adj = [TEMPORARY(adj),0.0]
        unihist_adj, sizes,mincntsperbin,minbinwid,xpts,vals,adj


        Plot_Numbers = fltarr(n_elements(xpts))
        Plot_Size = fltarr(n_elements(xpts))
        Plot_Area = fltarr(n_elements(xpts))
        Plot_Mass = fltarr(n_elements(xpts))

        for j=0,n_elements(xpts)-1,2 do begin
            ; Pull out the particles that are in each of the bing
            if j+1 eq n_elements(xpts)-1 then somarr=where(sizes ge xpts[j] ) $  ; Handle the last bin
                else somarr=where(sizes lt xpts[j+1] and sizes ge xpts[j]) ; Handle all but last bins

            if (n_elements(somarr) GE 2) then begin
                Plot_Numbers[j] = n_elements(somarr) / SVdefaultV / (xpts[j+1]-xpts[j]) ;   #/Liter/um
                Plot_Numbers[j+1] = Plot_Numbers[j]

                Plot_Size[j] =  total((sizes(somarr)  * adj(somarr)) / SVdefaultV / (xpts[j+1]-xpts[j])) ;   Size/Liter/um
                Plot_Size[j+1] = Plot_Size[j]

                ; Using this approach causea an out of  memory error
                ;tArea = (*alldataV).Area(somarr)   * pxsV * strbszV / 1000000.0D
                ;Plot_Area[j] =  total((tArea * adj(somarr)) / SVdefaultV / (xpts[j+1]-xpts[j])) ;   Area/Liter/um

                aa = (*alldataV).Area
                aa = temporary(aa(somarr)) * pxsV * strbszV / 1000000.0

                Plot_Area[j] =  total( (aa * adj(somarr))  / SVdefaultV / (xpts[j+1]-xpts[j])) ;   Area/Liter/um
                Plot_Area[j+1] = Plot_Area[j]

                Plot_Mass[j] =  total( ( (aa ^ beta)  * alpha * adj(somarr)) / SVdefaultV / (xpts[j+1]-xpts[j]) ) ;   Mass/Liter/um
                Plot_Mass[j+1] = Plot_Mass[j]

            endif else begin
                Plot_Numbers[j] =   0.0     ;   #/Liter/um
                Plot_Numbers[j+1] = 0.0

                Plot_Size[j] =   0.0;    Size/Liter/um
                Plot_Size[j+1] = 0.0

                Plot_Area[j] =   0.0;   Area/Liter/um
                Plot_Area[j+1] = 0.0

                Plot_Mass[j] =   0.0;   Mass/Liter/um
                Plot_Mass[j+1] = 0.0
            endelse

        endfor


        Plot, xpts, Plot_Numbers, $
            background= whitecolor, $
            Title = ' Vert.Particle Count Distribution', $
            XTitle = 'microns' , $
            YTitle = '#/Liter/um'  , $
            xrange = [xmin,xmax], $
            color = bluecolor, $
            XLOG = xlog_setting, $
            YLOG = ylog_setting

        Plot, xpts, Plot_Size, $
            background= whitecolor, $
            Title = ' Vert.Particle Size Distribution', $
            XTitle = 'microns' , $
            YTitle = 'microns/Liter/um'  , $
            xrange = [xmin,xmax], $
            color = bluecolor, $
            XLOG = xlog_setting, $
            YLOG = ylog_setting

        Plot, xpts, Plot_Area, $
            background= whitecolor, $
            Title = ' Vert.Particle Area Distribution', $
            XTitle = 'microns' , $
            YTitle = 'sq.mm./Liter/um'  , $
            xrange = [xmin,xmax], $
            color = bluecolor, $
            XLOG = xlog_setting, $
            YLOG = ylog_setting

        Plot, xpts, Plot_Mass, $
            background= whitecolor, $
            Title = ' Vert.Particle Mass Distribution', $
            XTitle = 'microns' , $
            YTitle = 'g/m^3/um'  , $
            xrange = [xmin,xmax], $
            color = bluecolor, $
            XLOG = xlog_setting, $
            YLOG = ylog_setting

        !P.MULTI = current_multi



    ENDIF


    ; TAF added to prevent empty Window 0 from popping up from device call
    Widget_Control, info.drawID, Get_Value=wid
    WSet,  wid
    WShow, wid

    device, CURSOR_STANDARD= !ARROW_CURSOR  ; show hourglass while we process the data

end
;*************************************************************************
;*************************************************************************

PRO SPEC_StripH_event, event

    Common  DataBlock,alldataH,alldataV,ADataH,ADataV,FirstData,LastData,StripData,FirstDataV,$
        LastDataV,StripDataV,OneHzH,OneHzV,avestripdata,avestripdataV,SHstripdata,SVstripdata,$
        Mstripdata

    Widget_Control, event.top, Get_UValue=strip_info, /No_Copy  ; get our info structure


    if(event.PRESS GE 1) then  begin
        if event.ID EQ strip_info.zoomID then print, "zoom window clicked"

    end

    if event.ID EQ strip_info.stripID then begin
        if (event.x GE 0 and event.X LE (!STRIP_WIDTH - 1)) then begin

            timestamp = SPEC_Nano2HMS(StripData(event.X).Nanoseconds)
            ;length = string(format = '(%"Length/L1(um) = %d")',StripData(event.X).ImageWords * 10)
            length = string(format = '(%"Length/L1n(um) = %d")',StripData(event.X).L1n * 10)
            height = string(format = '(%"Height/L2(um) = %d")',StripData(event.X).L2 * 10)
            area   = string(format = '(%"Area(sq.um) = %6.1d")',StripData(event.X).Area * 100)
            F1     = string(format = '(%"F1 = %1.1d")',StripData(event.X).F1)
            PC1    = string(format = '(%"PC1 = %3.1d")',StripData(event.X).PC1)
            L3    = string(format = '(%"L3 = %9.3f")',1.8*StripData(event.X).L3)
            L4    = string(format = '(%"L4 = %3.1d")',StripData(event.X).L4)
            L5    = string(format = '(%"L5 = %3.1d")',StripData(event.X).L5)
            L7    = string(format = '(%"L7 = %8.2f")',StripData(event.X).L7)
            L8    = string(format = '(%"L8 = %8.2f")',StripData(event.X).L8)
            PC3   = string(format = '(%"PC3 (x,y) = %9.3f,%9.3f")',StripData(event.X).PC3x,StripData(event.X).PC3y)
            PC4   = string(format = '(%"PC4 (x,y) = %9.3f,%9.3f")',StripData(event.X).PC4x,StripData(event.X).PC4y)
            A3    = string(format = '(%"A3 = %9.3f")',StripData(event.X).A3)
            Alexei =   string(format = '(%"A2 (alexei) = %6.1d")',StripData(event.X).Alexei1)

            WIDGET_CONTROL ,strip_info.stripLabel ,SET_VALUE= timestamp + !CRLF $
                + length + !CRLF $
                + height + !CRLF $
                + area   + !CRLF $
                + F1     + !CRLF $
                + PC1    + !CRLF $
                + L3     + !CRLF $
                + L4     + !CRLF $
                + L5     + !CRLF $
                + L7     + !CRLF $
                + L8     + !CRLF $
                + PC3    + !CRLF $
                + PC4    + !CRLF $
                + A3     + !CRLF $
                + Alexei

            if (StripData(event.X).ImageWords GT 0) then begin

                match = StripData(event.X).Nanoseconds
                beginning = LONG(event.X)
                index =  LONG(event.X)
                WHILE ((beginning GE 0) and (StripData(index).Nanoseconds EQ match)) DO BEGIN
                    beginning--
                    if --index LT 0 then index = 0
                ENDWHILE
                if beginning NE 0 then  beginning++

                theend = LONG(event.X)
                index = LONG(event.X)
                WHILE ((theend LE (!STRIP_WIDTH - 1)) and (StripData(index).Nanoseconds EQ match)) DO BEGIN
                    theend++
                    if ++index GT (!STRIP_WIDTH - 1) then index = (!STRIP_WIDTH - 1)  ; can't let us access StripData(600)
                ENDWHILE
                if theend NE (!STRIP_WIDTH - 1) then theend--



                Widget_Control, strip_info.stripID, Get_Value=wid
                WSet, wid

                center_y = long(StripData(event.X).PC1)
                center_x = long((theend-beginning)/2 + beginning)

                ;length = StripData(event.X).L2
                ;if (length LT StripData(event.X).ImageWords) then length = StripData(event.X).ImageWords


                ;start = long(StripData(event.X).PC1 -length)
                ;if (start LT 0) then start = 0

                ;length *= 2
                ;if (start + length) GT 127 then start = 128 - length
                ;height = StripData(event.X).ImageWords
                full_width = 22L
                half_width = 11L

                full_width = 44L
                half_width = 22L

                if (center_x LT half_width) then center_x = half_width
                if (center_x GT (!STRIP_WIDTH - 1)-half_width) then center_x = (!STRIP_WIDTH - 1)-half_width
                if (center_y LT half_width) then center_y = half_width
                if (center_y GT 127-half_width) then center_y = 127-half_width
                x1 = center_x - half_width
                y1 = center_y - half_width
                x_length = full_width
                y_length = full_width

                aa = where(StripData(*).nanoseconds eq match)
                x1 = aa[0]
                x_length = n_elements(aa)

                clip = TVRD(x1,y1,x_length,y_length)

                Widget_Control, strip_info.zoomID, Get_Value=wid
                WSet, wid
                erase,color='ffffff'x  ; get background to white
                ;bigger = rebin(clip,full_width*zoom_value,full_width*zoom_value,/SAMPLE)
                bigger = rebin(clip, x_length * !zoom_factor, y_length * !zoom_factor, /SAMPLE)
                tv,bigger

                Widget_Control, strip_info.stripID, Get_Value=wid
                WSet, wid
            end

        end
    end

    Widget_Control, event.top, Set_UValue=strip_info, /No_Copy  ; release our info structure
END
;-------------------------------------
pro ParticleStrip_Toolbar_Rewind, event
    Widget_Control, event.top, Get_UValue=strip_info, /No_Copy  ; get our info structure

    foffset = !FOFFSET
    newoffset = FillStrip(foffset, strip_info,-1,'H')

    if !FOFFSET EQ 0UL then begin
        WIDGET_CONTROL ,strip_info.stripLabel ,SET_VALUE= "BEGINNING OF FILE REACHED"
    end else begin
        WIDGET_CONTROL ,strip_info.stripLabel ,SET_VALUE= ""
    endelse



    Widget_Control, event.top, Set_UValue=strip_info, /No_Copy  ; release our info structure
end
;-------------------------------------
pro ParticleStrip_Toolbar_Forward, event

    Widget_Control, event.top, Get_UValue=strip_info, /No_Copy  ; get our info structure
    print, event.select

    foffset = !FOFFSET
    pre_offset = foffset
    newoffset = FillStrip(foffset, strip_info,1,'H')
    ; print, "pre,post",pre_offset,newoffset
    if pre_offset EQ foffset then begin
        WIDGET_CONTROL ,strip_info.stripLabel ,SET_VALUE= "END OF FILE REACHED"
    end else begin
        WIDGET_CONTROL ,strip_info.stripLabel ,SET_VALUE= ""
    endelse


    Widget_Control, event.top, Set_UValue=strip_info, /No_Copy  ; release our info structure

end;-------------------------------------


;=============================================================
;=============================================================
;== ParticleStrip ============================================
;=============================================================
;=============================================================

PRO ParticleStrip, info  ; this is for Hparticles

    stripID = 0L
    zoomID  = 0L
    stripBase = 0L
    stripLabel = 0L


    strip_info = {stripID:stripID, $               ; widget ID for the strip of bitmaps
        VstripID:0L,$
        zoomID:zoomID, $
        stripBase:stripBase, $
        stripLabel:stripLabel, $
        VstripBase:stripBase, $
        VstripLabel:stripLabel, $
        FR_button:0L, $
        SSR_button:0L, $
        SSF_button:0L, $
        FF_button:0L $
        }


    base = WIDGET_BASE(/COLUMN)
    strip_info.stripBase = base
    info.stripBase = base
    Hparticles = Widget_Draw(base,XSize=!STRIP_WIDTH, YSize=128, Event_Pro='SPEC_StripH_event', /BUTTON_EVENTS, /MOTION_EVENTS)

    strip_info.stripID = HParticles
    info.stripID = HParticles         ; we need to access this widget from the main window too



    STRIP_Toolbar = WIDGET_BASE(base, $
        COLUMN=8, $
        FRAME=1, $
        MAP=1, $
        UVALUE='BASE_Toolbar', $
        BASE_ALIGN_LEFT=1, $
        YSIZE=256)





    STRIP_dual_label = WIDGET_BASE(STRIP_Toolbar, $
        ROW=1, $
        ; FRAME=1, $
        MAP=1, $
        UVALUE='BASE_Toolbar', $
        BASE_ALIGN_LEFT=1, $
        YSIZE=240);;changer from 180 to 240; for showing L7, L8 , L5, PC1 etc.


    ; Generate a label on the bitmap widget that will display the parameters for a single particle

    LABEL_Info = WIDGET_TEXT(STRIP_dual_label, $
        VALUE="", $
        XSIZE = 40, $
        YSIZE=30)
    strip_info.stripLabel = LABEL_Info


    BUTTON_FastBackward = WIDGET_BUTTON( STRIP_Toolbar, $
        FONT='9x15', $
        UVALUE='FastRewind', $
        XSIZE=40, $
        TOOLTIP="Play Backwards", $
        VALUE=  !SPEC_3VCPI_DIR + '/RR.bmp', /BITMAP, $
        Event_Pro='play_reverse_timer_event')



    BUTTON_Rewind = WIDGET_BUTTON( STRIP_Toolbar, $
        FONT='9x15', $
        UVALUE='Rewind', $
        XSIZE=35, $
        TOOLTIP="Single Step Reverse", $
        VALUE=  !SPEC_3VCPI_DIR + '/SingleR.bmp', /BITMAP, $
        Event_Pro='ParticleStrip_Toolbar_Rewind')

    BUTTON_Forward = WIDGET_BUTTON( STRIP_Toolbar, $
        FONT='9x15', $
        UVALUE='Forward', $
        XSIZE=35, $
        TOOLTIP="Single Step Forward", $
        ;PUSHBUTTON_EVENTS=1, $
        VALUE= !SPEC_3VCPI_DIR + '/SingleF.bmp', /BITMAP, $
        Event_Pro='ParticleStrip_Toolbar_Forward')


    BUTTON_FastForward = WIDGET_BUTTON( STRIP_Toolbar, $
        FONT='9x15', $
        UVALUE='FastForward', $
        XSIZE=40, $
        TOOLTIP="Play Forwards", $
        VALUE=  !SPEC_3VCPI_DIR + '/FF.bmp', /BITMAP, $
        Event_Pro='play_forward_timer_event')


    spaceholder1 = WIDGET_LABEL(STRIP_Toolbar, $
        VALUE= " Speed (in Secs)", $
        XSIZE=110, $
        YSIZE=35)

    dd_list=['0.10','0.25','0.50','1.0']
    STRIP_Dropdown = WIDGET_DROPLIST(STRIP_Toolbar, $
        FRAME=1, $
        VALUE = dd_list, $
        Event_Pro = 'Bitmap_Playback_Speed_event')

    ; save so we can disble buttons during FF and Fast reverse
    strip_info.FR_button = BUTTON_FastBackward
    strip_info.SSR_button = BUTTON_Rewind
    strip_info.SSF_button = BUTTON_Forward
    strip_info.FF_button = BUTTON_FastForward

    Zoomer_base = WIDGET_BASE(STRIP_Toolbar, $
        MAP=1, $
        UVALUE='BASE_Zoomer', $
        BASE_ALIGN_LEFT=1, $
        YSIZE=!zoom_xsize, $
        XSIZE=!zoom_ysize)

    ;strip_info.zoomID = Widget_Draw(Zoomer_base,XSize=128 + 128, YSize=128 + 128, Event_Pro='SPEC_StripH_event', /BUTTON_EVENTS, /MOTION_EVENTS)
    strip_info.zoomID = Widget_Draw(Zoomer_base,XSize = !zoom_xsize, YSize = !zoom_ysize, Event_Pro='SPEC_StripH_event', /BUTTON_EVENTS, /MOTION_EVENTS)

    ;  strip_info.zoomID = Widget_Draw(Zoomer_base,XSize=128, YSize=128, Event_Pro='SPEC_StripH_event', /BUTTON_EVENTS, /MOTION_EVENTS)
    info.zoomID = strip_info.zoomID


    Widget_Control, base, Set_UValue=strip_info, /No_Copy

    WIDGET_CONTROL, base, /REALIZE
    XMANAGER, 'ParticleStrip', base, /JUST_REG, /no_block

END ; end of particlestrip,info
;*********************************************************************

;*******************************************************************
;== VParticleStrip ============================================
;=============================================================
;=============================================================

PRO VParticleStrip, info  ; this is for Vparticles

    stripID = 0L
    zoomID  = 0L
    stripBase = 0L
    stripLabel = 0L

    strip_info = {stripID:stripID, $               ; widget ID for the strip of bitmaps
        VstripID:0L,$
        zoomID:zoomID, $
        stripBase:stripBase, $
        stripLabel:stripLabel, $
        VstripBase:stripBase, $
        VstripLabel:stripLabel, $
        VzoomID:zoomID, $
        FR_button:0L, $
        SSR_button:0L, $
        SSF_button:0L, $
        FF_button:0L $
        }
    base = WIDGET_BASE(/COLUMN)
    strip_info.VstripBase = base
    info.VstripBase = base
    Vparticles = Widget_Draw(base,XSize=!STRIP_WIDTH, YSize=128, Event_Pro='VSPEC_StripV_event', /BUTTON_EVENTS, /MOTION_EVENTS)

    strip_info.VstripID = VParticles
    info.VstripID = VParticles         ; we need to access this widget from the main window too



    STRIP_Toolbar = WIDGET_BASE(base, $
        COLUMN=8, $
        FRAME=1, $
        MAP=1, $
        UVALUE='BASE_Toolbar', $
        BASE_ALIGN_LEFT=1, $
        YSIZE=256)


    STRIP_dual_label = WIDGET_BASE(STRIP_Toolbar, $
        ROW=1, $
        ; FRAME=1, $
        MAP=1, $
        UVALUE='BASE_Toolbar', $
        BASE_ALIGN_LEFT=1, $
        YSIZE=180)


    ; Generate a label on the bitmap widget that will display the parameters for a single particle

    LABEL_Info = WIDGET_TEXT(STRIP_dual_label, $
        VALUE="", $
        XSIZE = 40, $
        YSIZE=30)
    strip_info.VstripLabel = LABEL_Info


    BUTTON_FastBackward = WIDGET_BUTTON( STRIP_Toolbar, $
        FONT='9x15', $
        UVALUE='FastRewind', $
        XSIZE=40, $
        TOOLTIP="Play Backwards", $
        VALUE=  !SPEC_3VCPI_DIR + '/RR.bmp', /BITMAP, $
        Event_Pro='Vplay_reverse_timer_event')



    BUTTON_Rewind = WIDGET_BUTTON( STRIP_Toolbar, $
        FONT='9x15', $
        UVALUE='Rewind', $
        XSIZE=35, $
        TOOLTIP="Single Step Reverse", $
        VALUE=  !SPEC_3VCPI_DIR + '/SingleR.bmp', /BITMAP, $
        Event_Pro='VParticleStrip_Toolbar_Rewind')

    BUTTON_Forward = WIDGET_BUTTON( STRIP_Toolbar, $
        FONT='9x15', $
        UVALUE='Forward', $
        XSIZE=35, $
        TOOLTIP="Single Step Forward", $
        ;PUSHBUTTON_EVENTS=1, $
        VALUE= !SPEC_3VCPI_DIR + '/SingleF.bmp', /BITMAP, $
        Event_Pro='VParticleStrip_Toolbar_Forward')


    BUTTON_FastForward = WIDGET_BUTTON( STRIP_Toolbar, $
        FONT='9x15', $
        UVALUE='FastForward', $
        XSIZE=40, $
        TOOLTIP="Play Forwards", $
        VALUE=  !SPEC_3VCPI_DIR + '/FF.bmp', /BITMAP, $
        Event_Pro='Vplay_forward_timer_event')


    spaceholder1 = WIDGET_LABEL(STRIP_Toolbar, $
        VALUE= " Speed (in Secs)", $
        XSIZE=110, $
        YSIZE=35)

    dd_list=['0.10','0.25','0.50','1.0']
    STRIP_Dropdown = WIDGET_DROPLIST(STRIP_Toolbar, $
        FRAME=1, $
        VALUE = dd_list, $
        Event_Pro = 'VBitmap_Playback_Speed_event')

    ; save so we can disble buttons during FF and Fast reverse
    strip_info.FR_button = BUTTON_FastBackward
    strip_info.SSR_button = BUTTON_Rewind
    strip_info.SSF_button = BUTTON_Forward
    strip_info.FF_button = BUTTON_FastForward

    Zoomer_base = WIDGET_BASE(STRIP_Toolbar, $
        MAP=1, $
        UVALUE='BASE_Zoomer', $
        BASE_ALIGN_LEFT=1, $
        YSIZE=!zoom_xsize, $
        XSIZE=!zoom_ysize)


    strip_info.VzoomID = Widget_Draw(Zoomer_base,XSize = !zoom_xsize, YSize = !zoom_ysize, Event_Pro='VSPEC_StripV_event', /BUTTON_EVENTS, /MOTION_EVENTS)

    ;  strip_info.zoomID = Widget_Draw(Zoomer_base,XSize=128, YSize=128, Event_Pro='SPEC_StripH_event', /BUTTON_EVENTS, /MOTION_EVENTS)
    info.VzoomID = strip_info.zoomID

    Widget_Control, base, Set_UValue=strip_info, /No_Copy

    WIDGET_CONTROL, base, /REALIZE
    XMANAGER, 'VParticleStrip', base, /JUST_REG, /no_block

END  ; end of Vparticlestrip, info



PRO VSPEC_StripV_event, event

    Common DataBlock,alldataH,alldataV,ADataH,ADataV,FirstData,LastData,StripData,FirstDataV,$
        LastDataV,StripDataV,OneHzH,OneHzV,avestripdata,avestripdataV,SHstripdata,SVstripdata,$
        Mstripdata

    Widget_Control, event.top, Get_UValue=strip_info, /No_Copy  ; get our info structure


    if(event.PRESS GE 1) then  begin
        if event.ID EQ strip_info.VzoomID then print, "zoom window clicked"

    end

    if event.ID EQ strip_info.VstripID then begin
        if (event.x GE 0 and event.X LE (!STRIP_WIDTH - 1)) then begin

            timestamp = SPEC_Nano2HMS(StripDataV(event.X).Nanoseconds)
            ;length = string(format = '(%"Length/L1(um) = %d")',StripDataV(event.X).ImageWords * 10)
            length = string(format = '(%"Length/L1n(um) = %d")',StripDataV(event.X).L1n * 10)
            height = string(format = '(%"Height/L2(um) = %d")',StripDataV(event.X).L2 * 10)
            area   = string(format = '(%"Area(sq.um) = %6.1d")',StripDataV(event.X).Area * 100)
            F1     = string(format = '(%"F1 = %1.1d")',StripDataV(event.X).F1)
            PC1    = string(format = '(%"PC1 = %3.1d")',StripDataV(event.X).PC1)
            L3    = string(format = '(%"L3 = %9.3f")',1.8*StripDataV(event.X).L3)
            L4    = string(format = '(%"L4 = %3.1d")',StripDataV(event.X).L4)
            L5    = string(format = '(%"L5 = %3.1d")',StripDataV(event.X).L5)
            L7    = string(format = '(%"L4 = %3.1d")',StripDataV(event.X).L7)
            L8    = string(format = '(%"L5 = %3.1d")',StripDataV(event.X).L8)
            PC3   = string(format = '(%"PC3 (x,y) = %9.3f,%9.3f")',StripDataV(event.X).PC3x,StripDataV(event.X).PC3y)
            PC4   = string(format = '(%"PC4 (x,y) = %9.3f,%9.3f")',StripDataV(event.X).PC4x,StripDataV(event.X).PC4y)
            A3    = string(format = '(%"A3 = %9.3f")',StripDataV(event.X).A3)
            Alexei =   string(format = '(%"A2 (alexei) = %6.1d")',StripDataV(event.X).Alexei1)

            WIDGET_CONTROL ,strip_info.VstripLabel ,SET_VALUE= timestamp + !CRLF $
                + length + !CRLF $
                + height + !CRLF $
                + area   + !CRLF $
                + F1     + !CRLF $
                + PC1    + !CRLF $
                + L3     + !CRLF $
                + L4     + !CRLF $
                + L5     + !CRLF $
                + PC3    + !CRLF $
                + PC4    + !CRLF $
                + A3     + !CRLF $
                + Alexei

            if (StripDataV(event.X).ImageWords GT 0) then begin

                match = StripDataV(event.X).Nanoseconds
                beginning = LONG(event.X)
                index =  LONG(event.X)
                WHILE ((beginning GE 0) and (StripDataV(index).Nanoseconds EQ match)) DO BEGIN
                    beginning--
                    if --index LT 0 then index = 0
                ENDWHILE
                if beginning NE 0 then  beginning++

                theend = LONG(event.X)
                index = LONG(event.X)
                WHILE ((theend LE (!STRIP_WIDTH - 1)) and (StripDataV(index).Nanoseconds EQ match)) DO BEGIN
                    theend++
                    if ++index GT (!STRIP_WIDTH - 1) then index = (!STRIP_WIDTH - 1)  ; can't let us access StripDataV(600)
                ENDWHILE
                if theend NE (!STRIP_WIDTH - 1) then theend--

                Widget_Control, strip_info.VstripID, Get_Value=wid
                WSet, wid

                center_y = long(StripDataV(event.X).PC1)
                center_x = long((theend-beginning)/2 + beginning)

                ;length = StripDataV(event.X).L2
                ;if (length LT StripDataV(event.X).ImageWords) then length = StripDataV(event.X).ImageWords


                ;start = long(StripDataV(event.X).PC1 -length)
                ;if (start LT 0) then start = 0

                ;length *= 2
                ;if (start + length) GT 127 then start = 128 - length
                ;height = StripDataV(event.X).ImageWords
                full_width = 22L
                half_width = 11L

                full_width = 44L
                half_width = 22L

                if (center_x LT half_width) then center_x = half_width
                if (center_x GT (!STRIP_WIDTH - 1)-half_width) then center_x = (!STRIP_WIDTH - 1)-half_width
                if (center_y LT half_width) then center_y = half_width
                if (center_y GT 127-half_width) then center_y = 127-half_width
                x1 = center_x - half_width
                y1 = center_y - half_width
                x_length = full_width
                y_length = full_width

                aa = where(StripDataV(*).nanoseconds eq match)
                x1 = aa[0]
                x_length = n_elements(aa)

                clip = TVRD(x1,y1,x_length,y_length)

                Widget_Control, strip_info.VzoomID, Get_Value=wid
                WSet, wid
                erase,color='ffffff'x  ; get background to white
                ;bigger = rebin(clip,full_width*zoom_value,full_width*zoom_value,/SAMPLE)
                bigger = rebin(clip, x_length * !zoom_factor, y_length * !zoom_factor, /SAMPLE)
                tv,bigger

                Widget_Control, strip_info.VstripID, Get_Value=wid
                WSet, wid
            end

        end
    end

    Widget_Control, event.top, Set_UValue=strip_info, /No_Copy  ; release our info structure
END
;-------------------------------------
pro VParticleStrip_Toolbar_Rewind, event
    Widget_Control, event.top, Get_UValue=strip_info, /No_Copy  ; get our info structure
    foffset = !FOFFSET
    newoffset = FillStrip(foffset, strip_info,-1,'V')

    if !FOFFSET EQ 0UL then begin
        WIDGET_CONTROL ,strip_info.VstripLabel ,SET_VALUE= "BEGINNING OF FILE REACHED"
    end else begin
        WIDGET_CONTROL ,strip_info.VstripLabel ,SET_VALUE= ""
    endelse
    Widget_Control, event.top, Set_UValue=strip_info, /No_Copy  ; release our info structure
end
;-------------------------------------
pro VParticleStrip_Toolbar_Forward, event

    Widget_Control, event.top, Get_UValue=strip_info, /No_Copy  ; get our info structure
    print, event.select

    foffset = !FOFFSET
    pre_offset = foffset
    newoffset = FillStrip(foffset, strip_info,1,'V')
    ; print, "pre,post",pre_offset,newoffset
    if pre_offset EQ foffset then begin
        WIDGET_CONTROL ,strip_info.VstripLabel ,SET_VALUE= "END OF FILE REACHED"
    end else begin
        WIDGET_CONTROL ,strip_info.VstripLabel ,SET_VALUE= ""
    endelse

    Widget_Control, event.top, Set_UValue=strip_info, /No_Copy  ; release our info structure
end;-------------------------------------

pro Vplay_forward_timer_event, event
    forward_function  FillStrip

    IF (TAG_NAMES(event, /STRUCTURE_NAME) EQ 'WIDGET_TIMER') THEN BEGIN
        if (!PLAYBACK_ACTIVE eq 0) then begin



            Widget_Control, event.top, Get_UValue=strip_info, /No_Copy  ; get our info structure

            foffset = !FOFFSET
            pre_offset = foffset
            newoffset = FillStrip(foffset, strip_info,1,'V')
            ; print, "pre,post",pre_offset,newoffset
            if pre_offset EQ foffset then begin
                WIDGET_CONTROL ,strip_info.VstripLabel ,SET_VALUE= "END OF FILE REACHED"
                !PLAYBACK_ACTIVE = 1
                widget_control, event.ID, SET_VALUE = !SPEC_3VCPI_DIR + '/FF.bmp', /BITMAP

                WIDGET_CONTROL, strip_info.SSR_button, SENSITIVE=1
                WIDGET_CONTROL, strip_info.SSF_button, SENSITIVE=1
                WIDGET_CONTROL, strip_info.FR_button, SENSITIVE=1

            end else begin
                WIDGET_CONTROL ,strip_info.VstripLabel ,SET_VALUE= ""
                WIDGET_CONTROL, event.ID, TIMER= !PLAYBACK_SPEED
            endelse


            Widget_Control, event.top, Set_UValue=strip_info, /No_Copy  ; release our info structure

        endif
    endif else begin
        Widget_Control, event.top, Get_UValue=strip_info, /No_Copy  ; get our info structure
        if (!PLAYBACK_ACTIVE eq 1) then begin
            !PLAYBACK_ACTIVE = 0
            widget_control, event.ID, SET_VALUE = !SPEC_3VCPI_DIR + '/FFon.bmp', /BITMAP
            WIDGET_CONTROL, event.ID, TIMER= !PLAYBACK_SPEED

            WIDGET_CONTROL, strip_info.SSR_button, SENSITIVE=0
            WIDGET_CONTROL, strip_info.SSF_button, SENSITIVE=0
            WIDGET_CONTROL, strip_info.FR_button, SENSITIVE=0


            foffset = !FOFFSET
            pre_offset = foffset
            newoffset = FillStrip(foffset, strip_info,1,'V')
            ; print, "pre,post",pre_offset,newoffset
            if pre_offset EQ foffset then begin
                WIDGET_CONTROL ,strip_info.VstripLabel ,SET_VALUE= "END OF FILE REACHED"
            end else begin
                WIDGET_CONTROL ,strip_info.VstripLabel ,SET_VALUE= ""
            endelse




        endif else begin
            !PLAYBACK_ACTIVE = 1
            widget_control, event.ID, SET_VALUE = !SPEC_3VCPI_DIR + '/FF.bmp', /BITMAP

            WIDGET_CONTROL, strip_info.SSR_button, SENSITIVE=1
            WIDGET_CONTROL, strip_info.SSF_button, SENSITIVE=1
            WIDGET_CONTROL, strip_info.FR_button, SENSITIVE=1
        endelse
        Widget_Control, event.top, Set_UValue=strip_info, /No_Copy  ; release our info structure
    endelse
end
;*****************************************************************************
pro Vplay_reverse_timer_event, event

    forward_function  FillStrip

    IF (TAG_NAMES(event, /STRUCTURE_NAME) EQ 'WIDGET_TIMER') THEN BEGIN
        if (!REWIND_ACTIVE eq 0) then begin


            Widget_Control, event.top, Get_UValue=strip_info, /No_Copy  ; get our info structure

            foffset = !FOFFSET
            newoffset = FillStrip(foffset, strip_info,-1,'V')

            if !FOFFSET EQ 0UL then begin
                WIDGET_CONTROL ,strip_info.vstripLabel ,SET_VALUE= "BEGINNING OF FILE REACHED"
                !REWIND_ACTIVE = 1
                widget_control, event.ID, SET_VALUE = !SPEC_3VCPI_DIR + '/RR.bmp', /BITMAP
                WIDGET_CONTROL, strip_info.SSR_button, SENSITIVE=1
                WIDGET_CONTROL, strip_info.SSF_button, SENSITIVE=1
                WIDGET_CONTROL, strip_info.FF_button, SENSITIVE=1

            end else begin
                WIDGET_CONTROL ,strip_info.VstripLabel ,SET_VALUE= ""
                WIDGET_CONTROL, event.ID, TIMER= !PLAYBACK_SPEED
            endelse



            Widget_Control, event.top, Set_UValue=strip_info, /No_Copy  ; release our info structure

        endif
    endif else begin
        Widget_Control, event.top, Get_UValue=strip_info, /No_Copy  ; get our info structure
        if (!REWIND_ACTIVE eq 1) then begin
            !REWIND_ACTIVE = 0
            widget_control, event.ID, SET_VALUE = !SPEC_3VCPI_DIR + '/RRon.bmp', /BITMAP
            WIDGET_CONTROL, event.ID, TIMER= !PLAYBACK_SPEED

            WIDGET_CONTROL, strip_info.SSR_button, SENSITIVE=0
            WIDGET_CONTROL, strip_info.SSF_button, SENSITIVE=0
            WIDGET_CONTROL, strip_info.FF_button, SENSITIVE=0

            foffset = !FOFFSET
            newoffset = FillStrip(foffset, strip_info,-1,'V')

            if !FOFFSET EQ 0UL then begin
                WIDGET_CONTROL ,strip_info.VstripLabel ,SET_VALUE= "BEGINNING OF FILE REACHED"
            end else begin
                WIDGET_CONTROL ,strip_info.VstripLabel ,SET_VALUE= ""
            endelse




        endif else begin
            !REWIND_ACTIVE = 1
            widget_control, event.ID, SET_VALUE = !SPEC_3VCPI_DIR + '/RR.bmp', /BITMAP
            WIDGET_CONTROL, strip_info.SSR_button, SENSITIVE=1
            WIDGET_CONTROL, strip_info.SSF_button, SENSITIVE=1
            WIDGET_CONTROL, strip_info.FF_button, SENSITIVE=1

        endelse
        Widget_Control, event.top, Set_UValue=strip_info, /No_Copy  ; release our info structure
    endelse
end

pro VBitmap_Playback_Speed_event, event
    if event.index eq 0 then !PLAYBACK_SPEED = 0.1
    if event.index eq 1 then !PLAYBACK_SPEED = 0.25
    if event.index eq 2 then !PLAYBACK_SPEED = 0.50
    if event.index eq 3 then !PLAYBACK_SPEED = 1.0
end
;************* end of Vparticlestrip, info all parts of it.
;***********************************************************************

;=============================================================
;=============================================================
;== AverageWindow ============================================
;=============================================================
;=============================================================

PRO AverageWindow, info

    aveID = 0L
    zoomID  = 0L
    aveBase = 0L
    aveLabel = 0L
    charttype  = ""

    chart2controlpos = 0L
    IDAveMinBinSize = 0L
    IDAveMinCountPerBin = 0L
    IDAveXMin = 0L
    IDAveXMax = 0L
    IDAveXLog = 0L
    IDAveYLog = 0L

    AveMinBinSize = 10L
    AveMinCountPerBin = 100L
    AveXMin = 0L
    AveXMax = 100L
    AveXLog = 0b
    AveYLog = 0b

    ave_info = {aveID:aveID, $               ; widget ID for the average graphs
        zoomID:zoomID, $
        aveBase:aveBase, $
        aveLabel:aveLabel, $
        IDAveMinBinSize:IDAveMinBinSize, $
        IDAveMinCountPerBin:IDAveMinCountPerBin, $
        IDAveXMin:IDAveXMin, $
        IDAveXMax:IDAveXMax , $
        IDAveXLog:IDAveXLog, $
        IDAveYLog:IDAveYLog, $
        AveMinBinSize:AveMinBinSize, $
        AveMinCountPerBin:AveMinCountPerBin, $
        AveXMin:AveXMin, $
        AveXMax:AveXMax , $
        AveXLog:AveXLog, $
        AveYLog:AveYLog  $

        }




    base = WIDGET_BASE(/ROW, title="Averaged Results", /TLB_SIZE_EVENTS, /KBRD_Focus_Events,  /CONTEXT_EVENTS)
    ave_info.aveBase = base
    info.aveBase = base
    averages = Widget_Draw(base,XSize=600, YSize=500,Event_Pro='SPEC_Ave_event', /BUTTON_EVENTS, /MOTION_EVENTS)

    ave_info.aveID = averages
    info.aveID = averages         ; we need to access this widget from the main window too



    ;---------------------------------------------------


    AVE_QuadWindow = WIDGET_BASE(base, $
        ROW=10, $
        MAP=1, $
        UVALUE='BASE_Toolbar', $
        BASE_ALIGN_LEFT=1, $
        XSIZE = 100, $
        YSIZE=300)

    templabel = WIDGET_LABEL(AVE_QuadWindow, $
        VALUE="Settings", $
        XSIZE=90, $
        YSIZE=14)


    ;===========================================

    tlb_cb = Widget_Base(AVE_QuadWindow, Column=1, Scr_XSize=200, /NonExclusive)
    ave_info.IDAveXLog = Widget_Button(tlb_cb, Value='X Log')
    ave_info.IDAveYLog = Widget_Button(tlb_cb, Value='Y Log')
    Widget_Control, ave_info.IDAveXLog, Set_Button=0
    Widget_Control, ave_info.IDAveYLog, Set_Button=0

    tlb_tx = Widget_Base(AVE_QuadWindow, Column=1, xsize = 150)

    templabel = WIDGET_LABEL(tlb_tx, VALUE="Min X",XSIZE=50, ALIGN_LEFT = 1 ,YSIZE=14)
    IDAveXMin = Widget_TEXT(tlb_tx, EDITABLE=1, frame=40,  Scr_XSize = 50, Value='0')
    templabel = WIDGET_LABEL(tlb_tx, VALUE="Max X",XSIZE=50, ALIGN_LEFT = 1, YSIZE=14)
    IDAveXMax = Widget_TEXT(tlb_tx, EDITABLE=1, frame=40,  Scr_XSize = 50, Value='100')

    templabel = WIDGET_LABEL(AVE_QuadWindow, $
        VALUE="", $
        XSIZE=90, $
        YSIZE=40)

    templabel = WIDGET_LABEL(tlb_tx, VALUE="Min.Bin.Size",XSIZE=90, ALIGN_LEFT = 1, YSIZE=14)
    IDAveMinBinSize = Widget_TEXT(tlb_tx, EDITABLE=1, frame=40,  Scr_XSize = 50, Value='10')

    templabel = WIDGET_LABEL(tlb_tx, VALUE="Min.Counts/Bin",XSIZE=90, ALIGN_LEFT = 1, YSIZE=14)
    IDAveMinCountPerBin = Widget_TEXT(tlb_tx, EDITABLE=1, frame=40,  Scr_XSize = 50, Value='100')

    ave_info.IDAveXMin = IDAveXMin
    ave_info.IDAveXMax = IDAveXMax
    ave_info.IDAveMinBinSize = IDAveMinBinSize
    ave_info.IDAveMinCountPerBin = IDAveMinCountPerBin

    BUTTON_AveUpdate = WIDGET_BUTTON( AVE_QuadWindow, $
        FONT='9x15', $
        UVALUE='', $
        XSIZE=65, $
        TOOLTIP="Update", $
        VALUE="UPDATE", $
        Event_Pro='AveChartUpdate_event')



    ;--------------------------------------------------
    ;  Begin widgets for the lower average graph
    ;--------------------------------------------------



    WIDGET_CONTROL, base, /REALIZE, SHOW=0
    erase,color='ffffff'x  ; get background to white

    widget_control,ave_info.aveID,map=0  ; for now, hide the average window

    XMANAGER, 'AverageWindow', base, /JUST_REG,  Event_Handler='ave_events', /no_block


    Widget_Control, base, Set_UValue=ave_info, /No_Copy

END



; TAF 2014-09-22 v1.1.29 the functions below were moved to processparticle.pro, to be more cosistent with basescan

; pro index2xy, m, index, x, y
; function getangle, p1x, p1y, p2x, p2y, p3x, p3y
; PRO ProcessParticle, image_data, image, WordCount, Slices, Sums, L2, area, Midpoint, Edges, Err,$
;                     L3, L4, L5, PC3x, PC3y, PC4x, PC4y, A3, Roundness1, Roundness2, Roundness3, L1r, maxLength, maxWidth, num_perimeter_points, crystal_type, perimeterum, csp, $
;                     ml_p1x=ml_p1x, ml_p1y=ml_p1y, ml_p2x=ml_p2x, ml_p2y=ml_p2y, mw_p1x=mw_p1x, mw_p1y=mw_p1y, mw_p2x=mw_p2x, mw_p2y=mw_p2y, $
;                     perimeter_points_x=perimeter_points_x, perimeter_points_y=perimeter_points_y, num_satellites=num_satellites, HorV=HorV




PRO UpdateTimeSeries, info


    if   info.CurrentChart1 EQ !H_CHART then begin
        Widget_Control, info.drawID, Get_Value=wid
        WSet, wid
        SPEC_Hplot, info
    end


    if   info.CurrentChart1 EQ !V_CHART then begin

        SPEC_Vplot, info
    end


    if   info.CurrentChart1 EQ !NO_CHART then begin ; default to horizontal data

        Widget_Control, info.drawID, Get_Value=wid
        WSet, wid
        SPEC_Hplot, info
    end

END ; UpdateTimeSeries



pro a3vcpiview_Events, event

    ;return

    ;print , "In Event"
    theEvent = Tag_Names(event, /Structure_Name)


    if theEvent EQ 'WIDGET_BASE' then Begin
        ;----------- Handle resize event

        Widget_Control, event.top, Get_UValue=info, /No_Copy  ; get our info structure


        info.xscale(1) = event.x
        info.yscale(1) = event.y
        !CHAR_SIZE = (event.x / 300.0) * 0.3
        Widget_Control, info.drawID, Draw_XSize=event.x, Draw_YSize=event.y



        UpdateTimeSeries, info
        Widget_Control, event.top, Set_UValue=info, /No_Copy  ; release our info structure

    endif



    IF theEvent EQ 'WIDGET_KBRD_FOCUS' THEN BEGIN

        ; If losing keyboard focus, do nothing and RETURN.

        IF event.enter EQ 0 THEN RETURN

        ; Get the info structure and copy it here.

        Widget_Control, event.top, Get_UValue=info, /No_Copy

        ; Load the program's colors.

        ; TVLCT, info.r, info.g, info.b

        ; If this is other than 8-bit display, redraw graphic.

        ; Device, Get_Visual_Depth=theDepth
        ; IF theDepth GT 8 THEN BEGIN
        ;    WSet, info.wid
        ;    SpecGraph, *info.image, $
        ;     AxisColorName = info.axisColorName, $
        ;     BackColorName = info.backcolorName, $
        ;     Binsize = info.binsize, $
        ;     ;ColorTable=info.colortable, $
        ;     NoLoadCT=1, $
        ;     DataColorName=info.datacolorName, $
        ;     _Extra=*info.extra, $
        ;     ImageColors=info.imagecolors, $
        ;     Max_Value=info.max_value, $
        ;     XScale=info.xscale, $
        ;     YScale=info.yscale

        ; ENDIF

        ; Put the info structure back in its storage location.

        Widget_Control, event.top, Set_UValue=info, /No_Copy
    ENDIF

end ;======================================

;======================================================================================
;=== S A V E ==========================================================================
;======================================================================================
pro SPEC_Toolbar_Save, info, Kth;   TAF: FALSE statement to right, see below!: "This subroutine is for loading one sorted file only. for Batch archive, it does not comes to here."
    Common TASBlock, extTime,extTas


    if !use_external_TAS eq 1 and Kth eq 0 then begin;  for multi-archive,  only loads the TAS file once only.
        ;load a external data file to get a data file of two columns  of Time and NavTAS values. The Time is seconds, TAS is meter/S.
        print, ' load the external Time and TAS to replace the wrong TAS data imbeded in the sorted*.2DS data file .....'
        rr=dialog_message('Use external TAS  Keyword is entered, Please select a two column (Time, TAS(m/s)) ascii for your input!!',/information)
        load_navigation_time_tas_two_column_data; extTime and extTas is store in common data block alreay.
    endif

    ;first get the HK_tas from that whole ascii file.
    ; TAF this call is not needed for files with valid TAS, it was a field work-around for a DC8 project
    ;Get_dc8_TAS_time_series,info, Kth, HKseconds, HKtas
    ;note that info.currentPname='C:\DC8_Test_L1n\'; where is teh sorted file place.

    info.TotalParticles=0UL
    errcount = 0L

    struct = {_DerivedDataRev2, $
        PartID: 0 , $
        Nanosecsold:0ULL , $
        NanoSeconds:0ULL , $
        ImageOffset: 0UL , $
        ImageWords: 0 , $
        Area: 0.0 , $
        Nslice: 0 , $
        L2: 0 , $
        F1: 0 , $
        Adj1: 0.0, $
        Pc1: 0.0, $
        TAS: 0.0, $
        ProbeCounter:0ULL, $
        L3: 0.0, $
        L4: 0, $
        L5: 0, $
        PC3x: 0.0, $
        PC3y: 0.0, $
        PC4x: 0.0, $
        PC4y: 0.0, $
        A3: 0.0, $
        Alexei1: 0.0, $
        Alexei2: 0L, $
        reject: byte(0), $
        w8_t: 0LL, $
        L7: 0.0, $
        L8: 0.0, $
        roundness: 0.0, $
        focus: 0.0, $
        warn: 0.0, $  ; some day we will change this to byte value to svae memory.
        adj0: 1.0, $  ;added on May 8-2007 for w8_t cleaning.
        noisebyte:byte(0),$;; added in June 2007.
        AccDTtick:0ULL,$   ; added on oct5 2007
        w8_t_clean:0LL, $; added on oct5,2007.
        L1n: 0.0, $;    ; added on nov 9, 2007.
        largest_satellite_area: 0.0, $
        alexei1old: 0.0, $ ;added dec19,2007
        TASold: 0.0, $    ; added dec19,2007
        Roundness1: 0.0, $
        Roundness2: 0.0, $
        Roundness3: 0.0, $
        L1r: 0.0, $           ; Diameter of reconstructed images in pixels
        maxLength: 0.0, $     ; max chord length in pixels
        maxWidth: 0.0, $      ; 90 degrees to max chord length, max width in pixels
        perimeter: 0, $       ; number of perimeter pixels
        crystal_type: 0, $    ; crystal type flag, same as CPI View, see comment below for type codes
        perimeterum: 0.0, $   ; perimeter length in microns
        csp: 0.0, $           ; Combined Single Parameter: CSP = AW(2L + 2W)/P   or in IDL:   csp = Area * maxWidth * (2.*maxLength + 2.*maxWidth) / perimeterum
        num_satellites: 0.0, $
        round123: 0.0, $
        spare5: 0.0, $
        CPI_trigger: 0}
    ;add this tempary to fix the bug for generating the zoomed ascii
    ;**************************************************************************
    struct_ascii = {_AsciiData, $
        Nanosecsold:0ULL , $
        NanoSeconds:0ULL , $
        Area: 0.0 , $
        Nslice: 0 , $
        L2: 0 , $
        F1: 0 , $
        Adj1: 0.0, $
        Pc1: 0.0, $
        TAS: 0.0,$    ;added more in the following on oct 14 , 2005 by mo for outputing ascii data file.
        ProbeCounter:0ULL, $
        L3: 0.0, $
        L4: 0, $
        L5: 0, $
        PC3x: 0.0, $
        PC3y: 0.0, $
        PC4x: 0.0, $
        PC4y: 0.0, $
        A3: 0.0, $
        Alexei1: 0.0, $
        Alexei2: 0L,$
        L1n:0.0, $;  added nov 9, 2007.
        largest_satellite_area: 0.0, $    ;added dec19,2007
        alexei1old: 0.0, $ ;added dec19,2007
        TASold: 0.0 $    ; added dec19,2007
        }


    ;***************************************************************************



    struct2 = {_Timestamp, $
        Year: 0, $
        Month: 0, $
        Day: 0, $
        Hours: 0, $
        Minutes: 0, $
        Seconds: 0, $
        Milliseconds: 0, $
        Microseconds: 0, $
        Nanoseconds: 0 }

    horiz_count=0
    NanosecondsPerSecond = 1000000000ULL
    NanosecondsPerMinute = NanosecondsPerSecond * 60ULL
    NanosecondsPerHour   = NanosecondsPerMinute * 60ULL
    NanosecondsPerDay    = NanosecondsPerHour * 24ULL
    ;out_array = BytArr(20000) TAF this is another incorrect out_array allocation


    ;Widget_Control, event.top, Get_UValue=info, /No_Copy  ; get our info structure

    print, "File name is currently: ", info.currentFname

    ;get the sorted file's starttime and endtime
    substrs=strsplit(info.currentFname,'\',/extract)
    sortedFileName=substrs(n_elements(substrs)-1)
    datetimestring=strmid(sortedFilename,6,strlen(sortedfilename)-9); =yymmddhhmmss_hhmmss_hhmmss_000x

    if strlen(datetimestring) eq 32 then begin
        subtrs=strsplit(datetimestring,'_',/extract)
        Stime=subtrs(1)
        Etime=subtrs(2)
        FSTime=(strmid(Stime,0,2)*3600LL+strmid(Stime,2,2)*60LL+strmid(Stime,4,2)*1LL)*1000000000;nanoseconds
        FETime=(strmid(Etime,0,2)*3600LL+strmid(Etime,2,2)*60LL+strmid(Etime,4,2)*1LL)*1000000000
    endif;     FStime is full-seconds say: 50045000000000LL nanoseconds. FEtime ends with 12345.999999999

    WriteH_OK=0; once the the first double-fake particle's time is readed, it is Ok to write to the bin file.
    WriteV_Ok=0


    ; Put the info structure back in its storage location.

    Widget_Control, info.statusID, DYNAMIC_RESIZE=1, Set_Value="Opened: " + info.currentFname
    OpenR, lun, info.currentFname, /Get_Lun

    fileheader = bytarr(256)
    ReadU, lun, fileheader
    v_string = strmid(string(fileheader),4,4)
    reads, v_string, FORMAT='(%"%f")',cur_version
    if (cur_version ne 3.0) then begin
        r = dialog_message(['Warning, file you have selected is not a REV 3 SORTED file!','Use an older version of 2DSView to process this file..'],TITLE = 'File incompatible')
        close, lun
        return
    endif



    ; Test the SORTED file for the 50 micron H channel flag, vs the 10 micron H channel
    ; this section is called during Archiving ASCII

    ; The SORTED file header currently looks like this (Ver string below, annotated):

    ;                                                  49th character
    ;                                                  |      56th character
    ;                                                  |      |
    ; Ver 3.00 SORTED PARTICLE DATA   Realtime Version 1.1.17 H50Micron = False

    ; Thus strmid(string(fileheader), 56, 17) will extract: "H50Micron = False", "H50Micron = True"
    ; and strmid(string(fileheader), 49, 6) will extract: 1.1.17

    ; However, switching to tokenizing the string will be more robust to changes in character length, position


    if(!H_res_set eq 0) then begin ; test to ensure the pixel resolution is set only once, may consider moving this into the function, though unneeded function calls are logic errors, not sure
        if(!manual_override_H50vs10micron eq 0) then begin

            print, ""
            print, "determine pixel resolution automatically from SORTED, or fail here"

            Playback_version_string = strmid(string(fileheader), 49, 6)
            Playback_50_micron_flag = strmid(string(fileheader), 56, 17)

            print, ""
            print, "Playback Version from SORTED file: ", Playback_version_string
            print, Playback_50_micron_flag

            if(strcmp(Playback_50_micron_flag, "H50Micron = True ")) then begin
                print, "H 50 Micron detected in SORTED file"
                !H_resolution_microns = 50
            endif else if(strcmp(Playback_50_micron_flag, "H50Micron = False")) then begin
                print, "H 10 Micron detected in SORTED file"
                !H_resolution_microns = 10
            endif else begin
                print, ""
                print, "H50Micron flag not detected in SORTED file, switching to manual H 10 vs 50 micron selection"
                print, ""

                H50micron_pop_up
            endelse
        endif

        define_Pixel_resolutions
    endif

    basefile=file_basename(info.currentFname)
    substrs=strsplit(basefile,'.',/extract)

    DataKstr="Data"+strmid(substrs(0),6, strlen(substrs(0))-6)+"\"
    FILE_MKDIR,info.currentPname + DataKstr
    OpenW, horiz_lun, info.currentPname + DataKstr + "horiz" + info.datestamp + ".bin", /Get_Lun
    OpenW,  vert_lun, info.currentPname + DataKstr + "vert"  + info.datestamp + ".bin", /Get_Lun
    OpenW,    hk_lun, info.currentPname + DataKstr + "onehz" + info.datestamp + ".bin", /Get_Lun
    OpenW,  mask_lun, info.currentPname + DataKstr + "mask"  + info.datestamp + ".bin", /Get_Lun


    ;make a warning file when reading the sorted file if some obnormal thing happens.
    warningfile=info.currentPname + "sorted" + info.datestamp + "_warn.txt"
    ;OpenW, wlun, warningfile, /get  ; TAF might modify warning 22 to wlun, /get
    OpenW, 22, warningfile
    ;===========================================================
    ; Create Headers for the Ver 3.0 files
    ;===========================================================
    vheader = bytarr(256)
    vheader(*)=32 ; all spaces
    sheader = string(vheader)

    ;stop

    if!H_resolution_microns eq 10 then begin
        strput,sheader,'Ver 3.00 BIN PARTICLE DATA H50Micron = False ' + string(!SPEC_VERSION), 0  ; put info on the front
    endif else begin
        strput,sheader,'Ver 3.00 BIN PARTICLE DATA H50Micron = True  ' + string(!SPEC_VERSION), 0  ; put info on the front
    endelse


    WRITEU, horiz_lun, sheader
    WRITEU,  vert_lun, sheader
    WRITEU,    hk_lun, sheader
    WRITEU,  mask_lun, sheader
    ;stop

    ; TAF defining the input data field types here -- need to change for 48 bit timestamps on 3V-CPI
    ; some confusion about the Timestamps used here and the hi-res timestamp
    ; -- looks like "TimingWords" here are the 2D-S 32 bit values, though they are stored in UIntArr(2)
    ;    16 bit values?  Then the "_Timestamp" values are a struct storing the individual day:hour:minute:second:millisecond:microsecond:nanosecond

    DataType = 0  ; header of all record types is an integer with repeated letters
    TimingWord = UIntArr(2)
    StartTimingWord = UIntArr(2)
    EndTimingWord = UIntArr(2)
    Masks = UIntArr(16)
    HK_ints = UIntArr(48)   ; changed from 40->52 with version 5.0
    HK_timingwords = UIntArr(2)
    HK_TAS  = 0.0
    TAS = 0.0
    Tas_H=0.0
    Tas_V=0.0
    PartID = 0
    Slices = 0U
    WordCount =  0U
    Timestamp = { _Timestamp }
    HTimestamp = { _Timestamp }
    VTimestamp = { _Timestamp }
    StartTimestamp = { _Timestamp }
    EndTimestamp = { _Timestamp }
    pos = 0UL
    Area = 0UL
    Midpoint = 0
    Edges = 0

    pxsV = 0.01
    pxsV_in_microns = 10.0
    dwwV = !DWW_main; 63.0mm in 2D-S, 50.8 mm in 3V-CPI adjust in a3VCPIView.pro
    sa_defaultV = 128.0 * pxsV *  dwwV
    strbszV =0.01;   mm
    strbszV_squared =strbszV*strbszV ;   mm^2

    ; need to detect 50 microns here too
    pxsH = 0.05;  mm
    pxsH_in_microns = 50.0
    dwwH = !DWW_main; 63.0mm in 2D-S, 50.8 mm in 3V-CPI adjust in a3VCPIView.pro   ??????
    sa_defaultH = 128.0 * pxsH *  dwwH
    strbszH =0.05; mm
    strbszH_squared =strbszH*strbszH ;   mm^2

    Rollover = 0 ; 16 bit integer
    Rollover2= 0;  16 bit integer
    ProbeCount  = 0uL ; 32 bit unsigned lo 32
    ProbeCount2 = 0uL ; 32 bit unsigned hi 32
    CPI_trigger = 0 ; 16 bit integer
    temp_64 = 0ULL ; 32 bit result
    OneRollover = 4294967296ULL  ; 2^32
    onebyte = 33B

    L3 = 0.0
    L4 = 0
    L5 = 0
    PC3x = 0.0
    PC3y = 0.0
    PC4x = 0.0
    PC4y = 0.0
    A3 = 0.0
    lastH=10; keep a record of the last H particle written to the bin file.  possible values
    ; are 0, -104, -103, -101, -106, -108. 0 means it is a good particle. for wipe off decision.
    lastV=10



    ; TAF added to prevent empty Window 0 from popping up from device call
    Widget_Control, info.drawID, Get_Value=wid
    WSet,  wid
    WShow, wid

    device, CURSOR_STANDARD= !HOURGLASS_CURSOR  ; show hourglass while we process the data

    ;********************************************************
    OD = {_DerivedDataRev2}
    OD_block_size= N_TAGS( OD , /DATA_LENGTH )
    ;added above so that bad-data can be deleted. Oct 4, 2006.--by mo
    ;******************************************************
    backward=0

    counter = 0L
    skip_counter = 0L


    ; store the file size for comparison before each read -- this is for very careful file reads to ensure batch processing
    ; goes uniterrupted by corrupted or faulty files

    fstat_struct = fstat(lun)
    file_size = fstat_struct.size ; size in bytes
    point_lun, -lun, current_pos

    ;catch, error_status

    ;if(error_status ne 0) then begin
    ;    print, "error caught" + str(error_status)
    ;    stop
    ;    catch, /cancel
    ;endif

    ; Main SORTED File Read Loop

    WHILE(~EOF(lun) and (current_pos + 1 lt file_size)) DO BEGIN

        counter++
        if(counter mod 1000 eq 0) then wait, .001 ; pause for a moment to avoid spin locking IDL on read errors (allows breaking and GUI refreshes)

        ; test explicitly for end of file before each read
        ;next_read_length = n_tags(DataType, /data_length)
        next_read_length = 2

        point_lun, -lun, current_pos

        ; TAF break the while loop if the next read will exceed the file size, likely put this in place of the stops below too
        if(current_pos + 1 ge file_size) then begin
            print, "current_pos = ", current_pos, "file_size = ", file_size, "breaking"
            break
        endif

        if(current_pos + next_read_length gt file_size) then begin
            print, "file size exceeded reading datatype"
            stop
        endif

        ReadU, lun, DataType

        ;---------------------------
        ;added this block to handle new type of dummy particles that Chris added in september
        ;19th 2006, for those bad images ----tick or time is not monotonic.

        ; The 4 new deadtime indicaters are defined as:

        ;  V_DUMMY_DEADTIME_HEADER = &H1414 ' 5140 decimal   ; treat them as normal deadtime
        ;  H_DUMMY_DEADTIME_HEADER = &H1424 ' 5156 decimal   ; treat them as normal deadtime
        ;  V_BACKUP_DEADTIME_HEADER = &H2414 ' 9236 decimal  ; when I see this, throw out earlier
        ;  H_BACKUP_DEADTIME_HEADER = &H2424 ' 9252 decimal  ; raeded H or V particles.

        ; TAF would be nice to get the first TAS for the first particles, or have Playback write a TAS_HEADER as the first packet
        ;if(TAS eq 0.) then begin
        ;    print, "warning using HKTAS[0], from HK_TXT*.2DSCPI file"
        ;    print, "temporary workaround until Playback TAS_HEADERs are implemented"

        ;    TAS = hktas[0]
        ;endif


        ; main read SORTED data files

        SWITCH DataType OF
            5156: $    ;this way all dummy_dead_times are assigned as mask_flags.
                dummy_value = 42
            6425: $;   H_streaker_deadTime_header; no more backup type anymore.
                begin
            ;if datatype eq 6425 then  print, 'H_streaker_DEADTIME_HEADER'
            ; do delete last readed in H-particles from the array.
            ; point_lun,-horiz_lun, current_posH
            ; point_lun, horiz_lun,current_posH-OD_block_size
            ;    stop
            ; truncate_lun,Horiz_lun;   after we truncate it, we replaced it with the deadtime mark from the sorted
            ;    stop
            ;write out new type of fake particles -105 -106 for area


            ; test explicitly for end of file before each read
            next_read_length = n_tags(StartTimestamp, /data_length) + n_tags(EndTimestamp, /data_length)

            point_lun, -lun, current_pos

            if(current_pos + next_read_length gt file_size) then begin
                print, "file size exceeded in datatype", datatype
                stop
            endif

            ReadU, lun, StartTimestamp, EndTimestamp

            ; test explicitly for end of file before each read
            ;next_read_length = n_tags(Rollover,    /data_length) + n_tags(ProbeCount, /data_length) $
            ;                 + n_tags(ProbeCount2, /data_length) + n_tags(rollover2,  /data_length)

            next_read_length = 2 + 4 + 4 + 2

            point_lun, -lun, current_pos

            if(current_pos + next_read_length gt file_size) then begin
                print, "file size exceeded in datatype", datatype
                stop
            endif

            ReadU, lun, ProbeCount2, ProbeCount

            OD = {_DerivedDataRev2}


            OD.NanoSeconds = StartTimestamp.Nanoseconds                           + $
                (StartTimestamp.Microseconds * 1000ULL)              + $
                (StartTimestamp.Milliseconds * 1000000ULL)           + $
                (StartTimestamp.Seconds      * NanosecondsPerSecond) + $
                (StartTimestamp.Minutes      * NanosecondsPerMinute) + $
                (StartTimestamp.Hours        * NanosecondsPerHour)

            OD.Nanosecsold=OD.nanoseconds
            if datatype eq 5156 then OD.Area = -105 else OD.area=-107; -107 is bad type_6425
            OD.TAS = tas_H;  added "+tas" by mo at oct 5, 2006.
            OD.TASold=OD.TAS

            ;OD.ProbeCounter = Rollover * OneRollover + ProbeCount
            ;OD.ProbeCounter = ProbeCount2 * OneRollover + ProbeCount
            OD.ProbeCounter = ishft(Long64(ProbeCount2), 32) or Long64(ProbeCount)
            OD0=OD

            OD.NanoSeconds = EndTimestamp.Nanoseconds                           + $
                (EndTimestamp.Microseconds * 1000ULL)              + $
                (EndTimestamp.Milliseconds * 1000000ULL)           + $
                (EndTimestamp.Seconds      * NanosecondsPerSecond) + $
                (EndTimestamp.Minutes      * NanosecondsPerMinute) + $
                (EndTimestamp.Hours        * NanosecondsPerHour)

            OD.Nanosecsold=OD.nanoseconds ;= EndTimestamp.Year * 12 + EndTimestamp.Month
            if datatype eq 5156 then OD.Area = -106 else OD.area=-108; -108 is bad type_6425
            OD.TAS = tas_H
            OD.TASold=OD.TAS


            ReadU, lun, ProbeCount2, ProbeCount

            OD.ProbeCounter = ishft(Long64(ProbeCount2), 32) or Long64(ProbeCount)

            if OD.probecounter lt OD0.probecounter then begin
                OD.probecounter=OD0.probecounter
                OD.nanoseconds =OD0.nanoseconds
                OD.area=OD.area-100
                OD0.area=OD0.area-100
                printf,22,'at time=',OD.nanoseconds,' H Fake pair"s  end-tick less than start-tick  '
            endif

            writeu, horiz_lun,OD0
            WRITEU, horiz_lun, OD
            lastH=-106;  -106 and -108 is of the same type.

            break
        end  ; end of H-backup-deadtime particles.
        9252: $;   H_backup_deadTime_header
            begin
        ;print, 'H_Backup_DEADTIME_HEADER'

        ; do delete last readed in V-particles from the array.
        if lastH eq 0 then begin;  just romved one particle which is a good particle but time is wrong.
            point_lun,-horiz_lun, current_posH;   get the current file pointer position.
            point_lun, horiz_lun,current_posH-OD_block_size;  Note that OD_block_size is assigned
            ;block_size =174 bytes , it changes depending *****
            truncate_lun,Horiz_lun

        endif

        if lastH ne -104 and lastH lt 0 then begin;  need to remove the last pair of fake particles.
            point_lun,-horiz_lun, current_posH
            point_lun, horiz_lun,current_posH-2*OD_block_size;  Note that OD_block_size is assigned
            ;block_size =163 bytes ?
            truncate_lun,Horiz_lun

        endif

        ; begin reading the two time stamps and counts to write two bad particles.

        ; test explicitly for end of file before each read
        next_read_length = n_tags(StartTimestamp, /data_length) + n_tags(EndTimestamp, /data_length)

        point_lun, -lun, current_pos

        if(current_pos + next_read_length gt file_size) then begin
            print, "file size exceeded in datatype", datatype
            stop
        endif

        ReadU, lun,  StartTimestamp,EndTimestamp

        ; test explicitly for end of file before each read
        ;next_read_length = n_tags(Rollover,    /data_length) + n_tags(ProbeCount, /data_length) $
        ;                 + n_tags(ProbeCount2, /data_length) + n_tags(rollover2,  /data_length)

        next_read_length = 2 + 4 + 4 + 2

        point_lun, -lun, current_pos

        if(current_pos + next_read_length gt file_size) then begin
            print, "file size exceeded in datatype", datatype
            stop
        endif

        ReadU, lun, ProbeCount2, ProbeCount

        OD = {_DerivedDataRev2}


        OD.NanoSeconds = StartTimestamp.Nanoseconds + $
            (StartTimestamp.Microseconds * 1000ULL) + $
            (StartTimestamp.Milliseconds * 1000000ULL) + $
            (StartTimestamp.Seconds * NanosecondsPerSecond) + $
            (StartTimestamp.Minutes * NanosecondsPerMinute) + $
            (StartTimestamp.Hours * NanosecondsPerHour)
        OD.Nanosecsold=OD.nanoseconds ;= EndTimestamp.Year * 12 + EndTimestamp.Month
        OD.Area = -109
        OD.TAS = tas_H
        OD.TASold=OD.TAS

        ;OD.ProbeCounter = Rollover * OneRollover + ProbeCount
        ;OD.ProbeCounter = ProbeCount2 * OneRollover + ProbeCount
        OD.ProbeCounter = ishft(Long64(ProbeCount2), 32) or Long64(ProbeCount)

        OD0=OD

        OD.NanoSeconds = EndTimestamp.Nanoseconds + $
            (EndTimestamp.Microseconds * 1000ULL) + $
            (EndTimestamp.Milliseconds * 1000000ULL) + $
            (EndTimestamp.Seconds * NanosecondsPerSecond) + $
            (EndTimestamp.Minutes * NanosecondsPerMinute) + $
            (EndTimestamp.Hours * NanosecondsPerHour)
        OD.Nanosecsold=OD.nanoseconds ;= EndTimestamp.Year * 12 + EndTimestamp.Month
        OD.Area = -110

        OD.ProbeCounter = ishft(Long64(ProbeCount2), 32) or Long64(ProbeCount)

        if OD.probecounter lt OD0.probecounter then begin
            OD.probecounter=OD0.probecounter
            OD.nanoseconds =OD0.nanoseconds
            OD.area=OD.area-100
            OD0.area=OD0.area-100
            printf,22,'at time=',OD.nanoseconds,' H Fake pair"s  end-tick less than start-tick  '
        endif
        writeu, horiz_lun,OD0
        WRITEU, horiz_lun, OD
        lastH=-110

        break
    end; end of wipping off H-bad non_monotonic data and puting a deadtime pair inside.
    27756 :$   ; H_DEADTIME_HEADER (0x6C6C)
        dummy_value = 42
    ;        begin
    ;            ;dummy_value = 42
    ;            ; Put #SORTED_FNUM, , V_DEADTIME_HEADER
    ;            ; Put #SORTED_FNUM, , First_Overflow_Timestamp    'timestamp from first overflow
    ;            ; Put #SORTED_FNUM, , Second_Overflow_Timestamp
    ;            ; Put #SORTED_FNUM, , gPart_V.v_timing_word_hi
    ;            ; Put #SORTED_FNUM, , gPart_V.v_timing_word
    ;            ; Put #SORTED_FNUM, , First_Overflow_gPart_V.v_timing_word_hi
    ;            ; Put #SORTED_FNUM, , First_Overflow_gPart_V.v_timing_word

    ;            ; TAF 2013-10-08 handling of new Playback data types H and V_DEADTIME_HEADER, stub for now

    ;            print, "stub handling H_DEADTIME_HEADER"

    ;            next_read_length = n_tags(StartTimestamp, /data_length) + n_tags(EndTimestamp, /data_length)

    ;            point_lun, -lun, current_pos

    ;            if(current_pos + next_read_length gt file_size) then begin
    ;                print, "file size exceeded in datatype", datatype
    ;                stop
    ;            endif

    ;            ReadU, lun, StartTimestamp, EndTimestamp
    ;            ReadU, lun, ProbeCount2, ProbeCount
    ;            ReadU, lun, ProbeCount2, ProbeCount
    ;        break
    ;        end
    7196 : $   ; H_MASK (0x1C1C)
        begin
    ; test explicitly for end of file before each read
    next_read_length = n_tags(StartTimestamp, /data_length) + n_tags(EndTimestamp, /data_length)

    point_lun, -lun, current_pos

    if(current_pos + next_read_length gt file_size) then begin
        print, "file size exceeded in datatype", datatype
        stop
    endif

    ReadU, lun, StartTimestamp, EndTimestamp

    ; test explicitly for end of file before each read
    ;next_read_length = n_tags(Rollover,    /data_length) + n_tags(ProbeCount, /data_length) $
    ;                 + n_tags(ProbeCount2, /data_length) + n_tags(rollover2,  /data_length)

    next_read_length = 2 + 4 + 4 + 2

    point_lun, -lun, current_pos

    if(current_pos + next_read_length gt file_size) then begin
        print, "file size exceeded in datatype", datatype
        stop
    endif

    ReadU, lun, ProbeCount2, ProbeCount ; start probe counter



    ;if probecount2 eq probecount and rollover eq rollover2  then begin
    ;    break;  added this for verion 992 on Dec 15, 2006.
    ;endif

    ;if datatype eq 7196 then  print , "H MASK"
    OD = {_DerivedDataRev2}

    OD.NanoSeconds = StartTimestamp.Nanoseconds + $
        (StartTimestamp.Microseconds * 1000ULL) + $
        (StartTimestamp.Milliseconds * 1000000ULL) + $
        (StartTimestamp.Seconds * NanosecondsPerSecond) + $
        (StartTimestamp.Minutes * NanosecondsPerMinute) + $
        (StartTimestamp.Hours * NanosecondsPerHour)

    OD.Nanosecsold=OD.nanoseconds ;= EndTimestamp.Year * 12 + EndTimestamp.Month

    if DataType eq 27756 then OD.Area = !DEADTIME_START_FLAG else OD.Area = !MASK_START_FLAG
    OD.TAS = tas_H;  added "+tas" by mo at oct 5, 2006.
    OD.TASold=OD.TAS

    ;OD.ProbeCounter = Rollover * OneRollover + ProbeCount
    ;OD.ProbeCounter = ProbeCount2 * OneRollover + ProbeCount
    OD.ProbeCounter = ishft(Long64(ProbeCount2), 32) or Long64(ProbeCount)
    Tick0=OD.probecounter

    ;WRITEU, horiz_lun, OD
    OD0=OD


    ; TAF note 2013-10-22:  read end counter into different variables and then save into OD
    ReadU, lun, ProbeCount2, ProbeCount ; end probe counter

    OD.NanoSeconds = EndTimestamp.Nanoseconds + $
        (EndTimestamp.Microseconds * 1000ULL) + $
        (EndTimestamp.Milliseconds * 1000000ULL) + $
        (EndTimestamp.Seconds * NanosecondsPerSecond) + $
        (EndTimestamp.Minutes * NanosecondsPerMinute) + $
        (EndTimestamp.Hours * NanosecondsPerHour)

    OD.Nanosecsold=OD.nanoseconds ;= EndTimestamp.Year * 12 + EndTimestamp.Month

    if DataType eq 27756 then OD.Area = !DEADTIME_END_FLAG else OD.Area = !MASK_END_FLAG

    OD.TAS = tas_H
    OD.TASold=OD.TAS

    ;OD.ProbeCounter = Rollover2 * OneRollover + ProbeCount2
    ;OD.ProbeCounter = ProbeCount2 * OneRollover + ProbeCount
    OD.ProbeCounter = ishft(Long64(ProbeCount2), 32) or Long64(ProbeCount)

    if OD.probecounter lt OD0.probecounter then begin
        OD.probecounter=OD0.probecounter
        OD.nanoseconds =OD0.nanoseconds
        OD.area=OD.area-100
        OD0.area=OD0.area-100
        printf,22,'at time=',OD.nanoseconds,' H Fake pair"s  end-tick less than start-tick  '
    endif

    writeu, horiz_lun,OD0
    WRITEU, horiz_lun, OD
    lastH=-103;   -101 and -103 is the of the same type.

    break
end ; end of H-dead_time and H-mask
5140: $   ; V_DUMMY_DEADTIME_HEADER = &H1414 ' 5140 decimal
    dummy_value = 42
6168: $;   H_streaker_deadTime_header; no mre backup type anymore.
    begin
;if datatype eq 6168 then  print, 'V_streaker_DEADTIME_HEADER'
; begin reading the two time stamps and counts to write two bad particles.

; test explicitly for end of file before each read
next_read_length = n_tags(StartTimestamp, /data_length) + n_tags(EndTimestamp, /data_length)

point_lun, -lun, current_pos

if(current_pos + next_read_length gt file_size) then begin
    print, "file size exceeded in datatype", datatype
    stop
endif

ReadU, lun,  StartTimestamp,EndTimestamp

; test explicitly for end of file before each read
;next_read_length = n_tags(Rollover,    /data_length) + n_tags(ProbeCount, /data_length) $
;                 + n_tags(ProbeCount2, /data_length) + n_tags(rollover2,  /data_length)

next_read_length = 2 + 4 + 4 + 2

point_lun, -lun, current_pos

if(current_pos + next_read_length gt file_size) then begin
    print, "file size exceeded in datatype", datatype
    stop
endif

ReadU, lun, ProbeCount2, ProbeCount

OD = {_DerivedDataRev2}


OD.NanoSeconds = StartTimestamp.Nanoseconds + $
    (StartTimestamp.Microseconds * 1000ULL) + $
    (StartTimestamp.Milliseconds * 1000000ULL) + $
    (StartTimestamp.Seconds * NanosecondsPerSecond) + $
    (StartTimestamp.Minutes * NanosecondsPerMinute) + $
    (StartTimestamp.Hours * NanosecondsPerHour)
OD.Nanosecsold=OD.nanoseconds ;= EndTimestamp.Year * 12 + EndTimestamp.Month

if DataType eq 5140 then OD.Area = -105 else OD.Area =-107; ;-108 is V-streaker.
OD.TAS =tas_V
OD.TASold=OD.TAS
;OD.ProbeCounter = Rollover * OneRollover + ProbeCount
;OD.ProbeCounter = ProbeCount2 * OneRollover + ProbeCount
OD.ProbeCounter = ishft(Long64(ProbeCount2), 32) or Long64(ProbeCount)

tick5=OD.probecounter

;WRITEU, vert_lun, OD
OD0=OD

OD.NanoSeconds = EndTimestamp.Nanoseconds + $
    (EndTimestamp.Microseconds * 1000ULL) + $
    (EndTimestamp.Milliseconds * 1000000ULL) + $
    (EndTimestamp.Seconds * NanosecondsPerSecond) + $
    (EndTimestamp.Minutes * NanosecondsPerMinute) + $
    (EndTimestamp.Hours * NanosecondsPerHour)

OD.Nanosecsold=OD.nanoseconds ;= EndTimestamp.Year * 12 + EndTimestamp.Month

if DataType eq 5140 then OD.Area = -106 else OD.Area =-108; ;-108 is for backuptype Bad.

OD.TAS =TAS_V; added this "+tas" by mo on Oct 5, 2006.
OD.TASold=OD.TAS

;OD.ProbeCounter = Rollover2 * OneRollover + ProbeCount2
;OD.ProbeCounter = ProbeCount2 * OneRollover + ProbeCount

ReadU, lun, ProbeCount2, ProbeCount

OD.ProbeCounter = ishft(Long64(ProbeCount2), 32) or Long64(ProbeCount)

if OD.probecounter lt OD0.probecounter then begin
    OD.probecounter=OD0.probecounter
    OD.nanoseconds =OD0.nanoseconds
    OD.area=OD.area-100
    OD0.area=OD0.area-100
    printf,22,'at time=',OD.nanoseconds,' V Fake pair"s  end-tick less than start-tick  '
endif

writeu, vert_lun,OD0
WRITEU, vert_lun, OD
lastV=-106
break
end; end of type1 and type1 V Bad Particles.
9236: $;   V_backup_deadTime_header
    begin
;print, 'V_Backup_DEADTIME_HEADER'

; do delete last readed in V-particles from the array.
if lastV eq 0 then begin;  just romved one particle which is a good particle but time is wrong.
    point_lun,-vert_lun, current_posV
    point_lun, vert_lun,current_posV-OD_block_size;  Note that OD_block_size is assigned
    ;block_size =163 bytes ?
    truncate_lun,vert_lun
endif

if lastV ne -104 and lastV lt 0 then begin;  need to remove the last pair of fake particles.
    point_lun,-vert_lun, current_posV
    point_lun, vert_lun,current_posV-2*OD_block_size;  Note that OD_block_size is assigned
    ;block_size =163 bytes ?
    truncate_lun,vert_lun
endif

; begin reading the two time stamps and counts to write two bad particles.

; test explicitly for end of file before each read
next_read_length = n_tags(StartTimestamp, /data_length) + n_tags(EndTimestamp, /data_length)

point_lun, -lun, current_pos

if(current_pos + next_read_length gt file_size) then begin
    print, "file size exceeded in datatype", datatype
    stop
endif

ReadU, lun,  StartTimestamp,EndTimestamp

next_read_length = 2 + 4 + 4 + 2

point_lun, -lun, current_pos

if(current_pos + next_read_length gt file_size) then begin
    print, "file size exceeded in datatype", datatype
    stop
endif

ReadU, lun, ProbeCount2, ProbeCount
OD = {_DerivedDataRev2}

OD.NanoSeconds = StartTimestamp.Nanoseconds + $
    (StartTimestamp.Microseconds * 1000ULL) + $
    (StartTimestamp.Milliseconds * 1000000ULL) + $
    (StartTimestamp.Seconds * NanosecondsPerSecond) + $
    (StartTimestamp.Minutes * NanosecondsPerMinute) + $
    (StartTimestamp.Hours * NanosecondsPerHour)
OD.Nanosecsold=OD.nanoseconds ;= EndTimestamp.Year * 12 + EndTimestamp.Month

OD.Area = -109
OD.TAS = tas_V
OD.TASold=OD.TAS


;OD.ProbeCounter = Rollover * OneRollover + ProbeCount
;OD.ProbeCounter = ProbeCount2 * OneRollover + ProbeCount
OD.ProbeCounter = ishft(Long64(ProbeCount2), 32) or Long64(ProbeCount)

;WRITEU, vert_lun, OD
OD0=OD

OD.NanoSeconds = EndTimestamp.Nanoseconds + $
    (EndTimestamp.Microseconds * 1000ULL) + $
    (EndTimestamp.Milliseconds * 1000000ULL) + $
    (EndTimestamp.Seconds * NanosecondsPerSecond) + $
    (EndTimestamp.Minutes * NanosecondsPerMinute) + $
    (EndTimestamp.Hours * NanosecondsPerHour)
OD.Nanosecsold=OD.nanoseconds ;= EndTimestamp.Year * 12 + EndTimestamp.Month
OD.Area = -110
;OD.ProbeCounter = Rollover2 * OneRollover + ProbeCount2
;OD.ProbeCounter = ProbeCount2 * OneRollover + ProbeCount
OD.ProbeCounter = ishft(Long64(ProbeCount2), 32) or Long64(ProbeCount)
if OD.probecounter lt OD0.probecounter then begin
    OD.probecounter=OD0.probecounter
    OD.nanoseconds =OD0.nanoseconds
    OD.area=OD.area-100
    OD0.area=OD0.area-100
    ;print,OD0.area,': ',rollover, probecount
    ;print, OD.area,': ',rollover2, probecount2
    printf,22,'at time=',OD.nanoseconds,' V Fake pair"s  end-tick less than start-tick  '
endif

writeu, vert_lun,OD0
WRITEU, vert_lun, OD
lastV=-110
break
end; end of wipping off bad non_monotonic data and puting a deadtime pair inside.
31868 : $   ; V_DEADTIME (0x7C7C)
    dummy_value = 42
;        begin
;            ;dummy_value = 42
;            ; Put #SORTED_FNUM, , V_DEADTIME_HEADER
;            ; Put #SORTED_FNUM, , First_Overflow_Timestamp    'timestamp from first overflow
;            ; Put #SORTED_FNUM, , Second_Overflow_Timestamp
;            ; Put #SORTED_FNUM, , gPart_V.v_timing_word_hi
;            ; Put #SORTED_FNUM, , gPart_V.v_timing_word
;            ; Put #SORTED_FNUM, , First_Overflow_gPart_V.v_timing_word_hi
;            ; Put #SORTED_FNUM, , First_Overflow_gPart_V.v_timing_word

;            ; TAF 2013-10-08 handling of new Playback data types H and V_DEADTIME_HEADER, stub for now

;            print, "stub handling V_DEADTIME_HEADER"

;            next_read_length = n_tags(StartTimestamp, /data_length) + n_tags(EndTimestamp, /data_length)

;            point_lun, -lun, current_pos

;            if(current_pos + next_read_length gt file_size) then begin
;                print, "file size exceeded in datatype", datatype
;                stop
;            endif

;            ReadU, lun, StartTimestamp, EndTimestamp
;            ReadU, lun, ProbeCount2, ProbeCount
;            ReadU, lun, ProbeCount2, ProbeCount
;        break
;        end
11308 : $   ; V_MASK (0x2C2C)
    begin
next_read_length = n_tags(StartTimestamp, /data_length) + n_tags(EndTimestamp, /data_length)

point_lun, -lun, current_pos

if(current_pos + next_read_length gt file_size) then begin
    print, "file size exceeded in datatype", datatype
    stop
endif

ReadU, lun,  StartTimestamp, EndTimestamp

next_read_length = 2 + 4 + 4 + 2

point_lun, -lun, current_pos

if(current_pos + next_read_length gt file_size) then begin
    print, "file size exceeded in datatype", datatype
    stop
endif

ReadU, lun, ProbeCount2, ProbeCount ; start probe counter

;if probecount2 eq probecount and rollover eq rollover2  then begin
;    break;  added this for verion 992 on Dec 15, 2006.
;endif

OD = {_DerivedDataRev2}


OD.NanoSeconds = StartTimestamp.Nanoseconds + $
    (StartTimestamp.Microseconds * 1000ULL) + $
    (StartTimestamp.Milliseconds * 1000000ULL) + $
    (StartTimestamp.Seconds * NanosecondsPerSecond) + $
    (StartTimestamp.Minutes * NanosecondsPerMinute) + $
    (StartTimestamp.Hours * NanosecondsPerHour)

OD.Nanosecsold=OD.nanoseconds ;= StartTimestamp.Year * 12 + StartTimestamp.Month
OD.ImageOffset = 0UL
OD.ImageWords = 0L
if DataType eq 31868 then OD.Area = !DEADTIME_START_FLAG else OD.Area = !MASK_START_FLAG
OD.TAS = 0.0 + tas_V
OD.TASold=OD.TAS


;OD.ProbeCounter = Rollover * OneRollover + ProbeCount
;OD.ProbeCounter = ProbeCount2 * OneRollover + ProbeCount
OD.ProbeCounter = ishft(Long64(ProbeCount2), 32) or Long64(ProbeCount)
;WRITEU, vert_lun, OD
OD0=OD

ReadU, lun, ProbeCount2, ProbeCount ; end probe counter

OD.NanoSeconds = EndTimestamp.Nanoseconds + $
    (EndTimestamp.Microseconds * 1000ULL) + $
    (EndTimestamp.Milliseconds * 1000000ULL) + $
    (EndTimestamp.Seconds * NanosecondsPerSecond) + $
    (EndTimestamp.Minutes * NanosecondsPerMinute) + $
    (EndTimestamp.Hours * NanosecondsPerHour)

OD.Nanosecsold=OD.nanoseconds ;= EndTimestamp.Year * 12 + EndTimestamp.Month

if DataType eq 31868 then OD.Area = !DEADTIME_END_FLAG else OD.Area = !MASK_END_FLAG
OD.TAS = 0.0+TAS_V; added this "+tas" by mo on Oct 5, 2006.
OD.TASold=OD.TAS

;OD.ProbeCounter = Rollover2 * OneRollover + ProbeCount2
;OD.ProbeCounter = ProbeCount2 * OneRollover + ProbeCount
OD.ProbeCounter = ishft(Long64(ProbeCount2), 32) or Long64(ProbeCount)
if OD.probecounter lt OD0.probecounter then begin
    OD.probecounter=OD0.probecounter
    OD.nanoseconds =OD0.nanoseconds
    OD.area=OD.area-100
    OD0.area=OD0.area-100
    printf,22,'at time=',OD.nanoseconds,' V Fake pair"s  end-tick less than start-tick  '
endif

writeu, vert_lun,OD0
WRITEU, vert_lun, OD


;if DataType eq 31868 then stop

lastV=-103
break
end
19789 : $
    begin
next_read_length = 2*2 + 2*16

point_lun, -lun, current_pos

if(current_pos + next_read_length gt file_size) then begin
    print, "file size exceeded in datatype", datatype
    stop
endif

ReadU, lun, TimingWord, Masks
;print , "MASK"
break
end
16705 : $  ; TAS_HEADER_V
    dummy_value = 42
21588 : $  ; TAS_HEADER_H;  chris, is this the TAS change fake particle ? -104 ?
    BEGIN
next_read_length = 4

point_lun, -lun, current_pos

if(current_pos + next_read_length gt file_size) then begin
    print, "file size exceeded in datatype", datatype
    stop
endif

ReadU, lun , TAS
if DataType eq 16705 then TAS_V=tas else TAS_H=tas; added this on Nov.28, 2006.

if(TAS eq 0) then begin
    stop
endif

;Catch, onError
;IF onError NE 0 THEN BEGIN
;    Catch, /Cancel
;    result = Error_Message(Traceback=1)
;    stop
;    goto, close_and_free_luns
;ENDIF

next_read_length = n_tags(StartTimestamp, /data_length) + 2 + 4

point_lun, -lun, current_pos

if(current_pos + next_read_length gt file_size) then begin
    print, "file size exceeded in datatype", datatype
    stop
endif

ReadU, lun, StartTimestamp, ProbeCount2, ProbeCount ; TAF check these reads, verify with VB

OD = {_DerivedDataRev2}
OD.TASold=tas

OD.NanoSeconds = StartTimestamp.Nanoseconds                          + $
    (StartTimestamp.Microseconds * 1000ULL)              + $
    (StartTimestamp.Milliseconds * 1000000ULL)           + $
    (StartTimestamp.Seconds      * NanosecondsPerSecond) + $
    (StartTimestamp.Minutes      * NanosecondsPerMinute) + $
    (StartTimestamp.Hours        * NanosecondsPerHour)
OD.Nanosecsold=OD.nanoseconds ;= StartTimestamp.Year * 12 + StartTimestamp.Month
OD.Area = !TAS_FLAG
OD.TAS =0.0+tas;    added this by mo.
;OD.ProbeCounter = Rollover * OneRollover + ProbeCount
;OD.ProbeCounter = ProbeCount2 * OneRollover + ProbeCount
OD.ProbeCounter = ishft(Long64(ProbeCount2), 32) or Long64(ProbeCount)

;stop ; TAF 2013-11-19 debugging TAS
if (DataType eq 16705) then begin
    ;if OD.tas gt 0.0 then $
    WRITEU, vert_lun, OD

    lastV=-104
endif else begin
    ;if OD.tas gt 0.0 then $
    WRITEU, horiz_lun, OD
    lastH=-104

endelse
break
END
19275 : $  ; HK_HEADER
    BEGIN
;print, "HK_HEADER read"

; test explicitly for end of file before each read
;next_read_length = n_tags(HK_ints, /data_length) + n_tags(HK_TAS, /data_length) + n_tags(HK_timingwords, /data_length)

next_read_length = 2*48 + 4 + 2*2

point_lun, -lun, current_pos

if(current_pos + next_read_length gt file_size) then begin
    print, "file size exceeded in datatype", datatype
    stop
endif

ReadU, lun , HK_ints, HK_TAS, HK_timingwords

; test explicitly for end of file before each read
;next_read_length = n_tags(Rollover, /data_length) + n_tags(ProbeCount, /data_length)

next_read_length = 2 + 4

point_lun, -lun, current_pos

if(current_pos + next_read_length gt file_size) then begin
    print, "file size exceeded in datatype", datatype
    stop
endif

ReadU, lun, ProbeCount2,  ProbeCount; TAF check these reads, verify with VB
;print , "HK", HK_TAS
break
END
18504 : $  ; H_PARTICLE_HEADER
    BEGIN
next_read_length = 2 + 2 + 2 + n_tags(Timestamp, /data_length)

point_lun, -lun, current_pos

if(current_pos + next_read_length gt file_size) then begin
    print, "file size exceeded in datatype", datatype
    stop
endif

WordCount =  0U
ReadU, lun , PartID, WordCount, Slices, Timestamp

next_read_length = 2 + 4 + 2

point_lun, -lun, current_pos

if(current_pos + next_read_length gt file_size) then begin
    print, "file size exceeded in datatype", datatype
    stop
endif

ReadU, lun, ProbeCount2, ProbeCount, CPI_trigger  ; TAF check these reads, verify with VB

HTimestamp = Timestamp
if (WordCount GE 500) then begin
    ;print, "H WC=",Wordcount, info.TotalParticles
    WordCount = WordCount and 32767U
    ;stop
end

Nanoseconds=0LL
NanoSeconds = Timestamp.Nanoseconds + $
    Timestamp.Microseconds * 1000ULL + $
    Timestamp.Milliseconds * 1000000ULL + $
    Timestamp.Seconds      * NanosecondsPerSecond + $
    Timestamp.Minutes      * NanosecondsPerMinute + $
    Timestamp.Hours        * NanosecondsPerHour

if (Wordcount gt 0 and Slices gt 0) then begin
    out_byte_array = BytArr(128L, Slices)

    HImage = BytArr(WordCount * 2)
    HSums = IntArr(Slices)

    HMax  = 0

endif else begin
    Slices = 1000
endelse

if (Slices LT 1000) then begin

    POINT_LUN, -lun, pos

    ; test explicitly for end of file before each read
    ;next_read_length = n_tags(HImage, /data_length)

    next_read_length = WordCount * 2

    point_lun, -lun, current_pos

    if(current_pos + next_read_length gt file_size) then begin
        print, "file size exceeded in datatype", datatype
        stop
    endif

    ReadU, lun, HImage

    ; debugging check for some weird crashes
    if(wordcount eq 2 and slices eq 2) then begin
        if(himage[0] eq 0 and himage[1] eq 0 and himage[2] eq 0 and himage[3] eq 0) then begin
            ;stop
        endif
    endif


    nstop = 2

    Area = 0UL
    Midpoint = 0
    Edges = 0
    L3 = 0.0
    L4 = 0
    L5 = 0
    PC3x = 0.0
    PC3y = 0.0
    PC4x = 0.0
    PC4y = 0.0
    A3 = 0.0
    Roundness1 = 0.0
    Roundness2 = 0.0
    Roundness3 = 0.0
    L1r = 0.0
    maxLength = 0.0
    maxWidth  = 0.0
    perimeter = 0UL
    perimeterum = 0.
    csp = 0.
    num_satellites = 0.
    largest_satellite_area = 0.
    Err=0

    HorV = 'H'

    !mask_bit_filling = 0 ; disable for slight speedup -- mask bits are already filled in SORTED generation

    ProcessParticle, HImage, out_byte_array, Wordcount,Slices, HSums, HMax, Area, Midpoint, Edges,Err , $
        L3,L4,L5,PC3x,PC3y,PC4x,PC4y,A3, Roundness1, Roundness2, Roundness3, L1r, maxLength, maxWidth, perimeter, crystal_type, perimeterum, csp, $
        ml_p1x=ml_p1x, ml_p1y=ml_p1y, ml_p2x=ml_p2x, ml_p2y=ml_p2y, mw_p1x=mw_p1x, mw_p1y=mw_p1y, mw_p2x=mw_p2x, mw_p2y=mw_p2y, $
        perimeter_points_x=perimeter_points_x, perimeter_points_y=perimeter_points_y, num_satellites=num_satellites, HorV=HorV, largest_satellite_area=largest_satellite_area

    d = 0
    find_Image_Parameters, out_byte_array, d  ; this is to find alexi1 max area.


    ;print, Wordcount,Slices, HSums, HMax, Area, Midpoint, Edges,Err
    ;print, L3,L4,L5,PC3x,PC3y,PC4x,PC4y,A3, Roundness1, Roundness2, Roundness3
    ;print, L1r, maxLength, maxWidth, perimeter, crystal_type, perimeterum, csp
    ;print, ml_p1x, ml_p1y, ml_p2x, ml_p2y, mw_p1x, mw_p1y, mw_p2x, mw_p2y
    ;print, num_satellites, HorV, largest_satellite_area
    ;stop

    if (Err EQ 0) then begin

        Info.TotalParticles++
        ;a = string( format = '("Particles Read:" , I3)',info.TotalParticles)
        if info.TotalParticles mod 1000 EQ 0 then $
            Widget_Control, info.statusID, DYNAMIC_RESIZE=1, Set_Value= string( format = '("Particles Read (H):" , I8)',info.TotalParticles)

        OD = {_DerivedDataRev2}

        OD.PartID = PartID
        OD.NanoSeconds = Timestamp.Nanoseconds + $
            (Timestamp.Microseconds * 1000ULL) + $
            (Timestamp.Milliseconds * 1000000ULL) + $
            (Timestamp.Seconds * NanosecondsPerSecond) + $
            (Timestamp.Minutes * NanosecondsPerMinute) + $
            (Timestamp.Hours * NanosecondsPerHour)
        OD.Nanosecsold=OD.nanoseconds ;= Timestamp.Year * 12 + Timestamp.Month
        OD.ImageOffset = pos
        OD.ImageWords = WordCount
        OD.Area = Area
        OD.Nslice = Slices
        OD.L2 = HMax
        OD.F1 = Edges
        OD.Pc1 = Midpoint
        OD.TAS = TAS_H
        OD.TASold=OD.TAS
        factor=1.0

        if !use_external_TAS eq 1 then begin;  added this on 2015-01-12  -by Mo
            TAScorrected =interpol(ExtTAS,extTime, OD.nanoseconds/1000000000LL)
            factor =(TAScorrected >1.0)/(OD.TAS >1.0)
        endif

        ;OD.ProbeCounter = Rollover * OneRollover + ProbeCount
        ;OD.ProbeCounter = ProbeCount2 * OneRollover + ProbeCount
        OD.ProbeCounter = ishft(Long64(ProbeCount2), 32) or Long64(ProbeCount)
        OD.L3 = L3
        OD.L4 = L4
        OD.L5 = L5
        OD.PC3x = PC3x
        OD.PC3y = PC3y
        OD.PC4x = PC4x
        OD.PC4y = PC4y
        OD.A3 = A3
        OD.Alexei1= d

        OD.CPI_trigger = CPI_trigger


        This_L1n=((OD.Nslice+randomu(seed)-0.5)*factor) >1.0

        OD.L1n    =This_L1n
        OD.area   =area*factor
        OD.alexei1=   d*factor
        OD.Roundness1 = Roundness1
        OD.Roundness2 = Roundness2
        OD.Roundness3 = Roundness3

        OD.L1r = L1r
        OD.maxLength = maxLength
        OD.maxWidth  = maxWidth
        OD.perimeter = perimeter
        OD.crystal_type = crystal_type
        OD.perimeterum = perimeterum
        OD.csp = csp
        OD.num_satellites = num_satellites
        OD.largest_satellite_area = largest_satellite_area
        ;if OD.tas gt 0.0 then $
        WRITEU, horiz_lun, OD
        lastH=0
        horiz_count++

        ;if(OD.ProbeCounter eq 0) then stop

        ;if(OD.ProbeCounter ge 14870727283) then stop

    Endif ; end of if Err eq 0
endif
break
END
22102 : $  ; V_PARTICLE_HEADER
    BEGIN
;print, "V_PARTICLE_HEADER read"

; test explicitly for end of file before each read
next_read_length = n_tags(PartID, /data_length) + n_tags(WordCount, /data_length) + n_tags(Slices, /data_length) + n_tags(Timestamp, /data_length)

next_read_length = 2 + 2 + 2 + n_tags(Timestamp, /data_length)

point_lun, -lun, current_pos

if(current_pos + next_read_length gt file_size) then begin
    print, "file size exceeded in datatype", datatype
    stop
endif


WordCount =  0U
ReadU, lun , PartID, WordCount, Slices, Timestamp

; test explicitly for end of file before each read
next_read_length = n_tags(ProbeCount, /data_length) + n_tags(CPI_trigger, /data_length)

next_read_length = 2 + 4 + 2

point_lun, -lun, current_pos

if(current_pos + next_read_length gt file_size) then begin
    print, "file size exceeded in datatype", datatype
    stop
endif

ReadU, lun, ProbeCount2, ProbeCount, CPI_trigger  ; TAF check these reads, verify with VB

VTimestamp = Timestamp
if (WordCount GE 500) then begin
    ;    print, "V WC=",Wordcount, info.TotalParticles
    WordCount = WordCount and 32767U
endif

Nanoseconds=0LL
NanoSeconds = Timestamp.Nanoseconds + $
    Timestamp.Microseconds * 1000ULL + $
    Timestamp.Milliseconds * 1000000ULL + $
    Timestamp.Seconds * NanosecondsPerSecond + $
    Timestamp.Minutes * NanosecondsPerMinute + $
    Timestamp.Hours * NanosecondsPerHour

;--------------------------------------------------------------------------------------
; We'd like (in 2DS executable) to trap all goofy particles before they are placed into the SORTED
;  file but as new DSP error or data transmission errors occur, we may miss some. So
;  we are forced to double check here. If the particle slice count or integer data word
;  couint is zero, we clearly cannot proceed to create zero length arrays. However,
;  we still need to read the image, if there is one. So we force Slices =1000 to avoid
;  doing processing that would crash this program, and we ignore this particle.
;--------------------------------------------------------------------------------------
if (Wordcount gt 0 and Slices gt 0) then begin

    ;VImage = BytArr(WordCount*2)
    ;VSums = IntArr(Slices)
    ;out_array = bytarr(16L * Slices)
    ;out_byte_array = BytArr(128L, Slices)

    ;VMax  = 0


    out_byte_array = BytArr(128, Slices)


    VImage = BytArr(WordCount * 2)
    VSums = IntArr(Slices)
    VMax  = 0

endif else begin
    Slices = 1000
endelse

if (Slices LT 1000) then begin

    POINT_LUN, -lun, pos

    ; test explicitly for end of file before each read
    ;next_read_length = n_tags(VImage, /data_length)

    next_read_length = WordCount * 2

    point_lun, -lun, current_pos

    if(current_pos + next_read_length gt file_size) then begin
        print, "file size exceeded in datatype", datatype
        stop
    endif

    ReadU, lun, VImage

    ;if (Slices LT 1000) then begin

    Area = 0UL
    Midpoint = 0
    Edges = 0
    L3 = 0.0
    L4 = 0
    L5 = 0
    PC3x = 0.0
    PC3y = 0.0
    PC4x = 0.0
    PC4y = 0.0
    A3 = 0.0
    Roundness1 = 0.0
    Roundness2 = 0.0
    Roundness3 = 0.0
    L1r = 0.0
    maxLength = 0.0
    maxWidth  = 0.0
    perimeter = 0UL
    perimeterum = 0.
    csp = 0.
    num_satellites = 0.
    largest_satellite_area = 0.
    Err=0


    nstop = 3

    HorV = 'V'

    !mask_bit_filling = 0 ; disable for slight speedup -- mask bits are already filled in SORTED generation

    ProcessParticle, VImage, out_byte_array, Wordcount,Slices, VSums, VMax, Area, Midpoint, Edges,Err , $
        L3,L4,L5,PC3x,PC3y,PC4x,PC4y,A3, Roundness1, Roundness2, Roundness3, L1r, maxLength, maxWidth, perimeter, crystal_type, perimeterum, csp, $
        ml_p1x=ml_p1x, ml_p1y=ml_p1y, ml_p2x=ml_p2x, ml_p2y=ml_p2y, mw_p1x=mw_p1x, mw_p1y=mw_p1y, mw_p2x=mw_p2x, mw_p2y=mw_p2y, $
        perimeter_points_x=perimeter_points_x, perimeter_points_y=perimeter_points_y, num_satellites=num_satellites, HorV=HorV, largest_satellite_area=largest_satellite_area

    d = 0
    find_Image_Parameters, out_byte_array, d


    Info.TotalParticles++
    if info.TotalParticles mod 1000 EQ 0 then $
        Widget_Control, info.statusID, DYNAMIC_RESIZE=1, Set_Value= string( format = '("Particles Read (V):" , I8)',info.TotalParticles)

    OD = {_DerivedDataRev2}
    OD.PartID = PartID
    OD.NanoSeconds = Timestamp.Nanoseconds                         + $
        Timestamp.Microseconds * 1000ULL              + $
        Timestamp.Milliseconds * 1000000ULL           + $
        Timestamp.Seconds      * NanosecondsPerSecond + $
        Timestamp.Minutes      * NanosecondsPerMinute + $
        Timestamp.Hours        * NanosecondsPerHour
    OD.Nanosecsold=OD.nanoseconds ;= Timestamp.Year * 12 + Timestamp.Month

    OD.ImageOffset = pos
    OD.ImageWords = WordCount
    OD.Area = Area
    OD.Nslice = Slices
    OD.L2 = VMax
    OD.F1 = Edges
    OD.Pc1 = Midpoint
    OD.TAS = TAS_V
    OD.TASold=OD.TAS
    factor=1.0
    if !use_external_TAS eq 1 then begin;  added this on 2015-01-12  -by Mo
        TAScorrected =interpol(ExtTAS,extTime, OD.nanoseconds/1000000000LL)
        factor =(TAScorrected >1.0)/(OD.TAS >1.0)
    endif
    ;OD.ProbeCounter = Rollover * OneRollover + ProbeCount
    ;OD.ProbeCounter = ProbeCount2 * OneRollover + ProbeCount
    OD.ProbeCounter = ishft(Long64(ProbeCount2), 32) or Long64(ProbeCount)
    OD.L3 = L3
    OD.L4 = L4
    OD.L5 = L5
    OD.PC3x = PC3x
    OD.PC3y = PC3y
    OD.PC4x = PC4x
    OD.PC4y = PC4y
    OD.A3 = A3
    OD.Alexei1= d
    OD.CPI_trigger = CPI_trigger

    This_L1n=((OD.Nslice+randomu(seed)-0.5)*factor) > 1.0

    OD.L1n    =This_L1n
    OD.area   =area*factor
    OD.alexei1=   d*factor
    OD.Roundness1 = Roundness1
    OD.Roundness2 = Roundness2
    OD.Roundness3 = Roundness3
    OD.L1r = L1r
    OD.maxLength = maxLength
    OD.maxWidth = maxWidth
    OD.perimeter = perimeter
    OD.crystal_type = crystal_type
    OD.perimeterum = perimeterum
    OD.csp = csp
    OD.num_satellites = num_satellites
    OD.largest_satellite_area = largest_satellite_area
    ;***********************************************************
    ;if OD.tas gt 0.0 then $
    WRITEU, vert_lun, OD
    lastV=0
Endif
break
END
ELSE: begin
    ;stop

    ;        skip_counter++
    ;        if(skip_counter mod 1000 eq 0) then begin
    ;            print, 'attemping to skip ahead', counter, skip_counter
    ;            printh, current_pos
    ;            wait, .001
    ;        endif
    ;
    ;        point_lun, -lun, current_pos
    ;        point_lun,  lun, current_pos+1L ; advance one to try for realignment
    ;ReadU, lun, DataType
    break
end
ENDSWITCH
ENDWHILE


close_and_free_luns: ; breakout in here when End of File is reached inside the switch statement when it tries to reads more than there is.

Close ,lun
Close ,horiz_lun
Close ,vert_lun
Close ,hk_lun
Close ,mask_lun
close,22; added january 17, 2008.

Free_Lun, lun, horiz_lun, vert_lun,hk_lun,mask_lun,22

Widget_Control, info.statusID, DYNAMIC_RESIZE=1, Set_Value= string( format = '("File Process Completed, Particles Read:" , I8)',info.TotalParticles)

;Widget_Control, event.top, Set_UValue=info, /No_Copy

catch, /cancel



; TAF added to prevent empty Window 0 from popping up from device call
Widget_Control, info.drawID, Get_Value=wid
WSet,  wid
WShow, wid

; read_verticle_file
device, CURSOR_STANDARD= !ARROW_CURSOR  ; show hourglass while we process the data
;xdisplayfile,warningfile,title='Messages generated while loading the sorted data file'
end ;===== end of spec_toolbar_save make the horizon. vert_***bin files ===========================




pro SPEC_Toolbar_Plot_H, event

    Common  DataBlock,alldataH,alldataV,ADataH,ADataV,FirstData,LastData,StripData,FirstDataV,$
        LastDataV,StripDataV,OneHzH,OneHzV,avestripdata,avestripdataV,SHstripdata,SVstripdata,$
        Mstripdata

    Widget_Control, event.top, Get_UValue=info, /No_Copy  ; get our info structure


    info.CurrentChart1 = !H_CHART
    !CURRENT_CHANNEL = !H_CHART

    if widget_info(info.aveBase, /valid_id) EQ 1 then begin
        Widget_Control, info.aveBase, Get_UValue=ave_info, /No_Copy  ; get our info structure fredfred
        SPEC_DrawAverageChart, ave_info
        Widget_Control, info.aveBase, Set_UValue=ave_info, /No_Copy  ; get our info structure fredfred
    endif

    SPEC_Hplot, info
    Widget_Control, event.top, Set_UValue=info, /No_Copy
end



pro SPEC_Hplot, info
    Common  DataBlock,alldataH,alldataV,ADataH,ADataV,FirstData,LastData,StripData,FirstDataV,$
        LastDataV,StripDataV,OneHzH,OneHzV,avestripdata,avestripdataV,SHstripdata,SVstripdata,$
        Mstripdata

    ;  Xdata = (*alldataH).NanoSeconds ; - (*alldataH)(1:1).NanoSeconds
    ;print, Xdata

    ;  Ydata = (*alldataH).Area
    ;print, ndim

    ; TAF 2013-09-12 archiving was crashing below on the line:  Xdata = ADataH.Concentration
    ; should be a better solution before the call to SPEC_Hplot though
    if(n_elements(ADataH) eq 0) then begin
        print, " "
        print, "Archiving Done.  Exiting."
        print, " "

        return
    endif


    chart1 =    [0.15, 0.700, 0.95, 0.925]
    chart2 =    [0.15, 0.700 - 0.33, 0.95, 0.925-0.33]
    chart3 =    [0.15, 0.700 - 0.66, 0.95, 0.925-0.66]

    axisColor = GetColor(info.axisColorName, !D.Table_Size-2)
    dataColor = GetColor(info.datacolorName, !D.Table_Size-3)
    backColor = GetColor(info.backColorName, !D.Table_Size-4)
    ; to generate the correct character size, point to the main window
    Widget_Control, info.drawID, Get_Value=wid
    WSet, wid
    thisCharsize = Str_Size('A Sample String', 0.25)


    ; str = SPEC_Nano2HMS(FirstData.Nanoseconds)
    fullrange = 0ULL
    binwidth = 0ULL

    ; catch and avoid crash before file is loaded
    if(n_elements(LastData) eq 0) then begin
        return
    endif

    fullrange = LastData.Nanoseconds - FirstData.Nanoseconds
    binwidth = fullrange/8
    curbin = FirstData.Nanoseconds

    ;xlabel = SPEC_Nano2HMS(curbin)

    ;stop ; TAF stop here for debugging time 99 hours in plots

    tnames = StrArr(9)
    FOR i=0,8 DO BEGIN
        tnames(i) =  strtrim(strmid(SPEC_Nano2HMS(curbin),0,12),2)
        curbin += binwidth
    ENDFOR

    ;-------------------------------
    ;Set up X axis tick labels.
    ;-------------------------------
    !X.TICKNAME = tnames
    !X.TICKS = 8


    !P.Multi = [0,3,1,0,1]  ; Make sure we're plotting at the top position


    ; TAF added to halt execution instead of crashing.  This is the hangup after archiving is done bug.  Temporary workaround.
    if(n_elements(ADataH) eq 0) then begin
        print, "Archiving done"
        print, ""
        print, "poorly handled event, clearing events, exiting"
        print, ""
        print, "use .res to reset IDL, and @compile3vcpi to restart program"

        widget_control,info.wid,/clear_events
        widget_control,info.wid,/destroy

        retall
    endif

    Xdata = ADataH.Concentration
    main_title = 'Horizontal Particle Concentration'
    y_title = 'Concentration(#/L)'
    ;----------------------------------------------------
    ; Make sure the window we're plotting to is selected
    ;----------------------------------------------------
    Widget_Control, info.drawID, Get_Value=wid
    WSet, wid

    thisCharsize = 1.8
    ;add this line by mo to fix Xdata eq 1 point.
    if n_elements(Xdata) eq 1 then Xdata=[Xdata,Xdata]

    Plot, Xdata, $
        Background=backcolor, $           ; The background color of the display.
        Charsize=thisCharsize, $          ; The character size, as determined by Str_Size.
        Color=axiscolor, $                ; The color of the axes.
        XStyle=1, $                       ; Exact axis scaling. No autoscaled axes.
        ; XTickformat='(I6)', $
        Title = main_title, $
        XTitle = 'Bin' , $
        YTitle = y_title , $
        Position=chart1 ;, $
        ;XTICKFORMAT='MYXTICKS'

        Xdata = ADataH.Extinction
    main_title = 'Horizontal Particle Extinction'
    y_title = 'Extinction(1/km)'

    ;add this line by mo to fix Xdata eq 1 point.
    if n_elements(Xdata) eq 1 then Xdata=[Xdata,Xdata]

    Plot, Xdata, $
        Background=backcolor, $           ; The background color of the display.
        Charsize=thisCharsize, $          ; The character size, as determined by Str_Size.
        Color=axiscolor, $                ; The color of the axes.
        XStyle=1, $                       ; Exact axis scaling. No autoscaled axes.
        ; XTickformat='(I6)', $
        Title = main_title, $
        XTitle = 'Bin' , $
        YTitle = y_title , $
        Position=chart2 , $
        /NOERASE ; , $
        ; XTICKFORMAT='MYXTICKS'

        Xdata = ADataH.IWC
    main_title = 'Horizontal Particle IWC'
    y_title = 'Ice Water Content(g/m^3)'

    ;add this line by mo to fix Xdata eq 1 point.
    if n_elements(Xdata) eq 1 then Xdata=[Xdata,Xdata]

    Plot, Xdata, $
        Background=backcolor, $           ; The background color of the display.
        Charsize=thisCharsize, $          ; The character size, as determined by Str_Size.
        Color=axiscolor, $                ; The color of the axes.
        XStyle=1, $                       ; Exact axis scaling. No autoscaled axes.
        ; XTickformat='(I6)', $
        Title = main_title, $
        XTitle = 'Bin' , $
        YTitle = y_title , $
        Position=chart3 , $
        /NOERASE ; , $
        ;XTICKFORMAT='MYXTICKS'


        !X.TICKNAME = ''
    !X.TICKS = 0

end  ;  end of SPEC_HPlot


FUNCTION MYXTICKS,HorV,axis, index, value


    Common  DataBlock,alldataH,alldataV,ADataH,ADataV,FirstData,LastData,StripData,FirstDataV,$
        LastDataV,StripDataV,OneHzH,OneHzV,avestripdata,avestripdataV,SHstripdata,SVstripdata,$
        Mstripdata

    stop
    if strupcase(HorV) eq 'H' then begin
        this_lastData=lastData
        this_firsdata=FirstData
    endif else begin
        this_lastData=lastDataV
        this_firsdata=FirstDataV
    endelse


    ; str = SPEC_Nano2HMS(FirstData.Nanoseconds)
    fullrange = 0ULL
    binwidth = 0ULL
    fullrange = this_LastData.Nanoseconds - this_FirstData.Nanoseconds
    binwidth = fullrange/7
    curbin = this_FirstData.Nanoseconds  + (index * binwidth)


    ; WIDGET_CONTROL ,strip_info.stripLabel ,TLB_GET_SIZE= a
    ; widget_info(drawID,/GEOMETRY)

    xlabel = strmid(SPEC_Nano2HMS(curbin),0,12)
    ;xoffset = (index * ((0.95-0.15)/7.0)) + 0.153
    xoffset = (index * ((0.95-0.15)/7.0)) + 0.172


    yoffset = 0.681
    charsize = !CHAR_SIZE  * 0.85 ; use the sizing we calculated from the last window resize
    bluecolor = GetColor("Navy", !D.Table_Size-2)
    XYOutS, xoffset, yoffset, xlabel, color = bluecolor, $
        Alignment=1.0, Orientation=0.0,size = charsize, /NORMAL

    ;print, "MYXTICKS",axis,index,value
    ;return , string(index)
    return, ""

end

;*****************************************************
;*****************************************************
;*****************************************************
;*****************************************************
pro SPEC_Toolbar_Zoom, event

    Common  DataBlock,alldataH,alldataV,ADataH,ADataV,FirstData,LastData,StripData,FirstDataV,$
        LastDataV,StripDataV,OneHzH,OneHzV,avestripdata,avestripdataV,SHstripdata,SVstripdata,$
        Mstripdata

    Widget_Control, event.top, Get_UValue=info, /No_Copy  ; get our info structure
    ;print, "Zoom Button Pressed"
    if (!ZOOM_STATE eq 0) then   !ZOOM_STATE = 2  ; only activate if we're not in the middle of a zoom

    Widget_Control, event.top, Set_UValue=info, /No_Copy
end


;*****************************************************
;*****************************************************
;*****************************************************
;*****************************************************
pro SPEC_Toolbar_Averages, event

    Common  DataBlock,alldataH,alldataV,ADataH,ADataV,FirstData,LastData,StripData,FirstDataV,$
        LastDataV,StripDataV,OneHzH,OneHzV,avestripdata,avestripdataV,SHstripdata,SVstripdata,$
        Mstripdata

    Widget_Control, event.top, Get_UValue=info, /No_Copy  ; get our info structure

    widget_control,info.aveID,map=1  ; Make the averages screen appear

    Widget_Control, event.top, Set_UValue=info, /No_Copy
end


;*****************************************************
;*****************************************************
;*****************************************************
;*****************************************************
pro SPEC_Toolbar_Plot_V, event

    Common  DataBlock,alldataH,alldataV,ADataH,ADataV,FirstData,LastData,StripData,FirstDataV,$
        LastDataV,StripDataV,OneHzH,OneHzV,avestripdata,avestripdataV,SHstripdata,SVstripdata,$
        Mstripdata

    Widget_Control, event.top, Get_UValue=info, /No_Copy  ; get our info structure
    info.CurrentChart1 = !V_CHART
    !CURRENT_CHANNEL = !V_CHART

    if widget_info(info.aveBase, /valid_id) EQ 1 then begin

        Widget_Control, info.aveBase, Get_UValue=ave_info, /No_Copy  ; get our info structure fredfred
        SPEC_DrawAverageChart, ave_info
        Widget_Control, info.aveBase, Set_UValue=ave_info, /No_Copy  ; get our info structure fredfred
    endif
    SPEC_Vplot, info

    Widget_Control, event.top, Set_UValue=info, /No_Copy
end


;*****************************************************
;*****************************************************
;*****************************************************
;*****************************************************

pro SPEC_Vplot, info
    Common  DataBlock,alldataH,alldataV,ADataH,ADataV,FirstData,LastData,StripData,FirstDataV,$
        LastDataV,StripDataV,OneHzH,OneHzV,avestripdata,avestripdataV,SHstripdata,SVstripdata,$
        Mstripdata

    ;Widget_Control, event.top, Get_UValue=info, /No_Copy  ; get our info structure

    ; TAF 2013-09-26 archiving was crashing below on the line:  Xdata = ADataV.Concentration
    ; should be a better solution before the call to SPEC_Hplot though
    if(n_elements(ADataV) eq 0) then begin
        print, " "
        print, "Archiving Done.  Exiting."
        print, " "

        return
    endif

    Xdata = (*alldataV).NanoSeconds ; - (*alldataV)(1:1).NanoSeconds
    ;print, Xdata

    Ydata = (*alldataV).Area
    ndim = Size(image, /N_Dimensions)
    ;print, ndim

    chart1 =    [0.15, 0.700, 0.95, 0.925]
    chart2 =    [0.15, 0.700 - 0.33, 0.95, 0.925-0.33]
    chart3 =    [0.15, 0.700 - 0.66, 0.95, 0.925-0.66]

    axisColor = GetColor(info.axisColorName, !D.Table_Size-2)
    dataColor = GetColor(info.datacolorName, !D.Table_Size-3)
    backColor = GetColor(info.backColorName, !D.Table_Size-4)
    ; to generate the correct character size, point to the main window
    Widget_Control, info.drawID, Get_Value=wid
    WSet, wid
    thisCharsize = Str_Size('A Sample String', 0.25)


    ; str = SPEC_Nano2HMS(FirstData.Nanoseconds)
    fullrange = 0ULL
    binwidth = 0ULL
    fullrange = LastDataV.Nanoseconds - FirstDataV.Nanoseconds
    binwidth = fullrange/8
    curbin = FirstDataV.Nanoseconds

    ;xlabel = SPEC_Nano2HMS(curbin)

    tnames = StrArr(9)
    FOR i=0,8 DO BEGIN
        tnames(i) =  strtrim(strmid(SPEC_Nano2HMS(curbin),0,12),2)
        curbin += binwidth
    ENDFOR

    ;-------------------------------
    ;Set up X axis tick labels.
    ;-------------------------------
    !X.TICKNAME = tnames
    !X.TICKS = 8


    !P.Multi = [0,3,1,0,1]  ; Make sure we're plotting at the top position
    Xdata = ADataV.Concentration
    main_title = 'Vertical Particle Concentration'
    y_title = 'Concentration(#/L)'
    ;----------------------------------------------------
    ; Make sure the window we're plotting to is selected
    ;----------------------------------------------------
    Widget_Control, info.drawID, Get_Value=wid
    WSet, wid

    thisCharsize = 1.8
    ;********************************************************
    ;add this line by mo to fix Xdata eq 1 point.
    if n_elements(Xdata) eq 1 then Xdata=[Xdata,Xdata]

    Plot, Xdata, $
        Background=backcolor, $           ; The background color of the display.
        Charsize=thisCharsize, $          ; The character size, as determined by Str_Size.
        Color=axiscolor, $                ; The color of the axes.
        XStyle=1, $                       ; Exact axis scaling. No autoscaled axes.
        ; XTickformat='(I6)', $
        Title = main_title, $
        XTitle = 'Bin' , $
        YTitle = y_title , $
        Position=chart1; $
        ; XTICKFORMAT='MYXTICKS_V'; if I keep this, the Xtick labels will be very small.

        Xdata = ADataV.Extinction
    main_title = 'Vertical Particle Extinction'
    y_title = 'Extinction(1/km)'

    ;add this line by mo to fix Xdata eq 1 point.
    if n_elements(Xdata) eq 1 then Xdata=[Xdata,Xdata]

    Plot, Xdata, $
        Background=backcolor, $           ; The background color of the display.
        Charsize=thisCharsize, $          ; The character size, as determined by Str_Size.
        Color=axiscolor, $                ; The color of the axes.
        XStyle=1, $                       ; Exact axis scaling. No autoscaled axes.
        ; XTickformat='(I6)', $
        Title = main_title, $
        XTitle = 'Bin' , $
        YTitle = y_title , $
        Position=chart2; , $   /NOERASE, $   XTICKFORMAT='MYXTICKS_V'

    Xdata = ADataV.IWC
    main_title = 'Vertical Particle IWC'
    y_title = 'Ice Water Content(g/m^3)'

    ;add this line by mo to fix Xdata eq 1 point.
    if n_elements(Xdata) eq 1 then Xdata=[Xdata,Xdata]

    Plot, Xdata, $
        Background=backcolor, $           ; The background color of the display.
        Charsize=thisCharsize, $          ; The character size, as determined by Str_Size.
        Color=axiscolor, $                ; The color of the axes.
        XStyle=1, $                       ; Exact axis scaling. No autoscaled axes.
        ; XTickformat='(I6)', $
        Title = main_title, $
        XTitle = 'Bin' , $
        YTitle = y_title , $
        Position=chart3 ;, $   /NOERASE, $   XTICKFORMAT='MYXTICKS_V'

    !X.TICKNAME = ''
    !X.TICKS = 0

    ;Widget_Control, event.top, Set_UValue=info, /No_Copy
end ;----------------------------------------------------------------------

;==================================================================

;=============================================================================
;==================================================================
; LOAD 2D .BIN FILE AND GENERATE 300 BIN AVERAGE DATA
;==================================================================
;==================================================================
pro SPEC_Toolbar_Load, event

    Common  DataBlock,alldataH,alldataV,ADataH,ADataV,FirstData,LastData,StripData,FirstDataV,LastDataV,$
        StripDataV,OneHzH,OneHzV,avestripdata,avestripdataV,SHstripdata,SVstripdata,Mstripdata
    Common  HKdatablock,HKseconds,HKtas,num_hrz_dect,num_vrt_dect,pHKtimefpos,H50micron


    forward_function  CalcNumBins

    Widget_Control, event.top, Get_UValue=info, /No_Copy  ; get our info structure
    Widget_Control, info.statusID, DYNAMIC_RESIZE=1, Set_Value="Loading Processed Data" ; tell user what we're up to

    ;InitiateZoom, info, !ZOOM_START_INDEX_H, !ZOOM_END_INDEX_H


    ;----------------------------------------------------------------------------------
    ; Create the data structures we'll be using.
    ;  _DerivedData is the structure of the particle data in the .BIN files
    ; _AverageData is the structure of the 300 bin averaged results we'll be plotting
    ;----------------------------------------------------------------------------------
    struct = {_DerivedData, $
        NanoSecsold:0ULL , $
        NanoSeconds:0ULL , $
        ImageOffset: 0UL , $
        ImageWords: 0 , $
        Area: 0.0 , $
        Nslice: 0 , $
        L2: 0 , $
        F1: 0 , $
        Adj1: 0.0, $
        Pc1: 0.0, $
        TAS: 0.0, $
        L1n:0.0,$
        largest_satellite_area: 0.0, $    ;added dec19,2007
        alexei1old: 0.0, $ ;added dec19,2007
        TASold: 0.0 $    ; added dec19,2007
        } ; added nov 9, 2007.


    struct = {_DerivedDataRev2, $
        PartID: 0, $
        Nanosecsold:0ULL , $
        NanoSeconds:0ULL , $
        ImageOffset: 0UL , $
        ImageWords: 0 , $
        Area: 0.0 , $
        Nslice: 0 , $
        L2: 0 , $
        F1: 0 , $
        Adj1: 0.0, $
        Pc1: 0.0, $
        TAS: 0.0, $
        ProbeCounter:0ULL, $
        L3: 0.0, $
        L4: 0, $
        L5: 0, $
        PC3x: 0.0, $
        PC3y: 0.0, $
        PC4x: 0.0, $
        PC4y: 0.0, $
        A3: 0.0, $
        Alexei1: 0.0, $
        Alexei2: 0L, $
        reject: byte(0), $
        w8_t: 0LL, $
        L7: 0.0, $
        L8: 0.0, $
        roundness: 0.0, $
        focus: 0.0, $
        Warn: 0.0, $
        adj0: 1.0, $ ; for used w8_t cleaning; added in May 8, 2007
        noisebyte:byte(0),$;; added in June 2007.
        AccDTtick:0ULL,$   ; added on oct5 2007
        w8_t_clean:0LL, $; added on oct5,2007.
        L1n: 0.0, $   ; added nov 9, 2007.
        largest_satellite_area: 0.0, $
        alexei1old: 0.0, $ ;added dec19,2007
        TASold: 0.0, $     ; added dec19,2007
        Roundness1: 0.0, $
        Roundness2: 0.0, $
        Roundness3: 0.0, $
        L1r: 0.0, $
        maxLength: 0.0, $
        maxWidth: 0.0, $
        perimeter: 0, $
        crystal_type: 0, $
        perimeterum: 0.0, $
        csp: 0.0, $
        num_satellites: 0.0, $
        round123: 0.0, $
        spare5: 0.0, $
        CPI_trigger: 0  }

    struct = {_AverageData, $
        Concentration:0.0D, $
        Extinction:0.0D, $
        IWC:0.0D, $
        LWC:0.0D,$;  added this on Dec 5, 2005 by mo.
        PSize:0.0D, $
        PArea:0.0D, $
        PMass:0.0D, $
        FileOffset:0UL, $
        NSOffset:0ULL, $
        N:0UL, $
        ImageOffset: 0UL , $
        ImageWords: 0 , $
        Area: 0.0D , $
        Nslice: 0.0D , $
        L2: 0.0D , $
        F1: 0.0D , $
        Adj1: 0.0D, $
        Pc1: 0.0D,$
        L1n:0.0 }  ; added nov. 9, 2007.

    struct = {_AverageHzData, $
        Concentration:0.0D, $
        Extinction:0.0D, $
        IWC:0.0D, $
        Bins: DINDGEN(24), $  ; 23 PSD bins for #/L/micron
        FileOffset:0UL, $
        NSOffset:0ULL, $
        N:0UL, $
        TAS: 0.0, $
        Nslice: 0.0D , $
        Adj1: 0.0D, $
        Pc1: 0.0D, $
        Deadtime: 0.0D ,$
        L1n:0.0}; added nov 9, 2007.

    struct ={_HKdata,HKseconds:0LL,TAS:0.0}; added january 29, 2008.


    ;=================================================
    ; Let user pick which .BIN file to open
    ;=================================================

    filename = dialog_pickfile(filter='*.BIN',  /Read)

    IF filename NE '' then  BEGIN
        info.currentFname=filename
    ENDIF ELSE BEGIN
        Widget_Control, info.statusID, DYNAMIC_RESIZE=1, Set_Value= "No file selected, processing not completed"
        Widget_Control, event.top, Set_UValue=info, /No_Copy ; release the info structure since we are leaving early
        RETURN
    ENDELSE


    ; We let the user click on any of the 4 files (horiz,vert,mask, or onehz) in this selected
    ;  directory, then we extract the prefix and suffix of the filename as follows (horiz as example):
    ;   c:\RSI\mywork\horiz040505154705.bin
    ;   prefix = c:\RSI\mywork\
    ;   suffix = 040505154705.bin
    ; Now we can easily construct all 4 filename, and search to make sure they are all present



    end_of_path = STRPOS(filename, '\', /REVERSE_SEARCH)
    info.currentPname =  STRMID(filename,0,end_of_path+1)

    prefix =  STRMID(filename,0,end_of_path+1)

    ; Figure out which file they clicked on and generate the suffix for the creation of all 4 filenames
    if (strpos(filename,'horiz') NE -1) then begin
        suffix = strmid(filename,strpos(filename,'horiz')+5)
    end else begin
        if (strpos(filename,'vert') NE -1) then begin
            suffix = strmid(filename,strpos(filename,'vert')+4)
        end else begin
            if (strpos(filename,'onehz') NE -1) then begin
                suffix = strmid(filename,strpos(filename,'onehz')+5)
            end else begin
                suffix = strmid(filename,strpos(filename,'mask')+4)
            endelse
        endelse
    endelse

    horiz_filename = prefix + 'horiz' + suffix
    vert_filename = prefix + 'vert'  + suffix
    onehz_filename = prefix + 'onehz' + suffix
    mask_filename = prefix + 'mask'  + suffix
    oneHz_filename = strmid(onehz_filename,0,strlen(onehz_filename)-3) + '1hz'

    all_files_here = 1
    IF  SPEC_TestForFile(horiz_filename) NE 0 then all_files_here = 0
    IF  SPEC_TestForFile(vert_filename) NE 0 then all_files_here = 0
    ;IF  SPEC_TestForFile(onehz_filename) NE 0 then all_files_here = 0
    IF  SPEC_TestForFile(mask_filename) NE 0 then all_files_here = 0

    onehz_file_present =   SPEC_TestForFile(oneHz_filename) ; return -1 if file not found, 0 if it is found

    if (all_files_here NE 1) then begin
        Widget_Control, info.statusID, DYNAMIC_RESIZE=1, Set_Value="Unable to find the 4 related data files"
        Widget_Control, event.top, Set_UValue=info, /No_Copy
        return
    endif

    ;--------------------------------------------------------
    ; Added for version 0.30. As the files got bigger, it
    ; was important to clear out the previously loaded common
    ; data before you can start to load new data. This even
    ; happens when you exit the program, so shrinking the
    ; size of these two large arrays helps to avoid lack of memory problems.
    ;----------------------------------------------------
    if n_elements(alldataH) ge 1 then (*alldataH) = (*alldataH)(0)
    if n_elements(alldataV) ge 1 then (*alldataV) = (*alldataV)(0)
    ;==========================================
    ; READ HORIZONTAL DATA FILE
    ;==========================================
    OpenR, lun, horiz_filename, /Get_Lun
    ;=============================================
    ; New for REV 2, the .BIN files have 256 byte
    ;  text headers on them to show what REV they are.
    ; Since REV1 did not have the header, we read
    ;  in the first 256 bytes, if we see 3.00 then
    ;  it is REV 2, if not, then it is REV 1
    ;  and we need to reposition our file pointer
    ;  back to the start to begin processing.
    ;=============================================
    fileheader = bytarr(256)
    ReadU, lun, fileheader
    v_string = strmid(string(fileheader),4,4)
    if (v_string eq '3.00') then begin
        OD = {_DerivedDataRev2}
        !CURRENT_VERSION = 3.0
    endif else begin
        OD = {_DerivedData}
        POINT_LUN, lun, 0   ; rewind file pointer to start if version 1.0 (we didn't have the header then)
        !CURRENT_VERSION = 1.05
    endelse

    ;end_of_path = STRPOS(horiz_filename, '\', /REVERSE_SEARCH)
    ;info.currentPname =  STRMID(filename,0,end_of_path+1)  ; save this path to locate the SORTEDdatastamp.2DS file one directory up
    info.currentFname = horiz_filename                      ; save full filename for access when we click on the graph
    !CurrentHPname = info.currentPname
    !CurrentHFname = horiz_filename

    ; info.datestamp = strmid(horiz_filename,strpos(horiz_filename,"horiz")+5,12)  ; save this datestamp for locating the SORTEDdatestamp.2DS one directory above this
    substrsF=strsplit(horiz_filename,'\',/extract);  changed above by mo on sept 22, 2006.
    short_filename=substrsF(n_elements(substrsF)-1)

    info.datestamp = strmid(Short_filename,5,strlen(Short_filename)-9);  take way 9 letters,(horiz and .bin)
    ;info.datestamp = strmid(Short_filename,5,strlen(Short_filename)-12);  take way 12 letters,(horiz and .2DSCPI)
    ;stop

    ; one path up will be: strmid(info.currentPname,0,strlen(info.CurrentPname)-5)
    fdata = FSTAT(lun)
    StructSize = N_TAGS( OD , /DATA_LENGTH )

    ;==========================================
    ; READ VERTICAL DATA FILE
    ;==========================================
    OpenR, lunV, vert_filename, /Get_Lun
    if (v_string eq '3.00') then    ReadU, lunV, fileheader  ; for REV2 pull off the 256 byte header


    ; Test the binary file for the 50 micron H channel flag, vs the 10 micron H channel
    ; this section is called during Archiving ASCII

    ; The binary file header currently looks like this (Ver string below, annotated):

    ;                            27th character
    ;                            |                 45th character
    ;                            |                 |
    ; Ver 3.00 BIN PARTICLE DATA H50Micron = False 1.1.0 TAF

    ; Thus strmid(string(fileheader), 27, 17) will extract: "H50Micron = False", "H50Micron = True"
    ; and strmid(string(fileheader), 45, 15) will extract: 1.1.0 TAF

    ; However, switching to tokenizing the string will be more robust to changes in character length, position


    if(!H_res_set eq 0) then begin ; test to ensure the pixel resolution is set only once, may consider moving this into the function, though unneeded function calls are logic errors, not sure
        if(!manual_override_H50vs10micron eq 0) then begin

            print, ""
            print, "determine pixel resolution automatically from .bin files, or fail here"

            binary_50_micron_flag = strmid(string(fileheader), 27, 17)
            binary_version_string = strmid(string(fileheader), 45, 15)

            print, ""
            print, " 3V-CPI View Version from binary file: ", binary_version_string ; TAF note the space at the start of the print string is needed due to some weird IDL print bug!
            print, binary_50_micron_flag

            if(strcmp(binary_50_micron_flag, "H50Micron = True ")) then begin
                print, "H 50 Micron detected in binary file"
                !H_resolution_microns = 50
            endif else if(strcmp(binary_50_micron_flag, "H50Micron = False")) then begin
                print, "H 10 Micron detected in binary file"
                !H_resolution_microns = 10
            endif else begin
                print, ""
                print, "H50Micron flag not detected in binary file, switching to manual H 10 vs 50 micron selection"
                print, ""

                H50micron_pop_up
            endelse
        endif

        define_Pixel_resolutions
    endif



    fdataV = FSTAT(lunV)
    info.currentFnameV = vert_filename                      ; save full filename for access when we click on the graph
    !CurrentVPname = info.currentPname
    !CurrentVFname = vert_filename
    ;==========================================
    ; If we already have previously created the
    ;  1Hz data file, then skip the next code.
    ;  We'll have to check for version numbers
    ;  on the 1Hz file if it's already made.
    ; 1) Look for existing .1HZ file
    ; 2) Confirm version of 1HZ file is the most recent
    ;==========================================

    ;---------------------------------------------
    ; Look at the first and last particles in this
    ;  file for the range of timestamps.
    ;---------------------------------------------

    if (v_string eq '3.00') then begin
        FirstData = ({_DerivedDataRev2})
        num_data_points = long64( (fdata.Size-256) /StructSize)
    endif else if(v_string eq '3.00D') then begin
        print, 'SORTED file generated with Debug information, rerun Playback without Tools->Include particle offset in output checked'
        return
    endif else begin
        FirstData = ({_DerivedData})
        num_data_points = long64(fdata.Size/StructSize)
    endelse
    ReadU, lun, FirstData

    ; Sometimes the first data structure may not be a particle but a flag for a deadtime event
    ;  so read ahead until we get an actual particle
    no_string_in_H=0
    if (FirstData.Area lt 0) then begin
        ptr = 1L
        flag = 1
        while (flag eq 1) do begin
            if EOF(lun) eq 0 then begin
                ReadU, lun, FirstData
                if (FirstData.Area gt 0) and (FirstData.Nanoseconds gt 0) then begin
                    flag = 0  ; get us out of the loop
                endif else begin
                    ptr++
                    if (ptr ge num_data_points) then begin
                        ; r = dialog_message(['Error, cannot find a string timestamp in this data file!','Plese select a different data file".'],TITLE = 'Program is being stopped',/Cancel)
                        no_string_in_H=1
                    endif
                endelse
            endif else flag=0
        endwhile
    endif

    if (v_string eq '3.00') then begin
        LastData = ({_DerivedDataRev2})
        POINT_LUN, lun, ((num_data_points-1LL) * StructSize) + 256  ; include the header
    endif else begin
        LastData = ({_DerivedData})
        POINT_LUN, lun, (num_data_points-1LL) * StructSize
    endelse

    ReadU, lun, LastData

    ;---------------------------------------------
    ; Look at the first and last particles in this
    ;  file for the range of timestamps.
    ;---------------------------------------------
    if (v_string eq '3.00') then begin
        FirstDataV = ({_DerivedDataRev2})
        num_data_pointsV = long64( (fdataV.Size-256) /StructSize)

    endif else begin
        FirstDataV = ({_DerivedData})
        num_data_pointsV = long64(fdataV.Size/StructSize)
    endelse

    ReadU, lunV, FirstDataV

    ; Sometimes the first data structure may not be a particle but a flag for a deadtime event
    ;  so read ahead until we get an actual particle
    if (FirstDataV.Area lt 0) then begin
        ptr = 1L
        flag = 1
        while (flag eq 1) do begin
            if EOF(lunV) eq 0 then begin
                ReadU, lunV, FirstDataV
                if (FirstDataV.Area gt 0) and (FirstDataV.Nanoseconds gt 0) then begin
                    flag = 0  ; get us out of the loop
                endif else begin
                    ptr++
                    if (ptr ge num_data_pointsV) then begin
                        if no_string_in_H eq 1 then begin;  if there is one channel is working, we don't need to display this.
                            ;r = dialog_message(['Error, cannot find a string timestamp in both H and V channel !','Plese select a different data file".'],TITLE = 'Program is being stopped',/Cancel)
                        endif
                    endif
                endelse
            endif else flag =0
        endwhile
    endif

    if (v_string eq '3.00') then begin
        LastDataV = ({_DerivedDataRev2})
        POINT_LUN, lunV, (num_data_pointsV-1LL) * StructSize   + 256  ; include the header
        ;structsize=162;  num_data_pointsV=819718, 804538 ???
    endif else begin
        LastDataV = ({_DerivedData})
        POINT_LUN, lunV, (num_data_pointsV-1LL) * StructSize

    endelse
    ReadU, lunV, LastDataV


    ;------------------------------------------------
    ; Using the earlist and latest times from H and V
    ;  determine the full range of the data.
    ;------------------------------------------------
    if (FirstData.NanoSeconds gt FirstDataV.Nanoseconds) then min_time = FirstDataV.NanoSeconds else  min_time = FirstData.NanoSeconds
    if (LastData.NanoSeconds gt LastDataV.Nanoseconds) then max_time = LastData.NanoSeconds else max_time = LastDataV.NanoSeconds
    ;----------------------------------------------------------------------
    ; Now that we've got the timestamps from the first and last particles
    ; (may be from H or V stream), lets round those timestamps down at the
    ;  start, and up at the end so that we start and stop at exactly the
    ;  "top" of the second. This permits us to match our data more easily
    ;  to the other instruments collecting data at the same time.
    ;----------------------------------------------------------------------
    NanosecondsPerSecond = 1000000000.0D  ; use double for some calculations
    NPS = 1000000000ULL
    min_time = min_time - (min_time mod NPS)  ; Round down to time with milli,micro, and nano seconds = 0

    temp = max_time - (max_time mod NPS)      ; Round down to time with milli, micro and nano seconds = 0
    ;  If the upper limit happened to already be at exactly one second, don't bother to round up to next second

    if (temp NE max_time) then temp = temp + NPS  ; Round up to next full second
    max_time = temp

    full_time = max_time - min_time
    Seconds = long(ceil(full_time/NPS))


    FirstData.NanoSeconds  = min_time
    FirstDataV.NanoSeconds = min_time
    LastData.NanoSeconds   = max_time
    LastDataV.NanoSeconds  = max_time

    ;=========================================
    ; Before we generate the 1 Hz bins, create
    ;  and populate the complete contents of
    ;  the H & V particle file that we use
    ;  throughout the program..
    ;=========================================

    ; TAF 2013-09-26 apparently I need to re-re-reimplement the pointers in 3V-CPI again, as it is once again using straight array copys
    ; After pointers are reimplemented, the following lines will free memory before (re)allocating arrays, thus preventing the
    ; memory allocation crash bug.  Since the section tests for valid pointers, it does no harm having it implemented here before the full pointer
    ; system is back in place

    ; Free memory from previous iterations through archiving, before allocating new arrays

    if(ptr_valid(AllDataH)) then begin
        print, "Freeing AllDataH"

        ptr_free, AllDataH
    endif

    if(ptr_valid(AllDataV)) then begin
        print, "Freeing AllDataV"

        ptr_free, AllDataV
    endif


    Widget_Control, info.statusID, DYNAMIC_RESIZE=1, Set_Value="Loading Raw Data (Horizontal) " ; tell user what we're up to

    if (v_string eq '3.00') then begin
        alldataH = REPLICATE({_DerivedDataRev2},num_data_points)
        POINT_LUN, lun, 256  ; point back to start of file but past the header
    endif else begin
        alldataH = REPLICATE({_DerivedData},num_data_points)
        POINT_LUN, lun, 0   ; point back to start of file
    endelse
    ReadU, lun, alldataH
    alldataH = ptr_new(temporary(alldataH))

    Widget_Control, info.statusID, DYNAMIC_RESIZE=1, Set_Value="Loading Raw Data (Vertical) " ; tell user what we're up to

    if (v_string eq '3.00') then begin
        alldataV = REPLICATE({_DerivedDataRev2},num_data_pointsV)
        POINT_LUN, lunV, 256  ; point back to start of file but past the header
    endif else begin
        alldataV = REPLICATE({_DerivedData},num_data_pointsV)
        POINT_LUN, lunV, 0   ; point back to start of file
    endelse
    ReadU, lunV, alldataV
    alldataV = ptr_new(temporary(alldataV))

    Widget_Control, info.statusID, DYNAMIC_RESIZE=1, Set_Value="Closing Data Files" ; tell user what we're up to

    ; close the data files.---added by mo Feb 16, 2006.
    Free_lun,lun
    Free_lun,lunv
    ;re-fresh teh pop_up file name so that it will no over write agaian and again.
    pop_message_filename=strmid(!currentHFname,0,strlen(!currentHFname)-4)+'PopUpMessages.txt'
    openw,18,pop_message_filename
    close,18

    ; store the old values to keep them in the commond data block.
    (*alldataH).alexei1old =(*alldataH).alexei1

    (*alldataV).alexei1old =(*alldataV).alexei1



    Widget_Control, info.statusID, DYNAMIC_RESIZE=1, Set_Value="correct_HV_first_last_particle_ticks" ; tell user what we're up to

    ; TAF plotting unsorted times, in manual load SORTED file, not in archiving



    correct_HV_first_last_particles_ticks
    Plot_unsorted_particles_nanotimes_and_ticks_and_tas,'H'
    Plot_unsorted_particles_nanotimes_and_ticks_and_tas,'V'



    Widget_Control, info.statusID, DYNAMIC_RESIZE=1, Set_Value="correct_tas_of_HV_particles" ; tell user what we're up to

    if !use_external_TAS eq 0 then begin;  Do not need to correct TAS any more, external TAS is correct if !use_=1.
        correct_tas_of_HV_particles; including fake particles.
    endif

    Widget_Control, info.statusID, DYNAMIC_RESIZE=1, Set_Value="Label_all_saw_tooth_events" ; tell user what we're up to
    Label_all_saw_tooth_events,'H';  added on March 31, 2008.
    Label_all_saw_tooth_events,'V'

    Widget_Control, info.statusID, DYNAMIC_RESIZE=1, Set_Value="calculate_wait_time_of_all_HVparticles" ; tell user what we're up to
    calculate_wait_time_of_all_HVparticles,'H'; assigne warn=warn+10 and reject particles if w8_tless than 0.

    calculate_wait_time_of_all_HVparticles,'V'; f w8_t less than 0, it mean time is non-monotonic.

    Widget_Control, info.statusID, DYNAMIC_RESIZE=1, Set_Value="Reject_Particles_btw_SE_Mkpair_N_assign_irregular_Value100" ; tell user what we're up to
    Reject_Particles_btw_SE_Mkpair_N_assign_irregular_Value100,'H';  just assign warn=warn+100.
    Reject_Particles_btw_SE_Mkpair_N_assign_irregular_Value100,'V'

    ;if(!calibration eq 0) then begin
    ;    Widget_Control, info.statusID, DYNAMIC_RESIZE=1, Set_Value="correct_HV_particles_nanotimes" ; tell user what we're up to
    ;    correct_HV_particles_nanotimes,'H';  assigned warn=warn+1000; to time can not recovered particles.
    ;    correct_HV_particles_nanotimes,'V'; ; comment time correction  for bead testing
    ;endif

    ;************************************************************
    Widget_Control, info.statusID, DYNAMIC_RESIZE=1, Set_Value="calculate_L7_of_all_H_andV_particles" ; tell user what we're up to
    calculate_L7_of_all_H_andV_particles
    Widget_Control, info.statusID, DYNAMIC_RESIZE=1, Set_Value="calculate_L8_of_all_H_andV_particles" ; tell user what we're up to
    calculate_L8_of_all_H_andV_particles

    Widget_Control, info.statusID, DYNAMIC_RESIZE=1, Set_Value="calculate_AccDeadtime_of_all_H_andV_particles" ; tell user what we're up to
    calculate_AccDeadtime_of_all_H_andV_particles; added on oct 4, 2007.

    calculate_Adj1_of_all_H_andV_particles,'H'
    calculate_Adj1_of_all_H_andV_particles,'V'

    ; look_for_a_good_particle_in_a_bad_pc1_array
    ; stop

    ;****************************************************

    if(seconds gt 100000) then begin
        seconds = 50
        print, "TAF large seconds error, need to fix, processing should be OK"
    endif

    OneHzH = REPLICATE({_AverageHzData},Seconds)
    OneHzV = REPLICATE({_AverageHzData},Seconds)

    version_string = byte('VERSION 005')   ; Current version number for the .1HZ file
    version_string_number = '005'
    header_balance = BYTARR(245)

    ;=========================================================
    ;=========================================================
    if (onehz_file_present eq -1) then begin
        ;stop ; TAF 2013-09-30 when did this stop get inserted here?!
        generate_1hz_file,'H', Seconds,min_time,info,version_string,header_balance,oneHz_filename
        generate_1hz_file,'V', Seconds,min_time,info,version_string,header_balance,oneHz_filename
        num = Seconds  ; needed below for instances where we've just generated the 1Hz file rather than loading it from disk

    endif else begin

        OpenR, lunHz, onehz_filename, /Get_Lun
        header = bytarr(256)
        num = long(1)
        ReadU, lunHz, header
        version_number = string(header[8:10])

        if version_number eq version_string_number then begin
            ReadU, lunHz, num  ; this had better equal the Seconds variable
            if (num ne Seconds) then begin
                ; we'd better regenerate the file
                ; provide warning here
                r = dialog_message(['Warning, # seconds in one hz file does not equal the number in the file you have selected!','The One Hz file will now be regenerated.'],TITLE = 'File incompatible',/Cancel)
                if (R eq 'Cancel') then exit
                generate_1hz_file, 'H', Seconds,min_time,info,version_string,header_balance,oneHz_filename
                generate_1hz_file, 'V', Seconds,min_time,info,version_string,header_balance,oneHz_filename
                num = Seconds  ; needed below for instances where we've just generated the 1Hz file rather than loading it from disk

            endif else begin
                ReadU, lunHz, OneHzH
                ReadU, lunHz, OneHzV
            endelse
        endif else begin
            ; provide warning about version update
            r = dialog_message(['Warning, version # of saved file is not current!','The One Hz file will now be regenerated.'],TITLE = 'File is older version',/Cancel)
            if (R eq 'Cancel') then exit
            generate_1hz_file,'H', Seconds,min_time,info,version_string,header_balance,oneHz_filename
            generate_1hz_file,'V', Seconds,min_time,info,version_string,header_balance,oneHz_filename
            num = Seconds  ; needed below for instances where we've just generated the 1Hz file rather than loading it from disk

        endelse



        close, lunHz
        Free_Lun,lunHz
    endelse


    BinWidth = 0.0
    NumBins = CalcNumBins(num,BinWidth)  ; Binwidth also get returned from the function call !!!---mo.
    info.NanosPerBin = BinWidth * NPS   ; bin width in nanoseconds
    info.NumOneHzBins = NumBins   ; revised to hold the # of bins in the averaged 1Hz data


    ;NanosecondsPerSecond = 1000000000.0D  ; use double to allow us to round up on the next ceil
    ;Seconds = ceil(full_time/NanosecondsPerSecond)
    ;SecondsPerBin = ceil(Seconds/1000.0)   ; display up to 1000 bins, but make sure bins are some integer # of seconds wide
    ;NumBins = ceil(Seconds/DOUBLE(SecondsPerBin)) ; this will give us close to, but not more than, 1000 total bins
    NumBins_string = string(NumBins)


    ADataH = REPLICATE({_AverageData},NumBins)
    ADataV = REPLICATE({_AverageData},NumBins)

    StripData = REPLICATE({_DerivedDataRev2},!STRIP_WIDTH)
    StripDataV = REPLICATE({_DerivedDataRev2},!STRIP_WIDTH)
    aveStripData = REPLICATE({_DerivedDataRev2},!STRIP_WIDTH)
    aveStripDataV = REPLICATE({_DerivedDataRev2},!STRIP_WIDTH)
    SHstripdata=REPLICATE({_DerivedDataRev2},!STRIP_WIDTH)
    SVstripdata=REPLICATE({_DerivedDataRev2},!STRIP_WIDTH)
    Mstripdata=REPLICATE({_DerivedDataRev2},!STRIP_WIDTH)

    ;get the Kth value from prefix; ='C:\Bug_for_mo\Data\'
    ; substrs=strsplit(prefix,'\',/extract)
    ; datastr=substrs(n_elements(substrs)-1)
    ; if strlen(datastr) eq 4 then Kth=0 else Kth=round(strmid(datastr,4,strlen(datastr)-4)*1)
    ; get_dc8_TAS_time_series,info,Kth,HKseconds, HKtas; assigne HKseconds, and HKtas in here.
    ; comment above out on May 10th , 2008.

    Widget_Control, info.statusID, DYNAMIC_RESIZE=1, Set_Value="Generating Main Time Series display... " ; tell user what we're up to

    BinIndex = 0

    for Index = 0,NumBins-2 DO BEGIN
        ADataH(Index).Concentration = mean(OneHzH[BinIndex:BinIndex+BinWidth -1].Concentration)
        ADataH(Index).Extinction = mean(OneHzH[BinIndex:BinIndex+BinWidth -1].Extinction)
        ADataH(Index).IWC = mean(OneHzH[BinIndex:BinIndex+BinWidth -1].IWC)

        ADataH(Index).FileOffset =  OneHzH[BinIndex].FileOffset
        ADataH(Index).NSOffset = OneHzH[BinIndex].NSOffset
        ; print, "from->to ",BinIndex,BinIndex+BinWidth -1,"  Offset=",ADataH(Index).FileOffset, ' for index=',Index
        ADataV(Index).Concentration = mean(OneHzV[BinIndex:BinIndex+BinWidth -1].Concentration)
        ADataV(Index).Extinction = mean(OneHzV[BinIndex:BinIndex+BinWidth -1].Extinction)
        ADataV(Index).IWC = mean(OneHzV[BinIndex:BinIndex+BinWidth -1].IWC)
        ADataV(Index).FileOffset =  OneHzV[BinIndex].FileOffset
        ADataV(Index).NSOffset = OneHzV[BinIndex].NSOffset
        ;     print, index,ADataH(Index).Concentration
        BinIndex += BinWidth

    endfor
    print, BinIndex

    ;--------------------------------------------------------
    ; Handle the last bin
    ;--------------------------------------------------------
    ;added following line by mo to fix if numbins equal to 1  on July 7, 2006
    ;********************************************************
    if numbins eq 1 then index=0
    ;***********************************************************
    if (BinIndex ne Seconds) then begin
        ADataH(Index).Concentration = mean(OneHzH[BinIndex:Seconds -1].Concentration)
        ADataH(Index).Extinction = mean(OneHzH[BinIndex:Seconds -1].Extinction)
        ADataH(Index).IWC = mean(OneHzH[BinIndex:Seconds -1].IWC)
        ADataH(Index).FileOffset =  OneHzH[BinIndex].FileOffset
        ADataH(Index).NSOffset = OneHzH[BinIndex].NSOffset

        ADataV(Index).Concentration = mean(OneHzV[BinIndex:Seconds -1].Concentration)
        ADataV(Index).Extinction = mean(OneHzV[BinIndex:Seconds -1].Extinction)
        ADataV(Index).IWC = mean(OneHzV[BinIndex:Seconds -1].IWC)
        ADataV(Index).FileOffset =  OneHzV[BinIndex].FileOffset
        ADataV(Index).NSOffset = OneHzV[BinIndex].NSOffset
    endif


    ; Enable the buttons for the strip display once we have read in the data
    WIDGET_CONTROL, info.HButton, SENSITIVE=1
    WIDGET_CONTROL, info.VButton, SENSITIVE=1

    device, CURSOR_STANDARD= !ARROW_CURSOR  ; show hourglass while we process the data

    ; No Longer Using the Ave window with version 10
    ;     if widget_info(info.aveID, /valid_id) EQ 0 then begin
    ;        AverageWindow, info
    ;     Widget_Control, info.aveBase, Get_UValue=ave_info, /No_Copy  ; get our info structure fredfred
    ;     SPEC_DrawAverageChart, ave_info
    ;     Widget_Control, info.aveBase, Set_UValue=ave_info, /No_Copy  ; get our info structure fredfred
    ;   endif

    ; At startup, default to selecting H chart

    info.CurrentChart1 = !H_CHART
    !CURRENT_CHANNEL = !H_CHART

    UpdateTimeSeries, info

    ;Widget_Control,info.wid, BASE_SET_TITLE="Showing data from SORTED" + strmid(suffix,1,strlen(suffix)-5) + ".2DS"
    Widget_Control,info.wid, BASE_SET_TITLE="Showing data from SORTED" + strmid(suffix,1,strlen(suffix)-5) + ".2DSCPI"
    Widget_Control, info.statusID, DYNAMIC_RESIZE=1, Set_Value="Processed Data Loaded"
    Widget_Control, event.top, Set_UValue=info, /No_Copy

    ;*****************************************************
    ;*****************************************************
    ;pop_message_filename=strmid(!currentHFname,0,strlen(!currentHFname)-4)+'PopUpMessages.txt'
    ;if !autoarchive eq 0 then xdisplayfile, pop_message_filename, title='Messages generated while loading the data file'
    catch, /cancel
    ;*****************************************************
    ;*****************************************************
end  ;end of spec_toolbar_load

;******************************************************************************
;******************************************************************************
pro SPEC_Generate_Ascii_event, event;  changed this on feb.13, 2009 to generated multiple sorted files ascii data.
    Common  DataBlock,alldataH,alldataV
    forward_function archive_ice_pop_up
    Widget_Control, event.top, Get_UValue=info, /No_Copy  ; get our info structure
    ;This program load a bunch of sorted files,
    ;then make bin-files and generated the zoomed particle by particle ascii files, overNight.

    ;first get all the roi files.
    get_all_sorted_2DS_data_filenames,filenames,file_count

    if file_count eq 0 then begin
        return
    endif

    ;Then use a loop to process each file.

    for i=0, file_count-1 do begin
        Kth=i+1

        ;*********************************************************************************
        ;first assign the info and ten then call spec_toolbar_SAVE, info to generate all
        ;bin files

        ;Widget_Control, event.top, Get_UValue=info, /No_Copy  ; get our info structure
        filename = filenames(i);  dialog_pickfile(filter='SORTED*.2DS',  /Read)
        info.currentFname=filename
        substrsF=strsplit(filename,'\',/extract)
        short_filename=substrsF(n_elements(substrsF)-1)
        ;info.datestamp = strmid(filename,strpos(filename,"SORTED")+6,24); changed to 24 from 12.
        ;info.datestamp = strmid(Short_filename,6,strlen(Short_filename)-10)

        info.datestamp = strmid(Short_filename,6,strlen(Short_filename)-12)

        stop
        end_of_path = STRPOS(filename, '\', /REVERSE_SEARCH)
        info.currentPname =  STRMID(filename,0,end_of_path+1)

        ; folloowing line loads teh sorted filed and generate teh bin files.
        ; it can be comment out if the bin files have been generated before.
        ;look for dataKth directory in "info.currentPname\data\"  or info.currentPname\dataKth\ to see
        ; if there is data files in there. if no, there make new ones.

        currentPath=info.currentPname
        filter_str='Horiz'+info.datestamp+'.Bin';';   ;user comfirms
        filename=file_search(currentpath,filter_str,COUNT=file_count)


        if file_count eq 0 then begin
            SPEC_Toolbar_Save, info, Kth; change to this line on Nov. 14, 2007.
        endif
        ;now the bin files are generated, we need to load and open the bin-files do do the processing.
        ; in the next block.
        ;********************************************************************************

        ;Now open the BIN file and begin doing all of the processing
        SPEC_Toolbar_Load_archive,info

        ; for the ave_info  data to pass to the subroutine.

        startnanoseconds=(*alldataH)(0).nanoseconds < (*alldataV)(0).nanoseconds
        endnanoseconds=(*alldataH)(n_elements((*alldataH).tas)-1LL).nanoseconds > (*alldataV)(n_elements((*alldataV).tas)-1LL).nanoseconds
        ave_info={startnanoseconds:startnanoseconds,endnanoseconds:endnanoseconds,$
            startTimestring:strtrim(string(startnanoseconds),2),$
            endtimestring:strtrim(string(endnanoseconds),2),$
            Hpartcont:n_elements((*alldataH).tas),Vpartcont:n_elements((*alldataV).tas),$
            binfilename:info.CURRENTFNAME}

        datestamp=info.datestamp



        ascii_info={currentPname:!currentHPname,currentFname:!currentHFname,datestamp:datestamp}

        generate_zoomed_ascii, ascii_info, ave_info

    endfor

    Widget_Control, event.top, Set_UValue=info, /No_Copy
end



pro CPIView_button, event
    ; Open the 3V-CPI View CPI View GUI, for viewing and processing CPI data

    ;Widget_Control, event.top, Get_UValue=info, /No_Copy  ; get our info structure
    ;print, "Openning CPI View"
    ;print, "TOP: ", event.top
    ;print, "ID: ",  event.ID

    cpiview

    ;Widget_Control, event.top, Set_UValue=info, /No_Copy

end ; CPIView_button




pro basescan_batch, event
    Common  HKdatablock, HKseconds,HKtas,num_hrz_dect,num_vrt_dect,pHKtimefpos,H50micron
    forward_function spec_seconds2HHMMSS

    defsysv,'!showimg',0

    ; run the basescan program to convert base*.2DSCPI(HK) files to SORTED files

    ; basescan, 1 ; input flag to show images

    ; other basescan options can be set in the system variables in ./basescan/basescan.pro

    ;Widget_Control, event.top, Get_UValue=info, /No_Copy  ; get our info structure
    ;print, "Openning CPI View"
    ;print, "TOP: ", event.top
    ;print, "ID: ",  event.ID
    Widget_Control, event.top, Get_UValue=info, /No_Copy  ; get our info structure
    device, CURSOR_STANDARD= !ARROW_CURSOR  ; show hourglass while we process the data
    ;get the output options first for later to use.
    ice_archive_options_ptr=archive_ice_pop_up(value=1);   pop_up is in creatzoom.pro
    kkk = create_output_png_criteria(value=1)
    criteria_text = (*(*kkk).localstate).text

    ; get all data files and start a for loop.

    filter_str ='base*.2DSCPI'
    dirn=dialog_pickfile(filter=filter_str,get_path=pathnm)

    get_all_data_filenames,filter_str,pathnm,filenames,file_count
    This_info = {pHKData:ptr_new(), $          ; A pointer to the image data.
        StartTime:0L,$
        EndTime:0L,$
        DateStamp:' ', $;               info.Datestamp,$
        Pathname:pathnm,$
        ice:' ',$;                     processing treatment as if all are ice,
        Totalparticles: 0LL,$
        statusID:info.StatusID,$
        drawID:info.drawID,$
        currentFname:' ',$
        currentPname:' ',$
        currentFnameV:' ',$
        currentPnameV:' '}

    ptr=ptr_new(This_info,/no_copy); no extra copy is made.

    for i=0,file_count-1 do begin
        baseFilename =fileNames(i)
        (*ptr).currentFname=basefilename

        ; now get the HK data of the data file. one is image block fpos  and time
        ; and the other HK data is the House keeping data and the time.  temperature VS, diode voltage vs Time detected particle and Time.
        ; first get the Time and fpos  data and  get the house keeping data.

        scan_3v2ds_basefile_for_timefpos_and_Hkdata,pathnm,baseFilename
        HKdata={nanoseconds:HKseconds*1000000000LL,hrz_pd:num_hrz_dect,vrt_pd:num_vrt_dect}
        pHKdata  =ptr_new(HKdata)
        (*ptr).pHKdata=pHkdata
        Startfpos=0LL
        filepos=(*pHKtimefpos).fpos
        endfpos  =max(filepos)


        ;get the starttime end time string to make a direct for the processd time period.
        (*ptr).StartTime=HKseconds(0)
        (*ptr).EndTime=max(HKseconds)

        StartTimestring=SPEC_seconds2HHMMSS((*ptr).StartTime)
        EndTimeString  =SPEC_seconds2HHMMSS((*ptr).EndTime); CURRENTFNAME    STRING    'C:\bjwmo_playback\3V_2DS\base140714040900.2DSCPI'


        basefile=file_basename((*ptr).currentFname)
        substrs=strsplit(basefile,'.',/extract)
        extension=substrs(n_elements(substrs)-1);   the extension can be 2DS, 2DSCPI, 2D128 and HVPS.

        folder_name=strmid((*ptr).currentFname,0,strlen((*ptr).currentFname)-7)+'_'+StartTimestring+'_'+endtimestring+'\'

        ;find if there are any sorted file already genenerated in the directory for the same time period.
        results=file_search(folder_name+'Sorted*.'+extension,count=contG)

        if contG gt 0 then begin

            sortfile1=file_basename(results(0))
            if ContG ge 2 then sortfilen=file_basename(results(ContG-1)) else sortfilen='   '

            message=[['There are '+strtrim(string(contG),2)+ ' sorted file(s) already generated for this time period,'],$
                ['   '],$
                ['They are: '+sortfile1],$
                ['......    '+sortfilen],$
                ['      '],$
                ['Do you want to regenerate these SORTED files, or skip ahead ?'],$
                ['    '],$
                ['To skip ahead without redoing playback, Answer: Yes !'],$
                ['To regenerate the SORTED files,         Answer:  No !']]

            rrrr=dialog_message(message,/question)

        endif else rrrr='No'

        if rrrr eq 'No' then begin;  The folder is made in this call.
            basescan, basefilename, (*ptr).StartTime,(*ptr).EndTime,Startfpos, Endfpos
        endif

        ; above will finish generating all of the wanted sorted files.
        sortedfiles=file_search(folder_name,'Sorted*.'+extension)
        allOrKps='All'
        liqorice=(*ptr).ice
        info=(*ptr)

        ;now see if 1Hz files are needed to be generated.
        !autoarchive=1
        !autoarchive_png_only=0
        Do_overnight_auto_archiving_works_tc4,info,AllOrKps,liqorice,my_selection,basescanfiles=sortedfiles,criteria_text=criteria_text

        !autoarchive=0
        !autoarchive_png_only=1
        Do_overnight_auto_archiving_works_tc4,info,AllOrKps,liqorice,my_selection,basescanfiles=sortedfiles,criteria_text=criteria_text

        ; Please stop putting in popup messages!  Instead here is a printout and a stauts bar update
        print, 'SORTED File Generation Complete.'
        Widget_Control, info.statusID, DYNAMIC_RESIZE=1, Set_Value='SORTED File Generation Complete.'
        ;rrr=dialog_message('Processing Done !',/info)
        stop
    endfor;  end of for i=0, filecounts-1:

    Widget_Control, event.top, Set_UValue=info, /No_Copy
end ; basescan_batch

pro basescan_button, event
    Common  HKdatablock, HKseconds,HKtas,num_hrz_dect,num_vrt_dect,pHKtimefpos,H50micron

    ; run the basescan program to convert base*.2DSCPI(HK) files to SORTED files

    ; basescan, 1 ; input flag to show images

    ; other basescan options can be set in the system variables in ./basescan/basescan.pro

    ;Widget_Control, event.top, Get_UValue=info, /No_Copy  ; get our info structure
    ;print, "Openning CPI View"
    ;print, "TOP: ", event.top
    ;print, "ID: ",  event.ID
    Widget_Control, event.top, Get_UValue=info, /No_Copy  ; get our info structure

    if strlen(info.currentPname) eq 0 then begin
        baseFilename=dialog_pickfile(title='Please pick a *.2DSCPI data file',filter='*.2DSCPI',get_path=pathnm)
    endif else begin
        pathnm =info.currentPname
        baseFilename=dialog_pickfile(title='Please pick a *.2DSCPI data file',filter='*.2DSCPI',Path=pathnm)
    endelse

    if strlen(baseFilename) eq 0 then begin
        Widget_Control, event.top, Set_UValue=info, /No_Copy
        return
    endif

    ; now get the HL data of the data file. one is image block fpos  and time
    ; and the other HK data is the House keeping data and the time.  temperature VS, diode voltage vs Time detected particle and Time.
    ; first get the Time and fpos  data and  get the house keeping data.

    scan_3v2ds_basefile_for_timefpos_and_Hkdata,pathnm,baseFilename
    HKdata={nanoseconds:HKseconds*1000000000LL,hrz_pd:num_hrz_dect,vrt_pd:num_vrt_dect}
    pHKdata  =ptr_new(HKdata)

    browse_info = {pHKData:pHKdata, $          ; A pointer to the image data.
        Drawid:info.DrawID,$
        statusID:info.statusID,$
        click_a_point:0,$
        StartTime:0L,$
        DateStamp:' ', $;               info.Datestamp,$
        Pathname:pathnm,$
        currentFname:baseFileName,$
        currentPname:info.currentPname,$
        currentFnameV:' ',$
        currentPnameV:info.currentPnameV}

    a3vcpiview_browse_image, browse_info
    Widget_Control, event.top, Set_UValue=info, /No_Copy
end ; basescan_button


pro SPEC_Toolbar_Open, event
    Widget_Control, event.top, Get_UValue=info, /No_Copy  ; get our info structure
    ;print, "Open button hit"
    ;print, "TOP: ",event.top
    ;print, "ID: ",event.ID
    ;stop
    ;filename = dialog_pickfile(filter='SORTED*.2DS',  /Read)
    filename = dialog_pickfile(filter='SORTED*.2DSCPI',  /Read)
    print, "File name selected: ", filename
    if filename NE '' then begin
        info.currentFname=filename

        ;info.datestamp = strmid(filename,strpos(filename,"SORTED")+6,12)

        ;changed above by mo in sept 22, 2006.
        substrsF=strsplit(filename,'\',/extract)
        short_filename=substrsF(n_elements(substrsF)-1)
        ;info.datestamp = strmid(Short_filename,6,strlen(Short_filename)-10)
        info.datestamp = strmid(Short_filename,6,strlen(Short_filename)-13)

        end_of_path = STRPOS(filename, '\', /REVERSE_SEARCH)
        info.currentPname =  STRMID(filename,0,end_of_path+1)

        SPEC_Toolbar_Save, (info),0
        ; Put the info structure back in its storage location.
    endif else begin
        r = dialog_message('Raw data processing cancelled.',TITLE = 'No file selected')

    endelse

    Widget_Control, event.top, Set_UValue=info, /No_Copy

end

;******************************************************************************
;******************************************************************************

pro SPEC_Open, event  ; Open a new data file for analysis

    Widget_Control, event.top, Get_UValue=info, /No_Copy  ; get our info structure
    ;print, "Open button hit"
    ;print, "TOP: ",event.top
    ;print, "ID: ",event.ID
    ; file = DIALOG_PICKFILE(/READ, DEFAULT_EXTENSION="2DS", FILTER = '*.pro')
    filename = dialog_pickfile(filter='*.*', /Read)
    ;filename = Dialog_Pickfile(/Read, Filter'*.*')

    print, "File name selected: ", filename
    if filename NE '' then  info.currentFname=filename


    ; Put the info structure back in its storage location.

    Widget_Control, event.top, Set_UValue=info, /No_Copy
end

;******************************************************************************
;******************************************************************************
pro SPEC_Quit, event

    ;print, "Quit button hit"
    ;print, "TOP: ",event.top
    ;print, "ID: ",event.ID
    ; print, event,handler
    Widget_Control, event.top, /Destroy
end

;*****************************************************
;*****************************************************

;@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
pro SPEC_Open2, event  ; Open a new data file for analysis
    print, "Window 2 OPEN"
end

pro SPEC_Quit2, event
    print, "Window 2 QUIT"
end
pro SPEC_GraphClick2, event
    print, "IN SPEC_GraphClick2"
end
pro SPEC_Toolbar_Open2, event
    print, "Toolbar_Open2"
end
pro SPEC_Toolbar_Save2, event
    print, "Toolbar_Save2"
end

;@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
pro SPEC_GraphClick, event

    Common  DataBlock,alldataH,alldataV,ADataH,ADataV,FirstData,LastData,StripData,FirstDataV,$
        LastDataV,StripDataV,OneHzH,OneHzV,avestripdata,avestripdataV,SHstripdata,SVstripdata,$
        Mstripdata

    ; Detect empty events from early in the program execution, before data has been loaded and exit the event hanlder
    if(n_elements(alldataH) eq 0) then begin
        return
    endif


    Widget_Control, event.top, Get_UValue=info, /No_Copy  ; get our info structure

    ; Detect empty events from early in the program execution, before data has been loaded and exit the event hanlder
    if(n_elements(info) eq 0) then begin
        return
    endif


    ;print, "==============  SPEC_GraphClick MOUSE CLICK ======================="
    ;print, "SPEC_GraphClick TOP: ",event.top
    ;print, "SPEC_GraphClick ID: ",event.ID

    ;print, "SPEC_GraphClick TYPE,PRESS,RELESE,KEY,CH,MODIFIERS: ",event.TYPE,event.PRESS,event.RELEASE,event.KEY,event.CH,event.MODIFIERS
    ;print, "SPEC_GraphClick PRESS: ",event.PRESS
    ;print, "SPEC_GraphClick RELEASE: ",event.RELEASE
    ;print, "SPEC_GraphClick KEY: ",event.KEY

    ;print, "SPEC_GraphClick X: ",event.X , event.Y
    ;print, "SPEC_GraphClick Y: ",event.Y


    OneHzStartBin = 0UL
    OneHzEndBin = 0UL

    chart1 =    [0.15, 0.700, 0.95, 0.925]
    chart2 =    [0.15, 0.700 - 0.33, 0.95, 0.925-0.33]
    chart3 =    [0.15, 0.700 - 0.66, 0.95, 0.925-0.66]

    ; 2013-09-26 TAF the access to info below is printing error messages, handled now handled above
    ; once again, with code imported from 1.0.95_TAF into 1.0.102_TAF -- after much had been reverted by Mo over the summer!


    height = info.yscale(1)
    width = info.xscale(1)

    ongraph = 0B



    IF  info.CurrentChart1 EQ !H_CHART OR info.CurrentChart1 EQ !V_CHART then begin

        BinWidth = info.NanosPerBin
        NumBins = info.NumOneHzBins

        ;------------------------------------------------------------------
        ; Mouse on chart, figure out which chart
        ;------------------------------------------------------------------

        if ((event.X LE (width * 0.95)) and (event.X GE (width * 0.15))) THEN BEGIN


            xval = ULONG64(((event.X - (width * 0.15)) / ((width * 0.95) - (width * 0.15))) * NumBins)
            fill_status = 0
            if (xval GE 0) and (xval LE NumBins-1) then begin

                fill_status = 1
                IF ((event.Y LE (height * 0.925)) and (event.Y GE (height * 0.700))) THEN  BEGIN ; TOP CHART
                    item_label = '("Conc=" , D12.5)'
                    h_value = ADataH(xval).Concentration
                    v_value = ADataV(xval).Concentration
                ENDIF ELSE IF ((event.Y LE (height * (0.925-0.33))) and (event.Y GE (height * (0.700 - 0.33)))) THEN  BEGIN  ; MIDDLE CHART
                    item_label = '("Extinc=" , D12.5)'
                    h_value = ADataH(xval).Extinction
                    v_value = ADataV(xval).Extinction
                ENDIF ELSE  IF ((event.Y LE (height * (0.925-0.66))) and (event.Y GE (height * (0.700 - 0.66)))) THEN  BEGIN ; BOTTOM CHART
                    item_label = '("IWC=" , E12.5)'
                    h_value = ADataH(xval).IWC
                    v_value = ADataV(xval).IWC
                ENDIF ELSE BEGIN
                    WIDGET_CONTROL ,info.statusID ,SET_VALUE= "Move cursor to graph to select specific data"
                    fill_status = 0
                ENDELSE

            ENDIF

            ;if ((event.X LE (width * 0.95)) and (event.X GE (width * 0.15)))  $
            ;and ((event.Y LE (height * 0.925)) and (event.Y GE (height * 0.700))) THEN  BEGIN

            ;fredzoom

            IF (fill_status eq 1) then begin
                ongraph = 1B


                ; TAF added to prevent empty Window 0 from popping up from device call
                Widget_Control, info.drawID, Get_Value=wid
                WSet,  wid
                ;WShow, wid ; this line was pulling the back window to the front -- very annoying!!!

                ;Widget_Control, info.drawID, Get_Value=wid
                ;WSet, wid
                device, CURSOR_STANDARD= !ARROW_CURSOR
                ; Problem: if you close out the bitmap window, then move cursor over the horizontal Time Series Calculation
                ;   graph, the device statement causes a blank window to be generated!
                ;   so to get around this, you must set the main window to be the current

                added_message = ""
                IF (!ZOOM_STATE eq 2) THEN added_message = "[CLICK ON STARTING ZOOM-IN POINT]  "
                IF (!ZOOM_STATE eq 1) THEN added_message = "[CLICK ON ENDING ZOOM-IN POINT]  "


                xval = ULONG64(((event.X - (width * 0.15)) / ((width * 0.95) - (width * 0.15))) * NumBins)
                if (xval GE 0) and (xval LE NumBins-1) then begin
                    timestart = 0ULL
                    IF  info.CurrentChart1 EQ !H_CHART THEN BEGIN
                        timestart = strmid(SPEC_Nano2HMS(FirstData.Nanoseconds + (info.NanosPerBin * xval)),0,12)
                        timeend = strmid(SPEC_Nano2HMS(FirstData.Nanoseconds + (info.NanosPerBin * (xval+1)))  ,0,12)
                        a = added_message + string( format = item_label,h_value) + " between " +  $
                            timestart + " and " + timeend +  string(format = '(" (BIN ",I3)',xval) + ")"
                    END ELSE BEGIN
                        timestart = strmid(SPEC_Nano2HMS(FirstDataV.Nanoseconds + (info.NanosPerBin * xval)),0,12)
                        timeend = strmid(SPEC_Nano2HMS(FirstDataV.Nanoseconds + (info.NanosPerBin * (xval+1)))  ,0,12)
                        a = added_message + string( format = item_label,v_value) + " between " +  $
                            timestart + " and " + timeend +  string(format = '(" (BIN ",I3)',xval) + ")"
                    ENDELSE

                    WIDGET_CONTROL ,info.statusID , DYNAMIC_RESIZE=1,SET_VALUE= a
                endif
            ENDIF
            ;a = string( format = '("On Graph X=" , I3)',xval) +  string( format = '(" Y=" , I3)',yval)
            ;endif ELSE BEGIN
            ;   ;device, CURSOR_STANDARD= 32512
            ;   WIDGET_CONTROL ,info.statusID ,SET_VALUE= "Move cursor to graph to select specific data"
            ;ENDELSE


            thepoint = 0L
            IF (event.PRESS GE 1) and (ongraph EQ 1B) then  begin

                IF (!ZOOM_STATE gt 0) then begin  ; a shift key event
                    IF (event.PRESS GE 1) THEN BEGIN
                        !ZOOM_STATE--
                        IF (!ZOOM_STATE eq 1) THEN BEGIN
                            aa = where ((*alldataH)(*).Nanoseconds LT FirstData.Nanoseconds + (info.NanosPerBin * xval),thepoint)
                            !ZOOM_START_INDEX_H = thepoint
                            !ZOOM_START_NS = FirstData.Nanoseconds + (info.NanosPerBin * xval)

                            aa = where ((*alldataV)(*).Nanoseconds LT FirstData.Nanoseconds + (info.NanosPerBin * xval),thepoint)
                            !ZOOM_START_INDEX_V = thepoint
                            ;print, "ZOOM CLICK 1, Starting index = ", !ZOOM_START_INDEX
                            !ZOOM_START_STRING = timestart
                            !ZOOM_START_BIN_1HZ = xval * (info.NanosPerBin/1000000000ULL)   ; convert to index into the OneHzH/V array


                        ENDIF
                        IF (!ZOOM_STATE eq 0) THEN BEGIN
                            aa = where((*alldataH)(*).Nanoseconds LT FirstData.Nanoseconds + (info.NanosPerBin * xval),thepoint)
                            !ZOOM_END_INDEX_H = thepoint
                            !ZOOM_END_NS = FirstData.Nanoseconds + (info.NanosPerBin * xval)
                            aa = where((*alldataV)(*).Nanoseconds LT FirstData.Nanoseconds + (info.NanosPerBin * xval),thepoint)
                            !ZOOM_END_INDEX_V = thepoint
                            ;print, "ZOOM CLICK 2, Ending index = ", !ZOOM_END_INDEX
                            !ZOOM_END_STRING = timestart
                            !ZOOM_END_BIN_1HZ = xval * (info.NanosPerBin/1000000000ULL)   ; convert to index into the OneHzH/V array

                            H_count = !ZOOM_END_INDEX_H - !ZOOM_START_INDEX_H
                            V_count = !ZOOM_END_INDEX_V - !ZOOM_START_INDEX_V

                            if (V_count lt 300 or H_count lt 300) then begin
                                mo_message=[['Too few points selected for zoom.'],$
                                    [' H_count='+strtrim(H_count,2)+'  V_count='+strtrim(V_count)+'.'],$
                                    ['Click Yes to zoom it anyway,'],$
                                    ['or click No and then Click Zoom button and try again.']]
                                r = dialog_message(mo_message,TITLE = 'Insufficient points for plotting',/question)
                                if r eq 'Yes' then begin
                                    IF  info.CurrentChart1 EQ !H_CHART then  InitiateZoom, info, !ZOOM_START_INDEX_H, !ZOOM_END_INDEX_H, !ZOOM_START_BIN_1HZ, !ZOOM_END_BIN_1HZ
                                    IF  info.CurrentChart1 EQ !V_CHART then  InitiateZoom, info, !ZOOM_START_INDEX_V, !ZOOM_END_INDEX_V, !ZOOM_START_BIN_1HZ, !ZOOM_END_BIN_1HZ
                                endif
                            endif else begin

                                IF  info.CurrentChart1 EQ !H_CHART then  InitiateZoom, info, !ZOOM_START_INDEX_H, !ZOOM_END_INDEX_H, !ZOOM_START_BIN_1HZ, !ZOOM_END_BIN_1HZ
                                IF  info.CurrentChart1 EQ !V_CHART then  InitiateZoom, info, !ZOOM_START_INDEX_V, !ZOOM_END_INDEX_V, !ZOOM_START_BIN_1HZ, !ZOOM_END_BIN_1HZ

                            endelse

                            if (1 eq 2) then begin  ; old code for creating a zoomed data file, may need again so don't delete yet

                                aa = (*alldataH)(!ZOOM_START_INDEX_H:!ZOOM_END_INDEX_H)
                                bb = (*alldataH)(!ZOOM_START_INDEX_V:!ZOOM_END_INDEX_V)

                                ;aa = (*alldataH)(!ZOOM_START_INDEX_H:!ZOOM_END_INDEX_H-33990)  ; use 204-247 to generate 500 particles/file for testing
                                ;bb = (*alldataH)(!ZOOM_START_INDEX_V:!ZOOM_END_INDEX_V-32200)

                                fname1 = info.currentPname + 'horiz' + info.datestamp + '-zoom-' + !ZOOM_START_STRING + '-' + !ZOOM_END_STRING + '.bin'
                                fname2 = info.currentPname + 'vert' + info.datestamp + '-zoom-' + !ZOOM_START_STRING + '-' + !ZOOM_END_STRING + '.bin'
                                fname3 = info.currentPname + 'onehz' + info.datestamp + '-zoom-' + !ZOOM_START_STRING + '-' + !ZOOM_END_STRING + '.bin'
                                fname4 = info.currentPname + 'mask' + info.datestamp + '-zoom-' + !ZOOM_START_STRING + '-' + !ZOOM_END_STRING + '.bin'
                                fname1 = strrepl(fname1,':','-')
                                z = byte(fname1)
                                z(1) = 58
                                fname1 = string(z)

                                fname2 = strrepl(fname2,':','-')
                                z = byte(fname2)
                                z(1) = 58
                                fname2 = string(z)

                                fname3 = strrepl(fname3,':','-')
                                z = byte(fname3)
                                z(1) = 58
                                fname3 = string(z)

                                fname4 = strrepl(fname4,':','-')
                                z = byte(fname4)
                                z(1) = 58
                                fname4 = string(z)

                                OpenW, zoom_lun, fname1, /Get_Lun
                                WRITEU, zoom_lun, aa
                                Free_Lun, zoom_lun

                                OpenW, zoom_lun, fname2, /Get_Lun
                                WRITEU, zoom_lun, bb
                                Free_Lun, zoom_lun

                                OpenW, zoom_lun, fname3, /Get_Lun
                                Free_Lun, zoom_lun

                                OpenW, zoom_lun, fname4, /Get_Lun
                                Free_Lun, zoom_lun

                                dialog_text =  'Zoom files written to include ' +strtrim( STRING(!ZOOM_END_INDEX_H-!ZOOM_START_INDEX_H),1) + ' H and ' + $
                                    strtrim( STRING(!ZOOM_END_INDEX_V-!ZOOM_START_INDEX_V),1) + ' V particles'
                                r = dialog_message([dialog_text,'Files created:',fname1,fname2,fname3,fname4],TITLE = 'Zoom Result')
                            endif

                        ENDIF

                        Widget_Control, event.top, Set_UValue=info, /No_Copy
                        return
                    ENDIF
                ENDIF

                ; if widget_info(info.stripID, /valid_id) EQ 0 then ParticleStrip, info  ; only create the window the first time it's requested

                ; stop
                ; since user clicked on the graph, we want to draw into the strip, not the window with the graph
                ; Widget_Control, info.stripID, Get_Value=wid
                ; WSet, wid
                ; wShow, wid, 1

                ; xval = uint(((event.X - (width * 0.15)) / ((width * 0.95) - (width * 0.15))) * NumBins)
                ; if xval GE NumBins-1 then xval = NumBins-1

                ; Clicking a bin, we want to do the following to show particle data
                ; 1) locate ADataH(xval-1).FileOffset as an index into the horiz.bin file. If xval = 0 then
                ;       the FileOffset is by definition = 0
                ; 2) Read a _DerivedData structure from this location in horiz.bin. To draw multiple images, read
                ;     multiple _DerivedData structures from this file.
                ; 3) Use the DerivedData.ImageOffset to locate a file offset in the output_file.bin where the
                ;    image will be located. Create a byte array that is (DerivedData.Imagewords * 2) bytes long
                ;    which will hold the compressed image.
                ; 4) The output file (the one that contains one byte per pixel) should be 128 bytes wide by
                ;    the number of slices in the frame, or 128 by DerivedData.L1
                ;-------------------------------------------
                ; Call to ProcessParticle will require
                ;-------------------------------------------
                ; Image = the byte array that is Imagewords * 2 long, the input data, i.e. BytArr(DerivedData.Imagewords * 2)
                ; out_array - a 20000 byte array to hold the output decompressed image, i.e.  out_array = BytArr(10000)
                ; out_byte_array - a byte array that is 128 x L1 to hold the "byte per pixel" image created by the call to this routine.
                ; WordCount - DerivedData.ImageWords
                ; Slices = DerivedData.L1
                ; HSums = an integer array that is DerivedData.ImageWords integers long, i.e. IntArr(DerivedData.ImageWords)
                ; HMax  = 0   ; just define the following four parameter that will be filled in with the call to ProcessParticle
                ; Area = 0UL
                ; Midpoint = 0
                ; Edges = 0

                ;   ProcessParticle, Image, out_array, out_byte_array, Wordcount,Slices, HSums, HMax, Area, Midpoint, Edges
                fullrange = 0ULL
                ;binwidth = 0ULL

                IF  info.CurrentChart1 EQ !H_CHART THEN BEGIN
                    ;*********************************************************
                    if widget_info(info.stripID, /valid_id) EQ 0 then ParticleStrip, info  ; only create the window the first time it's requested
                    ; since user clicked on the graph, we want to draw into the strip, not the window with the graph
                    Widget_Control, info.stripID, Get_Value=wid
                    WSet, wid
                    wShow, wid, 1
                    xval = uint(((event.X - (width * 0.15)) / ((width * 0.95) - (width * 0.15))) * NumBins)
                    if xval GE NumBins-1 then xval = NumBins-1

                    ;*********************************************************
                    ;fullrange = LastData.Nanoseconds - FirstData.Nanoseconds
                    ;binwidth = fullrange/NumBins

                    curbin = FirstData.Nanoseconds
                    ;curbin += (xval * binwidth)
                    curbin += (xval * BinWidth)


                    ;BinWidth = info.NanosPerBin
                    ;NumBins = info.NumOneHzBins

                    clicktime = SPEC_Nano2HMS(curbin)
                    a = "Draw bitmap for:" + clicktime + " horiz.bin offset = " + string(ADataH(xval).FileOffset)
                    WIDGET_CONTROL ,info.statusID , DYNAMIC_RESIZE=1,SET_VALUE= a

                    foffset = 0UL  ; default to start of file
                    if (xval GT 0) then foffset = ADataH(xval).FileOffset ; use xval-1 cause FileOffset stores pointer to end of each bin
                    newoffset = FillStrip(foffset, info,1,'H')
                END ELSE BEGIN
                    ;***************************************************
                    if widget_info(info.VstripID, /valid_id) EQ 0 then VParticleStrip, info  ; only create the window the first time it's requested
                    Widget_Control, info.VstripID, Get_Value=wid
                    WSet, wid
                    wShow, wid, 1

                    xval = uint(((event.X - (width * 0.15)) / ((width * 0.95) - (width * 0.15))) * NumBins)
                    if xval GE NumBins-1 then xval = NumBins-1

                    ;*****************************************************
                    fullrange = LastDataV.Nanoseconds - FirstDataV.Nanoseconds
                    binwidth = fullrange/NumBins
                    curbin = FirstDataV.Nanoseconds
                    curbin += (xval * binwidth)

                    clicktime = SPEC_Nano2HMS(curbin)


                    a = "Draw bitmap for:" + clicktime + " vert.bin offset = " + string(ADataV(xval).FileOffset)
                    WIDGET_CONTROL ,info.statusID , DYNAMIC_RESIZE=1,SET_VALUE= a

                    foffset = 0UL  ; default to start of file
                    if (xval GT 0) then foffset = ADataV(xval).FileOffset ; use xval-1 cause FileOffset stores pointer to end of each bin
                    newoffset = FillStrip(foffset, info,1,'V')
                ENDELSE




                Widget_Control, event.top, Set_UValue=info, /No_Copy
                return
            ENDIF

        endif


    endif
    ;print,'**************graphicclick=',ave_info.startnanoseconds; ave_info is un-defined in here.

    Widget_Control, event.top, Set_UValue=info, /No_Copy


end ;== graphicclick====================================================================


PRO a3vcpiview_Cleanup, tlb

    ; The purpose of this procedure is to clean up pointers,
    ; objects, pixmaps, and other things in our program that
    ; use memory. This procedure is called when the top-level
    ; base widget is destroyed.
    Widget_Control, tlb, Get_UValue=info, /No_Copy  ; Note, we don't release this at the end of this cleanup routine
    IF N_Elements(info) EQ 0 THEN RETURN

    ; Remove any additional windows we may have created
    WHILE (!D.Window NE -1) DO wdelete, !D.Window
    ;IF (!D.Window EQ !STRIP_WINDOW) then   wdelete, !STRIP_WINDOW

    if widget_info(info.win2, /valid_id) then $
        widget_control, info.win2, /destroy

    if widget_info(info.stripID, /valid_id) then $
        widget_control, info.stripID, /destroy

    if widget_info(info.VstripID, /valid_id) then $
        widget_control, info.VstripID, /destroy

    if widget_info(info.stripBase, /valid_id) then $
        widget_control, info.stripBase, /destroy

    if widget_info(info.VstripBase, /valid_id) then $
        widget_control, info.VstripBase, /destroy

    if widget_info(info.aveBase, /valid_id) then $
        widget_control, info.aveBase, /destroy

    if widget_info(info.ZoomWindowID, /valid_id) then begin
        widget_control, info.ZoomWindowID, /destroy
    endif

    ; Free the pointers.
    Ptr_Free, info.image
    Ptr_Free, info.extra


END ;---------------------------------------------------------------------------


function strrepl,str,fromchar,tochar,IGNORECASE=ignorecase


    ON_ERROR,2    ; return to caller

    ; argument testing
    if n_params() lt 3 then begin
        message,'Usage: strrepl,str,fromchar,tochar[,/IGNORECASE])'
    endif


    ; make working copy of string and convert to a byte array
    bstr = byte(string(str))


    ; fromchar is given as character
    if size(fromchar,/TYPE) eq 7 then begin
        ; ignore case?
        if keyword_set(ignorecase) then begin
            ; call strrepl recursively w/o the IGNORE_CASE keyword
            res1 = strrepl(str,strupcase(fromchar),tochar)
            res2 = strrepl(res1,strlowcase(fromchar),tochar)
            return,res2
        endif else begin
            ; find all character occurences
            ; must be a single character - use the first
            bfc = (byte(fromchar))[0]
            ; go and search
            w = where(bstr eq bfc,count)
            ; if not found, return original string
            if count eq 0 then return,str
        endelse
    endif else begin
        ; fromchar is already an index array
        w = long(fromchar)
    endelse


    ; make sure index is in range
    test = where(w lt 0 OR w ge n_elements(bstr),tcount)
    if tcount gt 0 then begin
        message,'WARNING: Index out of range!',/Continue
        ; restrict to valid index values
        test = where(w ge 0 AND w lt n_elements(bstr),tcount)
        if tcount gt 0 then begin
            w = w[test]
        endif else begin
            ; no valid indices: return original string
            return,str
        endelse
    endif


    ; convert tochar to a byte value
    btc  = (byte(tochar))[0]


    ; replace
    bstr[w] = btc


    ; return result as string
    return,string(bstr)
end



pro Infocus_event,event
    Widget_Control, event.top, Get_UValue=info, /No_Copy  ; get our info structure

    if !infocus_on eq 0 then !infocus_on=1 else !infocus_on=0
    ; this info will get passed on to the subset data Hsubsetindex and Vsubsetindex
    ; when the zoom window is clicked on.
    Widget_Control, event.top, Set_UValue=info, /No_Copy  ; get our info structure
end


;******************************************************************
pro Infocus_droplist_event,event
    Widget_Control, event.top, Get_UValue=info, /No_Copy  ; get our info structure
    Widget_Control, event.id,  Get_UValue=uval  ; get our info structure
    ;***********************************************************
    ;thisvariable=(*info.infocus.variables)[event.index]
    if event.index eq 0 then  !focuspercent=0.9
    if event.index eq 1 then  !focuspercent=0.8
    if event.index eq 2 then  !focuspercent=0.7
    ;********************************************************
    Widget_Control, event.top, Set_UValue=info, /No_Copy  ; get our info structure
end
;*******************************************************
;******************************************************************
pro Archive_droplist_event,event
    forward_function archive_ice_pop_up
    Widget_Control, event.top, Get_UValue=info, /No_Copy  ; get our info structure
    Widget_Control, event.id,  Get_UValue=uval  ; get our info structure

    case event.index of

        0:begin;  this is for a single file archive option. no need to do cleaning.

        !autoarchive=1

        kkk=archive_ice_pop_up(value=1)
        my_selection=*((*kkk).localstate)

        if !mass_optionLiq eq 1 then begin
            IceorLiq ='liq'
        endif else begin;    for !mass_optionIWC = 1 or !mass_optioncsp =1  do ice archiving option
            IceorLiq ='ice'
        endelse



        if my_selection.M7 eq 1 then begin
            !autoarchive_method='M7'
            Do_Nsecond_Ultimate_archive_61binsHorV,info=info,'All',iceorliq,'H';;  This has extinction, mass psds with it.
            Do_Nsecond_Ultimate_archive_61binsHorV,info=info,'All',iceorliq,'V';;  This has extinction, mass psds with it.
        endif

        if my_selection.M6 eq 1 then begin
            !autoarchive_method='M6'
            Do_Nsecond_Ultimate_archive_61binsHorV,info=info,'All',iceorliq,'H';;  This has extinction, mass psds with it.
            Do_Nsecond_Ultimate_archive_61binsHorV,info=info,'All',iceorliq,'V';;  This has extinction, mass psds with it.
        endif
        if my_selection.M4 eq 1 then begin
            !autoarchive_method='M4'
            Do_Nsecond_Ultimate_archive_61binsHorV,info=info,'All',iceorliq,'H';;  This has extinction, mass psds with it.
            Do_Nsecond_Ultimate_archive_61binsHorV,info=info,'All',iceorliq,'V';;  This has extinction, mass psds with it.
        endif
        if my_selection.M2 eq 1 then begin
            !autoarchive_method='M2'
            Do_Nsecond_Ultimate_archive_61binsHorV,info=info,'All',iceorliq,'H';;  This has extinction, mass psds with it.
            Do_Nsecond_Ultimate_archive_61binsHorV,info=info,'All',iceorliq,'V';;  This has extinction, mass psds with it.
        endif
        if my_selection.M1 eq 1 then begin
            !autoarchive_method='M1'
            Do_Nsecond_Ultimate_archive_61binsHorV,info=info,'All',iceorliq,'H';;  This has extinction, mass psds with it.
            Do_Nsecond_Ultimate_archive_61binsHorV,info=info,'All',iceorliq,'V';;  This has extinction, mass psds with it.
        endif
        !autoarchive_method='M6';  return to default values.
        !autoarchive=0
    end

    1:begin  ; This is for multi archive.---do multiple files at one click.

    case !clean_option of
        0:  AllorKps='All'
        1:  AllorKPs='KeepSplashes'
        2:  AllorKps='NoClean'
    endcase



    !autoarchive=1
    kkk = archive_ice_pop_up(value=1)
    my_selection=*((*kkk).localstate)

    if !mass_optionLiq eq 1 then begin
        IceorLiq ='liq'
    endif else begin;    for !mass_optionIWC = 1 or !mass_optioncsp =1  do ice archiving option
        IceorLiq ='ice'
    endelse

    if(!roundness_criteria_checked) then begin
        Do_overnight_auto_archiving_works_Tc4, info, AllorKps, iceorliq,my_selection, round_flag = 2 ; not round output
        Do_overnight_auto_archiving_works_Tc4, info, AllorKps, iceorliq,my_selection, round_flag = 1 ;     round output
    endif else begin
        Do_overnight_auto_archiving_works_Tc4, info, AllorKps, iceorliq,my_selection ;  do multiple file ice archive, clean all..
    endelse

    !autoarchive=0;                             ; this includes extinction mass psds with it.
end

endcase

Widget_Control, event.top, Set_UValue=info, /No_Copy  ; get our info structure
end
;*******************************************************

;******************************************************************
pro Archive_pngdroplist_event,event
    Common  DataBlock,alldataH,alldataV
    ;;forward_function archive_ice_pop_up
    Widget_Control, event.top, Get_UValue=info, /No_Copy  ; get our info structure
    Widget_Control, event.id,  Get_UValue=uval  ; get our info structure
    ;***********************************************************
    case event.index of
        0:begin
        ; for RICO use. do one file at a time, it should be executed after cleaning is done.
        if strlen(!currentHFname) gt 4 then begin
            rrr=dialog_message([['Have you choosen your clean options yet ?'],$
                ['If not and you click Yes to continue, It '],$
                ['will make the pngs pages in whatever the '],$
                ['current cleaning status is. If you click '],$
                ['Then you need to click cleanAll or cleanNoise'],$
                ['to do the cleaning first.                '],$
                ['                                         '],$
                ['  Continue anyway ?'                      ]],/question)

            if rrr eq  'No' then goto, END_of_Archive_pngDrop_List
            ;get number of how many images per second you want to output to a ping page.
            mytitle='Enter dt (default=0.0001 second) to show images at least dt seconds apart (enter 0 to show all images):'
            !Dt=(Moqixu_input_single_number_pop_up(title=mytitle,label=mytitle,value=['0.00']))*1.0

            if(!disable_time_popups eq 1) then begin
                !Time_synced = 1
                !TDS_time_offset=0
            endif else begin
                mytitle='Enter 1 (default=0) if 2DS Time has been Synced):'
                !Time_synced=(Moqixu_input_single_number_pop_up(title=mytitle,label=mytitle,value=['0']))*1.0

                if !Time_synced gt 0 then begin
                    mytitle='Enter 2DS time offset N: (postive number means 2DS_Time_new=Time_original+N)'
                    !TDS_time_offset=(Moqixu_input_single_number_pop_up(title=mytitle,label=mytitle,value=['0.000']))*1.0
                endif else begin
                    !TDS_time_offset=0
                endelse
            endelse

            !autoarchive_png_only=1
            ;get all accepted indexs
            ; Hsubindex=where((*alldataH).reject eq 0 and (*alldataH).L1n gt 0,contG)
            ; ave_info={Hsubindex:ptr_new(),$
            ;         Vsubindex:ptr_new()}
            ;**************************************************************************************************
            kkk=create_output_png_criteria(value=1)
            criteria_text=(*(*kkk).localstate).text

            criteria={on:1,text:criteria_text(0),Hfactor:1,Vfactor:1,label:' '}
            startnanoseconds=min([min((*alldataH).nanoseconds),min((*alldataV).nanoseconds)])
            endnanoseconds  =max([max((*alldataH).nanoseconds),max((*alldataV).nanoseconds)])
            ave_info={criteria:criteria,$
                startnanoseconds:startnanoseconds,$
                endnanoseconds:endnanoseconds,$
                Hallindex:ptr_new(),$
                Hsubindex:ptr_new(),$
                Vallindex:ptr_new(),$
                Vsubindex:ptr_new()$
                }
            Get_allindexs_and_boolean_subsetindexs_CLEAN,ave_info,Hallindex,Hsubindex,Hfactor,Vallindex,VsubIndex,Vfactor
            ;****************************************************************************************************************
            if Hsubindex(0) ge 0 then begin
                ave_info.Hsubindex=ptr_new(Hsubindex)
                output_png_images_page,ave_info,'H'
            endif

            if Vsubindex(0) ge 0 then begin
                ave_info.Vsubindex=ptr_new(Vsubindex)
                output_png_images_page,ave_info,'V'
            endif
            !autoarchive_png_only=0
            ;********************************************************************************
            ;********************************************************************************
        endif
    end

    1:begin  ; This is for liqorice multi archive.
    !autoarchive_png_only=1

    kkk = archive_ice_pop_up(value=1)
    my_selection=*((*kkk).localstate)
    if !mass_optionliq eq 1 then begin
        liqorice='liq'
    endif else begin
        liqorice ='ice'
    endelse

    ;get number of how many images per second you want to output to a ping page.
    ; mytitle='First few ---up to the Max Number of Images Per Second To Put on the Png='
    ;!N_images_Per_Second_to_output=(Moqixu_input_single_number_pop_up(title=mytitle,label=mytitle,value=['2']))*1
    mytitle='Enter dt (default=0.0001 second) to show images at least dt seconds apart (enter 0 to show all images):'

    if(!disable_time_popups eq 1) then begin
        !Time_synced = 1
        !TDS_time_offset=0
    endif else begin
        ; the whole point of !disable_time_popups is to skip all of these popups so we can process
        ; without having to keep interacting with the program
        !Dt=(Moqixu_input_single_number_pop_up(title=mytitle,label=mytitle,value=['0.000']))*1.0

        mytitle='Enter 1 (default=0) if 2DS Time has been Synced):'
        !Time_synced=(Moqixu_input_single_number_pop_up(title=mytitle,label=mytitle,value=['0']))*1.0

        if !Time_synced gt 0 then begin
            mytitle='Enter 2DS time offset N: (postive number means 2DS_Time_new=Time_original+N)'
            !TDS_time_offset=(Moqixu_input_single_number_pop_up(title=mytitle,label=mytitle,value=['0.000']))*1.0
        endif else begin
            !TDS_time_offset=0
        endelse
    endelse
    ;stop
    Do_overnight_auto_archiving_works_Tc4,info,'All',liqorice,my_selection;  do multiple file archive, clean all..
    !autoarchive_png_only=0
    !TDS_time_offset=0
    !Time_synced=0;       ; this includes extinction mass psds with it.
end
endcase
END_of_Archive_pngDrop_List:  ; Do nothing here.
Widget_Control, event.top, Set_UValue=info, /No_Copy  ; get our info structure
end
;*******************************************************

;******************************************************************
pro MakeAscii_droplist_event,event
    Common  DataBlock,alldataH,alldataV
    forward_function archive_ice_pop_up
    Widget_Control, event.top, Get_UValue=info, /No_Copy  ; get our info structure
    Widget_Control, event.id,  Get_UValue=uval  ; get our info structure
    ;***********************************************************
    case event.index of
        0:begin  ; this just displays teh help instructions
        rrr=dialog_message([['This droplits is for making particle by particle ascii files'],$
            ['user can choose to not clean--few particles get rejected as bad'],$
            ['user can choose to do liq clean--ice particles will get rejected '],$
            ['user can choose to do ice clean--- '],$
            ['user can load a bounch of files to let the computer to generate'],$
            ['an H-ascii and a V-Ascii file for each of the sorted***.2DS file'],$
            ['continue ?                        ']],/information)
        goto,  END_of_makeAscii_Drop_List
    end

    1:begin;   This is for liq_clean
    AllOrKps='All'
    liqorice='liq'
end

2:begin  ; This is for ice archive. TC4.
AllOrKps='All'
liqorice='ice'
end

;1:begin;   This is for no_clean
;  AllOrKps='NoClean'
;  liqorice='DontKnow'
;  end

;3:begin; This is for cleaning the noise only---keep the splash particles as good.
;  AllOrKps='KeepSplashes'
; liqorice='liq'
; end


5:begin; This is for cleaning the noise only---keep the splash particles as good.
AllOrKps='KeepSplashes'
liqorice='ice'
end
endcase
;next load a bunch of sorted files,
;then make bin-files and generated the zoomed particle by particle ascii files, overNight.

;first get all the roi files.
get_all_sorted_2DS_data_filenames,filenames,file_count

if file_count eq 0 then  goto, END_of_makeAscii_Drop_List


;Then use a loop to process each file.

for i=0, file_count-1 do begin
    Kth=i+1

    ;*********************************************************************************
    ;first assign the info and ten then call spec_toolbar_SAVE, info to generate all
    ;bin files

    ;Widget_Control, event.top, Get_UValue=info, /No_Copy  ; get our info structure
    filename = filenames(i);  dialog_pickfile(filter='SORTED*.2DS',  /Read)
    info.currentFname=filename
    substrsF=strsplit(filename,'\',/extract)
    short_filename=substrsF(n_elements(substrsF)-1)
    ;info.datestamp = strmid(filename,strpos(filename,"SORTED")+6,24); changed to 24 from 12.
    info.datestamp = strmid(Short_filename,6,strlen(Short_filename)-10)

    end_of_path = STRPOS(filename, '\', /REVERSE_SEARCH)
    info.currentPname =  STRMID(filename,0,end_of_path+1)

    ; folloowing line loads teh sorted filed and generate teh bin files.
    ; it can be comment out if the bin files have been generated before.
    ;look for dataKth directory in "info.currentPname\data\"  or info.currentPname\dataKth\ to see
    ; if there is data files in there. if no, there make new ones.

    currentPath=info.currentPname
    filter_str='Horiz'+info.datestamp+'.Bin';';   ;user comfirms
    filename=file_search(currentpath,filter_str,COUNT=file_count)


    if file_count eq 0 then begin
        SPEC_Toolbar_Save, info,Kth; change to this line on Nov. 14, 2007.
    endif
    ;now the bin files are generated, we need to load and open the bin-files do do the processing.
    ; in the next block.
    ;********************************************************************************

    ;Now open the BIN file and begin doing all of the processing
    SPEC_Toolbar_Load_archive,info

    Auto_clean_oneParticle_by_oneParticle,'H',info,AllOrKps,liqorice;  This is for cleaning all both ice or liq
    print,'Done H channel Cleaning '+AllOrKps+'****';         with user specified roundness
    Auto_clean_oneParticle_by_oneParticle,'V',info,AllorKps,liqorice
    print,'Done V channel Cleaning '+AllOrKps+'****'

    ; for the ave_info  data to pass to the subroutine.

    startnanoseconds=(*alldataH)(0).nanoseconds < (*alldataV)(0).nanoseconds
    endnanoseconds=(*alldataH)(n_elements((*alldataH).tas)-1LL).nanoseconds > (*alldataV)(n_elements((*alldataV).tas)-1LL).nanoseconds
    ave_info={startnanoseconds:startnanoseconds,endnanoseconds:endnanoseconds,$
        startTimestring:strtrim(string(startnanoseconds),2),$
        endtimestring:strtrim(string(endnanoseconds),2),$
        Hpartcont:n_elements((*alldataH).tas),Vpartcont:n_elements((*alldataV).tas),$
        binfilename:info.CURRENTFNAME}

    datestamp=info.datestamp
    ascii_info={currentPname:!currentHPname,currentFname:!currentHFname,datestamp:datestamp}

    generate_zoomed_ascii, ascii_info, ave_info

endfor

END_of_makeAscii_Drop_List:  ; Do nothing here.
Widget_Control, event.top, Set_UValue=info, /No_Copy  ; get our info structure

end;  ; end of makeAscii_droplist_event.pro



;==============================================================
;==============================================================
;==   MAIN PROCEDURE  a3vcpiview      ===========================
;==============================================================
;==============================================================
pro a3vcpiview, precip=precip
    ; keyword shattering for paul's shattering research, Precip for taking away has_big=1 requirement.
    ; above keywords are added on Feb 15, 2008.

    ; detect if 3V-CPI View is running in the IDL Virtual Machine and load relevant variables
    ; doing the restore in normal development IDL causes it to be difficult to change the system variables without commenting out the restore
    if(lmgr(/vm)) then begin
        restore, 'a3vcpiview_vars.sav' ; uncomment for generating the .sav for IDL Virtual Machine, otherwise leave commented out or there could be problems with system variables
    endif

    ; fetch the CPI resolution from ./Libraries/define_physical_constants.pro
    define_physical_constants

    ; TAF math error check
    EXCEPT = 1
    ; setting !EXCEPT = 2 causes undebuggable program hang

    DEFSYSV, '!PIXELS', 128 ; 128 pixels in 3V-CPI diode arrays
    DEFSYSV, '!zoom_factor', 3 ; the multiplier for the size of the zoomed images in the H, V strips
    DEFSYSV, '!zoom_xsize', !PIXELS * !zoom_factor ; x size of the zoom window
    DEFSYSV, '!zoom_ysize', !PIXELS * !zoom_factor ; y size of the zoom window
    DEFSYSV, '!NO_CHART', 0
    DEFSYSV, '!H_CHART', 1
    DEFSYSV, '!V_CHART', 2
    DEFSYSV, '!HOURGLASS_CURSOR' , 32514
    DEFSYSV, '!ARROW_CURSOR' , 32512
    DEFSYSV, '!STRIP_WINDOW' , 3
    DEFSYSV, '!STRIP_WIDTH' , 1000 ; 600
    DEFSYSV, '!CHAR_SIZE' , 0.4
    DEFSYSV, '!CurrentHPname' , ''
    DEFSYSV, '!CurrentHFname' , ''
    DEFSYSV, '!CurrentVPname' , ''
    DEFSYSV, '!CurrentVFname' , ''
    DEFSYSV, '!FOFFSET', 0UL  ; default to start of file
    DEFSYSV, '!START_FOFFSET', 0UL  ; default to start of file
    DEFSYSV, '!StereoFOFFset',[0UL,0UL]
    DEFSYSV, '!CRLF', string(13B) + string(10B)
    DEFSYSV, '!CURRENT_CHANNEL' , 1   ; 0=none, 1=horiz, 2=vert
    DEFSYSV, '!ZOOM_STATE' , 0b
    DEFSYSV, '!ZOOM_START_INDEX_H',0L
    DEFSYSV, '!ZOOM_END_INDEX_H', 0L
    DEFSYSV, '!ZOOM_START_INDEX_V',0L
    DEFSYSV, '!ZOOM_END_INDEX_V', 0L
    DEFSYSV, '!ZOOM_START_STRING' , ""
    DEFSYSV, '!ZOOM_END_STRING' , ""
    DEFSYSV, '!ZOOM_START_BIN_1HZ',0L
    DEFSYSV, '!ZOOM_END_BIN_1HZ', 0L
    DEFSYSV, '!DEADTIME_START_FLAG' , -100
    DEFSYSV, '!DEADTIME_END_FLAG' ,   -101
    DEFSYSV, '!MASK_START_FLAG' ,     -102
    DEFSYSV, '!MASK_END_FLAG' ,       -103
    DEFSYSV, '!TAS_FLAG' ,            -104
    DEFSYSV, '!CURRENT_VERSION',       1.0

    DEFSYSV, '!PLAYBACK_ACTIVE' , 1
    DEFSYSV, '!REWIND_ACTIVE' , 1
    DEFSYSV, '!PLAYBACK_SPEED', 0.10
    DEFSYSV, '!HI_LO_STATE' , 0   ; 0=low, 1=high

    DEFSYSV, '!ZOOM_START_NS',0ULL
    DEFSYSV, '!ZOOM_END_NS', 0ULL

    DEFSYSV, '!clean_status',0;  for celaning stuprious particles.

    DEFSYSV, '!Infocus_on',0;    for doing infocus only analysis.
    DEFSYSV, '!focuspercent',0.9; chosing infocus threshold.
    DEFSYSV, '!AutoArchive',0;   Program mode to supress the dialog_message.Dec 5, 2006.
    DEFSYSV, '!AutoArchive_png_only',0;   Program mode to supress the dialog_message.August 22, 2007.
    DEFSYSV, '!roundness_threshold1',0.5;for if (L1 gt 0.5*L5 and L5 gt 0.5*L1) then roundness=1
    DEFSYSV, '!roundness_threshold2',50.0;  for if (L1 gt 0.5*L5 and L1 gt 50_pixel) then roundness=1
    DEFSYSV, '!filter',0; if filter=0, all adj0=1; if filter=1, adj0=clean1's adj0 values. 1.0 to 1.1
    DEFSYSV, '!AutoArchive_method','M6';   added this in July 11, 2007 for TC4.
    DEFSYSV, '!archive_ice_pop_up_clicked',0;   make sure it is click at list once.
    DEFSYSV, '!Dt',0.0; all images flag
    DEFSYSV, '!set_has_big_always_eq_1',0;  When keyword precip is set, ignore_has_big_eq_1 is set  to 1.
    DEFSYSV, '!use_nav_tas_correct_L1n',0;  when keyword NavTAS is set, load Nav TAS to correct L1n alexei1 and area.
    ;DEFSYSV, '!w8_T_Shatter_factor',1.0; when Kweyword shattering is set, This values in assigned user input value.
    ;DEFSYSV, '!DWW_main', 50.8D ; mm Distance Window to Window for defining the Depth of Field (DOF), 50.8mm for 3V-CPI, 63.0mm for 2D-S
    ;DEFSYSV, '!DWW_2DS',  63.0D ; mm Distance Window to Window for defining the Depth of Field (DOF) 63.0mm for 2D-S
    DEFSYSV, '!DT_CPI',  0L ;  ; change this values  so that CPI image Time are in sync with all Other Times. airplane Time 2DS time , ect.
    DEFSYSV, '!TDS_time_offset',0
    DEFSYSV, '!Time_synced',0;  ; if Time is synced,  assign this value to 1.0



    ; save the startup directory for the local calls
    ; this is partly for IDL 8.0 compatibility
    cd, "", CURRENT = current_directory
    DEFSYSV, '!SPEC_3VCPI_DIR', current_directory

    DEFSYSV, '!calibration', 0; user must change this to 1 to do the bead test calibration to make the pixel psd and overlap plots.
    DEFSYSV, '!beadsize',   100; user must change to appropriate size to do teh calibration.

    DEFSYSV, '!poriginal', !P ; store the original !P plot settings for restoration

    ; TAF actually below might be wrong -- there is likely an adjust for the cone angle of the lenses, but this should be the same
    ; for this wavelength laser
    ; TAF 2011-12-14 a correction to the dof calculated above based on the ration in dww between 2D-S and 3V-CPI
    ; use this correction until we get the proper explanation of the Fdof coefficients (5.13 etc) from Brad
    ;DEFSYSV, '!Fdof_2DS', 5.13D ; 1/microns used for size dependent DOF correct ask Brad Baker how this number was derrived
    ;DEFSYSV, '!Fdof_3VCPI', 4.13657D ; 1/microns corrected from 2D-S 5.13 * 50.8 / 63.0
    if keyword_set(precip) then !set_has_big_always_eq_1=1
    if keyword_set(TAScorrect) then !use_nav_tas_correct_L1n=1
    DEFSYSV,'!TDS_time_offset',0
    ;DEFsysv,'!Time_Synced',0

    ; system variables for Roundness
    defsysv, '!Roundness1_min',  .8
    defsysv, '!Roundness1_max', 2.
    defsysv, '!Roundness2_min', 0.
    defsysv, '!Roundness2_max',  .2
    defsysv, '!Roundness3_min', 0.
    defsysv, '!Roundness3_max',  .25

    defsysv, '!roundness_criteria_checked', 0

    defsysv, '!showperimeterH', 0 ; used by zoomed H strip as a boolean for the Show Perimeter check boxes
    defsysv, '!showperimeterV', 0 ; used by zoomed V strip as a boolean for the Show Perimeter check boxes

    ;if keyword_set(shattering) then begin
    ;my_message=[['This Option is for Shattering Analysis, Next:  .........................'],$
    ;            ['Please input a factor K that will cleans up w8_t particles according to:'],$
    ;            [' if w8_t(I_temp) le k*C1 or w8_t(Next_I_temp) le k*C1 Then reject  or '],$
    ;            [' if both W8_t(I_temp) and W8_t(Next_I_temp) le k*C2 Then reject.      '],$
    ;            ['  where C1=smaller of {0.05*averaged_W8_t, 2000}  ...................'],$
    ;            ['        C2=smaller of {0.2 *avearged_w8_t,8000}  ....................']]
    ;rrr=dialog_message(my_message,/information)
    ;!w8_t_shatter_factor=(moqixu_input_single_number_pop_up(title='For Shattering Analysis Only',label='factor:',value='1.0'))*1.0
    ;endif

    Catch, onError
    IF onError NE 0 THEN BEGIN
        Catch, /Cancel
        result = Error_Message(Traceback=1)
        RETURN
    ENDIF

    IF N_Elements(currentFname) EQ 0 then currentFname = ""
    IF N_Elements(currentPname) EQ 0 then currentPname = ""
    IF N_Elements(currentFnameV) EQ 0 then currentFnameV = ""
    IF N_Elements(currentPnameV) EQ 0 then currentPnameV = ""
    datestamp = ""
    statusID=0
    ; Check for positional parameter. Define if necessary.
    ; Make sure it is correct size.

    IF N_Elements(image) EQ 0 THEN image = LoadData(7)
    ndim = Size(image, /N_Dimensions)
    IF ndim NE 2 THEN Message, '2D Image Variable Required.', /NoName

    ; Check for histogram keywords.

    IF N_Elements(binsize) EQ 0 THEN BEGIN
        range = Max(image) - Min(image)
        binsize = 2.0 > (range / 128.0)
    ENDIF

    IF N_Elements(max_value) EQ 0 THEN max_value = 5000.0
    IF N_Elements(title) EQ 0 THEN title = 'Histo_GUI Program'

    ; Check for image scale parameters.

    s = Size(image, /Dimensions)
    IF N_Elements(xscale) EQ 0 THEN xscale = [0, s[0]]
    IF N_Elements(xscale) NE 2 THEN Message, 'XSCALE must be 2-element array', /NoName
    IF N_Elements(yscale) EQ 0 THEN yscale = [0, s[1]]
    IF N_Elements(yscale) NE 2 THEN Message, 'YSCALE must be 2-element array', /NoName

    ; Check for color keywords.

    IF N_Elements(dataColorName) EQ 0 THEN dataColorName = "Red"
    IF N_Elements(axisColorName) EQ 0 THEN axisColorName = "Navy"
    IF N_Elements(backcolorName) EQ 0 THEN backcolorName = "White"
    IF N_Elements(colortable) EQ 0 THEN colortable = 4
    colortable = 0 > colortable < 40
    imagecolors = !D.Table_Size-4

    IF N_Elements(CurrentChart1) EQ 0 THEN CurrentChart1 = 0
    IF N_Elements(CurrentChart2) EQ 0 THEN CurrentChart2 = 0
    IF N_Elements(CurrentChart3) EQ 0 THEN CurrentChart3 = 0

    TotalParticles = 0UL



    ;@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
    tlb2= Widget_Base(Column=1, TLB_Size_Events=1, Title="SPEC 2DSView - Average Charts Version 0.59", MBar=menubarID2, /KBRD_Focus_Events,  /CONTEXT_EVENTS )
    drawID2 = Widget_Draw(tlb2,XSize=400, YSize=400, Event_Pro='SPEC_GraphClick2', /BUTTON_EVENTS, /MOTION_EVENTS )


    fileID2 = Widget_button(menubarID2, Value='File', /Menu)
    openID2 = Widget_button(fileID2, Value='Open', Event_Pro='SPEC_Open2')
    quitID2 = Widget_button(fileID2, Value='Quit', Event_Pro='SPEC_Quit2')


    BASE_Toolbar2 = WIDGET_BASE(tlb2, $
        COLUMN=3, $
        FRAME=1, $
        MAP=1, $
        UVALUE='BASE_Toolbar', $
        BASE_ALIGN_LEFT=1, $
        YSIZE=35)

    BUTTON_Open2 = WIDGET_BUTTON( BASE_Toolbar2, $
        FONT='9x15', $
        UVALUE='open_file', $
        XSIZE=35, $
        TOOLTIP="Open Raw File", $
        VALUE=!SPEC_3VCPI_DIR + '/SORTED_to_bin.bmp', /BITMAP, $
        Event_Pro='SPEC_Toolbar_Open2')

    BUTTON_Save2 = WIDGET_BUTTON(BASE_Toolbar2, $
        FONT='9x15', $
        UVALUE='save_file', $
        XSIZE=35, $
        TOOLTIP="Convert SORTED/PAIRS...2DSCPI files into .BIN files", $ ;TOOLTIP="Convert SORTED/PAIRS...2DS files into .BIN files", $
        VALUE=!SPEC_3VCPI_DIR + '/sort2.bmp', /BITMAP,  $
        Event_Pro='SPEC_Toolbar_Save2')

    BUTTON_New2 = WIDGET_BUTTON( BASE_Toolbar2, $
        FONT='9x15', $
        UVALUE='new_file', $
        XSIZE=35, $
        TOOLTIP="Load Processed .BIN Data for plotting", $
        VALUE=!SPEC_3VCPI_DIR + '/bin_open.bmp', /BITMAP)


    BUTTON_Plot2 = WIDGET_BUTTON( BASE_Toolbar2, $
        FONT='9x15', $
        UVALUE='new_file', $
        XSIZE=35, $
        TOOLTIP="Plot Data", $
        VALUE=!SPEC_3VCPI_DIR + '/graphs.bmp', /BITMAP)

    Widget_Control, drawID2, Get_Value=wid2
    WSet, wid2
    Widget_Control, tlb2, /Realize
    ;@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@


    ; Generate our main window for the graphs

    tlb = Widget_Base(Column=1, TLB_Size_Events=1, Title="SPEC 3VCPIView - Version " + !SPEC_VERSION, MBar=menubarID, /KBRD_Focus_Events,  /CONTEXT_EVENTS )
    ; , /KBRD_Focus_Events /CONTEXT_EVENTS
    ; Add menus

    fileID = Widget_button(menubarID, Value='File', /Menu)
    openID = Widget_button(fileID, Value='Open', Event_Pro='SPEC_Open')
    quitID = Widget_button(fileID, Value='Quit', Event_Pro='SPEC_Quit')



    BASE_Toolbar = WIDGET_BASE(tlb, $
        COLUMN=14, $;  added more columns to make more buttons.
        FRAME=1, $
        MAP=1, $
        UVALUE='BASE_Toolbar', $
        BASE_ALIGN_LEFT=1, $
        YSIZE=35)

    BUTTON_CPIView = WIDGET_BUTTON( BASE_Toolbar, $
        FONT='9x15', $
        UVALUE='CPIView_button', $
        XSIZE=35, $
        TOOLTIP="Open CPI Processing", $
        VALUE=!SPEC_3VCPI_DIR + '/snowflake1.bmp', /BITMAP, $
        Event_Pro='CPIView_button')

    BUTTON_basebatch = WIDGET_BUTTON( BASE_Toolbar, $
        FONT='9x15', $
        UVALUE='Batch', $
        XSIZE=45, $
        TOOLTIP="batch processing all base files to make ascii and Png pages", $
        VALUE='Batch', $
        Event_Pro='basescan_Batch')

    BUTTON_basescan= WIDGET_BUTTON( BASE_Toolbar, $
        FONT='9x15', $
        UVALUE='basescan_button', $
        XSIZE=35, $
        TOOLTIP="Convert base*.2DSCPI(HK) files to SORTED", $
        VALUE=!SPEC_3VCPI_DIR + '/base_to_SORTED.bmp', /BITMAP, $
        Event_Pro='basescan_button')


    BUTTON_Open = WIDGET_BUTTON( BASE_Toolbar, $
        FONT='9x15', $
        UVALUE='open_file', $
        XSIZE=35, $
        TOOLTIP="Convert SORTED....2DSCPI files into .BIN files", $;TOOLTIP="Convert SORTED....2DS files into .BIN files", $
        VALUE=!SPEC_3VCPI_DIR + '/SORTED_to_bin.bmp', /BITMAP, $
        Event_Pro='SPEC_Toolbar_Open')


    BUTTON_New = WIDGET_BUTTON( BASE_Toolbar, $
        FONT='9x15', $
        UVALUE='new_file', $
        XSIZE=35, $
        TOOLTIP="Load .BIN files for plotting", $
        VALUE= !SPEC_3VCPI_DIR + '/bin_open.bmp', /BITMAP, $
        Event_Pro='SPEC_Toolbar_Load')

    BUTTON_Plot_H = WIDGET_BUTTON( BASE_Toolbar, $
        FONT='9x15', $
        UVALUE='new_file', $
        XSIZE=35, $
        YSIZE=29, $
        TOOLTIP="Plot Data", $
        SENSITIVE = 0, $
        VALUE= "H", $ ; 'c:\graphs.bmp', /BITMAP, $
        Event_Pro='SPEC_Toolbar_Plot_H')


    BUTTON_Plot_V = WIDGET_BUTTON( BASE_Toolbar, $
        FONT='9x15', $
        UVALUE='new_file', $
        XSIZE=35, $
        YSIZE=29, $
        TOOLTIP="Plot Data", $
        SENSITIVE = 0, $
        VALUE= "V", $ ; 'c:\graphs.bmp', /BITMAP, $
        Event_Pro='SPEC_Toolbar_Plot_V')
    BUTTON_Plot_Zoom = WIDGET_BUTTON( BASE_Toolbar, $
        FONT='9x15', $
        UVALUE='new_file', $
        XSIZE=35, $
        YSIZE=29, $
        TOOLTIP="Zoom into Data", $
        VALUE= !SPEC_3VCPI_DIR + '/zoom.bmp', /BITMAP, $
        Event_Pro='SPEC_Toolbar_Zoom')

    BUTTON_Auto_clean_oneby_one = WIDGET_BUTTON( BASE_Toolbar, $
        FONT='9x15', $
        UVALUE='auto_clean_1by1', $
        XSIZE=60, $
        YSIZE=29, $
        TOOLTIP="Remove all noisy and shattered particles for the current loaded data file", $
        VALUE= 'CleanAll', $
        Event_Pro='Auto_clean_one_by_one_event')

    BUTTON_Auto_clean_oneby_one4 = WIDGET_BUTTON( BASE_Toolbar, $
        FONT='9x15', $
        UVALUE='autoClean4by4', $
        XSIZE=75, $
        YSIZE=29, $
        TOOLTIP="Remove_statisticNoise_But_keep_shattered particle for current loaded data file", $
        VALUE= 'CleanNoise', $
        Event_Pro='Auto_clean_one_by_one4_event')

    ;*******************************************************************************

    ;*******************************************************************************
    infocus= {   variables:ptr_new(),$
        vDlistID:0 $
        }
    ;************************************************************
    b341 = widget_base(Base_toolbar,/frame,/row)
    b341a=widget_base(b341,col=1,/nonexclusive)
    infocusID = widget_button(b341a, value='Options(default is all):', Event_pro='Infocus_event')
    widget_control, infocusID, set_button=0

    variables = ['use focus >=0.9 (in focus) ','use focus >=0.8',' use focus >=0.7']
    pvariables=ptr_new(strarr(n_elements(variables)))
    *pvariables=variables
    infocus.variables=pvariables
    infocus.vDListID = widget_droplist(b341, value=variables, Event_Pro='infocus_droplist_event')

    ;*************************************************************

    b342 = widget_base(Base_toolbar,/frame,/row)
    ; A_variables = ['Archive_this_liq_ascii','Archive_Multi_liq_ascii','Archive_this_ice_ascii','Archive_multi_ice_ascii',$
    ;               'Archive_multi_ice_ascii_WS','Archive_multi_ice_ascii_NC',$
    ;              'Archive_multi_liq_ascii_WS','Archive_multi_liq_ascii_NC']
    ; A_variables = ['Archive_this_ice_ascii','Archive_multi_ice_ascii'];; use this since 2015-07-13

    A_variables = ['Archive_this_ascii','Archive_Multi_ascii']; 20160208
    Archive_DListID = widget_droplist(b342,title='',value=A_variables,Event_Pro='archive_droplist_event')

    b343 = widget_base(Base_toolbar,/frame,/row)
    A_variables = ['Archive_this_png','Archive_multi_png']
    ;A_variables = ['Archive_this_png','Archive_multi_ice_png']  ; use this since 20150713
    Archive_DListID = widget_droplist(b343,title='',value=A_variables,Event_Pro='archive_pngdroplist_event')

    ; added following in feb. 16th, 2009.
    b344 = widget_base(Base_toolbar,/frame,/row)
    ;A_variables = ['Make_N_ascii_help','Make_N_ascii_NoClean','Make_N_ascii_liq','Make_N_ascii_liq_WS','Make_N_ascii_ice','Make_N_ascii_ice_WS']
    ;A_variables = ['Make_N_ascii_help','Make_N_ascii_ice'];  ; use this since 2015-07-13
    A_variables = ['Make_N_ascii_help','Make_N_ascii_liq','Make_N_ascii_ice']

    Archive_DListID = widget_droplist(b344,title='',value=A_variables,Event_Pro='MakeAscii_droplist_event')

    ;************************************************************************
    starting_window_x = 1040  ; changed from 900 to 1050 on 2015-07-21
    starting_window_y = 500

    drawID = Widget_Draw(tlb,XSize=starting_window_x, YSize=starting_window_y, Event_Pro='SPEC_GraphClick', /BUTTON_EVENTS, /MOTION_EVENTS, KEYBOARD_EVENTS=2)
    Widget_Control, tlb, /Realize


    ; TAF added to prevent empty Window 0 from popping up from device call
    Widget_Control, drawID, Get_Value=wid
    WSet,  wid
    WShow, wid

    device, CURSOR_STANDARD= !ARROW_CURSOR

    ; Bring main window to current focus

    Widget_Control, drawID, Get_Value=wid
    WSet, wid



    ;   SpecGraph, image, $
    ;   AxisColorName = axisColorName, $
    ;   BackColorName = backcolorName, $
    ;   Binsize = binsize, $
    ;   ColorTable=colortable, $
    ;   DataColorName=datacolorName, $
    ;   _Extra=extra, $
    ;   ImageColors=imagecolors, $
    ;   Max_Value=max_value, $
    ;   XScale=xscale, $
    ;   YScale=yscale

    erase,color='ffffff'x  ; get background to white

    win2 = 0L
    stripID = 0L
    zoomID = 0L
    stripBase = 0L
    stripLabel = 0L

    aveID = 0L
    aveBase = 0L
    aveLabel = 0L

    ZoomWindowID = 0L
    ZoomWindowDrawID = 0L






    HButton = BUTTON_Plot_H
    VButton = BUTTON_Plot_V

    NanosPerBin = 0ULL
    NumOneHzBins = 0ULL

    ; Create the info structure with the information
    ; required to run the program.

    info = { image:Ptr_New(image), $          ; A pointer to the image data.
        axisColorName:axisColorName, $   ; The name of the axis color.
        backColorName:backcolorName, $   ; The name of the background color.
        binsize:binsize, $               ; The histogram bin size.
        dataColorName:datacolorName, $   ; The name of the data color.
        imageColors:imagecolors, $       ; The number of colors used to display the image.
        max_value:max_value, $           ; The maximum value of the histogram plot.
        xscale:xscale, $                 ; The X scale of the image axis.
        title:title, $                   ; The window title.
        yscale:yscale, $                 ; The Y scale of the image axis.
        extra:Ptr_New(extra), $          ; A pointer to "extra" keywords.
        drawID:drawID, $                 ; The identifier of the draw widget.
        currentFname:currentFname, $     ; If not "", then the currently opened file
        currentPname:currentPname, $     ; If not "", then the currently opened file
        currentFnameV:currentFnameV, $     ; If not "", then the currently opened file
        currentPnameV:currentPnameV, $     ; If not "", then the currently opened file
        datestamp:datestamp, $           ; the yymmddhhmmss part of the file we're currently working on
        statusID:statusID, $             ; for access to status bar from events
        TotalParticles:TotalParticles, $
        win2:win2, $
        stripID:stripID, $               ; widget ID for the H-strip of bitmaps
        zoomID:zoomID, $
        stripBase:stripBase, $
        stripLabel:stripLabel, $
        VstripID:0L, $              ; widget ID for the V-strip of bitmaps
        VstripBase:0L, $            ;widget ID for the V-strip of bitmaps title  --Mo Nov 30, 2005
        VstripLabel:stripLabel, $
        VzoomID:0L, $
        aveID:aveID, $               ; widget ID for the average charts
        aveBase:aveBase, $
        aveLabel:aveLabel, $
        NanosPerBin:NanosPerBin, $        ; # of summed 1Hz bins in each bin of the main time series display (as nanoseconds)
        NumOneHzBins:NumOneHzBins, $      ; # of 1Hz bins in OneHzH and OneHzV (same # for each array, same as n_elements(OneHzH)
        CurrentChart1:CurrentChart1, $
        CurrentChart2:CurrentChart2, $
        CurrentChart3:CurrentChart3, $
        HButton:Hbutton, $
        VButton:VButton, $
        ZoomWindowID:ZoomWindowID, $
        ZoomWindowDrawID:ZoomWindowDrawID, $
        wid:wid, $                        ; The window index number of the graphics window.
        infocus:infocus $
        }



    info.currentFname = ""
    info.CurrentChart1 = 0
    info.CurrentChart2 = 0
    info.CurrentChart3 = 0
    info.wid = tlb
    info.win2 = tlb2



    ; set xscale and yscale to the size of the window (and change this when we resize)
    info.xscale(1) = starting_window_x
    info.yscale(1) = starting_window_y

    ; Add the Status bar to the bottom of the window
    BASE_Status = WIDGET_BASE(tlb, $
        COLUMN=3, $
        FRAME=1, $
        MAP=1, $
        UVALUE='BASE_Status', $
        ;BASE_ALIGN_CENTER=1, $
        ;ALIGN_BOTTOM=1, $
        YSIZE=20      )

    info.statusID = WIDGET_LABEL(BASE_Status,VALUE="Status bar                         ")


    ; Store the info structure in the user value of the TLB.

    Widget_Control, tlb, Set_UValue=info, /No_Copy


    widget_control, tlb2, /destroy

    ; Register the event loop

    XManager, 'a3vcpiview', tlb, /No_Block, Event_Handler='a3vcpiview_Events', $
        Cleanup = 'a3vcpiview_Cleanup'

    cd, !SPEC_3VCPI_DIR


END ; a3vcpiview ;==================================================================

pro  get_all_data_filenames,filter_str,pathnm,filenames,file_count
    ;input is filter_str, pathnm
    ;output is filenames,file_count
    substrs=strsplit(filter_str,'.',/extract)
    extension=substrs(1)

    whole_filter_str=pathnm+filter_str
    filenames=findfile(whole_filter_str,COUNT=file_count)
    ;find all files with filter_str and counts the number of these files

    ; display the files in the screen for user to review
    print,'***********************************************************'
    print,'********  Here are data files with ',filter_str, '   ******'
    print,'*** found in current directory for input by this program ***'
    print,'************************************************************'

    for j=0, (file_count-1)  do begin
        print, filenames(j)
    endfor


    print,'                            '
    print,' done,get the files,.....Processing,  please wait ! '
    print,'
end; endof get_all_data_filenames,filter_str,filenames,pathnm
;****************************************************************************
